c  Fichier base_qua_c.f
c
c************************************************************
      subroutine FBASEC_2D_c(BASEC,det,N1,N2,N3,N4,xch,ych,x,y)
c************************************************************
c
c   Cette procedure calcule la valeur des quatres fonctions de bases
c   au point (xch,ych) appartenant au carre de reference, ainsi que
c   le jacobien de l'application: carre de reference --> quadrilatere.
c
c   BASEC: tableau contenant les valeurs des quatres fonctions de 
c          base au point (xch,ych)
c   det: jacobien de la transformation: carre de reference --> quadrilatere
c   N1,N2,N3,N4: contient les numeros, ainsi que les coordonnees des 
c                quatre points qui constituent le quadrilatere
c   xch,ych: coordonnees du point de calcul dans le carre de reference
c   x,y: coordonnees du point dans le quadrilatere
c
c
      implicit none

      real*8    BASEC,xch,ych,det,N1,N2,N3,N4
      real*8    J11,J12,J21,J22,a1,a2,b1,b2,x,y
      dimension BASEC(*),N1(*),N2(*),N3(*),N4(*)
      
      a1=(1.d0+xch)
      a2=(1.d0-xch)
      b1=(1.d0+ych)
      b2=(1.d0-ych)
c
c fonctions de bases
c
      BASEC(1)=(a2*b2)/4.d0
      BASEC(2)=(a1*b2)/4.d0
      BASEC(3)=(a1*b1)/4.d0
      BASEC(4)=(a2*b1)/4.d0
c
c coordonnees reelle du point de calcul
c
      x=(BASEC(1)*N1(2)+BASEC(2)*N2(2)+BASEC(3)*N3(2)+BASEC(4)*N4(2))
      y=(BASEC(1)*N1(3)+BASEC(2)*N2(3)+BASEC(3)*N3(3)+BASEC(4)*N4(3))
c
c calcul du jacobien de la transformation
c
      J11=-N1(2)+N2(2)+N3(2)-N4(2)+ych*(N1(2)-N2(2)+N3(2)-N4(2))
      J11=J11/4.d0

      J12=-N1(3)+N2(3)+N3(3)-N4(3)+ych*(N1(3)-N2(3)+N3(3)-N4(3)) 
      J12=J12/4.d0

      J21=-N1(2)-N2(2)+N3(2)+N4(2)+xch*(N1(2)-N2(2)+N3(2)-N4(2))
      J21=J21/4.d0

      J22=-N1(3)-N2(3)+N3(3)+N4(3)+xch*(N1(3)-N2(3)+N3(3)-N4(3))
      J22=J22/4.d0

      det=(J11*J22)-(J21*J12)
               
      RETURN
      end                                        

