C Fichier gradcp_c.f
C
c********************************************************************
c            FICHIERS NECESSAIRES A LA SUBROUTINE gradc
c********************************************************************

      INCLUDE'prod_mat_c.f'
      INCLUDE'resol_sys_c.f'

C***********************************************************************
      SUBROUTINE GradC_c(n,aa,ia,ja,al,b,x,g,h,cg,kMax,eps)
C***********************************************************************
c
C Ce sous-programme resout le systeme lineaire Ax=b par
C gradient conjugue preconditionne, avec au plus kMax 
C iterations. Le test d"arret est: |Ax^k-b|<eps|Ax^0-b|.
C La matrice A du systeme lineaire, de taille n*n
C est stockee sous forme Morse dans le tableau aa;
C ce stockage necessite aussi les tableaux auxiliaires
C d"indices ia et ja.
C La matrice C de preconditionnement est la factorisation
C de Choleski incomplete de A (C=L^* (L^*)^{T})  et la
C matrice L^* est stockee sous forme Morse dans le 
C tableau al, avec les memes tableaux d"indices ia et 
C ja que A.
C
C  aa: matrice A du systeme stockee sous forme Morse avec 
C      comme tableau d'indices ia et ja
C  al: factorisee de Choleski incomplete de A avec 
C      comme tableau d'indices ia et ja 
C  b: second membre du systeme lineaire
C  kMax: nombre maximal d"iterations
C  eps: precision du calcul
C  x:tableau contenant les valeurs de la solution 
C  gcg:scalaire contenant le carre de la norme de
C      g (en pratique ce nombre doit decroitre au 
C         cours des iterations).
C  g, cg: tableaux contenant les valeurs de g^n et
C         de C g^n respectivement;
C  h: tableau contenant la direction de descente h^n

      implicit none
      INTEGER n,ia,ja,kmax,i,k
      REAL*8  aa,al,b,x,g,h,cg,gcg,hah,ro,gamma,eps

      DIMENSION x(*),b(*),g(*),h(*),cg(*)
      DIMENSION aa(*),ja(*),ia(*),al(*)

      DO  k=1, kMax

        IF (k.EQ.1) THEN
          CALL IAmul_c(n,aa,ia,ja,x,h)
          DO  i=1,n
            cg(i)=b(i)-h(i)
            g(i)=cg(i)
          enddo
          CALL ISolvSys_c(n,g,al,ia,ja)
          gcg=0.d0
          DO  i=1,n
            gcg=gcg + g(i)*cg(i)
            h(i)=g(i)
          enddo
        ENDIF
       
        CALL IAmul_c(n,aa,ia,ja,h,b)
        hAh=0.d0
        DO i=1,n		
          hAh=hAh+h(i)*b(i)
        enddo
        ro=gcg/hAh
        DO  i=1,n
          cg(i) = cg(i) -ro*b(i)
        enddo
        CALL ISolvSys_c(n,b,al,ia,ja)
        DO  i=1,n
          x(i)=x(i)+ro*h(i)
          g(i)=g(i)- ro*b(i)
        enddo
        gamma=gcg
        gcg=0.d0
        DO  i=1,n
          gcg=gcg + g(i)*cg(i)
        enddo

        IF (gcg.LT.eps) THEN
c            write(*,*)'nombre d''iteration:',k,'  gcg=',gcg             
                         RETURN
        endif

        gamma=gcg/gamma
        DO  i=1,n
          h(i)=g(i)+gamma*h(i)
        enddo
      enddo

      write(*,*)'le solveur gradiant conjugue ne converge pas.' 
      write(17,*)'le solveur gradiant conjugue ne converge pas.'
      STOP

      END

