import java.awt.*;
import java.awt.event.*;

public class BuildAMapDialog extends PositionalDialog implements ActionListener {

  public BuildAMapDialog(Frame parent, String title, int nx, int ny, int nz) {

    // set up the dialog box
    super(parent, title, true, WIDTH, HEIGHT);
    setBackground(color);
    state = UNSELECTED;

    nmapx = nx;
    nmapy = ny;
    nmapz = nz;

    Panel p1 = new Panel();
    p1.setLayout(new GridLayout(0,1));
    Panel p2 = new Panel();
    p2.setLayout(new GridLayout(1,0));

    // set up the panels
    p1.add(label);
    p1.add(gridLabel);

    p2.add(new Label("X"));
    p2.add(xBlocksTextField);
    p2.add(new Label("Y"));
    p2.add(yBlocksTextField);
    p2.add(new Label("Z"));
    p2.add(zBlocksTextField);
    xBlocksTextField.setText(new Integer(nmapx).toString());
    xBlocksTextField.setBackground(Color.white);
    xBlocksTextField.setForeground(Color.black);
    yBlocksTextField.setText(new Integer(nmapy).toString());
    yBlocksTextField.setBackground(Color.white);
    yBlocksTextField.setForeground(Color.black);
    zBlocksTextField.setText(new Integer(nmapz).toString());
    zBlocksTextField.setBackground(Color.white);
    zBlocksTextField.setForeground(Color.black);

    Panel p3 = new Panel();
    p3.setLayout(new GridLayout(1,0));
    p3.add(new Label("Background Map Value"));
    p3.add(backgroundValueTextField);
    backgroundValueTextField.setBackground(Color.white);
    backgroundValueTextField.setForeground(Color.black);

    Panel p4 = new Panel();
    p4.setLayout(new GridLayout(1,0));
    okay = new Button("Okay");
    cancel = new Button("Cancel");
    p4.add(okay);
    p4.add(cancel);

    setSize(WIDTH, HEIGHT);

    // set the layout
    GridBagLayout gb = new GridBagLayout();
    GridBagConstraints gbc = new GridBagConstraints();
    int insets = 2;
    gbc.insets = new Insets(insets,insets,insets,insets);
    gbc.gridheight = 10;
    gbc.gridwidth = 40;
    setLayout(gb);

    // add the current folder information pieces
    gbc.gridx = 0;
    gbc.gridy = 0;
    gb.setConstraints(p1,gbc);
    add(p1);
    gbc.gridx = 0;
    gbc.gridy = 10;
    gb.setConstraints(p2, gbc);
    add(p2);
    gbc.gridx = 0;
    gbc.gridy = 20;
    gb.setConstraints(p3, gbc);
    add(p3);
    gbc.gridx = 0;
    gbc.gridy = 30;
    gb.setConstraints(p4, gbc);
    add(p4);

    okay.addActionListener(this);
    cancel.addActionListener(this);

    requestFocus();

  }

  public boolean getState () {
    if (state == OK_RESPONSE) return true;
    else return false;
  }

  public int getNumberOfXBlocks() {
    if(state == OK_RESPONSE) {
      int nx;
      String temp = xBlocksTextField.getText();
      // try to build the integer from the text field
      if (temp != null && !temp.equals("")) {
        try{nx = Integer.valueOf(temp).intValue();
        } catch (NumberFormatException nfe) {
          System.out.println("Number Format Exception");
          return -1;
        }
        return nx;
      } else {
        return -1;
      }
    } else {
      return 0;
    }
  }

  public int getNumberOfYBlocks() {
    if(state == OK_RESPONSE) {
      int ny;
      String temp = yBlocksTextField.getText();
      // try to build the integer from the text field
      if (temp != null && !temp.equals("")) {
        try{ny = Integer.valueOf(temp).intValue();
        } catch (NumberFormatException nfe) {
          System.out.println("Number Format Exception");
          return -1;
        }
        return ny;
      } else {
        return -1;
      }
    } else {
      return 0;
    }
  }

  public int getNumberOfZBlocks() {
    if(state == OK_RESPONSE) {
      int nz;
      String temp = zBlocksTextField.getText();
      // try to build the integer from the text field
      if (temp != null && !temp.equals("")) {
        try{nz = Integer.valueOf(temp).intValue();
        } catch (NumberFormatException nfe) {
          System.out.println("Number Format Exception");
          return -1;
        }
        return nz;
      } else {
        return -1;
      }
    } else {
      return 0;
    }
  }

  public Double getBackgroundValue() {
    Double value = new Double(0.0);
    String temp = backgroundValueTextField.getText();
    // try to build the Double object from the text field
    if (temp != null && !temp.equals("")) {
      try{value = Double.valueOf(temp);
      } catch (NumberFormatException nfe) {
        return value;
      }
      return value;
    } else {
      return value;
    }
  }

  public void actionPerformed (ActionEvent event) {
    Object source = event.getSource();

    if (source == okay) {
      state = OK_RESPONSE;
      setVisible(false);
    } else if (source == cancel) {
      state = CANCEL_RESPONSE;
      setVisible(false);
    }
  }

  public static void main(String args[]) {
    Frame f = new Frame("BuildAMap Object Test Frame");
    BuildAMapDialog bam = new BuildAMapDialog(f,"test string", 4, 3, 2);
    bam.pack();
    bam.show();
    // get the data from the object upon return
    System.out.println("State: " + bam.getState()); 
    System.out.println("X Blocks: " + bam.getNumberOfXBlocks()); 
    System.out.println("Y Blocks: " + bam.getNumberOfYBlocks()); 
    System.out.println("Z Blocks: " + bam.getNumberOfZBlocks()); 
    System.out.println("Background Value: " + bam.getBackgroundValue()); 
    bam.dispose();
    System.exit(0);
  }

  // local variables

  // variables used to store user input
  int nmapx;
  int nmapy;
  int nmapz;
  Double [] mapValues;

  // selection defaults
  private final int UNSELECTED = -1;
  private final int OK_RESPONSE = 1;
  private final int CANCEL_RESPONSE = 0;

  private static final int WIDTH = 350;
  private static final int HEIGHT = 170;

  private Color color = new Color(96, 128, 255);
  private Label label = new Label("Map Set Up Input");
  private Label gridLabel = new Label("Grid Input");
  private TextField xBlocksTextField = new TextField(2);
  private TextField yBlocksTextField = new TextField(2);
  private TextField zBlocksTextField = new TextField(2);
  private TextField backgroundValueTextField = new TextField(2);

  // stuff for ending the function of the object
  private Button           okay, cancel;
  private int              state;

}
