import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.StringTokenizer;

public class EllipticProblemDataInputDialog extends Dialog
         implements ActionListener, ItemListener {

  public EllipticProblemDataInputDialog(Frame parent, int xoff, int yoff) {
    super(parent, "BC Input", true);
    this.parent = parent;

    panel = new Panel();
    panel.setLayout(new BorderLayout());
    panel.setBackground(panelColor);

      checkboxPanel = new Panel();
      checkboxPanel.setLayout(new GridLayout(0,1));
        pointSourceCheckbox = new Checkbox(POINT_SOURCE, true);
        pointSourceCheckbox.addItemListener(this);
        pointSourceCheckbox.setBackground(labelColor);
          bcType = POINT_SOURCE;
        xFluxCheckbox = new Checkbox(X_FLUX, false);
        xFluxCheckbox.addItemListener(this);
        xFluxCheckbox.setBackground(labelColor);
        yFluxCheckbox = new Checkbox(Y_FLUX, false);
        yFluxCheckbox.addItemListener(this);
        yFluxCheckbox.setBackground(labelColor);
        zFluxCheckbox = new Checkbox(Z_FLUX, false);
        zFluxCheckbox.addItemListener(this);
        zFluxCheckbox.setBackground(labelColor);
      checkboxPanel.add(pointSourceCheckbox);
      checkboxPanel.add(xFluxCheckbox);
      checkboxPanel.add(yFluxCheckbox);
      checkboxPanel.add(zFluxCheckbox);

      buttonPanel = new Panel();
      buttonPanel.setLayout(new GridLayout(1,0));
      buttonPanel.setBackground(panelColor);
        okay = new Button("Okay");
        okay.addActionListener(this);
        quit = new Button("Quit");
        quit.addActionListener(this);
      buttonPanel.add(okay);
      buttonPanel.add(quit);

    panel.add("Center", checkboxPanel);
    panel.add("South", buttonPanel);

    state = GOOD;

    setLayout(new GridLayout(0,1));

    setBackground(panelColor);
    add(panel);

    // grab the font information
    width = 175;
    height = 160;

    setSize(width, height);

    // Position it centered over the parent Frame
    if (parent != null) {
      Dimension d = parent.getSize();
      int xOffset = d.width / 2 - width / 2;
      int yOffset = d.height / 2 - height / 2;
      if(xOffset < 0) xOffset = 0;
      if(yOffset < 0) yOffset = 0;
      Point p = parent.getLocation();
      p.translate(xOffset+xoff, yOffset+yoff);
      setLocation(p);
    }

  }

  public boolean getState () {
    return state;
  }

  public String getEllipticProblemData() {
    return bcType;
  }

  public void setEllipticProblemData(String s) {
    bcType = s;
    if(bcType.equals(POINT_SOURCE)) {
      pointSourceCheckbox.setState(true);
      xFluxCheckbox.setState(false);
      yFluxCheckbox.setState(false);
      zFluxCheckbox.setState(false);
    } else if(bcType.equals(X_FLUX)) {
      pointSourceCheckbox.setState(false);
      xFluxCheckbox.setState(true);
      yFluxCheckbox.setState(false);
      zFluxCheckbox.setState(false);
    } else if(bcType.equals(Y_FLUX)) {
      pointSourceCheckbox.setState(false);
      xFluxCheckbox.setState(false);
      yFluxCheckbox.setState(true);
      zFluxCheckbox.setState(false);
      bcType = Y_FLUX;
    } else if(bcType.equals(Z_FLUX)) {
      pointSourceCheckbox.setState(false);
      xFluxCheckbox.setState(false);
      yFluxCheckbox.setState(false);
      zFluxCheckbox.setState(true);
    }
  }

  public void actionPerformed (ActionEvent event) {
    Object source = event.getSource();
    if(source == okay) setVisible(false);
    if(source == quit) {
      state = BAD;
      setVisible(false);
    }
  }

  public void itemStateChanged(ItemEvent event) {
    Object source = event.getSource();

    if(source == pointSourceCheckbox) {
      pointSourceCheckbox.setState(true);
      xFluxCheckbox.setState(false);
      yFluxCheckbox.setState(false);
      zFluxCheckbox.setState(false);
      bcType = POINT_SOURCE;
    }

    if(source == xFluxCheckbox) {
      pointSourceCheckbox.setState(false);
      xFluxCheckbox.setState(true);
      yFluxCheckbox.setState(false);
      zFluxCheckbox.setState(false);
      bcType = X_FLUX;
    }

    if(source == yFluxCheckbox) {
      pointSourceCheckbox.setState(false);
      xFluxCheckbox.setState(false);
      yFluxCheckbox.setState(true);
      zFluxCheckbox.setState(false);
      bcType = Y_FLUX;
    }

    if(source == zFluxCheckbox) {
      pointSourceCheckbox.setState(false);
      xFluxCheckbox.setState(false);
      yFluxCheckbox.setState(false);
      zFluxCheckbox.setState(true);
      bcType = Z_FLUX;
    }

  }

  public static void main(String args[]) {
    // test for the inclusion of an argument which should be the location
    // of the jar file on PC implementations
    // pretty simple test
    Frame f = new Frame("Input Dialog Test");
    EllipticProblemDataInputDialog bci1
                  = new EllipticProblemDataInputDialog(f, 100, 100);
    bci1.show();
    System.out.println("Selection State:    " + bci1.getState());
    System.out.println("BC Type:            " + bci1.getEllipticProblemData());
    System.exit(0);

  }

  // local variables
                                        
  private static final int WIDTH = 350;
  private static final int HEIGHT = 200;
  private int width;
  private int height;

  // input gui stuff
  private Frame parent;
  private Panel panel;

    private Panel checkboxPanel;
      private CheckboxGroup bcCheckbox;
        private Checkbox pointSourceCheckbox;
        private Checkbox xFluxCheckbox;
        private Checkbox yFluxCheckbox;
        private Checkbox zFluxCheckbox;

    private Panel buttonPanel;
      private Button okay;
      private Button quit;

  // color variables
  private Color backGroundColor = Color.white;
  private Color foreGroundColor = Color.black;
  private Color panelColor = new Color(196,196,132);
  private Color labelColor = new Color(110,132,255);

  private boolean state;
  private final boolean BAD = false;
  private final boolean GOOD = true;

  public String bcType = "Point Source";
  public final static String POINT_SOURCE = "Point Source";
  public final static String X_FLUX = "X Direction Flux";
  public final static String Y_FLUX = "Y Direction Flux";
  public final static String Z_FLUX = "Z Direction Flux";

}
