import java.io.*;
import java.util.Vector;

public class ExternalProcess {

  public ExternalProcess(String s, int st) {
    errorMessages = new Vector();
    command = s;
    if(command != null) runCommand();
    sleepTime = st;
    if(sleepTime <= 0) sleepTime = MINIMUM_SLEEP_TIME;
    if(sleepTime > MAXIMUM_SLEEP_TIME) sleepTime = MAXIMUM_SLEEP_TIME;
  }

  public void setCommand(String s) {
    command = s;
  }

  public String getCommand() {
    return command;
  }

  public void killCommand() {
    proc.destroy();
  }

  public void runCommand() {

    try{
      // get the process and input and output streams
      proc = Runtime.getRuntime().exec(command);
      input = proc.getInputStream();
      error = proc.getErrorStream();
      output = proc.getOutputStream();

      startInput();
      startError();
      startOutput();

      // outside the independent threads
      proc.waitFor();
      exitValue = proc.exitValue();
      if(debug) System.out.println("Process Exited - exit value = " +exitValue);

    } catch(IOException ioe) {}
      catch(InterruptedException ie) {}

    // kill the process - to restart or rerun use runCommand
    if(proc != null) proc.destroy();
    proc = null;

  }

  public void startInput() {
    // start of the input information
    new Thread(
        new Runnable() {
          public void run() {
            try {
              int len = input.read(buf);
              while(len > 0) {
              // do what you want with this stuff
                if(debug) System.out.println("output from process \n\n"
                                      + new String(buf,0,len));
                len = input.read(buf);
              }
            } catch(IOException ioe) {}
          }
        }).start();
  }

  public void startError() {
    // start of the error information
    new Thread(
        new Runnable() {
          public void run() {
            try {
              int len = error.read(ebuf);
              while(len > 0) {
                // do what you want with this stuff
                String s = new String(ebuf,0,len);
                if(debug) System.out.println("error from process: \n\n" + s);
                errorMessages.add(s);
                len = error.read(ebuf);
              }
            } catch(IOException ioe) {}
          }
        }).start();
  }

  public void startOutput() {
    // start of the output information
    new Thread(
        new Runnable() {
          public void run() {
            try {
              Thread.sleep(sleepTime);
              if(debug) System.out.println("About to ping.....");
              output.write(("ping"
                      + System.getProperty("line.separator")).getBytes());
              output.flush();
            } catch(InterruptedException ie) {}
              catch(IOException ioe) {}
          }
        }).start();
  }

  public boolean getStatus() {
    if(proc == null) {
      return false;
    } else {
      return true;
    }
  }

  public int getExitValue() {
    return exitValue;
  }

  public void setDebug(boolean b) {
    if(b) {
      debug = true;
    } else {
      debug = false;
    }
  }

  public Vector getErrorMessages() {
    return errorMessages;
  }
 
  public static void main(String args[]) {
    ExternalProcess ep = new ExternalProcess("ls -al", 900);
    ep.setCommand("echo ok");
    ep.runCommand();
  }

  // local variables
  private byte [] buf = new byte[1000];
  private byte [] ebuf = new byte[1000];
  private Process proc;
  private InputStream input;
  private InputStream error;
  private OutputStream output;
  private String command = null;
  private Vector errorMessages = null;

  private int exitValue = 0;

  private int sleepTime;
  private static final int MINIMUM_SLEEP_TIME = 100;
  private static final int MAXIMUM_SLEEP_TIME = 5000;

  private boolean debug = false;

}
