import java.awt.*;
import java.awt.event.*;

public class HomogenizationMaskDialog extends PositionalDialog
    implements ActionListener, MouseListener {

  public HomogenizationMaskDialog(Frame p,
                                  String [] m,
                                  int dmi,
                                  int nx,
                                  int ny,
                                  int nz) {
    this(p, null, m, dmi, nx, ny, nz);
  }

  public HomogenizationMaskDialog(Frame p,
                                  int [] hm,
                                  String [] m,
                                  int nx,
                                  int ny,
                                  int nz) {
    this(p, hm, m, 0, nx, ny, nz);
  }

  public HomogenizationMaskDialog(Frame p,
                                  int [] hm,
                                  String [] m,
                                  int dmi,
                                  int nx,
                                  int ny,
                                  int nz) {
    // set up the dialog box
    super(p, "Homogenization Mask", true, WIDTH, HEIGHT);
    parent = p;

    // set up the homogenization grid
    this.nx = nx;
    this.ny = ny;
    this.nz = nz;
    nxny = nx * ny;
    nhom = nxny * nz;
    // set the inital slice to look at
    slice = 0;

    // get the averaging methods
    this.methods = m;
    noOfMethods = m.length;

    // test the input mask array to see if it is there and the correct
    // size
    boolean hmok = true;
    if(hm == null) {
      hmok = false;
    } else {
      if(hm.length != nhom) hmok = false;
    }

    // if the input mask is ok use it, otherwise set up a new one
    if(hmok) {
      homogenizationMask = hm;
      currentMethodIndex = hm[0];
    } else {
      homogenizationMask = new int [nhom];
      for(int i=0; i<nhom; i++) homogenizationMask[i] = currentMethodIndex;
      currentMethodIndex = dmi;
    }
    if(dmi < 1) currentMethodIndex = 1;
    if(dmi > methods.length) currentMethodIndex = methods.length;
    currentSliceMask = new int [nhom];

    // set up the gui
    displayPanel = new Panel();
    displayPanel.setLayout(new BorderLayout());

      blockByBlockLabel = new Label("Block Description of Averaging Methods");

      methodCanvas = new ImageCanvas(canvasWidth, canvasHeight);
      methodCanvas.addMouseListener(this);

      buttonPanel = new Panel();
      buttonPanel.setLayout(new GridLayout(0,1));
        methodButtons = new Button [noOfMethods];
        for(int i=0; i<noOfMethods; i++) {
          methodButtons[i] = new Button(methods[i]);
          methodButtons[i].setBackground(colorArray[i]);
          methodButtons[i].addActionListener(this);
          buttonPanel.add(methodButtons[i]);
        }

      controlPanel = new Panel();
      controlPanel.setLayout(new GridLayout(1,0));
        okay = new Button("Okay");
        okay.addActionListener(this);
        sliceTextField = new TextField("1", 4);
          sliceTextField.setBackground(Color.white);
          sliceTextField.setForeground(Color.black);
        previous= new Button("Previous");
        previous.addActionListener(this);
        next = new Button("Next");
        next.addActionListener(this);
        methodTextField = new TextField(methods[currentMethodIndex], 4);
          methodTextField.setBackground(Color.white);
          methodTextField.setForeground(Color.black);
        setAll = new Button("Set All");
        setAll.addActionListener(this);
        setSlice= new Button("Set Slice");
        setSlice.addActionListener(this);
        quit = new Button("Quit");
        quit.addActionListener(this);

      controlPanel.add(okay);
      controlPanel.add(sliceTextField);
      controlPanel.add(previous);
      controlPanel.add(next);
      controlPanel.add(methodTextField);
      controlPanel.add(setSlice);
      controlPanel.add(setAll);
      controlPanel.add(quit);

    displayPanel.add("North", blockByBlockLabel);
    displayPanel.add("Center", methodCanvas);
    displayPanel.add("East", buttonPanel);
    displayPanel.add("South", controlPanel);

    add(displayPanel);

    setSize(WIDTH, HEIGHT);

    // set up the grid
    setCurrentSlice();

    drawImage();

  }

  public int [] getHomogenizationMask() {
    return homogenizationMask;
  }

  public boolean getState() {
    return state;
  }

  public void paint(Graphics g) {
    drawImage();
  }

  public void drawImage() {
    // build the off screen image to draw to for the computations
    offScreenImage = null;
    offScreenImage = parent.createImage(canvasWidth, canvasHeight);
    // get the graphics context
    offScreenImageGraphics = offScreenImage.getGraphics();
    // compute the number of pixels that are needed for each block
    int dx = canvasWidth / nx;
    int dy = canvasHeight / ny;
    offScreenImageGraphics.setColor(resetColor);
    offScreenImageGraphics.fillRect(0, 0, canvasWidth, canvasHeight);
    // run over the blocks
    for(int j=0; j<ny; j++) {
      int j2 = j * nx;
      int yl = j * dy;
      int yr = yl + dy;
      for(int i=0; i<nx; i++) {
        int xl = i * dx;
        int xr = xl + dx;
        // get the method index
        int index = currentSliceMask[i+j2];
        offScreenImageGraphics.setColor(colorArray[index]);
        offScreenImageGraphics.fillRect(xl, yl, xr, yr);
      }
    }

    // draw the boundary
    offScreenImageGraphics.setColor(black);

      offScreenImageGraphics.drawLine(0,0,400,0);
      offScreenImageGraphics.drawLine(0,0,0,400);
      offScreenImageGraphics.drawLine(0,400,400,400);
      offScreenImageGraphics.drawLine(400,0,400,400);

      // draw the horizontal grid lines
      int j = 1;
      int yval = 0;
      while (j<ny) {
        yval = yval + dy;
        offScreenImageGraphics.drawLine(0, yval, canvasWidth, yval);
        j++;
      }
      // draw the horizontal grid lines
      int i = 1;
      int xval = 0;
      while (i<nx) {
        xval = xval + dx;
        offScreenImageGraphics.drawLine(xval, 0, xval, canvasHeight);
        i++;
      }

    // get rid of the graphics context
    offScreenImageGraphics.dispose();

    methodCanvas.redraw(0, 0, offScreenImage);

  }
 
  public void setMethodValue() {
    int indx;
    int indy;
    int index;

    // get the current location in the map
    int xLocation = methodCanvasX;
    if(xLocation < 0) xLocation = 0;
    if(xLocation > canvasWidth) xLocation = canvasWidth;
    int yLocation = methodCanvasY;
    if(yLocation < 0) yLocation = 0;
    if(yLocation > canvasHeight) yLocation = canvasHeight;

    // compute the appropriate indices
    int dx = canvasWidth / nx;
    int dy = canvasHeight / ny;
    indx = (int) xLocation / dx + 1;
    indy = (int) yLocation / dy + 1;
    index = indx + ( indy - 1 ) * nx - 1;

    // check the index to make sure that it is in range
    if(index < 0 || index >= nxny) return;
    currentSliceMask[index] = currentMethodIndex;
    index = index + slice * nxny;
    homogenizationMask[index] = currentMethodIndex;

    // update the image with the new value
    drawImage();

  }

  // set up the mouse event handler
  public void mousePressed(MouseEvent e) {
    Point tmpPoint = e.getPoint();
    if(tmpPoint != null) {
      methodCanvasX = tmpPoint.x;
      methodCanvasY = tmpPoint.y;
      setMethodValue();
    }
  }
  public void mouseReleased(MouseEvent e) {
  }
  public void mouseEntered(MouseEvent e) {
  }
  public void mouseExited(MouseEvent e) {
  }
  public void mouseClicked(MouseEvent e) {
  }

  public void actionPerformed (ActionEvent event) {
    Object source = event.getSource();
    if(source.equals(okay)) {
      setVisible(false);
      state = true;
      return;
    }
    if(source.equals(previous)) {
      for(int j=0; j<ny; j++) {
        int j2 = j * nx + slice * nx * ny;
        for(int i=0; i<nx; i++) {
          int index = i + j2;
        }
      }
      saveCurrentSlice();
      setPreviousSlice();
      setCurrentSlice();
      drawImage();
      return;
    }
    if(source.equals(next)) {
      for(int j=0; j<ny; j++) {
        int j2 = j * nx + slice * nx * ny;
        for(int i=0; i<nx; i++) {
          int index = i + j2;
        }
      }
      saveCurrentSlice();
      setNextSlice();
      setCurrentSlice();
      drawImage();
      return;
    }
    if(source.equals(setSlice)) {
      for(int i=0; i<nxny; i++) currentSliceMask[i] = currentMethodIndex;
      drawImage();
      return;
    }
    if(source.equals(setAll)) {
      for(int i=0; i<nhom; i++) homogenizationMask[i] = currentMethodIndex;
      for(int i=0; i<nxny; i++) currentSliceMask[i] = currentMethodIndex;
      drawImage();
      return;
    }
    if(source.equals(quit)) {
      setVisible(false);
      state = false;
      return;
    }
    for(int i=0; i<noOfMethods; i++) {
      if(source.equals(methodButtons[i])) {
        currentMethodIndex = i;
        methodTextField.setText(methods[currentMethodIndex]);
      }
    }
  }

  public void setPreviousSlice() {
    slice--;
    if(slice < 0) slice = nz - 1;
    sliceTextField.setText(new Integer(slice+1).toString());
  }

  public void setNextSlice() {
    slice++;
    if(slice >= nz) slice = 0;
    sliceTextField.setText(new Integer(slice+1).toString());
  }

  public void setCurrentSlice() {
    for(int j=0; j<ny; j++) {
      int j2 = j * nx; 
      int j22 = j2 + slice * nx * ny; 
      for(int i=0; i<nx; i++) {
        currentSliceMask[i+j2] = homogenizationMask[i+j22];
      }
    }
  }

  public void saveCurrentSlice() {
    for(int j=0; j<ny; j++) {
      int j2 = j * nx; 
      int j22 = j2 + slice * nx * ny; 
      for(int i=0; i<nx; i++) {
        homogenizationMask[i+j22] = currentSliceMask[i+j2];
      }
    }
  }

  public static void main(String args[]) {
    Frame f = new Frame("Homogenization Mask Object Test Frame");
    f.setSize(400,500);
    String [] methods = {"Periodic",
                         "Arithmetic",
                         "Harmonic",
                         "Geometric",
                         "Aperiodic"};
    HomogenizationMaskDialog
        hmd = new HomogenizationMaskDialog(f, methods, 2, 4, 3, 4);
    hmd.show();
    hmd.dispose();
    // output the indices to make sure it all works
    int [] homogenzationMask = hmd.getHomogenizationMask();
    int n = homogenzationMask.length;
    for(int i=0; i<n; i++) {
      System.out.println("value:   " + i + "   " + homogenzationMask[i]);
    }
    System.exit(0);
  }

  // local variables

  // variables used to store user input
  private int noOfMethods;
  private int nx;
  private int ny;
  private int nz;
  private int nxny;
  private int nhom;
  private int slice = 0;
  private String [] methods;
  private static int DEFAULT_METHOD_INDEX = 0;
  private int currentMethodIndex = 0;
  private int [] homogenizationMask;
  private int [] currentSliceMask;

  private static final int WIDTH = 550;
  private static final int HEIGHT = 475;

  private Color color = new Color(96, 128, 255);
  private Color black = Color.black;
  private Color [] colorArray = {Color.red,
                                 Color.cyan,
                                 Color.white,
                                 Color.green,
                                 Color.lightGray,
                                 Color.magenta,
                                 Color.blue,
                                 Color.orange,
                                 Color.yellow,
                                 Color.pink,
                                 Color.gray};

  // stuff for ending the function of the object
  private Frame parent;
  private Panel displayPanel;
    private Label blockByBlockLabel;
    private ImageCanvas methodCanvas;
            private int methodCanvasX;
            private int methodCanvasY;
    private Panel buttonPanel;
      private Button [] methodButtons;
    private Panel controlPanel;
      private Button okay;
      private TextField sliceTextField;
      private Button previous;
      private Button next;
      private TextField methodTextField;
      private Button setSlice;
      private Button setAll;
      private Button quit;

  private int canvasWidth = 400;
  private int canvasHeight = 400;

  // variables for the off screen image being created
  private Image offScreenImage;
  private Graphics offScreenImageGraphics;
  private Color resetColor = new Color(35, 122, 255);

  private boolean state = false;

}
