import java.awt.*;
import java.awt.event.MouseListener;
import java.awt.event.MouseEvent;

public class ImageCanvas extends Canvas implements MouseListener {

  /*
   * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
   *                                                                         *
   *    constructors for the class                                           *
   *                                                                         *
   * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
   */
  public ImageCanvas(int width, int height) {

    lastImageX = 0;
    lastImageY = 0;
    lastImageWidth = 0;
    lastImageHeight = 0;
    currentImageX = 0;
    currentImageY = 0;
    currentImageWidth = 0;
    currentImageHeight = 0;

    // set the canvas width and height
    imageCanvasWidth = width;
    imageCanvasHeight = height;

    // add the mouse event listener for the canvas
    addMouseListener(this);

  }

  /*
   * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
   *                                                                         *
   *    redraw()                                                             *
   *                                                                         *
   * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
   */
  public void redraw(int x, int y, Image img) {

    // set the last locations to be the current
    lastImageX = currentImageX;
    lastImageY = currentImageY;
    lastImageWidth = currentImageWidth;
    lastImageHeight = currentImageHeight;
    currentImageX = x;
    currentImageY = y;
    this.image = img;
    if(image != null) {
      currentImageWidth = image.getWidth(this);
      currentImageHeight = image.getHeight(this);
      repaint();
    }
  }

  public void clearCanvas() {
    Graphics g;
    g = getGraphics();
    g.clearRect(0,0,imageCanvasWidth,imageCanvasHeight);
    g.dispose();
  }

  /*
   * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
   *                                                                         *
   *    paint()                                                              *
   *                                                                         *
   * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
   */
  public void paint(Graphics g) {

    // set up the two areas in which drawing and coverup must be done
    old_area = new Rectangle(lastImageX, lastImageY,
                                         lastImageWidth, lastImageHeight);
    new_area = new Rectangle(currentImageX, currentImageY,
                                    currentImageWidth, currentImageHeight);

    // check to see if the rectangles are the same
    if(old_area.equals(new_area)) 
      update_area = new Rectangle(new_area);
    else
      update_area = new_area.union(old_area);

    // get the graphics context for the canvas
    g.clipRect(update_area.x, update_area.y,
                              update_area.width, update_area.height);
 
    // draw the image to the graphs context
    if(image != null) g.drawImage(image, currentImageX, currentImageY, this);
 
  }

  /*
   * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
   *                                                                         *
   *    parameter getting methods                                            *
   *                                                                         *
   * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
   */
  public Point getPoint() {
    return point;
  }

  /*
   * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
   *                                                                         *
   *    parameter setting methods                                            *
   *                                                                         *
   * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
   */
  public void setImageCanvasWidth(int width) {
    imageCanvasWidth = width;
  }
  public void setImageCanvasHeight(int height) {
    imageCanvasHeight = height;
  }
  public void setImageCanvasSize() {
    setSize(imageCanvasWidth,imageCanvasHeight);
  }
  public void setImageCanvasSize(int width, int height) {
    imageCanvasWidth = width;
    imageCanvasHeight = height;
    setSize(width, height);
  }
  public void setImageCanvasImage(Image img) {
    image = img;
  }
  public void setPoint(Point p) {
    point = p;
  }

  public void mousePressed(MouseEvent e) {
    int x = e.getX();
    int y = e.getY();
    if(point == null) {
      point = new Point(x,y);
    } else {
      point.x = x;
      point.y = y;
    }
  }

  public void mouseReleased(MouseEvent e) {
  }

  public void mouseEntered(MouseEvent e) {
  }

  public void mouseExited(MouseEvent e) {
  }

  public void mouseClicked(MouseEvent e) {
  }


  // the local object variables
  private Image image;
  private int lastImageX;
  private int lastImageY;
  private int lastImageWidth;
  private int lastImageHeight;
  private int currentImageX;
  private int currentImageY;
  private int currentImageWidth;
  private int currentImageHeight;
  private int imageCanvasWidth;
  private int imageCanvasHeight;
  private Rectangle old_area;
  private Rectangle new_area;
  private Rectangle update_area;
  private Point point = null;

}
