import java.awt.*;
import java.awt.event.*;
public class OKPromptDialog extends Dialog implements ActionListener {
  
  public OKPromptDialog(Frame frame, String title, String s) {
    this(frame, title, s, null);
  }
  
  public OKPromptDialog(Frame frame, String title, String s, Color color) {

    // call the dialog constructor from the super
    super(frame, title, true);

    // set the state before trying stuff out
    state = UNSELECTED;

    // set up the panel to display
    mainPanel = new Panel();
    mainPanel.setLayout(new BorderLayout());

    // set the background color
    if(color != null) {
      mainPanel.setBackground(color);
    } else {
      mainPanel.setBackground(Color.white);
    }
    if (frame != null) {
      Dimension d = frame.getSize();
      int       xOffset = d.width / 2 - width / 2;
      int       yOffset = d.height / 2 - height / 2;
      Point     p = frame.getLocation();
      p.translate(xOffset, yOffset);
      setLocation(p);
    }

    // add the message to show the user
    message = new TextArea(s, 40, 5, TextArea.SCROLLBARS_VERTICAL_ONLY);
    mainPanel.add("Center", message);

    // add a button to release control back to the program
    buttonPanel = new Panel();
    buttonPanel.setLayout(new GridLayout(1,0));
    okButton = new Button("Okay");
    okButton.setBackground(Color.black);
    okButton.setForeground(Color.white);
    okButton.addActionListener(this);
    buttonPanel.add(okButton);
    cancelButton = new Button("Cancel");
    cancelButton.setBackground(Color.black);
    cancelButton.setForeground(Color.white);
    cancelButton.addActionListener(this);
    buttonPanel.add(cancelButton);
    mainPanel.add("South", buttonPanel);

    // add the panel, set the size of the box and show it
    add(mainPanel);

    setSize(width, height);
    show();

  }

  public boolean getState() {
    if(state == OK) {
      return true;
    } else {
      return false;
    }
  }

  public void setMessage(String s) {
    message.setText(s);
    setVisible(true);
  }

  public void actionPerformed (ActionEvent event) {
    Object source = event.getSource();
    if(source == okButton) {
      state = OK;
    } else {
      state = CANCEL;
    }
    setVisible(false);
  }

  public static void main(String args[]) {
    // pretty simple test
    Frame f = new Frame("OKPrompt Dialog Test");
    String s = "A very long string to test things and such";
    OKPromptDialog wd1 = new OKPromptDialog(f, "Test OKPromptDialog", s);
    OKPromptDialog wd2 = new OKPromptDialog(f, "Test OKPromptDialog"
           , "test", Color.red);
    System.exit(0);

  }

  // local variables
  private Panel mainPanel;
  private Panel buttonPanel;
  private int width = 300;
  private int height = 125;
  private TextArea message;
  private Button okButton;
  private Button cancelButton;
  private int state;

  // static variables that do not change
  private static final int UNSELECTED = -1;
  private static final int OK = 1;
  private static final int CANCEL = 0;

}
