public class PiecewiseG1Linear1D extends Basis1DObject {
  // constructor with all arguments
  public PiecewiseG1Linear1D() {
    this(0.0, 1.0);
  }
  public PiecewiseG1Linear1D(double xl, double xr) {
    computeParameters(xl, xr);
  }
  public double evaluate(double x) {
    if(x>= xleft && x <= xright) return ( x - xg1 ) / dxg;
    return 0.0;
  }
  public double evaluate(double x, double xl, double xr) {
    computeParameters(xl,xr);
    if(x>= xleft && x <= xright) return ( x - xg1 ) / dxg;
    return 0.0;
  }
  public double derivative(double x) {
    if(x>= xleft && x <= xright) return 1.0 / dxg;
    return 0.0;
  }
  public double derivative(double x, double xl, double xr) {
    computeParameters(xl,xr);
    if(x>= xleft && x <= xright) return 1.0 / dxg;
    return 0.0;
  }
  public void computeParameters(double xl, double xr) {
    if(xl < xr) {
      xleft = xl;
      xright = xr;
    } else {
      xleft = xr;
      xright = xl;
    }
    xmid = 0.5 * ( xleft + xright );
    dx = xright - xleft;
    // compute the Gauss point info
    double delx = 0.5 * dx / Math.sqrt(3.0);
    xg1 = xmid - delx;
    xg2 = xmid + delx;
    dxg = xg2 - xg1;
  }
  public void setDomain(double xl, double xr) {
    computeParameters(xl, xr);
  }
  public double getLeftDomain() {
    return xleft;
  }
  public double getRightDomain() {
    return xright;
  }
  public static void main(String args[]) {
    PiecewiseG1Linear1D pl1d = new PiecewiseG1Linear1D();
    System.out.println(pl1d.toString());
    double dx = 0.01;
    double xleft = 0.0;
    double xright = 1.0;
    for(int i=-4; i<105; i++) {
      double xval = i * dx;
      System.out.println("  " + i + "  " + pl1d.evaluate(xval,xleft,xright)
                        + "  " + pl1d.derivative(xval,xleft,xright));
    }
    xleft = 0.31;
    xright = 0.94;
    for(int i=-4; i<105; i++) {
      double xval = i * dx;
      System.out.println("  " + i + "  " + pl1d.evaluate(xval,xleft,xright)
                        + "  " + pl1d.derivative(xval,xleft,xright));
    }
  }
  private double xleft;
  private double xright;
  private double xmid;
  private double dx;
  private double xg1;
  private double xg2;
  private double dxg;
}
