public class PiecewiseGQuadratic1D extends Basis1DObject {
  public PiecewiseGQuadratic1D() {
    this(0.0, 1.0);
  }
  public PiecewiseGQuadratic1D(double xl, double xr) {
    computeParameters(xl, xr);
  }
  public double evaluate(double x) {
    double dxsq = dx * dx;
    if(x>= xleft && x <= xright)
          return ( 4.0 * ( x - xleft ) * ( xright - x ) ) / dxsq;
    return 0.0;
  }
  public double evaluate(double x, double xl, double xr) {
    computeParameters(xl, xr);
    double dxsq = dx * dx;
    if(x>= xleft && x <= xright)
          return ( 4.0 * ( x - xleft ) * ( xright - x ) ) / dxsq;
    return 0.0;
  }
  public double derivative(double x) {
    double dxsq = dx * dx;
    if(x>= xleft && x <= xright)
        return (4.0 / dxsq) * ( xright + xleft - 2.0 * x);
    return 0.0;
  }
  public double derivative(double x, double xl, double xr) {
    computeParameters(xl, xr);
    double dxsq = dx * dx;
    if(x>= xleft && x <= xright)
        return (4.0 / dxsq) * ( xright + xleft - 2.0 * x);
    return 0.0;
  }
  public void computeParameters(double xl, double xr) {
    if(xl < xr) {
      xleft = xl;
      xright = xr;
    } else {
      xleft = xr;
      xright = xl;
    }
    xmid = 0.5 * ( xleft + xright );
    dx = xright - xleft;
    // compute the Gauss point info
    double delx = 0.5 * dx / Math.sqrt(3.0);
    xg1 = xmid - delx;
    xg2 = xmid + delx;
    dxg = xg2 - xg1;
  }
  public void setDomain(double xl, double xr) {
    computeParameters(xl, xr);
  }
  public double getLeftDomain() {
    return xleft;
  }
  public double getRightDomain() {
    return xright;
  }
  public String toString() {
    String s = "";
    s = "\n Piecewise Linear Gauss Basis Function #1\n";
    s = s + "\n left endpoint = " + new Double(xleft).toString();
    s = s + "\n right endpoint = " + new Double(xright).toString();
    s = s + "\n first guass pt = " + new Double(xg1).toString();
    s = s + "\n second guass pt = " + new Double(xg2).toString();
    s = s + "\n";
    return s;
  }
  public static void main(String args[]) {
    PiecewiseGQuadratic1D pl1d = new PiecewiseGQuadratic1D();
    System.out.println(pl1d.toString());
    double dx = 0.01;
    for(int i=-4; i<105; i++) {
      double xval = i * dx;
      System.out.println("  " + i + "  " + pl1d.evaluate(xval) + "  "
                              + pl1d.derivative(xval));
    }
    pl1d = new PiecewiseGQuadratic1D(-.3, 1.4);
    System.out.println(pl1d.toString());
    dx = 0.01 * ( 1.4 + .3 );
    for(int i=-4; i<105; i++) {
      double xval = -0.3 + i * dx;
      System.out.println("  " + i + "  " + pl1d.evaluate(xval) + "  "
                              + pl1d.derivative(xval));
    }
  }

  // local variables
  private double xleft;
  private double xright;
  private double xmid;
  private double dx;
  private double xg1;
  private double xg2;
  private double dxg;

}
