public class PiecewiseLinear1D extends Basis1DObject {
  public PiecewiseLinear1D() {
    this(0.0, 0.5, 1.0);
  }
  public PiecewiseLinear1D(double xl, double xr) {
    this(xl, 0.5 * ( xl + xr ), xr);
  }
  public PiecewiseLinear1D(double xl, double xm, double xr) {
    xmid = xm;
    computeParameters(xl, xr);
  }
  public double evaluate(double x) {
    if(x>= xleft && x < xmid) return ( x - xleft ) / dxl;
    if(x>= xmid && x < xright) return ( xright - x) / dxr;
    return 0.0;
  }
  public double evaluate(double x, double xl, double xr) {
    computeParameters(xl, xr);
    if(x>= xleft && x < xmid) return ( x - xleft ) / dxl;
    if(x>= xmid && x < xright) return ( xright - x) / dxr;
    return 0.0;
  }
  public double derivative(double x) {
    if(x>= xleft && x < xmid) return 1.0 / dxl;
    if(x>= xmid && x < xright) return -1.0 / dxr;
    return 0.0;
  }
  public double derivative(double x, double xl, double xr) {
    computeParameters(xl, xr);
    if(x>= xleft && x < xmid) return 1.0 / dxl;
    if(x>= xmid && x < xright) return -1.0 / dxr;
    return 0.0;
  }
  public void computeParameters(double xl, double xr) {
    if(xl < xr) {
      xleft = xl;
      xright = xr;
    } else {
      xleft = xr;
      xright = xl;
    }
    dx = xright - xleft;
    dxl = xmid - xleft;
    dxr = xright - xmid;
  }
  public void setDomain(double xl, double xr) {
    xmid = 0.5 * ( xl + xr );
    computeParameters(xl, xr);
  }
  public double getLeftDomain() {
    return xleft;
  }
  public double getRightDomain() {
    return xright;
  }
  public String toString() {
    String s = "";
    s = "\n Piecewise Linear Basis Function\n";
    s = s + "\n left endpoint = " + new Double(xleft).toString();
    s = s + "\n right endpoint = " + new Double(xright).toString();
    s = s + "\n";
    return s;
  }
  public static void main(String args[]) {
    PiecewiseLinear1D pl1d = new PiecewiseLinear1D();
    System.out.println(pl1d.toString());
    double dx = 0.01;
    for(int i=-4; i<105; i++) {
      double xval = i * dx;
      System.out.println("  " + i + "  " + pl1d.evaluate(xval) + "  "
                              + pl1d.derivative(xval));
    }
    pl1d = new PiecewiseLinear1D(-.3, 1.4);
    System.out.println(pl1d.toString());
    dx = 0.01 * ( 1.4 + .3 );
    for(int i=-4; i<105; i++) {
      double xval = -0.3 + i * dx;
      System.out.println("  " + i + "  " + pl1d.evaluate(xval) + "  "
                              + pl1d.derivative(xval));
    }
  }

  // local variables
  private double xleft;
  private double xright;
  private double xmid;
  private double dx;
  private double dxl;
  private double dxr;

}
