import java.awt.*;
import java.awt.event.*;

public class SetFractureValuesDialog extends PositionalDialog implements
 ActionListener {

  public SetFractureValuesDialog(Frame parent, double fv, double mv, int fw) {

    // set up the dialog box
    super(parent, "Fracture/Matrix Value Dialog", true, WIDTH, HEIGHT);
    setBackground(color);
    state = UNSELECTED;

    setLayout(new BorderLayout());

    // initialize the values
    fractureValue = fv;
    matrixValue = mv;
    fractureWidth = fw;

    Panel p1 = new Panel();
    p1.setLayout(new GridLayout(0,1));

    p1.add(new Label("Fracture Value"));
      fractureValueTextField.setBackground(Color.white);
      fractureValueTextField.setForeground(Color.black);
    p1.add(fractureValueTextField);
    p1.add(new Label("Matrix Value"));
      matrixValueTextField.setBackground(Color.white);
      matrixValueTextField.setForeground(Color.black);
    p1.add(matrixValueTextField);
    p1.add(new Label("No. of Blocks in Fracture"));
      fractureWidthTextField.setBackground(Color.white);
      fractureWidthTextField.setForeground(Color.black);
    p1.add(fractureWidthTextField);
    fractureValueTextField.setText(new Double(fractureValue).toString());
    matrixValueTextField.setText(new Double(matrixValue).toString());
    fractureWidthTextField.setText(new Integer(1).toString());

    Panel p2 = new Panel();
    p2.setLayout(new GridLayout(1,0));
      okay = new Button("Okay");
      okay.addActionListener(this);
      cancel = new Button("Cancel");
      cancel.addActionListener(this);
    p2.add(okay);
    p2.add(cancel);

    add("Center", p1);
    add("South", p2);

    setSize(WIDTH, HEIGHT);

    requestFocus();

  }

  public boolean getState () {
    if (state == OK_RESPONSE) return true;
    else return false;
  }

  public double getFractureValue() {
    return fractureValue;
  }

  public double getMatrixValue() {
    return matrixValue;
  }

  public int getFractureWidth() {
    return fractureWidth;
  }

  public void setFractureValue(double d) {
    fractureValue = d;
    fractureValueTextField.setText(new Double(fractureValue).toString());
  }

  public void setMatrixValue(double d) {
    matrixValue = d;
    matrixValueTextField.setText(new Double(matrixValue).toString());
  }

  public void setFractureWidth(int f) {
    fractureWidth = f;
    fractureWidthTextField.setText(new Integer(1).toString());
  }

  public void setData() {
    double fv = fractureValue;
    double mv = matrixValue;
    int fw = 1;
    String s = null;
    try {
      s = fractureValueTextField.getText();
      fv = Double.valueOf(s).doubleValue();
    } catch(NumberFormatException nfe) {
    }
    fractureValue = fv;
    try {
      s = matrixValueTextField.getText();
      mv = Double.valueOf(s).doubleValue();
    } catch(NumberFormatException nfe) {
    }
    matrixValue = mv;
    try {
      s = fractureWidthTextField.getText();
      fw = Integer.valueOf(s).intValue();
    } catch(NumberFormatException nfe) {
    }
    fractureWidth = fw;
  }

  public void actionPerformed (ActionEvent event) {
    Object source = event.getSource();

    if (source == okay) {
      state = OK_RESPONSE;
      setData();
      setVisible(false);
    } else if (source == cancel) {
      state = CANCEL_RESPONSE;
      setVisible(false);
    }
  }

  public static void main(String args[]) {
    Frame f = new Frame("SetFractureValues Object Test Frame");
    f.setSize(500,500);
    SetFractureValuesDialog sfv = new SetFractureValuesDialog(f, 100., 1., 1);
    sfv.pack();
    sfv.show();
    // get the data from the object upon return
    System.out.println("State: " + sfv.getState()); 
    System.out.println("Fracture Value: " + sfv.getFractureValue()); 
    System.out.println("Matrix  Value: " + sfv.getMatrixValue()); 
    System.out.println("Fracture Width: " + sfv.getFractureWidth()); 
    sfv.dispose();
    System.exit(0);
  }

  // local variables

  // variables used to store user input
  private double fractureValue;
  private double matrixValue;
  private int fractureWidth;

  // selection defaults
  private final int UNSELECTED = -1;
  private final int OK_RESPONSE = 1;
  private final int CANCEL_RESPONSE = 0;

  private static final int WIDTH = 350;
  private static final int HEIGHT = 250;

  private Color color = new Color(96, 128, 255);
  private TextField fractureValueTextField = new TextField(2);
  private TextField matrixValueTextField = new TextField(2);
  private TextField fractureWidthTextField = new TextField(2);

  // stuff for ending the function of the object
  private Button           okay, cancel;
  private int              state;

}
