public class TensorProduct3D extends Basis3DObject{
  // constructor with all arguments
  public TensorProduct3D() {
    this(null, null, null);
  }
  public TensorProduct3D(Basis1DObject bx, Basis1DObject by, Basis1DObject bz) {
    basisx = bx;
    basisy = by;
    basisz = bz;
  }
  public double evaluate(double x, double y, double z) {
    double xval = basisx.evaluate(x);
    double yval = basisy.evaluate(y);
    double zval = basisz.evaluate(z);
    return xval * yval * zval;
  }
  public double evaluate(double x, double xleft, double xright,
                         double y, double yleft, double yright,
                         double z, double zleft, double zright) {
    double xval = basisx.evaluate(x, xleft, xright);
    double yval = basisy.evaluate(y, yleft, yright);
    double zval = basisz.evaluate(z, zleft, zright);
    return xval * yval * zval;
  }
  public double dbydx(double x, double y, double z) {
    double xval = basisx.derivative(x);
    double yval = basisy.evaluate(y);
    double zval = basisz.evaluate(z);
    return xval * yval * zval;
  }
  public double dbydx(double x, double xleft, double xright,
                      double y, double yleft, double yright,
                      double z, double zleft, double zright) {
    double xval = basisx.derivative(x, xleft, xright);
    double yval = basisy.evaluate(y, yleft, yright);
    double zval = basisz.evaluate(z, zleft, zright);
    return xval * yval * zval;
  }
  public double dbydy(double x, double y, double z) {
    double xval = basisx.evaluate(x);
    double yval = basisy.derivative(y);
    double zval = basisz.evaluate(z);
    return xval * yval * zval;
  }
  public double dbydy(double x, double xleft, double xright,
                      double y, double yleft, double yright,
                      double z, double zleft, double zright) {
    double xval = basisx.evaluate(x, xleft, xright);
    double yval = basisy.derivative(y, yleft, yright);
    double zval = basisz.evaluate(z, zleft, zright);
    return xval * yval * zval;
  }
  public double dbydz(double x, double y, double z) {
    double xval = basisx.evaluate(x);
    double yval = basisy.evaluate(y);
    double zval = basisz.derivative(z);
    return xval * yval * zval;
  }
  public double dbydz(double x, double xleft, double xright,
                      double y, double yleft, double yright,
                      double z, double zleft, double zright) {
    double xval = basisx.evaluate(x, xleft, xright);
    double yval = basisy.evaluate(y, yleft, yright);
    double zval = basisz.derivative(z, zleft, zright);
    return xval * yval * zval;
  }
  public void setDomain(double xl, double xr,
                        double yl, double yr,
                        double zl, double zr) {
    basisx.setDomain(xl,xr);
    basisy.setDomain(yl,yr);
    basisz.setDomain(zl,zr);
  }
  public double getXLeftDomain() {
    return basisx.getLeftDomain();
  }
  public double getXRightDomain() {
    return basisx.getRightDomain();
  }
  public double getYLeftDomain() {
    return basisy.getLeftDomain();
  }
  public double getYRightDomain() {
    return basisy.getRightDomain();
  }
  public double getZLeftDomain() {
    return basisz.getLeftDomain();
  }
  public double getZRightDomain() {
    return basisz.getRightDomain();
  }
  public static void main(String args[]) {
    PiecewiseConstant1D pwcx = new PiecewiseConstant1D();
    PiecewiseConstant1D pwcy = new PiecewiseConstant1D();
    PiecewiseConstant1D pwcz = new PiecewiseConstant1D();
    TensorProduct3D tp3d
         = new TensorProduct3D((Basis1DObject) pwcx, (Basis1DObject) pwcy,
                               (Basis1DObject) pwcz);
    double dx = 0.01;
    double xleft = 0.0;
    double xright = 1.0;
    double dy = 0.05;
    double yleft = -0.5;
    double yright = 1.0;
    double dz = 0.05;
    double zleft = -0.5;
    double zright = 1.0;
    for(int k=-4; k<10; k++) {
      double zval = k * dy;
      for(int j=-4; j<10; j++) {
        double yval = j * dy;
        for(int i=-4; i<10; i++) {
          double xval = i * dx;
          System.out.println("  " + i + "  "
             + tp3d.evaluate(xval,xleft,xright,
                             yval,yleft,yright,
                             zval,zleft,zright)
             + "  " + tp3d.dbydx(xval,xleft,xright,
                                 yval,yleft,yright,
                                 zval,zleft,zright)
             + "  " + tp3d.dbydy(xval,xleft,xright,
                                 yval,yleft,yright,
                                 zval,zleft,zright)
             + "  " + tp3d.dbydz(xval,xleft,xright,
                                 yval,yleft,yright,
                                 zval,zleft,zright));
        }
      }
    }
  }

  private Basis1DObject basisx;
  private Basis1DObject basisy;
  private Basis1DObject basisz;

}
