c
c this common block contains the storage for the velocity mass matrices
c of the form
c
c         [  m11  m12  m13  ]
c     m = [  m21  m22  m23  ]
c         [  m31  m32  m33  ]
c
c and the right hand side of the equations obtained in the mixed
c finite element method, the matrices have a beginning letter a, but then
c have the exact name as above for the 2-4 characters of the name of the
c array. the d in the name stands for diagonal and the s and u stand for
c super and sub diagonals respectively.
c
c   am11_ - the diagonal for the matrix in the x component of the
c           velocity
c   am22_ - the diagonal for the matrix in the y component of the
c           velocity
c   am33_ - the diagonal for the matrix in the z component of the
c           velocity
c   am12 - the contributions due to the (1,2) entry in the inverted permeability
c          tensor. note that this is not the matrix but is information used in
c          the matrix multiply to get the action of the matrix
c   am13 - the contributions due to the (1,3) entry in the inverted permeability
c          tensor. note that this is not the matrix but is information used in
c          the matrix multiply to get the action of the matrix
c   am21 - the contributions due to the (2,1) entry in the inverted permeability
c          tensor. note that this is not the matrix but is information used in
c          the matrix multiply to get the action of the matrix
c   am23 - the contributions due to the (2,3) entry in the inverted permeability
c          tensor. note that this is not the matrix but is information used in
c          the matrix multiply to get the action of the matrix
c   am31 - the contributions due to the (3,1) entry in the inverted permeability
c          tensor. note that this is not the matrix but is information used in
c          the matrix multiply to get the action of the matrix
c   am32 - the contributions due to the (3,2) entry in the inverted permeability
c          tensor. note that this is not the matrix but is information used in
c          the matrix multiply to get the action of the matrix
c   rhs - the right hand side to the resulting discrete pressure
c         equation in the mixed finite element method
c   d - the time dependent part on the left hand side in the pressure
c       equation. note that this adds to the diagonal of the mixed
c       finite element pressure equation matrix
c   imobmat - a flag to determine if the mobility matrix need to be recomputed
c             at the ned time step
c             imobmat = 0; no need to recompute the matrix 
c             imobmat = 1; need to recompute the matrix 
c   imferdc - this parameter tells the code to reduce the mass matrix for the
c             diagonal velocity blocks
c             imferdc = 1; reduce
c             imferdc = anything else; no reduction
c
      common /matrix/ am11d(nvelx),am11s(nvelx),am11u(nvelx)
     1               ,am22d(nvely),am22s(nvely),am22u(nvely)
     1               ,am33d(nvelz),am33s(nvelz),am33u(nvelz)
     1               ,am12(npres),am13(npres),am21(npres)
     1               ,am23(npres),am31(npres),am32(npres)
     1               ,rhs(nnvar),d(npres),imobmat,imferdc
c
c mass matrix storage
c
c   admssx,asmssx - arrays containing the mass matrix in the x direction
c   admssy,asmssy - arrays containing the mass matrix in the y direction
c   admssz,asmssz - arrays containing the mass matrix in the z direction
c
     1               ,admssx(nvelx),asmssx(nvelx)
     1               ,admssy(nvely),asmssy(nvely)
     1               ,admssz(nvelz),asmssz(nvelz)
