c
c this common block contains information and storage for the
c porosity for use in the the simulator.
c
c General comments about the data structures:
c
c 1. the arrays are one-dimensional for all cases. the arrays are ordered
c    lexicographically in the spatial grid.
c
c 2. The description of the variables is given below.
c
c 3. the parameter statements are included in this file to keep things a little
c    easier in terms of coding subroutines.
c
c 4. this common block MUST appear before all other porosity common blocks
c    like homog_common so that the parameter statement can be loaded for
c    setting up the array lengths
c
c parameter definitions for the porosity blocks
c
c  the following parameter statements set up the storage and the like for the
c 3-d porosity code
c
c   nporo - the maximum number of porosity nodes allowable in the simulation
c   nodepor - the maximum number of nodes allowed in either coordinate
c             direction. Note that this should be the maximum of the
c             desired nodes in the x direction and the y direction
c
c relationships
c
c   nnporo = nxpor * nypor * nzpor
c
c and then
c
c   nnporo >= nnpor
c   nodepor >= max(nxporo,nyporo,nzporo)
c
c note that upon change of the parameter file here, the entire library
c and all drivers must be recompiled and linked in order to ensure that
c the simulators are on the same wavelength. There are facilities
c already built to handle this in the utility directory under sim2p2d.
c
      parameter(nodepor=300, nporo=50000)
c
c porosity variables
c
c   por0 - the array of constant porosity values
c   por1 - the array of linear porosity values
c   nxpor - the number of porosity nodes in the first coordinate direction
c   nypor - the number of porosity nodes in the second coordinate direction
c   nzpor - the number of porosity nodes in the third coordinate direction
c
      common /porosity / por0(nporo),por1(nporo)
     1,nxpor,nypor,nzpor,nxporm1,nyporm1,nzporm1,nxporp1,nyporp1
     1,nzporp1,nnpor
c
c some extra parameters for the higher dimensional comptuations
c
c   nxnypor = nxpor * nypor
c   nynzpor = nypor * nzpor
c   nxnzpor = nxpor * nzpor
c
     1,nxnypor,nynzpor,nxnzpor
c
c porosity mesh variables
c
c   xpor - the cell centers of the first coordinate in the porosity mesh
c   ypor - the cell centers of the second coordinate in the porosity mesh
c   zpor - the cell centers of the third coordinate in the porosity mesh
c   xgrpor - the grid lines of the first coordinate in the porosity mesh
c   ygrpor - the grid lines of the second coordinate in the porosity mesh
c   zgrpor - the grid lines of the third coordinate in the porosity mesh
c   dxpor - the cell sizes in the first coordinate in the porosity mesh
c   dypor - the cell sizes in the second coordinate in the porosity mesh
c   dzpor - the cell sizes in the third coordinate in the porosity mesh
c
     1,xpor(0:nodepor+1),ypor(0:nodepor+1),zpor(0:nodepor+1)
     1,xgrpor(0:nodepor+1),ygrpor(0:nodepor+1),zgrpor(0:nodepor+1)
     1,dxpor(nodepor),dypor(nodepor),dzpor(nodepor)
c
c mask array for the porosity variable
c
c   maskpor - vector to mask out some porosity nodes - normally used to
c             communicate input to the code or to fix porosity that are
c             constant and known in the domain
c
     1,maskpor(nporo)
c
c the temporary mask vectors for the porosity and velocity variables
c
c   ibitpor - temporary masking vector for the porosity variable
c
     1,ibitpor(nporo)
c
c numerical methods information for the permeability routine
c
c   methavg - the method to use for the permeability translation
c             methavg=1; homogenization
c             methavg=2; arithmetic averaging
c             methavg=3; harmonic averaging
c             methavg=4; geometric averaging
c   prmcgtol - the tolerance to use in the conjugate gradient routine for
c              convergence
c
     1,methavg,porcgtol
c
c zone map information - if there are only a few different values of the
c porosity a porosity zone map can be created.
c
c   ipormap - in integer array that differentiates between various zones
c              of porosity - this is a single array
c  
     1,ipormap(nporo)
c
c minimum and maximum values
c
c   porpormin - the minimum pressure computed in the simulation
c   porpormax - the maximum pressure computed in the simulation
c
     1,porpormin,porpormax
