c
c this common block contains information and storage for the
c sedimentary deposition code.
c
c General comments about the data structures:
c
c 1. The description of the variables is given below.
c
c 2. the parameter statements are included in this file to keep things a little
c    easier in terms of coding subroutines.
c
c parameter definitions for the sedimentary blocks
c
c  the following parameter statements set up the storage and the like for the
c 2-d sedimentary code
c
c   nsed - the maximum number of sedimentary map nodes allowable in the
c          simulation
c   nodesed - the maximum number of nodes allowed in either coordinate
c              direction. Note that this should be the maximum of the
c              desired nodes in the x direction and the y direction
c
c relationships
c
c   nnsed = nxsed * nysed * nzsed
c   nnvx = ( nxsed + 1 ) * nysed * nzsed
c   nnvy = ( nysed + 1 ) * nxsed * nzsed
c   nnvz = ( nzsed + 1 ) * nxsed * nysed
c
c and then
c
c   nsed >= nnsed
c   nodesed >= max(nxsed,nysed,nzsed)
c
c for the next to lowest order mixed finite element method the values
c are computed internally and can be taken by multiplying nxsed and nysed by
c two. also, one should make sure that nvelx is the maximum number of
c the velocity nodes, since it is used in any ambiguous definitions,
c such as routines that are symmetrically in the x and y components.
c routines lusol and reduce are examples of such routines.
c
c note that upon change of the parameter file here, the entire library
c and all drivers must be recompiled and linked in order to ensure that
c the simulators are on the same wavelength. There are facilities
c already built to handle this in the utility directory under sim2p2d.
c
      parameter(nodesed=5000, nsed=1000000)
      parameter(nsedd=200,nseda=20,nsedl=20,nsedcdf=100)
c
c sedimentary map variables
c
c   sedmap - the array of sedimentary map at the next time step (2d array)
c   nxsed - the number of sedimentary map nodes in first coordinate direction
c   nysed - the number of sedimentary map nodes in second coordinate direction
c   nzsed - the number of sedimentary map nodes in third coordinate direction
c
      common /sed/ sedmap(nsed),nxsed,nysed,nzsed,nxsedm1,nysedm1,nzsedm1
     1,nxsedp1,nysedp1,nzsedp1,nnsed,nnvx
c
c some extra parameters for the higher dimensional comptuations
c
c   nxnysed = nxsed * nysed
c   nynzsed = nysed * nzsed
c   nxnzsed = nxsed * nzsed
c
     1,nxnysed,nynzsed,nxnzsed
c
c sedimentary mesh variables
c
c   xsed - the cell centers of the first coordinate in the sedimentary mesh
c   ysed - the cell centers of the second coordinate in the sedimentary map mesh
c   zsed - the cell centers of the third coordinate in the sedimentary map mesh
c   xgrsed - the grid lines of the first coordinate in the sedimentary map mesh
c   ygrsed - the grid lines of the second coordinate in the sedimentary map mesh
c   zgrsed - the grid lines of the third coordinate in the sedimentary map mesh
c   dxsed - the cell sizes of the first coordinate in the sedimentary map mesh
c   dysed - the cell sizes of the second coordinate in the sedimentary map mesh
c   dzsed - the cell sizes of the third coordinate in the sedimentary map mesh
c   sedxleft - the original left end point for the domain in first coord.
c   sedxright - the original right end point for the domain in first coord.
c   sedyleft - the original left end point for the domain in second coord.
c   sedyright - the original right end point for the domain in second coord.
c   sedzleft - the original left end point for the domain in third coord.
c   sedzright - the original right end point for the domain in third coord.
c
     1,xsed(0:nodesed+1),ysed(0:nodesed+1),zsed(0:nodesed+1)
     1,xgrsed(0:nodesed+1),ygrsed(0:nodesed+1),zgrsed(0:nodesed+1)
     1,dxsed(nodesed),dysed(nodesed),dzsed(nodesed),sedxleft,sedxright
     1,sedyleft,sedyright,sedzleft,sedzright
c
c information about the top and the bottom of the formation
c
c   sedbotmean - the mean vertical location for the bottom boundary of the
c                formation 
c   sedbotdev - the maximum deviation in the bottom boundary of the formation 
c   sedtopmean - the mean vertical location for the top boundary of the
c                formation 
c   sedtopdev - the maximum deviation in the top boundary of the formation 
c
     1,sedbotmean,sedbotdev,sedtopmean,sedtopdev
c
c top and bottom indices for the formation
c
c   jsedbot - the indices in the grid for the bottom of the formation
c   jsedtop - the indices in the grid for the top of the formation
c
     1,jsedbot(nodesed),jsedtop(nodesed)
c
c probability of an interior bedform occuring
c
c   sedtakeoffprob - the probability that a boundary intersecting the boundary
c                    will have an interior boundary start from it
c
     1,sedtakeoffprob
c
c type of structural input
c
c  isedtype - type of input to the bed generation program
c             isedtype = 0; complicated statistical data input
c             isedtype = 1; basic geometry,statistical data input (the default)
c             isedtype = 2; archetypal data input
c  isedbndtype - the type of boundary condition to use
c             isedbndtype = 0; start the boundary bedform about anywhere
c             isedbndtype = 1; start the boundary bedform at the first quartile
c             isedbndtype = 2; start the boundary bedform at the second quartile
c             isedbndtype = 3; start the boundary bedform at the third quartile
c             isedbndtype =-1; start the boundary bedform randomly between the
c                              top and the location of the first quartile
c             isedbndtype =-2; start the boundary bedform randomly between the
c                              location of the first and second quartiles
c             isedbndtype =-3; start the boundary bedform randomly between the
c                              location of the second and third quartiles
c             isedbndtype =-4; start the boundary bedform randomly between the
c                              location of the third quartile and the bottom
c
     1,isedtype,isedbndtype
c
c bed flip parameter
c
c   isedflip beds
c
     1,isedflipbeds
c
c data used for the type 0 bedform generation
c
c bedform geometry information
c
c   sedminbedwidth - the minimum length of the polygonal logistic form - this is
c                    a horizontal scaling of the curve
c   sedmaxbedwidth - the maximum length of the polygonal logistic form - this is
c                    a horizontal scaling of the curve
c   sedminbedangle - the minimum angle a form can have along the vertical
c                   boundary of the form - this is on the upwind end of the
c                   formation
c   sedmaxbedangle - the maximum angle a form can have along the vertical
c                   boundary of the form - this is on the upwind end of the
c                   formation
c   sedminintangle - the minimum angle made by an interior bed at the upper
c                    end of the form.
c   sedmaxintangle - the maximum angle made by an interior bed at the upper
c                    end of the form.
c
c the variables that end with the string 'diff' are just the differences in the
c previous two variables
c
c
c the maximum and minimum thicknesses of the bed forms
c
c   sedthkminv - the minimum thickness along the vertical boundary
c   sedthkmaxv - the maximum thickness along the vertical boundary
c   sedthkminh - the minimum thickness along the horizontal boundary
c   sedthkmaxh - the maximum thickness along the horizontal boundary
c
c interior boundary angle information
c
c   sedmintakeoffang - the minimum angle that an interior boundary can take off
c                      from an existing boundary
c   sedmaxtakeoffang - the maximum angle that an interior boundary can take off
c                      from an existing boundary
c
c
     1,sedminbedwidth,sedmaxbedwidth,sedbedwidthdiff
     1,sedminbedangle,sedmaxbedangle,sedbedanglediff
     1,sedminintangle,sedmaxintangle,sedintanglediff
     1,sedthkminv,sedthkmaxv,sedthkminh,sedthkmaxh
     1,sedmintakeoffang,sedmaxtakeoffang,sedtakeoffangdiff
c
c data for the type 1 bedform generation - basic geometry with simple statistics
c
c bed formation minimum and maximum measurements
c
c   sedbedlenmin - the minimum bed length allowed
c   sedbedlenmax - the maximum bed length allowed
c   sedbedlendist - an integer to tell the code the type of probability
c                   distribution to sample from for the length
c                   0 -> uniform
c                   1 -> normal
c                   2 -> log-normal
c   sedbedlens1 - first statistical parameter for the bed length
c                 uniform - minimum value
c                 normal - mean value
c                 log-normal - mean value
c   sedbedlens2 - second statistical parameter for the bed length
c                 uniform - maximum value
c                 normal - standard deviation
c                 log-normal - standard deviation
c   sedhq1min - the minimum thickness at the first quartile of the bedform
c               measured from the top of the bedform
c   sedhq1max - the maximum thickness at the first quartile of the bedform
c               measured from the top of the bedform
c   sedhq1dist - an integer to tell the code the type of probability
c                distribution to sample from for the 1st quartile thickness 
c                0 -> uniform
c                1 -> normal
c                2 -> log-normal
c   sedhq1s1 - first statistical parameter for the 1st quartile thickness
c              uniform - minimum value
c              normal - mean value
c              log-normal - mean value
c   sedhq1s2 - second statistical parameter for the 1st quartile thickness
c              uniform - maximum value
c              normal - standard deviation
c              log-normal - standard deviation
c   sedhq2min - the minimum thickness at the second quartile of the bedform
c               measured from the top of the bedform
c   sedhq2max - the maximum thickness at the second quartile of the bedform
c               measured from the top of the bedform
c   sedhq2dist - an integer to tell the code the type of probability
c                distribution to sample from for the 2nd quartile thickness 
c                0 -> uniform
c                1 -> normal
c                2 -> log-normal
c   sedhq2s1 - first statistical parameter for the 2nd quartile thickness
c              uniform - minimum value
c              normal - mean value
c              log-normal - mean value
c   sedhq2s2 - second statistical parameter for the 2nd quartile thickness
c              uniform - maximum value
c              normal - standard deviation
c              log-normal - standard deviation
c   sedhq3min - the minimum thickness at the third quartile of the bedform
c               measured from the top of the bedform
c   sedhq3max - the maximum thickness at the third quartile of the bedform
c               measured from the top of the bedform
c   sedhq3dist - an integer to tell the code the type of probability
c                distribution to sample from for the 3rd quartile thickness 
c                0 -> uniform
c                1 -> normal
c                2 -> log-normal
c   sedhq3s1 - first statistical parameter for the 3rd quartile thickness
c              uniform - minimum value
c              normal - mean value
c              log-normal - mean value
c   sedhq3s2 - second statistical parameter for the 3rd quartile thickness
c              uniform - maximum value
c              normal - standard deviation
c              log-normal - standard deviation
c
     1,sedbedlenmin,sedbedlenmax,sedbedlendist,sedbedlens1,sedbedlens2
     1,sedhq1min,sedhq1max,sedhq1dist,sedhq1s1,sedhq1s2
     1,sedhq2min,sedhq2max,sedhq2dist,sedhq2s1,sedhq2s2
     1,sedhq3min,sedhq3max,sedhq3dist,sedhq3s1,sedhq3s2
c
c data for the type 2 bedform generation - archetypes
c
c   nsedarchedata - the number of data points used in the determination of the
c                   archetypes
c   nsedarches - the number of archetypes to read in
c   nsedarchelen - the length of the archetype vectors
c   sedarches - the vector of archetypes
c   sedarchebeta - the coefficients for the construction of the archetypes from
c                  the data
c   sedarchealpha - the coefficients for computing the projections of the data
c                   to the archetypes 
c   sedarcheamean - the means for the archetype multipliers
c   sedarcheastdv - the standard deviations for the archetype multipliers
c   isedarchedist - the type of distribution to sample from
c                   0 -> uniform
c                   1 -> normal
c                   2 -> log-normal
c   sedarchecdf - cdf values for the archetype projection coefficients - alpha's
c
     1,nsedarchedata,nsedarches,nsedarchelen,sedarches(nsedl,nseda)
     1,sedarchebeta(nsedd,nseda),sedarchealpha(nseda,nsedd)
     1,sedarcheamean(nseda),sedarcheastdv(nseda),isedarchedist
     1,sedarchecdf(0:nsedcdf+1,nseda)
c
c permeability values on the sedimentary map grid
c
c   sedkxx - (1,1) tensor entry in the permeability
c   sedkxy - (1,2) tensor entry in the permeability
c   sedkxz - (1,3) tensor entry in the permeability
c   sedkyx - (2,1) tensor entry in the permeability
c   sedkyy - (2,2) tensor entry in the permeability
c   sedkyz - (2,3) tensor entry in the permeability
c   sedkzx - (3,1) tensor entry in the permeability
c   sedkzy - (3,2) tensor entry in the permeability
c   sedkzz - (3,3) tensor entry in the permeability
c                sedimentary map simulator.
c
     1,sedkxx(nsed),sedkxy(nsed),sedkxz(nsed)
     1,sedkyx(nsed),sedkyy(nsed),sedkyz(nsed)
     1,sedkzx(nsed),sedkzy(nsed),sedkzz(nsed)
     1,t1sedkxx(nsed),t1sedkxy(nsed),t1sedkxz(nsed)
     1,t1sedkyx(nsed),t1sedkyy(nsed),t1sedkyz(nsed)
     1,t1sedkzx(nsed),t1sedkzy(nsed),t1sedkzz(nsed)
c
c restart parameter
c
c   isedrstrt - a parameter to tell the code whether or not to restart the
c               simulator from the last output step
c   sedrstrt = restart string:
c              RESTART -> restart the simulation
c              RUN COMPLETE -> the simulation is complete and there is no need
c                              to run.
c              any thing else will let the simulation start from the beginning
c
     1,isedrstrt,sedrstrt
      character*12 sedrstrt
c
c force the map to be integers
c ----------------------------
c
      integer sedmap,sedbedlendist,sedhq1dist,sedhq2dist,sedhq3dist
      data sedarchebw / 0.05 /
