c     Programme dispersion2d.f
c     
c     Programme principal permettant de traiter des cas 
c     de milieux poreux fractures en 2 D
c
c     Pour calcul du tenseur de dispersion dans un milieu fracture
c
c     
c     Ecrit par Benedicte VIDAL
c     Stage IPSN
c         suivi par Brahim AMAZIANE et Christophe SERRES
c     
c     avril - septembre 2001
      
      program dispersion
      
*     definition des parameters ou nv est la dimension des vecteurs ie le nombre d inconnues 
*     et nm est la dimension de la matrice       
* pour site ASPO : MAIS TAILLE DES VECTEURS TROP GRANDE OU TROP DE VECTEURS DE GRANDE TAILLE
*      parameter (nv=25000002,nm=225000002)
*     pour sites TOURNEMIRE, VIENNE
      parameter (nv=26000,nm=230000)
      
*      implicit none
      
c     declaration des variables
*     gh pour gradient hydraulique
*     vf=(vf1,vf2) vecteur vitesse v0
*     kvf=(kvf1,kvf2) vecteur moyenne de vf
*     p11,p22 correspondent aux elements k11 et k22 de K
*     kp11,kp22 correspondent aux elements k11* et k22* de K*
*     xx et yy contiennent les milieux des elements (nxh*nhr,nyh*nyhr)
*     
      integer nxh,nyh,nxhf,nyhf,nxhr,nyhr
      integer nmat,nmorse,nvalx,nvaly,nvalfx,nvalfy,ntot,ntotf,
     *     ntotv,nvar,nvarf,nfxr,nfyr,nfx,nfy
      real diff,difm,ghx,ghy,p11,p22,kp11,kp22
      real x(nv),y(nv),xx(nv+2),yy(nv+2)
      real vf1,vf2,kvf1,kvf2
      real dx(nv),dy(nv),dxy(nv),dyx(nv)
      real v1(nv),v2(nv)
      real zx,zy
      real zcx,zcxy,zcyx,zcy
      character*200 buf
      integer i
      
      common/morse/nmat,nmorse
      common/morse1/nvalx,nvaly,nvalfx,nvalfy
      common/morse2/ntot,ntotf,ntotv,nvar,nvarf
      common/morse3/nfxr,nfyr
      
c     Lecture des donnees dans le fichier de donnees
      open (10,FILE='input.dat',STATUS='old')
      read (10,*) buf,buf,nxh,buf,nyh,buf,nxhf,buf,nyhf,buf,nxhr,buf,
     *  nyhr,buf,p11,buf,kp11,buf,diff,buf,difm,buf,ghx,buf,ghy
      close (10)  
      
      p22=p11
      kp22=kp11
      
*     cas si les diffusions dans la matrice et la fracture sont identiques
      if (diff .eq. difm) then
         difm=difm+1e-24
      end if   
      
c     Calcul des milieux des cellules
      CALL mil_cel(nxh,nyh,nxhr,nyhr,x,y,xx,yy)
      
c     calcul de termes intervenant dans le calcul du tenseur D*
      nvalx=nxh*nxhr
      nvaly = nyh * nyhr
      nvalfx = nxhf * nxhr
      nvalfy = nyhf * nyhr
      ntot = nxh * nyh
      ntotf = nxhf * nyhf
      ntotv = ntot - ntotf
      nvar = nvalx * nvaly
      nvarf = nvalfx * nvalfy
      nfx = 0.5 * ( nxh - nxhf )
      nfy = 0.5 * ( nyh - nyhf )
      nfxr = nfx * nxhr
      nfyr = nfy * nyhr
      
*     nombre d inconnues
      nmat = nvar
*     taille de la matrice morse
      nmorse=nmat*9
      
c     Remplissage des vecteurs dx,dy,dxy,dyx 
      CALL remplissage_d(diff,difm,nxh,nyh,nxhf,nyhf,nfx,nfy,dx,dy,
     *     dxy,dyx)
      
*     calcul de la vitesse
      vf1=-p11*ghx
      vf2=-p22*ghy

      
      write(*,*)'vf1',vf1,'\t','\tvf2',vf2      
      
*     calcul de la vitesse macro
      kvf1=-ghx*kp11
      kvf2=-ghy*kp22

      
      write(*,*)'<vf1>',kvf1,'\t','\t<vf2>',kvf2
      pause
      
*     remplissage des vecteurs pour la vitesse      
      call remplissage_v(vf1,vf2,nxh,nyh,
     *     nfx,nfy,v1,v2)       

c     appel de la procedure pour le calcul de D**
      CALL homogab2_disp(nv,nm,v1,v2,kvf1,kvf2,dx,dxy,dyx,dy,x,y,xx,
     *     yy,nxh,nyh,nxhf,nyhf,nxhr,nyhr,zcx,zcxy,zcyx,zcy,zx,zy,0)
      
      
*     affichage a l ecran du tenseur homogeneise D**
      write(*,*)'*****************************************************'
     *     ,'*******'
      write(*,*)'\t \t TENSEUR HOMOGENEISE D**'
      write(*,*)
      write(*,*)zcx,'\t',zcxy
      write(*,*)zcyx,'\t',zcy
      write(*,*)
      write(*,*)'**************************************************'
     *     ,'**********'
      pause 'FIN DU PROGRAMME'
      
      end
c     fin du programme principal
      
      

********************************************************************************
c     Calcul des milieux des sous cellules
      
      subroutine mil_cel(nxh,nyh,nxhr,nyhr,x,y,xx,yy)

* nxh : nombre de sous cellules selon x
* nyh : nombre de sous cellules selon y
* nxhr : raffinement selon x
* nyhr : raffinement selon y
*   x : vecteur contenant les abscisses des milieux des sous cellules
*   y : vecteur contenant les ordonnees des milieux des sous cellules
*  xx : vecteur contenant les abscisses des milieux des elements
*  yy : vecteur contenant les ordonnees des milieux des elements
      
      implicit none
      
      real x(*),y(*),xx(*),yy(*)
      integer i,nxh,nyh,nxhr,nyhr
      
      do 1 i=1,nxh
         x(i)=(2*FLOAT(i)-1)/(2*nxh)
 1    continue
      
      do 2 i=1,nyh
         y(i)=(2*FLOAT(i)-1)/(2*nyh)
 2    continue
      
      xx(1)=0.0
      yy(1)=0.0
      do 3 i=2,(nxh+1)*nxhr
         xx(i)=(FLOAT(i-1))/(nxh*nxhr)
 3    continue
      
      do 4 i=2,(nyh+1)*nyhr
         yy(i)=(FLOAT(i-1))/(nyh*nyhr)
 4    continue
      
      end
      
*********************************************************************************
c     Remplissage des vecteurs dx,dy,dxy,dyx 
      
      subroutine remplissage_d(diff,difm,nxh,nyh,nxhf,nyhf,
     *     nfx,nfy,dx,dy,dxy,dyx)

* diff : diffusion dans la fracture
* difm : diffusion dans la matrice
*  nxh : nombre de sous cellules selon x
*  nyh : nombre de sous cellules selon y
* nxhf : nombre de sous cellules constituant la matrice selon x
* nyhf : nombre de sous cellules constituant la matrice selon y
*  nfx : taille de la fracture selon x (ie nombre de sous cellules entre la frontiere exterieure de la cellule et la frontiere de la fracture et la matrice) 
*  nfy : taille de la fracture selon y (ie nombre de sous cellules entre la frontiere exterieure de la cellule et la frontiere de la fracture et la matrice) 
*   dx : vecteur contenant l'element 1,1 du tenseur de diffusion correspondant a chaque sous cellules
*  dxy : vecteur contenant l'element 1,2 du tenseur de diffusion correspondant a chaque sous cellules
*  dyx : vecteur contenant l'element 2,1 du tenseur de diffusion correspondant a chaque sous cellules   
*   dy : vecteur contenant l'element 2,2 du tenseur de diffusion correspondant a chaque sous cellules   
      
      implicit none
      
      real dx(*),dy(*),dxy(*),dyx(*)
      real diff,difm
      integer i,nxh,nyh,nxhf,nyhf,j,nfx,nfy
      
      j=1
      
      do 1 i=1,nxh*nyh
         if (((i .ge. 1) .and. (i .le. nfy*nxh+nfx)) .or. 
     *        ((i . ge. (nyh-nfy)*nxh) 
     *        .and. (i .le. nxh*nyh))) then
            dx(i)=diff
            dy(i)=dx(i)
            
         else
            if ((j .le. nfx) .or. (j .ge. nxh-nfx+1)) then
               dx(i)=diff
               dy(i)=dx(i)
               
            else
               dx(i)=difm
               dy(i)=dx(i)
               
            end if        
         end if 
         
         dxy(i)=0.
         dyx(i)=0.
         
         if (mod(i,nxh) .eq. 0) then
            j=1
         else
            j=j+1
         end if
 1    continue
      
      end
      
***********************************************************************************
      
      SUBROUTINE remplissage_v(vf1,vf2,nxh,nyh,
     *     nfx,nfy,v1,v2)
      
      implicit none

*  vf1 : premiere coordonnees du vecteur vitesse
*  vf2 : seconde coordonnees du vecteur vitesse
*  nxh : nombre de sous cellules selon y
*  nyh : nombre de sous cellules selon y
*  nfx : taille de la fracture selon x (ie nombre de sous cellules entre la frontiere exterieure de la cellule et la frontiere de la fracture et la matrice) 
*  nfy : taille de la fracture selon y (ie nombre de sous cellules entre la frontiere exterieure de la cellule et la frontiere de la fracture et la matrice) 
*   v1 : vecteur contenant la premiere coordonne du vecteur vitesse pour chaque sous cellule
*   v2 : vecteur contenant la seconde coordonne du vecteur vitesse pour chaque sous cellule
      
      real vf1,vf2,v1(*),v2(*)
      integer j,i,nfx,nfy,nxh,nyh
      
      j=1
      
      do 1 i=1,nxh*nyh
         if (((i .ge. 1) .and. (i .le. nfy*nxh+nfx)) .or. 
     *        ((i . ge. (nyh-nfy)*nxh) 
     *        .and. (i .le. nxh*nyh))) then
            v1(i)=vf1
            v2(i)=vf2
         else
            if ((j .le. nfx) .or. (j .ge. nxh-nfx+1)) then
               v1(i)=vf1
               v2(i)=vf2             
            else
               v1(i)=0.
               v2(i)=0.             
            end if        
         end if 
         
         if (mod(i,nxh) .eq. 0) then
            j=1
         else
            j=j+1
         end if
 1    continue
      
      END





