C Fichier gmres.f
C
C---------------------------------------------
      subroutine gmres(ns,a,mia,mja,x,eps,nkr,nit,b)
C---------------------------------------------
C Ce sous-programme resout le systeme lineaire $Ax=b$
C par la methode GMRES(nkr), i.e. avec une dimension nk 
C de l"espace de Krylov limitee a nkr, ce qui necessite
C plusieurs reinitialisations successives, dont on limite
C arbitrairement le nombre a nit. Le test d"arret de la 
C methode porte sur la norme euclidienne du residu.
C
C Cette methode est basee sur la construction, selon la 
C methode d"Arnoldi, d"une base orthonormee de l"espace
C de Krylov, base qui est stockee dans le tableau v; plus 
C precisement, v(i,j) contient la i eme composante du  
C j eme vecteur de base. 
C
C Simultanement, est construit le tableau h, qui,
C au depart, est defini par h(i,j)=$(A v_i,v_j)$;
C la j eme colonne de ce tableau h represente alors
C la j eme colonne de la matrice de Hessenberg $\bar H^{nk}$,
C matrice qui est d"ordre $(nk+1)\times nk$ et qui est
C ensuite factorisee sous forme $Q^{nk} R^{nk}$, avec $Q^{nk}$ 
C matrice orthogonale d"ordre $(nk+1)\times (nk+1)$ et
C $R^{nk}$ matrice triangulaire superieure d"ordre 
C $(nk+1)\times nk$, de derniere ligne nulle, et dont tous
C les elements diagonaux sont strictement positifs.
C
C Dans un deuxieme temps, h(.,j) contient la colonne 
C d"indice j de la matrice triangulaire superieure $R^{nk}$,
C apres avoir effectue la j eme rotation (dont le cosinus
C de l"angle est note cos, le sinus etant 
C note sin), rotation dont le but est d"annuler le 
C coefficient d"indice $(j+1)j$ dans la matrice 
C de Hessenberg $\bar H^{kr}$.
C
C La tableau r contient le second membre 
C $g^{nk}=(Q^{nk})^T ||r^0||  e_1^{nk+1}$ du probleme 
C de minimisation d"inconnue $y^{nk}$ a resoudre.  
C Ensuite, ce tableau r contient la solution $y^{nk}$ qui
C est calculee par resolution d"un systeme triangulaire 
C superieur, de matrice $R^{nk}$. Cette operation se fait
C dans le sous-programme soln.
C
C La solution approchee a l"etape nk de l"algorithme est
C stockee dans le tableau x, tableau qui contient au depart
C l"initialisation $x^0$ de la solution. Cette operation se
C fait egalement dans le sous-programme soln.
C
C Nous calculons ensuite le "vecteur residu" qui nous est 
C necessaire pour l"initialisation de l"etape it suivante; 
C simultanement sa norme nous sert de test d"arret.
C
C Remarque 1:
C -----------
C Nous avons ici choisi de construire d"abord la base
C de l"espace de Krylov puis de factoriser  dans un
C deuxieme temps la matrice de Hessenberg.
C
C Remarque 2:
C -----------
C La matrice $A$ de taille ns*ns, ainsi que la second membre 
C $b$ de taille ns du systeme lineaire a resoudre,
C n"intervenant que dans le calcul du residu initial,
C puis dans la construction des vecteurs de base, nous ne
C les avons pas stockes dans ce sous programme.
C Ils ne figurent donc que dans un sous-programme, que nous
C avons note "pmat(ns,x,ax,ind)": pour un vecteur $x(ns)$
C donne (stocke dans le tableau x(ns)), et pour un 
C indicateur fixe ind, le sous programme pmat calcule et 
C stocke dans le tableau ax:
C - le produit $A * x$, si ind vaut 0,
C - la combinaison lineaire $b-A*x$, si ind vaut 1.
C Ce sous-programme dependant du type de stockage de la 
C matrice $A$ envisage, nous ne le donnons pas ici. Il est a
C creer par l"utilisateur en fonction de ses besoins.
C 
C 
C          Programme ecrit par Pascal Joly
C          (Laboratoire d"Analyse Numerique,
C          Universite Pierre et Marie Curie),
C          modifie par Brigitte Lucquin (Juin 95).
C 
C 

      implicit none

      integer ns,nkr
      real  x(ns+2),v(ns+2,nkr+2),a(*),
     >           h(nkr+2,nkr+2),r(nkr+2)
      real aux(ns+2),aux1(ns+2)
      integer ind,nit,i,j,it,nk,il,ip
      real eps,prods
      integer mia(*),mja(*)
      real b(*)
      real res,rec,zero,tem,sin,cos,hij,hipj,raux,gamma,dem,hii,hipi

      DATA zero/1.e-8/
*      rec    = zero  
      rec=eps       
*      write(*,*)ns
*      pause

C
C...initialisation
C
      ind=1

      do 100 i=1,ns
         aux(i)=0.0
 100  continue

      CALL pmat(ns,x,aux,ind,a,mia,mja,b)
*      write(*,*)'DANS GMRES'
*      write(*,*)'AUX APRES PMAT'
*      write(*,*)(aux(i),i=1,ns)
      res= prods(ns,aux,aux)
      res= sqrt(res)

      IF(res .LT. rec) then
         write(*,*)'RES<REC'
         GOTO 4
      end if
*      rec = max(rec,res*eps)
      write(*,*)'res=',res
      write(*,*)'res*eps=',res*eps 
           
C
C...demarrage des iterations
C      
      write(*,*)nit                 
      DO 1 it=1,nit

        nk=nkr
C                                                                     
C...orthogonalisation d'Arnoldi
C
        DO 10 i=1,ns
          aux(i) = aux(i)/res
          r(i)=0.
10      CONTINUE
        r(1)   = res

        DO 11 j=1,nkr
          DO 21 i=1,ns                                                     
            v(i,j) = aux(i)                                              
21        CONTINUE

          ind=0
                           
          CALL pmat(ns,aux,aux1,ind,a,mia,mja,b)  

          DO 22 i=1,ns
            aux(i)=aux1(i)
22        CONTINUE  
          
          DO 23 i=1,j
            tem= prods(ns,aux1,v(1,i))
            h(i,j) = tem
            DO 24 il=1,ns
              aux(il)=aux(il)-tem*v(il,i)
24          CONTINUE
23        CONTINUE
          
          dem= prods(ns,aux,aux)
          dem=sqrt(dem)
          h(j+1,j) = dem
          IF (dem .LT. rec) THEN
            nk=j
            GOTO 5
          ENDIF
          DO 25 i=1,ns
            aux(i) = aux(i)/dem
25          CONTINUE
11      CONTINUE

C
C...triangularisation et modif. du second membre
C
5       CONTINUE
        DO 31 i=1,nk
          ip=i+1
          hii=h(i,i)
          hipi=h(ip,i)
          gamma=hii*hii+hipi*hipi
          gamma=sqrt(gamma)
          gamma=1./gamma
          cos=hii*gamma
          sin=-hipi*gamma
          DO 41 j=i,nk
            hij=h(i,j)
            hipj=h(ip,j)
            h(i,j) = cos*hij - sin*hipj
            h(ip,j) = sin*hij + cos*hipj
41        CONTINUE
          raux=r(i)
          r(i)=cos*raux
          r(ip)=sin*raux
31      CONTINUE

C
C...resolution du systeme triangulaire superieur
C
        CALL soln(nk,ns,r,h,v,x)

c
C...calcul du residu
C
        ind=1

        CALL pmat(ns,x,aux,ind,a,mia,mja,b)
        res= prods(ns,aux,aux)
        res= sqrt(res)
        if ((mod(it,100) .eq. 0) .or. (mod(it,nit) .eq. 0 )) then
           write(*,*)res
        end if
        IF(res .LT. rec) THEN
          GOTO 4
        ELSEIF(it.GE.nit) THEN
          write(*,*)'nbre d iterations =',it
          write(*,*)'eps=',rec
          write(*,*)'res=',res
          write(*,*) '*********  NON CONVERGENCE *********'
          pause 'FIN DE GMRES'
          GOTO 6
        ENDIF
1     CONTINUE
C
C...sortie des resultats
C
4     CONTINUE
      write(*,*) 'RES<REC'
      write(*,*)'nbre d iterations =',it
      write(*,*)'eps=',rec
      write(*,*)'res=',res
      pause 'FIN DE GMRES'
      write(*,*)

6     CONTINUE
C
C
      END
C
C
C**********************************
C
C     BIBLIOTHEQUE
C
C**********************************
C
C

C
C---------------------------------------------
      SUBROUTINE soln(nk,ns,r,h,v,x)
C---------------------------------------------
C Ce sous-programme resout le systeme triangulaire 
C superieur permettant de calculer la solution 
C du probleme de minimisation dans l"algorithme 
C GMRES. La solution approchee a l"etape nk est 
C calculee et stokee dans le vecteur x.
C
C   
*      PARAMETER (nsMax=4000,nkrMax=5000)
      
      implicit none

      integer ns,nk
*      real  x(nsmax),v(nsmax,nkrmax),
*     >           h(nkrmax,nkrmax),r(nkrmax)
      real  x(*),v(ns+2,nk+2),
     >           h(nk+2,nk+2),r(*)
      integer i,j,il
      real tem
C
*      write(*,*)'DANS SOLN, nk=',nk
*      write(*,*)'H'
*      write(*,*)(h(i,i),i=1,nk)
*      pause
      DO 1 i=nk,1,-1                                                
        tem = r(i)/h(i,i)
        r(i)=tem                                                    
        DO 2 j=i-1,1,-1
          r(j)=r(j)-h(j,i)*tem                                                    
2       CONTINUE
1     CONTINUE
      
*      write(*,*)(r(i),i=1,nk)
*      pause
C     
      DO 3 i=1,nk
        tem = r(i)
        DO 4 il=1,ns
          x(il)=x(il)+tem*v(il,i)
4       CONTINUE
3     CONTINUE
      
*      write(*,*)(x(i),i=1,nk)
*      pause
      END
C
C--------------------------------------
      real FUNCTION   prods(n,x,y)                                           
C--------------------------------------
C Cette fonction calcule le produit scalaire de 2 vecteurs
C x et y de taille n.
C
C
      implicit none
      integer n,i
      real x(n+2),y(n+2) 
      real som

      som = 0.                                                    
      DO 1 i=1,n                                                      
1       som = som + x(i)*y(i)                                         
      prods = som                                                     
      END
