*******************************************************
******               gracpmorse.f              ********
*
C Ce sous-programme resout le systeme lineaire $Ax=b$ par
C gradient conjugue preconditionne, avec au plus kMax 
C iterations. Le test d"arret est: $|Ax^k-b|<eps|Ax^0-b|$.
C La matrice $A$ du systeme lineaire, de taille $n \times n$
C est stockee sous forme Morse dans le tableau aa;
C ce stockage necessite aussi les tableaux auxiliaires
C d"indices ia et ja, comme explique dans le chapitre III.
C La matrice $C$ de preconditionnement est la factorisation
C de Choleski incomplete de $A$ ($C=L^* (L^*)^{T}$)  et la
C matrice $L^*$ est stockee sous forme Morse dans le 
C tableau al, avec les memes tableaux d"indices ia et 
C ja que $A$.
C
C     entree:
C     --------
C       matrice $A$ du systeme stockee sous forme Morse 
C         dans les tableaux (aa,ia,ja),
C       factorisee de Choleski incomplete de $A$ stockee
C         dans les tableaux (al,ia,ja);
C       second membre de l"equation stocke dans le 
C         tableau b;
C       nombre maximal kMax d"iterations, precision eps
C         du calcul;
C
C     N.B.: le calcul du second membre ne servant qu"a 
C       l"etape d"initialisation, le tableau b est ensuite
C       ecrase et utilise comme tableau auxiliaire de  
C       calcul, en particulier pour le stockage du produit 
C       de la matrice $A$ par $h$, puis pour le stockage 
C       de la solution y du systeme $C y= A h$.
C       
C                            
C     sortie:
C     ------- 
C       x:tableau contenant les valeurs de la solution 
C         en chacun des sommets du maillage;
C       gcg:scalaire contenant le carre de la norme de
C         $g$ (en pratique ce nombre doit decroitre au 
C         cours des iterations).
C                            
C     autres tableaux:
C     ----------------
C       g, cg: tableaux contenant les valeurs de $g^n$ et
C         de $C g^n$ respectivement;
C       h: tableau contenant la direction de descente $h^n$
C         (au tout debut du programme, ce tableau sert a 
C          stocker le produit de $A$ par $x^0$);
C       
C
C     sous-programmes:
C     ---------------- 
C       IAmul:effectue le produit d"une matrice par 
C         un vecteur, la matrice etant stockee sous
C         forme Morse,
C       ISolvSys:resolution du systeme lineaire 
C         $L L^T x=y$ par descente remontee, pour
C         une matrice $L$ stockee sous forme Morse;
C       Ces deux sous-programmes sont dans le fichier
C       GCPLaplac.for.
C
C

*	de 'INTRODUCTION AU CALCUL SCIENTIFIQUE' 
*	B. LUCQUIN
*	O. PIRONNEAU
*	MASSON, 1996


      subroutine gradcpmorse(n,aa,ia,ja,al,sc,x,g,h,cg,kmax,eps)
      
      implicit none
      
      integer n,i,k,kmax
      real x(*),b(n),sc(*),g(n),h(n),cg(n)
      real aa(*),al(*)
      integer ja(*),ia(*)
      real gcg,eps,hAh,ro,gamma

      do 100 i=1,n
         b(i)=sc(i)
         x(i)=0.00
         cg(i)=0.0
         h(i)=0.0
         g(i)=0.0
         
 100  continue

      do 1 k=1,kmax

c        cas ou k=1
         if(k .eq. 1) then
c           appel de la procedure pour la multiplication mat/vecteur
            call IAmul(n,aa,ia,ja,x,h)

            do 2 i=1,n
               cg(i)=b(i)-h(i)
               g(i)=cg(i)
 2          continue
            
c           appel de la procedure pour resoudre al.z=g (g=second membre et sol)
            call ISolvSys(n,g,al,ia,ja)

            gcg=0.
            do 3 i=1,n
               gcg=gcg+g(i)*cg(i)
               h(i)=g(i)
 3          continue
 
         end if
      
c        appel de la procedure pour la multiplication mat/vecteur
         call IAmul(n,aa,ia,ja,h,b)
        
         hAh=0.
         do 10 i=1,n
            hAh=hAh+h(i)*b(i)
 10      continue
         
         ro=gcg/hAh
         
         do 11 i=1,n
            cg(i)=cg(i)-ro*b(i)
 11      continue
         
c     appel de la procedure pour resoudre al.z=b (g=second membre et sol) 
         call ISolvSys(n,b,al,ia,ja)

         do 12 i=1,n
            x(i)=x(i)+ro*h(i)
            g(i)=g(i)-ro*b(i)
 12      continue

         gamma=gcg
         gcg=0.
         do 13 i=1,n
            gcg=gcg+g(i)*cg(i)
 13      continue
         
         if (k .eq. 1) eps=eps*gcg
         if (gcg .lt. eps) then
             write(*,*)'GCG<EPS'
             write(*,*)'nbre d iterations k=',k
             write(*,*)'eps=',eps
             write(*,*)'gcg=',gcg
             write(*,*)'FIN GRADCP'
             write(*,*)
             write(*,*)
*             pause 
             
            return
         end if
         gamma=gcg/gamma
         do 14 i=1,n
            h(i)=g(i)+gamma*h(i)
 14      continue

 1    continue

      write(*,*)'K=KMAX'
      write(*,*)'nbre d iterations k=',k
      write(*,*)'eps=',eps
      write(*,*)'gcg=',gcg
      write(*,*) 'FIN GRADCP'
      write(*,*)
      write(*,*)
       
      return
      end










