c\begin{verbatim}
      subroutine homogab2_permfrac(nv,nm,kx,kxy,kyx,ky,x,y,nxh,
     1nyh,nxhf,nyhf,nxhr,nyhr,zkx,zkxy,zkyx,zky,zx,zy,iopt)
c
c * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
c
c crit par : joe koebbe 
c et modifie par : benedicte VIDAL
c
c   Cette procedure permet de calculer pour un MILIEU POREUX FRACTURE 
c le tenseur homogeneise.
c
c Le tenseur contenant les permeabilite est de la forme :
c
c                 [  kx    kxy  ]
c             K = [             ]
c                 [  kyx    ky  ]
c
c  Le tenseur homogeneise est donne en sortie sous la forme de la matrice K#
c                          #   [  zkx    zkxy  ] 
c                         K  = [               ]
c                              [  zkyx   zky   ]
c
c
c * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
c
      
      implicit none

      common /morse/ nmat,nmorse
      common /morse1/nvalx,nvaly,nvalfx,nvalfy
      common /morse2/ntot,ntotf,ntotv,nvar,nvarf

c     declaration des variables
c-------------------------------
      integer nxh,nyh,nxhf,nyhf,nxhr,nyhr,iopt,ntot,nv,nm
      integer n1,n2,n1r,n2r
      real sixth,quarter,third
      real xvalm,xvalp,yvalm,yvalp,supval
      real kx(*),kxy(*),kyx(*),ky(*)
      real x(*),y(*)
      real zkx,zkxy,zkyx,zky
      real rkx,rkxy,rkyx,rky
      real zx,zy
      real cgeps,eps,constant
      real sum,cellsize,term1,term2,delx,dely,del
      integer n1f,n2f,nvalfx,nvalfy,ntotf,ntotv,nvar,nmat,nvarf
      integer nvalx,nvaly
      integer nfx,nfy,nfxr,nfyr,nmorse
      integer kmax,i,ii,j2,j,ind1,ind2,ind3,ind4,indx,indy,indprm
c     variables pour le stockage morse  
c     vecteur pour la matrice morse
      integer mia(nv),mja(nm)                     
      real maa(nm),mal(nm)
      real hrhs(nv,2),hx(nv,2)
      integer first,place,bool
c     variables pour le gradient conjugue preconditionne
      real h(nv),cg(nv),g(nv)

c
c     definition de parametres utilises dans la suite
c -----------------------------------------------------
c
      data sixth,quarter,third
     1          / 0.166666666666660,.25,0.333333333333333 /
      data cgeps / 0.000001 /
c
c     definition du nombre d'iteration maximum
c ----------------------------------------------
c
      print *,'got here'
      kmax=nmat    
c
c     si le nombre d'elements ou le raffinement selon un axe 
c     est nul le mettre a 1
c -----------------------------------------------------------
c
      if(nxh .le. 0) nxh = 1
      if(nyh .le. 0) nyh = 1
      if(nxhr .le. 0) nxhr = 1
      if(nyhr .le. 0) nyhr = 1
c
c     posons des variables pour la suite
c --------------------------------------
c
      n1 = nxh
      n2 = nyh
      n1r = nxhr
      n2r = nyhr
      n1f = nxhf
      n2f = nyhf
      nfx = 0.5 * ( nxh - nxhf )
      nfy = 0.5 * ( nyh - nyhf )
      nfxr = nfx * nxhr
      nfyr = nfy * nyhr
 
c
c     calcul de la taille de la cellule de base 
c     pour une utilisation ulterieure
c -----------------------------------------------
c
      if(nxh .eq. 1) then
         xvalm = 0.0
         xvalp = 1.0
      else
         xvalm = 1.5 * x(1) - 0.5 * x(2)
         xvalp = 1.5 * x(nxh) - 0.5 * x(nxh-1)
      end if
      if(nyh .eq. 1) then
         yvalm = 0.0
         yvalp = 1.0
      else
         yvalm = 1.5 * y(1) - 0.5 * y(2)
         yvalp = 1.5 * y(nyh) - 0.5 * y(nyh-1)
      end if
      cellsize = ( xvalp - xvalm ) * ( yvalp - yvalm )

c     
c     calcul des coordonnees du nouveau centre de la cellule
c -----------------------------------------------------------
c
      zx = 0.5 * ( xvalp + xvalm )
      zy = 0.5 * ( yvalp + yvalm )

c
c     recherche du maximum dans les vecteurs
c -------------------------------------------
c
      supval = 0.0
      do 1 i=1,ntotv
         if(abs(kx(i)) .gt. supval)  supval = kx(i)
         if(abs(kxy(i)) .gt. supval) supval = kxy(i)
         if(abs(kyx(i)) .gt. supval) supval = kyx(i)
         if(abs(ky(i)) .gt. supval)  supval = ky(i)
    1 continue

c
c     verification des valeurs nulles sur la diagonale en entree
c ---------------------------------------------------------------
c
      constant = 0.000001 * supval
      do 2 i=1,ntotv
c
c     somme en valeur absolue des entrees
c -------------------------------------------
c
         sum = abs(kx(i))  + abs(kxy(i))
     1       + abs(kyx(i)) + abs(ky(i))
c
c     verification de la somme
c --------------------------------
c
         if(sum .lt. constant) then
           kx(i) = constant
           ky(i) = constant
         end if

    2 continue
      
c
c     initialisation de la matrice et du second membre
c ----------------------------------------------------
c
      do 3 i=1,nmat
         hrhs(i,1) = 0.0
         hrhs(i,2) = 0.0
         maa(i)=0.0
         mja(i)=0
         mia(i)=0  
 3    continue
     
      do 31 i=nmat+1,nmorse
         maa(i)=0.0
         mja(i)=0
 31   continue
      mia(nmat+1)=nmorse+1
      
c
c     boucles sur les noeuds
c ---------------------------
c   
      do 4 j=1,nvaly
         j2 = ( j - 1 ) * nvalx
c
c     recherche de la seconde coordonnee dans la grille de permeabilite
c -------------------------------------------------------------------------
c
         indy = ( j + nyhr - 1 ) / nyhr
c
c     calcul de la taille d'un element selon y
c ------------------------------------------------
c
         if(nyh .eq. 1) then
           yvalm = 0.0
           yvalp = 1.0
         else
           if(indy .eq. 1) then
             yvalm = 1.5 * y(1) - 0.5 * y(2)
             yvalp = 0.5 * ( y(indy) + y(indy+1) )
           end if
           if(indy .gt. 1 .and. indy .lt. nyh) then
             yvalm = 0.5 * ( y(indy-1) + y(indy) )
             yvalp = 0.5 * ( y(indy) + y(indy+1) )
           end if
           if(indy .eq. nyh) then
             yvalm = 0.5 * ( y(indy-1) + y(indy) )
             yvalp = 1.5 * y(indy) - 0.5 * y(indy-1)
           end if
         end if
         dely = ( yvalp - yvalm ) / float(nyhr)


      if (j .le. nfyr) then
       
      do 40 i=1,nvalx
c
c     recherche de la premiere coordonnee dans la grille de permeabilite
c --------------------------------------------------------------------------
c
         indx = ( i + nxhr - 1 ) / nxhr
c
c     calcul de la taille d'un element selon x
c ------------------------------------------------
c
         if(nxh .eq. 1) then
           xvalm = 0.0
           xvalp = 1.0
         else
           if(indx .eq. 1) then
             xvalm = 1.5 * x(1) - 0.5 * x(2)
             xvalp = 0.5 * ( x(indx) + x(indx+1) )
           end if
           if(indx .gt. 1 .and. indx .lt. nxh) then
             xvalm = 0.5 * ( x(indx-1) + x(indx) )
             xvalp = 0.5 * ( x(indx) + x(indx+1) )
           end if
           if(indx .eq. nxh) then
             xvalm = 0.5 * ( x(indx-1) + x(indx) )
             xvalp = 1.5 * x(indx) - 0.5 * x(indx-1)
           end if
         end if
         delx = ( xvalp - xvalm ) / float(nxhr)
c
c     attribution de la permeabilite
c -------------------------------------
c
         indprm = indx + ( indy - 1 ) * nxh
         rkx = kx(indprm)
         rkxy = kxy(indprm)
         rkyx = kyx(indprm)
         rky = ky(indprm)

c
c     calcul de la contribution de la diagonale pour l'element
c
c     calcul des indices des 4 sommets de l'element 
c     en utilisant la propriete de la periodicite
c --------------------------------------------------------------
c
         ind1 = i + j2
         if(i .eq. nvalx) then
           ind2 = 1 + j2
         else
           ind2 = ind1 + 1
         end if
         if( j .eq. nvaly) then
           ind3 = i
         else
           ind3 = ind1 + nvalx
         end if
         if(j .eq. nvaly) then
           if(i .eq. nvalx) then
             ind4 = 1
           else
             ind4 = i + 1
           end if
         else
           ind4 = ind2 + nvalx
         end if        
c
c     calcul du vecteur MIA pour ZONE 1
c---------------------------------------
c
         if (j .eq. 1) then
            if (i .eq. 1) then
               mia(ind1)=1
               mia(ind2)=2
               mia(ind2+1)=4
               mia(ind3)=1+(nvalx-2)*2+3+1
               mia(ind4)=mia(ind3)+4
               mia(ind4+1)=mia(ind4)+5
            else
               if ((i+2 .ge. 2) .and. (i+1 .le. nvalx-1)) then
                  mia(ind2+1)=mia(ind2)+2
                  mia(ind4+1)=mia(ind4)+5
               else
                  if (i+1 .eq. nvalx) then
                     mia(ind4+1)=mia(ind4)+6
                  end if
               end if
            end if
         else
            if (i .eq. 1) then
               mia(ind4)=mia(ind3)+4
               mia(ind4+1)=mia(ind4)+5
            else
              if ((i+2 .ge. 2) .and. (i+1 .le. nvalx-1)) then
                  mia(ind4+1)=mia(ind4)+5
               else
                  if (i+1 .eq. nvalx) then
                     mia(ind4+1)=mia(ind4)+6
                  end if 
               end if
            end if
         end if
c     
c     calcul du meme terme pour tous les elements de la diagonale 
c ---------------------------------------------------------------
c
         term1 = third * ( dely * rkx / delx + delx * rky / dely )
     1          + quarter * ( rkxy + rkyx )
         term2 = third * ( dely * rkx / delx + delx * rky / dely )
     1          - quarter * ( rkxy + rkyx )

c     calcul du vecteur maa
         maa(mia(ind1+1)-1)=maa(mia(ind1+1)-1)+term1
         maa(mia(ind2+1)-1)=maa(mia(ind2+1)-1)+term2
         maa(mia(ind3+1)-1)=maa(mia(ind3+1)-1)+term2
         maa(mia(ind4+1)-1)=maa(mia(ind4+1)-1)+term1
c     calcul du vecteur mja
         mja(mia(ind1+1)-1)=ind1
         mja(mia(ind2+1)-1)=ind2
         mja(mia(ind3+1)-1)=ind3
         mja(mia(ind4+1)-1)=ind4
c
c     premiere diagonale
c -----------------------
c
         if (ind1 .ge. ind2) then
            call remplissage_maa(ind1,ind2,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              - third * dely * rkx / delx 
     1              + quarter * ( rkxy - rkyx )
     1              + sixth * delx * rky / dely
            else
               maa(first)=maa(first)
     1              - third * dely * rkx / delx 
     1              + quarter * ( rkxy - rkyx )
     1              + sixth * delx * rky / dely
               mja(first)=ind2
            end if
         else
            call remplissage_maa(ind2,ind1,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              - third * dely * rkx / delx 
     1              + quarter * ( rkyx - rkxy )
     1              + sixth * delx * rky / dely
            else
               maa(first)=maa(first)
     1              - third * dely * rkx / delx 
     1              + quarter * ( rkyx - rkxy )
     1              + sixth * delx * rky / dely
               mja(first)=ind1
            end if
         end if
         
         if (ind3 .ge.ind4) then
            call remplissage_maa(ind3,ind4,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              - third * dely * rkx / delx 
     1              + quarter * ( rkyx - rkxy )
     1              + sixth * delx * rky / dely
            else
               maa(first)=maa(first)
     1              - third * dely * rkx / delx 
     1              + quarter * ( rkyx - rkxy )
     1              + sixth * delx * rky / dely
               mja(first)=ind4
            end if
         else
            call remplissage_maa(ind4,ind3,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              - third * dely * rkx / delx 
     1              + quarter * ( rkxy - rkyx )
     1              + sixth * delx * rky / dely 
            else
               maa(first)=maa(first)
     1              - third * dely * rkx / delx 
     1              + quarter * ( rkxy - rkyx )
     1              + sixth * delx * rky / dely
               mja(first)=ind3
            end if
         end if
c     
c     seconde diagonale
c     -------------------------
c
         if (ind2 .ge. ind3) then 
            call remplissage_maa(ind2,ind3,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              - sixth * ( dely * rkx / delx + delx * rky / dely )
     1              + quarter * ( rkxy + rkyx )
            else
               maa(first)=maa(first)
     1              - sixth * ( dely * rkx / delx + delx * rky / dely )
     1              + quarter * ( rkxy + rkyx )
               mja(first)=ind3
            end if
         else
            call remplissage_maa(ind3,ind2,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              - sixth * ( dely * rkx / delx + delx * rky / dely )
     1              + quarter * ( rkxy + rkyx ) 
            else
               maa(first)=maa(first)
     1              - sixth * ( dely * rkx / delx + delx * rky / dely )
     1              + quarter * ( rkxy + rkyx )
               mja(first)=ind2
            end if
         end if
c     
c     troisieme diagonale
c ---------------------------
c
         if (ind1 .ge. ind3) then
            call remplissage_maa(ind1,ind3,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              + sixth * dely * rkx / delx 
     1              + quarter * ( rkyx - rkxy )
     1              - third * delx * rky / dely
            else
               maa(first)=maa(first)
     1              + sixth * dely * rkx / delx 
     1              + quarter * ( rkyx - rkxy )
     1              - third * delx * rky / dely
               mja(first)=ind3
            end if
         else
            call remplissage_maa(ind3,ind1,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              + sixth * dely * rkx / delx 
     1              + quarter * ( rkxy - rkyx )
     1              - third * delx * rky / dely
            else
               maa(first)=maa(first)
     1              + sixth * dely * rkx / delx 
     1              + quarter * ( rkxy - rkyx )
     1              - third * delx * rky / dely
               mja(first)=ind1
            end if
         end if
         
         if (ind2 .ge. ind4) then
            call remplissage_maa(ind2,ind4,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              + sixth * dely * rkx / delx 
     1              + quarter * ( rkxy - rkyx )
     1              - third * delx * rky / dely
            else
               maa(first)=maa(first)
     1              + sixth * dely * rkx / delx 
     1              + quarter * ( rkxy - rkyx )
     1              - third * delx * rky / dely
               mja(first)=ind4
            end if
         else
            call remplissage_maa(ind4,ind2,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              + sixth * dely * rkx / delx 
     1              + quarter * ( rkyx - rkxy )
     1              - third * delx * rky / dely
            else
               maa(first)=maa(first)
     1              + sixth * dely * rkx / delx 
     1              + quarter * ( rkyx - rkxy )
     1              - third * delx * rky / dely
               mja(first)=ind2
            end if
         end if
c     
c     quatrieme diagonale
c ---------------------------
c
         if (ind1 .ge. ind4) then
            call remplissage_maa(ind1,ind4,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              - sixth * ( dely * rkx / delx + delx * rky / dely )
     1              - quarter * ( rkxy + rkyx )
            else
               maa(first)=maa(first)
     1              - sixth * ( dely * rkx / delx + delx * rky / dely )
     1              - quarter * ( rkxy + rkyx )
               mja(first)=ind4
            end if
         else
            call remplissage_maa(ind4,ind1,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              - sixth * ( dely * rkx / delx + delx * rky / dely )
     1              - quarter * ( rkxy + rkyx )
            else
               maa(first)=maa(first)
     1              - sixth * ( dely * rkx / delx + delx * rky / dely )
     1              - quarter * ( rkxy + rkyx )
               mja(first)=ind1
            end if
         end if 
c     
c     calcul du second membre pour chaque probleme
c     ------------------------------------------------
c
         hrhs(ind1,1) = hrhs(ind1,1)
     1            + 0.5 * ( dely * rkx / delx + delx * rkyx / dely )
         hrhs(ind1,2) = hrhs(ind1,2)
     1            + 0.5 * ( dely * rkxy / delx + delx * rky / dely )
c
         hrhs(ind2,1) = hrhs(ind2,1)
     1            - 0.5 * ( dely * rkx / delx - delx * rkyx / dely )
         hrhs(ind2,2) = hrhs(ind2,2)
     1            - 0.5 * ( dely * rkxy / delx - delx * rky / dely )
c
         hrhs(ind3,1) = hrhs(ind3,1)
     1            + 0.5 * ( dely * rkx / delx - delx * rkyx / dely )
         hrhs(ind3,2) = hrhs(ind3,2)
     1            + 0.5 * ( dely * rkxy / delx - delx * rky / dely )
c
         hrhs(ind4,1) = hrhs(ind4,1)
     1            - 0.5 * ( dely * rkx / delx + delx * rkyx / dely )
         hrhs(ind4,2) = hrhs(ind4,2)
     1            - 0.5 * ( dely * rkxy / delx + delx * rky / dely )

c
 40   continue

      end if

      if ((j .ge. nfyr+1)   .and.
     *    (j .le. (nvaly-nfyr))) then
     
      do 41 i=1,nfxr
c
c     recherche de la premiere coordonnee dans la grille de permeabilite
c --------------------------------------------------------------------------
c
         indx = ( i + nxhr - 1 ) / nxhr
c
c        calcul de la taille d'un element selon x
c ------------------------------------------------
c
         if(nxh .eq. 1) then
           xvalm = 0.0
           xvalp = 1.0
         else
           if(indx .eq. 1) then
             xvalm = 1.5 * x(1) - 0.5 * x(2)
             xvalp = 0.5 * ( x(indx) + x(indx+1) )
           end if
           if(indx .gt. 1 .and. indx .lt. nxh) then
             xvalm = 0.5 * ( x(indx-1) + x(indx) )
             xvalp = 0.5 * ( x(indx) + x(indx+1) )
           end if
           if(indx .eq. nxh) then
             xvalm = 0.5 * ( x(indx-1) + x(indx) )
             xvalp = 1.5 * x(indx) - 0.5 * x(indx-1)
           end if
         end if
         delx = ( xvalp - xvalm ) / float(nxhr)
c
c        attribution de la permabilite
c -------------------------------------
c
         if (( j .ge. nfyr+1) .and. ( j .le. (nfyr + nyhr)) ) then
            indprm = indx + ( indy - 1 ) * nxh
         else
            indprm = indx + ( indy - 1 ) * nxh - (indy-1 -nfy) * nxhf
         end if
         rkx = kx(indprm)
         rkxy = kxy(indprm)
         rkyx = kyx(indprm)
         rky = ky(indprm)
c
c     calcul de la contribution de la diagonale pour la cellule
c
c     calcul des indices des 4 sommets de l'element
c     en utilisant la propriete de la periodicite
c ----------------------------------------------------------
c

         if ((j .eq. nfyr+1).or. ( j .eq. (nfyr +2 ))) then
            ind1 = i + j2
         else
            ind1 = i + j2-(j-(nfyr+2))*(nvalfx-1)
         end if
         ind2 = ind1 + 1
         if( j .eq. (nfyr + 1)) then
           ind3 = ind1 + nvalx
         else
           ind3 = ind1 + nvalx - ( nvalfx - 1 )
         end if
         ind4=ind3+1

c
c     calcul du vecteur MIA pour ZONE 2
c---------------------------------------
c

         if ((j .ne. nvaly-nfyr+1) .and. (j .ne. nvaly-1)) then
            if (i .eq. 1) then
               mia(ind4)=mia(ind3)+4
            end if
            if ((i+2 .ge. 2) .and. (i+2 .le. nfxr+1)) then
               mia(ind4+1)=mia(ind4)+5
            else
               mia(ind4+1)=mia(ind4)+4
               if (j .eq. nvaly-nfyr) then
                  do 410 ii=2,nvalfx
                     mia(ind4+ii)=mia(ind4+ii-1)+2
 410              continue
               end if             
            end if 
         else
            if (i .eq. 1) then
               mia(ind4)=mia(ind3)+7
            end if
            if (i+1 .eq. nfxr+1) then
               mia(ind4+1)=mia(ind4)+7
               if (j .eq. nvaly-nfyr) then
                  do 411 ii=2,nvalfx
                     mia(ind4+ii)=mia(ind4+ii-1)+5
 411              continue
               end if
            else
               if ((i+2 .ge. 2) .and. (i+2 .le. nfxr+2)) then
                  mia(ind4+1)=mia(ind4)+5
               end if
            end if
         end if
c
c     calcul du meme terme pour tous les elements de la diagonale
c ---------------------------------------------------------------
c
         term1 = third * ( dely * rkx / delx + delx * rky / dely )
     1          + quarter * ( rkxy + rkyx )
         term2 = third * ( dely * rkx / delx + delx * rky / dely )
     1          - quarter * ( rkxy + rkyx )
c
c     calcul du vecteur maa
         maa(mia(ind1+1)-1)=maa(mia(ind1+1)-1)+term1
         maa(mia(ind2+1)-1)=maa(mia(ind2+1)-1)+term2
         maa(mia(ind3+1)-1)=maa(mia(ind3+1)-1)+term2
         maa(mia(ind4+1)-1)=maa(mia(ind4+1)-1)+term1
c     calcul du vecteur mja
         mja(mia(ind1+1)-1)=ind1
         mja(mia(ind2+1)-1)=ind2
         mja(mia(ind3+1)-1)=ind3
         mja(mia(ind4+1)-1)=ind4

c
c     premiere diagonale
c -----------------------
c
         if (ind1 .ge. ind2) then
            call remplissage_maa(ind1,ind2,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              - third * dely * rkx / delx 
     1              + quarter * ( rkxy - rkyx )
     1              + sixth * delx * rky / dely
            else
               maa(first)=maa(first)
     1              - third * dely * rkx / delx 
     1              + quarter * ( rkxy - rkyx )
     1              + sixth * delx * rky / dely
               mja(first)=ind2
            end if
         else
            call remplissage_maa(ind2,ind1,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              - third * dely * rkx / delx 
     1              + quarter * ( rkyx - rkxy )
     1              + sixth * delx * rky / dely
            else
               maa(first)=maa(first)
     1              - third * dely * rkx / delx 
     1              + quarter * ( rkyx - rkxy )
     1              + sixth * delx * rky / dely
               mja(first)=ind1
            end if
         end if
         
         if (ind3 .ge.ind4) then
            call remplissage_maa(ind3,ind4,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              - third * dely * rkx / delx 
     1              + quarter * ( rkyx - rkxy )
     1              + sixth * delx * rky / dely
            else
               maa(first)=maa(first)
     1              - third * dely * rkx / delx 
     1              + quarter * ( rkyx - rkxy )
     1              + sixth * delx * rky / dely
               mja(first)=ind4
            end if
         else
            call remplissage_maa(ind4,ind3,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              - third * dely * rkx / delx  
     1              + quarter * ( rkxy - rkyx )
     1              + sixth * delx * rky / dely 
            else
               maa(first)=maa(first)
     1              - third * dely * rkx / delx 
     1              + quarter * ( rkxy - rkyx )
     1              + sixth * delx * rky / dely
               mja(first)=ind3
            end if
         end if
c     
c     seconde diagonale
c -------------------------
c
         if (ind2 .ge. ind3) then 
            call remplissage_maa(ind2,ind3,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              - sixth * ( dely * rkx / delx + delx * rky / dely )
     1              + quarter * ( rkxy + rkyx )
            else
               maa(first)=maa(first)
     1              - sixth * ( dely * rkx / delx + delx * rky / dely )
     1              + quarter * ( rkxy + rkyx )
               mja(first)=ind3
            end if
         else
            call remplissage_maa(ind3,ind2,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              - sixth * ( dely * rkx / delx + delx * rky / dely )
     1              + quarter * ( rkxy + rkyx ) 
            else
               maa(first)=maa(first)
     1              - sixth * ( dely * rkx / delx + delx * rky / dely )
     1              + quarter * ( rkxy + rkyx )
               mja(first)=ind2
            end if
         end if
         
c     
c     troisieme diagonale
c ---------------------------
c
         if (ind1 .ge. ind3) then
            call remplissage_maa(ind1,ind3,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              + sixth * dely * rkx / delx 
     1              + quarter * ( rkyx - rkxy )
     1              - third * delx * rky / dely
            else
               maa(first)=maa(first)
     1              + sixth * dely * rkx / delx 
     1              + quarter * ( rkyx - rkxy )
     1              - third * delx * rky / dely
               mja(first)=ind3
            end if
         else
            call remplissage_maa(ind3,ind1,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              + sixth * dely * rkx / delx 
     1              + quarter * ( rkxy - rkyx )
     1              - third * delx * rky / dely
            else
               maa(first)=maa(first)
     1              + sixth * dely * rkx / delx 
     1              + quarter * ( rkxy - rkyx )
     1              - third * delx * rky / dely
               mja(first)=ind1
            end if
         end if
         
         if (ind2 .ge. ind4) then
            call remplissage_maa(ind2,ind4,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              + sixth * dely * rkx / delx 
     1              + quarter * ( rkxy - rkyx )
     1              - third * delx * rky / dely
            else
               maa(first)=maa(first)
     1              + sixth * dely * rkx / delx 
     1              + quarter * ( rkxy - rkyx )
     1              - third * delx * rky / dely
               mja(first)=ind4
            end if
         else
            call remplissage_maa(ind4,ind2,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              + sixth * dely * rkx / delx 
     1              + quarter * ( rkyx - rkxy )
     1              - third * delx * rky / dely
            else
               maa(first)=maa(first)
     1              + sixth * dely * rkx / delx 
     1              + quarter * ( rkyx - rkxy )
     1              - third * delx * rky / dely
               mja(first)=ind2
            end if
         end if
c     
c     quatrieme diagonale
c ---------------------------
c
         if (ind1 .ge. ind4) then
            call remplissage_maa(ind1,ind4,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              - sixth * ( dely * rkx / delx + delx * rky / dely )
     1              - quarter * ( rkxy + rkyx )
            else
               maa(first)=maa(first)
     1              - sixth * ( dely * rkx / delx + delx * rky / dely )
     1              - quarter * ( rkxy + rkyx )
               mja(first)=ind4
            end if
         else
            call remplissage_maa(ind4,ind1,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              - sixth * ( dely * rkx / delx + delx * rky / dely )
     1              - quarter * ( rkxy + rkyx )
            else
               maa(first)=maa(first)
     1              - sixth * ( dely * rkx / delx + delx * rky / dely )
     1              - quarter * ( rkxy + rkyx )
               mja(first)=ind1
            end if
         end if 
c     
c     calcul du second membre pour chacun des problemes
c     -----------------------------------------------------
c     
         hrhs(ind1,1) = hrhs(ind1,1)
     1            + 0.5 * ( dely * rkx / delx + delx * rkyx / dely )
         hrhs(ind1,2) = hrhs(ind1,2)
     1            + 0.5 * ( dely * rkxy / delx + delx * rky / dely )
c
         hrhs(ind2,1) = hrhs(ind2,1)
     1            - 0.5 * ( dely * rkx / delx - delx * rkyx / dely )
         hrhs(ind2,2) = hrhs(ind2,2)
     1            - 0.5 * ( dely * rkxy / delx - delx * rky / dely )
c
         hrhs(ind3,1) = hrhs(ind3,1)
     1            + 0.5 * ( dely * rkx / delx - delx * rkyx / dely )
         hrhs(ind3,2) = hrhs(ind3,2)
     1            + 0.5 * ( dely * rkxy / delx - delx * rky / dely )
c
         hrhs(ind4,1) = hrhs(ind4,1)
     1            - 0.5 * ( dely * rkx / delx + delx * rkyx / dely )
         hrhs(ind4,2) = hrhs(ind4,2)
     1            - 0.5 * ( dely * rkxy / delx + delx * rky / dely )

 41   continue   

      do 42 i=nvalx-nfxr+1,nvalx
c
c        recherche de la premiere coordonnee dans la grille de permeabilite
c --------------------------------------------------------------------------
c
         indx = ( i + nxhr - 1 ) / nxhr

c
c        calcul de la taille d'un element selon x
c ------------------------------------------------
c
         if(nxh .eq. 1) then
           xvalm = 0.0
           xvalp = 1.0
         else
           if(indx .eq. 1) then
             xvalm = 1.5 * x(1) - 0.5 * x(2)
             xvalp = 0.5 * ( x(indx) + x(indx+1) )
           end if
           if(indx .gt. 1 .and. indx .lt. nxh) then
             xvalm = 0.5 * ( x(indx-1) + x(indx) )
             xvalp = 0.5 * ( x(indx) + x(indx+1) )
           end if
           if(indx .eq. nxh) then
             xvalm = 0.5 * ( x(indx-1) + x(indx) )
             xvalp = 1.5 * x(indx) - 0.5 * x(indx-1)
           end if
         end if
         delx = ( xvalp - xvalm ) / float(nxhr)
c
c        attribution de la permabilite
c -------------------------------------
c
         indprm = indx + ( indy - 1 ) * nxh - (indy -nfy) * nxhf
         rkx = kx(indprm)
         rkxy = kxy(indprm)
         rkyx = kyx(indprm)
         rky = ky(indprm)

c
c     calcul de la contribution de la diagonale pour la cellule
c     
c     calcul des indices des 4 sommets de l'element 
c     en utilisant la propriete de la periodicite
c --------------------------------------------------------------
c
         if (j .eq. nfyr+1) then
            ind1 = i + j2
         else
            ind1 = i + j2-(j-(nfyr+1))*(nvalfx-1)
         end if
         if (i .eq. nvalx) then
            if (j .eq. nfyr+1) then
               ind2=1+j2
            else
               ind2=ind1-(nvalx-nvalfx)
            end if
         else
            ind2 = ind1 + 1
         end if
         if( j .eq. (nvaly-nfyr)) then
           ind3 = ind1 + nvalx
         else
           ind3 = ind1 + (nvalx-nvalfx) + 1 
         end if
         if (i .eq. nvalx) then
            if (j .eq. nvaly-nfyr) then
               ind4 = ind3 - nvalx +1
            else
               ind4=ind3-(nvalx-nvalfx)
            end if
         else
            ind4=ind3+1
         end if

c
c     calcul du vecteur MIA pour ZONE 2 bis
c-------------------------------------------
c
         if (j .ne. nvaly-nfyr) then
            if (i .eq. nvalx-nfxr+1) then
               if (i .eq. nvalx) then
                  mia(ind3+1)=mia(ind3)+4
               end if
               if (i+1 .eq. nvalx) then
                  mia(ind4)=mia(ind3)+3
                  mia(ind4+1)=mia(ind4)+6
               end if
               if (i+2 .le. nvalx) then
                  mia(ind4)=mia(ind3)+3
                  mia(ind4+1)=mia(ind4)+5
               end if
            else
               if (i .ge. nvalx-nfxr+2) then
                  if ((i+2 .ge. nvalx-nfxr+2) 
     *                 .and. (i+1 .le. nvalx-1)) then
                     mia(ind4+1)=mia(ind4)+5
                  end if
                  if (i+1 .eq. nvalx) then
                     mia(ind4+1)=mia(ind4)+6
                  end if
               end if
            end if
         else
            if ((j .eq. nvaly-nfyr) .and. (j .ne. nvaly-1)) then
               if (i .eq. nvalx-nfxr+1) then
                  if (i .eq. nvalx) then
                     mia(ind3+1)=mia(ind3)+5
                  else
                     if (i+1 .eq. nvalx) then
                        mia(ind4)=mia(ind3)+4
                        mia(ind4+1)=mia(ind4)+6
                     end if
                     if (i+2 .le. nvalx) then
                        mia(ind4)=mia(ind3)+4
                        mia(ind4+1)=mia(ind4)+5
                     end if
                  end if
               else
                  if (i+1 .eq. nvalx) then
                     mia(ind4+1)=mia(ind4)+6
                  else
                     if (i+2 .le. nvalx) then
                        mia(ind4+1)=mia(ind4)+5
                     end if
                  end if
               end if
            else
               if ((j .eq. nvaly-nfyr) .and. (j .eq. nvaly-1)) then
                  if (i .eq. nvalx-nfxr+1) then
                     if (i+1 .eq. nvalx) then
                        mia(ind4)=mia(ind3)+7
                     end if
                     if (i+2 .le. nvalx) then
                        mia(ind4)=mia(ind3)+7
                        mia(ind4+1)=mia(ind4)+8
                     end if
                  else
                     if (i+1 .eq. nvalx) then
                        mia(ind4)=mia(ind3)+8
                     end if
                     if (i+2 .le. nvalx) then
                        mia(ind4)=mia(ind3)+8
                        mia(ind4+1)=mia(ind4)+8
                     end if
                  end if
               end if
            end if
         end if
            
c
c     calcul du meme terme pour tous les elements de la diagonale
c ---------------------------------------------------------------
c
         term1 = third * ( dely * rkx / delx + delx * rky / dely )
     1          + quarter * ( rkxy + rkyx )
         term2 = third * ( dely * rkx / delx + delx * rky / dely )
     1          - quarter * ( rkxy + rkyx )

c     calcul du vecteur maa
         maa(mia(ind1+1)-1)=maa(mia(ind1+1)-1)+term1
         maa(mia(ind2+1)-1)=maa(mia(ind2+1)-1)+term2
         maa(mia(ind3+1)-1)=maa(mia(ind3+1)-1)+term2
         maa(mia(ind4+1)-1)=maa(mia(ind4+1)-1)+term1
c     calcul du vecteur mja
         mja(mia(ind1+1)-1)=ind1
         mja(mia(ind2+1)-1)=ind2
         mja(mia(ind3+1)-1)=ind3
         mja(mia(ind4+1)-1)=ind4

c
c     premiere diagonale
c -----------------------
c
         if (ind1 .ge. ind2) then
            call remplissage_maa(ind1,ind2,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              - third * dely * rkx / delx 
     1              + quarter * ( rkxy - rkyx )
     1              + sixth * delx * rky / dely
            else
               maa(first)=maa(first)
     1              - third * dely * rkx / delx 
     1              + quarter * ( rkxy - rkyx )
     1              + sixth * delx * rky / dely
               mja(first)=ind2
            end if
         else
            call remplissage_maa(ind2,ind1,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              - third * dely * rkx / delx 
     1              + quarter * ( rkyx - rkxy )
     1              + sixth * delx * rky / dely
            else
               maa(first)=maa(first)
     1              - third * dely * rkx / delx 
     1              + quarter * ( rkyx - rkxy )
     1              + sixth * delx * rky / dely
               mja(first)=ind1
            end if
         end if
         
         if (ind3 .ge.ind4) then
            call remplissage_maa(ind3,ind4,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              - third * dely * rkx / delx 
     1              + quarter * ( rkyx - rkxy )
     1              + sixth * delx * rky / dely
            else
               maa(first)=maa(first)
     1              - third * dely * rkx / delx 
     1              + quarter * ( rkyx - rkxy )
     1              + sixth * delx * rky / dely
               mja(first)=ind4
            end if
         else
            call remplissage_maa(ind4,ind3,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              - third * dely * rkx / delx 
     1              + quarter * ( rkxy - rkyx )
     1              + sixth * delx * rky / dely 
            else
               maa(first)=maa(first)
     1              - third * dely * rkx / delx 
     1              + quarter * ( rkxy - rkyx )
     1              + sixth * delx * rky / dely
               mja(first)=ind3
            end if
         end if
c     
c     seconde diagonale
c -------------------------
c
         if (ind2 .ge. ind3) then 
            call remplissage_maa(ind2,ind3,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              - sixth * ( dely * rkx / delx + delx * rky / dely )
     1              + quarter * ( rkxy + rkyx )
            else
               maa(first)=maa(first)
     1              - sixth * ( dely * rkx / delx + delx * rky / dely )
     1              + quarter * ( rkxy + rkyx )
               mja(first)=ind3
            end if
         else
            call remplissage_maa(ind3,ind2,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              - sixth * ( dely * rkx / delx + delx * rky / dely )
     1              + quarter * ( rkxy + rkyx ) 
            else
               maa(first)=maa(first)
     1              - sixth * ( dely * rkx / delx + delx * rky / dely )
     1              + quarter * ( rkxy + rkyx )
               mja(first)=ind2
            end if
         end if
         
c     
c     troisieme diagonale
c ---------------------------
c
         if (ind1 .ge. ind3) then
            call remplissage_maa(ind1,ind3,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              + sixth * dely * rkx / delx 
     1              + quarter * ( rkyx - rkxy )
     1              - third * delx * rky / dely
            else
               maa(first)=maa(first)
     1              + sixth * dely * rkx / delx 
     1              + quarter * ( rkyx - rkxy )
     1              - third * delx * rky / dely
               mja(first)=ind3
            end if
         else
            call remplissage_maa(ind3,ind1,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              + sixth * dely * rkx / delx 
     1              + quarter * ( rkxy - rkyx )
     1              - third * delx * rky / dely
            else
               maa(first)=maa(first)
     1              + sixth * dely * rkx / delx 
     1              + quarter * ( rkxy - rkyx )
     1              - third * delx * rky / dely
               mja(first)=ind1
            end if
         end if
         
         if (ind2 .ge. ind4) then
            call remplissage_maa(ind2,ind4,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              + sixth * dely * rkx / delx 
     1              + quarter * ( rkxy - rkyx )
     1              - third * delx * rky / dely
            else
               maa(first)=maa(first)
     1              + sixth * dely * rkx / delx 
     1              + quarter * ( rkxy - rkyx )
     1              - third * delx * rky / dely
               mja(first)=ind4
            end if
         else
            call remplissage_maa(ind4,ind2,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              + sixth * dely * rkx / delx 
     1              + quarter * ( rkyx - rkxy )
     1              - third * delx * rky / dely
            else
               maa(first)=maa(first)
     1              + sixth * dely * rkx / delx 
     1              + quarter * ( rkyx - rkxy )
     1              - third * delx * rky / dely
               mja(first)=ind2
            end if
         end if
c     
c     quatrieme diagonale
c     ---------------------------
c     
         if (ind1 .ge. ind4) then
            call remplissage_maa(ind1,ind4,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              - sixth * ( dely * rkx / delx + delx * rky / dely )
     1              - quarter * ( rkxy + rkyx )
            else
               maa(first)=maa(first)
     1              - sixth * ( dely * rkx / delx + delx * rky / dely )
     1              - quarter * ( rkxy + rkyx )
               mja(first)=ind4
            end if
         else
            call remplissage_maa(ind4,ind1,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              - sixth * ( dely * rkx / delx + delx * rky / dely )
     1              - quarter * ( rkxy + rkyx )
            else
               maa(first)=maa(first)
     1              - sixth * ( dely * rkx / delx + delx * rky / dely )
     1              - quarter * ( rkxy + rkyx )
               mja(first)=ind1
            end if
         end if 
c     
c     calcul du second membre pour chacun des problemes
c -----------------------------------------------------
c

         hrhs(ind1,1) = hrhs(ind1,1)
     1            + 0.5 * ( dely * rkx / delx + delx * rkyx / dely )
         hrhs(ind1,2) = hrhs(ind1,2)
     1            + 0.5 * ( dely * rkxy / delx + delx * rky / dely )
c
         hrhs(ind2,1) = hrhs(ind2,1)
     1            - 0.5 * ( dely * rkx / delx - delx * rkyx / dely )
         hrhs(ind2,2) = hrhs(ind2,2)
     1            - 0.5 * ( dely * rkxy / delx - delx * rky / dely )
c
         hrhs(ind3,1) = hrhs(ind3,1)
     1            + 0.5 * ( dely * rkx / delx - delx * rkyx / dely )
         hrhs(ind3,2) = hrhs(ind3,2)
     1            + 0.5 * ( dely * rkxy / delx - delx * rky / dely )
c
         hrhs(ind4,1) = hrhs(ind4,1)
     1            - 0.5 * ( dely * rkx / delx + delx * rkyx / dely )
         hrhs(ind4,2) = hrhs(ind4,2)
     1            - 0.5 * ( dely * rkxy / delx + delx * rky / dely )

 42   continue

      end if


      if ((j .ge. (nvaly-nfyr+1)) .and. 
     *    (j.le. nvaly)) then

      do 43 i=1,nvalx
c
c        recherche de la premiere coordonnee dans la grille de permeabilite
c --------------------------------------------------------------------------
c
         indx = ( i + nxhr - 1 ) / nxhr
c
c        calcul de la taille d'un element selon x
c ------------------------------------------------
c
         if(nxh .eq. 1) then
           xvalm = 0.0
           xvalp = 1.0
         else
           if(indx .eq. 1) then
             xvalm = 1.5 * x(1) - 0.5 * x(2)
             xvalp = 0.5 * ( x(indx) + x(indx+1) )
           end if
           if(indx .gt. 1 .and. indx .lt. nxh) then
             xvalm = 0.5 * ( x(indx-1) + x(indx) )
             xvalp = 0.5 * ( x(indx) + x(indx+1) )
           end if
           if(indx .eq. nxh) then
             xvalm = 0.5 * ( x(indx-1) + x(indx) )
             xvalp = 1.5 * x(indx) - 0.5 * x(indx-1)
           end if
         end if
         delx = ( xvalp - xvalm ) / float(nxhr)
c
c        attribution de la permabilite
c -------------------------------------
c
         indprm = indx + ( indy - 1 ) * nxh - ntotf
         rkx = kx(indprm)
         rkxy = kxy(indprm)
         rkyx = kyx(indprm)
         rky = ky(indprm)

c
c     calcul de la contribution de la diagonale pour la cellule
c
c     calcul des indices des 4 sommets de l'element 
c     en utilisant la propriete de la periodicite
c --------------------------------------------------------------
c
         ind1 = i + j2 - ( nvalfx - 1 )*( nvalfy - 1)
         if (i .eq. nvalx) then
            ind2 = ind1 - nvalx + 1
         else
            ind2 = ind1 + 1
         end if
         if (j .eq. nvaly) then
            ind3 = i
         else
            ind3 = ind1 + nvalx
         end if
         if (i .eq. nvalx) then
            if ( j .eq. nvaly ) then
               ind4 = 1
            else
               ind4=ind2 + nvalx
            end if
         else
            ind4 = ind3 + 1
         end if

c
c     calcul du vecteur MIA pour ZONE 3
c---------------------------------------
c
         if ((j .ge. nvaly-nfyr+1) .and. (j .ne. nvaly)) then
            if (j+1 .lt. nvaly) then
               if (i .eq. 1) then
                  mia(ind4)=mia(ind3)+4
                  mia(ind4+1)=mia(ind4)+5
               end if
               if ((i+2 .ge. 2) .and. (i+1 .lt. nvalx)) then
                  mia(ind4+1)=mia(ind4)+5
               else
                  if (i+1 .eq. nvalx) then
                     mia(ind4+1)=mia(ind4)+6
                  end if
               end if
            end if
            if (j+1 .eq. nvaly) then
               if (i .eq. 1) then
                  mia(ind4)=mia(ind3)+7
                  mia(ind4+1)=mia(ind4)+8
               else
                  if ((i+2 .ge. 2) .and. (i+2 .le. nvalx)) then
                     mia(ind4+1)=mia(ind4)+8
                  end if
               end if
            end if
         end if
c
c     calcul du meme terme pour tous les elements de la diagonale
c ---------------------------------------------------------------
c
         term1 = third * ( dely * rkx / delx + delx * rky / dely )
     1          + quarter * ( rkxy + rkyx )
         term2 = third * ( dely * rkx / delx + delx * rky / dely )
     1          - quarter * ( rkxy + rkyx )

c     calcul du vecteur maa
         maa(mia(ind1+1)-1)=maa(mia(ind1+1)-1)+term1
         maa(mia(ind2+1)-1)=maa(mia(ind2+1)-1)+term2
         maa(mia(ind3+1)-1)=maa(mia(ind3+1)-1)+term2
         maa(mia(ind4+1)-1)=maa(mia(ind4+1)-1)+term1
c     calcul du vecteur mja
         mja(mia(ind1+1)-1)=ind1
         mja(mia(ind2+1)-1)=ind2
         mja(mia(ind3+1)-1)=ind3
         mja(mia(ind4+1)-1)=ind4
c
c     premiere diagonale
c -----------------------
c
         if (ind1 .ge. ind2) then
            call remplissage_maa(ind1,ind2,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              - third * dely * rkx / delx 
     1              + quarter * ( rkxy - rkyx )
     1              + sixth * delx * rky / dely
            else
               maa(first)=maa(first)
     1              - third * dely * rkx / delx 
     1              + quarter * ( rkxy - rkyx )
     1              + sixth * delx * rky / dely
               mja(first)=ind2
            end if
         else
            call remplissage_maa(ind2,ind1,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              - third * dely * rkx / delx 
     1              + quarter * ( rkyx - rkxy )
     1              + sixth * delx * rky / dely
            else
               maa(first)=maa(first)
     1              - third * dely * rkx / delx 
     1              + quarter * ( rkyx - rkxy )
     1              + sixth * delx * rky / dely
               mja(first)=ind1
            end if
         end if
         
         if (ind3 .ge.ind4) then
            call remplissage_maa(ind3,ind4,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              - third * dely * rkx / delx 
     1              + quarter * ( rkyx - rkxy )
     1              + sixth * delx * rky / dely
            else
               maa(first)=maa(first)
     1              - third * dely * rkx / delx 
     1              + quarter * ( rkyx - rkxy )
     1              + sixth * delx * rky / dely
               mja(first)=ind4
            end if
         else
            call remplissage_maa(ind4,ind3,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              - third * dely * rkx / delx 
     1              + quarter * ( rkxy - rkyx )
     1              + sixth * delx * rky / dely 
            else
               maa(first)=maa(first)
     1              - third * dely * rkx / delx 
     1              + quarter * ( rkxy - rkyx )
     1              + sixth * delx * rky / dely
               mja(first)=ind3
            end if
         end if
c     
c     seconde diagonale
c     -------------------------
c     
         if (ind2 .ge. ind3) then 
            call remplissage_maa(ind2,ind3,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              - sixth * ( dely * rkx / delx + delx * rky / dely )
     1              + quarter * ( rkxy + rkyx )
            else
               maa(first)=maa(first)
     1              - sixth * ( dely * rkx / delx + delx * rky / dely )
     1              + quarter * ( rkxy + rkyx )
               mja(first)=ind3
            end if
         else
            call remplissage_maa(ind3,ind2,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              - sixth * ( dely * rkx / delx + delx * rky / dely )
     1              + quarter * ( rkxy + rkyx ) 
            else
               maa(first)=maa(first)
     1              - sixth * ( dely * rkx / delx + delx * rky / dely )
     1              + quarter * ( rkxy + rkyx )
               mja(first)=ind2
            end if
         end if
         
c     
c     troisieme diagonale
c     ---------------------------
c     
         if (ind1 .ge. ind3) then
            call remplissage_maa(ind1,ind3,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              + sixth * dely * rkx / delx 
     1              + quarter * ( rkyx - rkxy )
     1              - third * delx * rky / dely
            else
               maa(first)=maa(first)
     1              + sixth * dely * rkx / delx 
     1              + quarter * ( rkyx - rkxy )
     1              - third * delx * rky / dely
               mja(first)=ind3
            end if
         else
            call remplissage_maa(ind3,ind1,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              + sixth * dely * rkx / delx 
     1              + quarter * ( rkxy - rkyx )
     1              - third * delx * rky / dely
            else
               maa(first)=maa(first)
     1              + sixth * dely * rkx / delx 
     1              + quarter * ( rkxy - rkyx )
     1              - third * delx * rky / dely
               mja(first)=ind1
            end if
         end if
         
         if (ind2 .ge. ind4) then
            call remplissage_maa(ind2,ind4,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              + sixth * dely * rkx / delx 
     1              + quarter * ( rkxy - rkyx )
     1              - third * delx * rky / dely
            else
               maa(first)=maa(first)
     1              + sixth * dely * rkx / delx 
     1              + quarter * ( rkxy - rkyx )
     1              - third * delx * rky / dely
               mja(first)=ind4
            end if
         else
            call remplissage_maa(ind4,ind2,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              + sixth * dely * rkx / delx 
     1              + quarter * ( rkyx - rkxy )
     1              - third * delx * rky / dely
            else
               maa(first)=maa(first)
     1              + sixth * dely * rkx / delx 
     1              + quarter * ( rkyx - rkxy )
     1              - third * delx * rky / dely
               mja(first)=ind2
            end if
         end if
c     
c     quatrieme diagonale
c ---------------------------
c     
         if (ind1 .ge. ind4) then
            call remplissage_maa(ind1,ind4,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              - sixth * ( dely * rkx / delx + delx * rky / dely )
     1              - quarter * ( rkxy + rkyx )
            else
               maa(first)=maa(first)
     1              - sixth * ( dely * rkx / delx + delx * rky / dely )
     1              - quarter * ( rkxy + rkyx )
               mja(first)=ind4
            end if
         else
            call remplissage_maa(ind4,ind1,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              - sixth * ( dely * rkx / delx + delx * rky / dely )
     1              - quarter * ( rkxy + rkyx )
            else
               maa(first)=maa(first)
     1              - sixth * ( dely * rkx / delx + delx * rky / dely )
     1              - quarter * ( rkxy + rkyx )
               mja(first)=ind1
            end if
         end if 
c     
c     calcul du second membre pour chacun des problemes
c -----------------------------------------------------
c
         hrhs(ind1,1) = hrhs(ind1,1)
     1            + 0.5 * ( dely * rkx / delx + delx * rkyx / dely )
         hrhs(ind1,2) = hrhs(ind1,2)
     1            + 0.5 * ( dely * rkxy / delx + delx * rky / dely )
c
         hrhs(ind2,1) = hrhs(ind2,1)
     1            - 0.5 * ( dely * rkx / delx - delx * rkyx / dely )
         hrhs(ind2,2) = hrhs(ind2,2)
     1            - 0.5 * ( dely * rkxy / delx - delx * rky / dely )
c
         hrhs(ind3,1) = hrhs(ind3,1)
     1            + 0.5 * ( dely * rkx / delx - delx * rkyx / dely )
         hrhs(ind3,2) = hrhs(ind3,2)
     1            + 0.5 * ( dely * rkxy / delx - delx * rky / dely )
c
         hrhs(ind4,1) = hrhs(ind4,1)
     1            - 0.5 * ( dely * rkx / delx + delx * rkyx / dely )
         hrhs(ind4,2) = hrhs(ind4,2)
     1            - 0.5 * ( dely * rkxy / delx + delx * rky / dely )

 43   continue

      end if

 4    continue
 

c
c     initialisation des solutions initiales et vecteurs iteratifs
c ----------------------------------------------------------------
c

      do 5 i=1,nmat
         hx(i,1) = 0.0
         hx(i,2)=0.0
         h(i)=0.0
         g(i)=0.0
         cg(i)=0.0
  5   continue
      
c
c     tri des indices
c     ---------------
c
      do 6  i=1,nmat
         call Tri(mja(mia(i)),maa(mia(i)),mia(i+1)-mia(i))
 6    continue
      
c
c     decomposition de choleski incomplet
c -----------------------------------------
c     
      call ICholeski(nmat,maa,mia,mja,mal)

c     
c     resolution du premier probleme
c ------------------------------
c    
      eps=cgeps
      write(*,*)'precision : ',eps
      write(*,*)'Kmax=',kmax
      call gradcpmorse(nmat,maa,mia,mja,mal,hrhs(1,1),hx(1,1)
     *     ,g,h,cg,kmax,eps)

c     
c     reinitialisation
c ------------------------
c     
      do 7 i=1,nmat
         hx(i,2)=0.0
         h(i)=0.0
         g(i)=0.0
         cg(i)=0.0
 7    continue
c
c     resolution du second probleme
c ---------------------------------
c
      eps=cgeps
      
      call gradcpmorse(nmat,maa,mia,mja,mal,hrhs(1,2),hx(1,2)
     *,g,h,cg,kmax,eps)
      
c
c     retour au programme principal sans calcul du tenseur 
c --------------------------------------------------------
c
      if(iopt .ne. 0) return
c
c     calcul final de la moyenne basee sur les elements finis
c
c     initialisation des valeurs
c ------------------------------
c
       zkx = 0.0
       zkxy = 0.0
       zkyx = 0.0
       zky = 0.0
c
c     boucles sur les noeuds
c --------------------------
c
       do 8 j=1,nvaly
          j2 = ( j - 1 ) * nvalx
c
c     recherche de la seconde coordonnee dans la grille de permeabilite
c ---------------------------------------------------------------------
c
          indy = ( j + nyhr - 1 ) / nyhr
c
c     calcul de la taille d'un element selon y
c --------------------------------------------
c
          if(nyh .eq. 1) then
             yvalm = 0.0
             yvalp = 1.0
          else
            if(indy .eq. 1) then
              yvalm = 1.5 * y(1) - 0.5 * y(2)
              yvalp = 0.5 * ( y(indy) + y(indy+1) )
            end if
            if(indy .gt. 1 .and. indy .lt. nyh) then
              yvalm = 0.5 * ( y(indy-1) + y(indy) )
              yvalp = 0.5 * ( y(indy) + y(indy+1) )
            end if
            if(indy .eq. nyh) then
              yvalm = 0.5 * ( y(indy-1) + y(indy) )
              yvalp = 1.5 * y(indy) - 0.5 * y(indy-1)
            end if
          end if
          dely = ( yvalp - yvalm ) / float(nyhr)


         if (j .le. nfyr) then
         
            do 80 i=1,nvalx
c
c     recherche de la premiere coordonnee dans la grille de permeabilite
c ----------------------------------------------------------------------
c
               indx = ( i + nxhr - 1 ) / nxhr
c
c     calcul de la taille d'un element selon x
c --------------------------------------------
c
          if(nxh .eq. 1) then
            xvalm = 0.0
            xvalp = 1.0
         else
            if(indx .eq. 1) then
               xvalm = 1.5 * x(1) - 0.5 * x(2)
               xvalp = 0.5 * ( x(indx) + x(indx+1) )
           end if
           if(indx .gt. 1 .and. indx .lt. nxh) then
              xvalm = 0.5 * ( x(indx-1) + x(indx) )
              xvalp = 0.5 * ( x(indx) + x(indx+1) )
            end if
            if(indx .eq. nxh) then
              xvalm = 0.5 * ( x(indx-1) + x(indx) )
              xvalp = 1.5 * x(indx) - 0.5 * x(indx-1)
            end if
         end if
          delx = ( xvalp - xvalm ) / float(nxhr)
c
c     calcul du volume de la sous cellule en cours
c ------------------------------------------------
c
          del = delx * dely
c
c     attribution de la permeabilite
c ----------------------------------
c
          indprm = indx + ( indy - 1 ) * nxh
          rkx = kx(indprm)
          rkxy = kxy(indprm)
          rkyx = kyx(indprm)
          rky = ky(indprm)
c
c     calcul des indices des 4 sommets de l'element 
c     en utilisant la propriete de la periodicite
c --------------------------------------------------------------
c
          ind1 = i + j2
          if(i .eq. nvalx) then
            ind2 = 1 + j2
          else
            ind2 = ind1 + 1
          end if
          if( j .eq. nvaly) then
            ind3 = i
          else
            ind3 = ind1 + nvalx
          end if
          if(j .eq. nvaly) then
            if(i .eq. nvalx) then
              ind4 = 1
            else
              ind4 = i + 1
            end if
          else
            ind4 = ind2 + nvalx
          end if
c
c     ajout du terme respectif
c ----------------------------
c
          zkx = zkx + del
     *         * ( rkx + rkx * 0.5 * ( hx(ind2,1) - hx(ind1,1) 
     *         + hx(ind4,1) - hx(ind3,1) ) + rkxy * 0.5 * ( hx(ind3,1)
     *         - hx(ind1,1) + hx(ind4,1) - hx(ind2,1) ) )
c
          zkxy = zkxy + del
     *         * ( rkxy + rkx * 0.5 * ( hx(ind2,2) - hx(ind1,2)
     *         + hx(ind4,2) - hx(ind3,2) ) + rkxy * 0.5 * ( hx(ind3,2)
     *         - hx(ind1,2) + hx(ind4,2) - hx(ind2,2) ) )
c
          zkyx = zkyx + del
     *         * ( rkyx + rkyx * 0.5 * ( hx(ind2,1) - hx(ind1,1)
     *         + hx(ind4,1) - hx(ind3,1) ) + rky * 0.5 * ( hx(ind3,1)
     *         - hx(ind1,1) + hx(ind4,1) - hx(ind2,1) ) )
c
          zky = zky + del
     *         * ( rky + rkyx * 0.5 * ( hx(ind2,2) - hx(ind1,2) 
     *         + hx(ind4,2) - hx(ind3,2) ) + rky * 0.5 * ( hx(ind3,2)
     *         - hx(ind1,2) + hx(ind4,2) - hx(ind2,2) ) )
 80    continue

      end if


      if ((j .ge. nfyr +1 ) .and. (j .le. (nvaly-nfyr))) then
         do 81 i=1,nfxr
c
c     recherche de la premiere coordonnee dans la grille de permeabilite
c ----------------------------------------------------------------------
c
          indx = ( i + nxhr - 1 ) / nxhr
c
c     calcul de la taille d'un element selon x
c --------------------------------------------
c
          if(nxh .eq. 1) then
            xvalm = 0.0
            xvalp = 1.0
          else
            if(indx .eq. 1) then
              xvalm = 1.5 * x(1) - 0.5 * x(2)
              xvalp = 0.5 * ( x(indx) + x(indx+1) )
            end if
            if(indx .gt. 1 .and. indx .lt. nxh) then
              xvalm = 0.5 * ( x(indx-1) + x(indx) )
              xvalp = 0.5 * ( x(indx) + x(indx+1) )
            end if
            if(indx .eq. nxh) then
              xvalm = 0.5 * ( x(indx-1) + x(indx) )
              xvalp = 1.5 * x(indx) - 0.5 * x(indx-1)
            end if
         end if
          delx = ( xvalp - xvalm ) / float(nxhr)
c
c     calcul du volume de la sous cellule en cours
c ------------------------------------------------
c
          del = delx * dely
c
c     attribution de la permeabilite
c ---------------------------------
c
         if (( j .ge. nfyr+1) .and. ( j .le. (nfyr + nyhr)) ) then
            indprm = indx + ( indy - 1 ) * nxh
         else
            indprm = indx + ( indy - 1 ) * nxh - (indy-1 -nfy) * nxhf
         end if
          rkx = kx(indprm)
          rkxy = kxy(indprm)
          rkyx = kyx(indprm)
          rky = ky(indprm)

c     
c     calcul des indices des 4 sommets de l'element 
c     en utilisant la propriete de la periodicite
c --------------------------------------------------------------
c
         if ((j .eq. nfyr+1).or. ( j .eq. (nfyr +2 ))) then
            ind1 = i + j2
         else
            ind1 = i + j2-(j-(nfyr+2))*(nvalfx-1)
         end if
         ind2 = ind1 + 1
         if( j .eq. (nfyr + 1)) then
           ind3 = ind1 + nvalx
         else
           ind3 = ind1 + nvalx - ( nvalfx - 1 )
         end if
         ind4=ind3+1
c
c     ajout du terme respectif
c ----------------------------
c
          zkx = zkx + del
     *         * ( rkx + rkx * 0.5 * ( hx(ind2,1) - hx(ind1,1) 
     *         + hx(ind4,1) - hx(ind3,1) ) + rkxy * 0.5 * ( hx(ind3,1)
     *         - hx(ind1,1) + hx(ind4,1) - hx(ind2,1) ) )
c
          zkxy = zkxy + del
     *         * ( rkxy + rkx * 0.5 * ( hx(ind2,2) - hx(ind1,2)
     *         + hx(ind4,2) - hx(ind3,2) ) + rkxy * 0.5 * ( hx(ind3,2)
     *         - hx(ind1,2) + hx(ind4,2) - hx(ind2,2) ) )
c
          zkyx = zkyx + del
     *         * ( rkyx + rkyx * 0.5 * ( hx(ind2,1) - hx(ind1,1)
     *         + hx(ind4,1) - hx(ind3,1) ) + rky * 0.5 * ( hx(ind3,1)
     *         - hx(ind1,1) + hx(ind4,1) - hx(ind2,1) ) )
c
          zky = zky + del
     *         * ( rky + rkyx * 0.5 * ( hx(ind2,2) - hx(ind1,2) 
     *         + hx(ind4,2) - hx(ind3,2) ) + rky * 0.5 * ( hx(ind3,2)
     *         - hx(ind1,2) + hx(ind4,2) - hx(ind2,2) ) )

 81      continue

         do 82 i=nvalx-nfxr+1, nvalx
c
c     recherche de la premiere coordonnee dans la grille de permeabilite
c ----------------------------------------------------------------------
c
          indx = ( i + nxhr - 1 ) / nxhr
c
c     calcul de la taille d'un element selon x
c --------------------------------------------
c
          if(nxh .eq. 1) then
            xvalm = 0.0
            xvalp = 1.0
          else
            if(indx .eq. 1) then
              xvalm = 1.5 * x(1) - 0.5 * x(2)
              xvalp = 0.5 * ( x(indx) + x(indx+1) )
            end if
            if(indx .gt. 1 .and. indx .lt. nxh) then
              xvalm = 0.5 * ( x(indx-1) + x(indx) )
              xvalp = 0.5 * ( x(indx) + x(indx+1) )
             end if
            if(indx .eq. nxh) then
              xvalm = 0.5 * ( x(indx-1) + x(indx) )
              xvalp = 1.5 * x(indx) - 0.5 * x(indx-1)
            end if
         end if
          delx = ( xvalp - xvalm ) / float(nxhr)
c
c     calcul du volume de la sous cellule en cours
c ------------------------------------------------
c
          del = delx * dely
c
c     attribution de la permeabilite
c ----------------------------------
c
          indprm = indx + ( indy - 1 ) * nxh - (indy -nfy) * nxhf
          rkx = kx(indprm)
          rkxy = kxy(indprm)
          rkyx = kyx(indprm)
          rky = ky(indprm)
c
c     calcul des indices des 4 sommets de l'element 
c     en utilisant la propriete de la periodicite
c --------------------------------------------------
c
         if (j .eq. nfyr+1) then
            ind1 = i + j2
         else
            ind1 = i + j2-(j-(nfyr+1))*(nvalfx-1)
         end if
         if (i .eq. nvalx) then
            if (j .eq. nfyr+1) then
               ind2=1+j2
            else
               ind2=ind1-(nvalx-nvalfx)
            end if
         else
            ind2 = ind1 + 1
         end if
         if( j .eq. (nvaly-nfyr)) then
           ind3 = ind1 + nvalx
         else
           ind3 = ind1 + (nvalx-nvalfx) + 1 
         end if
         if (i .eq. nvalx) then
            if (j .eq. nvaly-nfyr) then
               ind4 = ind3 - nvalx +1
            else
               ind4=ind3-(nvalx-nvalfx)
            end if
         else
            ind4=ind3+1
         end if
c
c     ajout du terme respectif
c ----------------------------
c
          zkx = zkx + del
     *         * ( rkx + rkx * 0.5 * ( hx(ind2,1) - hx(ind1,1) 
     *         + hx(ind4,1) - hx(ind3,1) ) + rkxy * 0.5 * ( hx(ind3,1)
     *         - hx(ind1,1) + hx(ind4,1) - hx(ind2,1) ) )
c
          zkxy = zkxy + del
     *         * ( rkxy + rkx * 0.5 * ( hx(ind2,2) - hx(ind1,2)
     *         + hx(ind4,2) - hx(ind3,2) ) + rkxy * 0.5 * ( hx(ind3,2)
     *         - hx(ind1,2) + hx(ind4,2) - hx(ind2,2) ) )
c
          zkyx = zkyx + del
     *         * ( rkyx + rkyx * 0.5 * ( hx(ind2,1) - hx(ind1,1)
     *         + hx(ind4,1) - hx(ind3,1) ) + rky * 0.5 * ( hx(ind3,1)
     *         - hx(ind1,1) + hx(ind4,1) - hx(ind2,1) ) )
c
          zky = zky + del
     *         * ( rky + rkyx * 0.5 * ( hx(ind2,2) - hx(ind1,2) 
     *         + hx(ind4,2) - hx(ind3,2) ) + rky * 0.5 * ( hx(ind3,2)
     *         - hx(ind1,2) + hx(ind4,2) - hx(ind2,2) ) )

 82      continue

      end if


      if ((j .ge. (nvaly-nfyr+1)) .and. (j.le. nvaly)) then
         do 83 i=1, nvalx
c
c     recherche de la premiere coordonnee dans la grille de permeabilite
c ----------------------------------------------------------------------
c
          indx = ( i + nxhr - 1 ) / nxhr
c
c     calcul de la taille d'un element selon x
c --------------------------------------------
c
          if(nxh .eq. 1) then
            xvalm = 0.0
            xvalp = 1.0
          else
            if(indx .eq. 1) then
              xvalm = 1.5 * x(1) - 0.5 * x(2)
              xvalp = 0.5 * ( x(indx) + x(indx+1) )
            end if
            if(indx .gt. 1 .and. indx .lt. nxh) then
              xvalm = 0.5 * ( x(indx-1) + x(indx) )
              xvalp = 0.5 * ( x(indx) + x(indx+1) )
            end if
            if(indx .eq. nxh) then
              xvalm = 0.5 * ( x(indx-1) + x(indx) )
              xvalp = 1.5 * x(indx) - 0.5 * x(indx-1)
            end if
         end if
          delx = ( xvalp - xvalm ) / float(nxhr)
c
c     calcul du volume de la sous cellule en cours
c ------------------------------------------------
c
          del = delx * dely
c
c     attribution de la permeabilite
c ----------------------------------
c
          indprm = indx + ( indy - 1 ) * nxh - ntotf
          rkx = kx(indprm)
          rkxy = kxy(indprm)
          rkyx = kyx(indprm)
          rky = ky(indprm)
c
c     calcul des indices des 4 sommets de l'element 
c     en utilisant la propriete de la periodicite
c ---------------------------------------------------
c
         ind1 = i + j2 - ( nvalfx - 1 )*( nvalfy - 1)
         if (i .eq. nvalx) then
            ind2 = ind1 - nvalx + 1
         else
            ind2 = ind1 + 1
         end if
         if (j .eq. nvaly) then
            ind3 = i
         else
            ind3 = ind1 + nvalx
         end if
         if (i .eq. nvalx) then
            if ( j .eq. nvaly ) then
               ind4 = 1
            else
               ind4=ind2 + nvalx
            end if
         else
            ind4 = ind3 + 1
         end if
c
c     ajout du terme respectif
c ----------------------------
c
          zkx = zkx + del
     *         * ( rkx + rkx * 0.5 * ( hx(ind2,1) - hx(ind1,1) 
     *         + hx(ind4,1) - hx(ind3,1) ) + rkxy * 0.5 * ( hx(ind3,1)
     *         - hx(ind1,1) + hx(ind4,1) - hx(ind2,1) ) )
c
          zkxy = zkxy + del
     *         * ( rkxy + rkx * 0.5 * ( hx(ind2,2) - hx(ind1,2)
     *         + hx(ind4,2) - hx(ind3,2) ) + rkxy * 0.5 * ( hx(ind3,2)
     *         - hx(ind1,2) + hx(ind4,2) - hx(ind2,2) ) )
c
          zkyx = zkyx + del
     *         * ( rkyx + rkyx * 0.5 * ( hx(ind2,1) - hx(ind1,1)
     *         + hx(ind4,1) - hx(ind3,1) ) + rky * 0.5 * ( hx(ind3,1)
     *         - hx(ind1,1) + hx(ind4,1) - hx(ind2,1) ) )
c
          zky = zky + del
     *         * ( rky + rkyx * 0.5 * ( hx(ind2,2) - hx(ind1,2) 
     *         + hx(ind4,2) - hx(ind3,2) ) + rky * 0.5 * ( hx(ind3,2)
     *         - hx(ind1,2) + hx(ind4,2) - hx(ind2,2) ) )

 83      continue
      end if


 8     continue
c
c     division par la taille de la cellule
c ---------------------------------------
c
      zkx =   zkx / cellsize
      zkxy =  zkxy / cellsize
      zkyx =  zkyx / cellsize
      zky =   zky / cellsize

      return
      end
c\end{verbatim}


