c    Fichier th.f
c************************************************
c
c    Pour compiler taper: f90 -o th th.f -O  
c
c****************************************************************
c            FICHIERS NECESSAIRES AU PROGRAMME PRINCIPAL
c****************************************************************
c       
c       INCLUDE'./BIB/permequi.f'
c       INCLUDE'./BIB/nfr_nint.f'
c       INCLUDE'./BIB/cond_bor.f'
c       INCLUDE'./BIB/2nd_memb.f'
c       INCLUDE'./BIB/aire.f'
c       INCLUDE'./BIB/mise_a_0.f'
c       INCLUDE'./BIB/ecr_vect.f'
c       INCLUDE'./BIB/assemb_a.f'
c       INCLUDE'./BIB/prelimi.f'
c       INCLUDE'./BIB/prelimi1.f'
c       INCLUDE'./BIB/prelimi21.f'
c       INCLUDE'./BIB/prelimi22.f'
c       INCLUDE'./BIB/assemb_c.f'
c       INCLUDE'./BIB/info_ele.f'
c       INCLUDE'./BIB/triangle.f'
c       INCLUDE'./BIB/tr_compl.f'
c       INCLUDE'./BIB/quadrila.f'
c       INCLUDE'./BIB/qu_compl.f'
c       INCLUDE'./BIB/gradcp.f'
c       INCLUDE'./BIB/icholes.f'
c       INCLUDE'./BIB/tri.f'
c       INCLUDE'./BIB/fonct_2m.f'
c       INCLUDE'./BIB/jc.f'

c       INCLUDE'./BIB/norme.f'

c****************************************************************
c                       PROGRAMME  PRINCIPAL
c****************************************************************        

      program homogeneisation_2D

c    Ce programme calcule un tenseur homogeneise suivant la
c    methode de bamberger.
c    La methode de bamberger consiste d'abord a calculer les
c    solutions les deux problemes locaux suivant:
c
c        -div[K(x).grad(Pi)]=0    dans le domaine
c                                                              pour i=1,2
c              Pi=Xi              sur la frontiere du domaine
c
c      Xi  :designe le ieme axe de coordonnee
c      Pi  :designe la pression
c      K(x):designe le tenseur des permeabilites
c
c
c    l'element generique du tenseur homogeneise est alors donne
c    par le calcul suivant:
c
c     (K)ij=(1/|domaine|)*INTEGRALE(grad(Pi)*grad(Pj))  pour tout 1=< i,j =<2
c
c     Variable du programme principal
c    **********************************
c
c
c   i,j,chele :indice de boucle
c   ntn :nombre total de noeud du maillage du domaine
c   nte :nombre total d'element
c   nti :nombre total d'inconnues du systeme lineaire
c   nnf :nombre de noeud situes sur la frontiere
c   nelemA :dimension du tableau aa
c   nelemB :dimension du tableau bb
c   nelemC :dimension du tableau cc
c   kmax :nombre maximum d'iteration du solveur gradiant conjugue
c         preconditionne
c   K :tableau 2*2 contenant le tenseur des permeabilites
c   N1,N2,N3,N4 : tableau contenant le numero de l'element ainsi
c                 que les numeros des noeuds qui le constituent
c   aire :aire de la grosse maille
c   eps :precision du solveur
c   perm1,perm2: termes diagonaux des permeabilites lu dans le fichier
c                perm.dat affectes au tableau K
c   ELEMENTBIS: vecteur de bouleens indiquant la "position" d'un noeud
c                false : noeud situe sur la frontiere
c                true  : noeud situe a l'interieur du domaine
c   ord_gr :dimension du vecteur VECT1
c   nbndele: entier qui differencie un element triangulaire d'un element
c            quadrangulaire
c            nbndele=5 :element quadrangulaire
c            nbndele=4 :element triangulaire
c   INDICE: tableau donnant le rang d'un noeud appartenant a VECTI ou VECTF
c   nbpoint: nombre de point de la grosse maille
c   aire: aire de la grosse maille
c   TABele: tableau contenant les elements du maillage au travers des numeros
c           des noeuds qui constituent l'element
c   VECTI: vecteur contenant les numeros des noeuds situes a l'interieur
c          de la grosse maille
c   VECTF: vecteur contenant les numeros des noeuds situes sur la frontiere
c          de la grosse maille
c   aa: tableau contenant les contributions int-int (c'est la matrice A
c       du systeme lineaire)
c   ia(k): indice dans le tableau aa du premier element
c        de la i eme ligne de A stocke dans aa,
c   ja(k): indice j de colonne de l"element Aij stocke
c          dans le tableau aa a l"indice k
c   bb: tableau contenant les contributions bord-bord
c   cc: tableau contenant les contributions bord-int
c   ib,jb: idem que ia et ja pour le tableau bb
c   ic,jc: idem que ia et ja pour le tableau cc
c   SMinter: vecteur contenant le second membre du systeme lineaire
c            sans les conditions aux limites
c   SM :vecteur SMinter avec le sconditions aux limites
c   X1: solution du probleme local 1
c   X2 :solution du probleme local 2
c   G: tableau contenant les coordonnees des points qui constituent la
c      grosse maille
c   COORD: tableaux contenants les valeurs des vecteurs directeurs
c   CONDB1,CONDB2: tableaux contenant les conditions initiales
c                  sur le bord suivant le probleme local
c   al : tableau contenant la factorise de choleski incomplete de la
c        matrice A stockee sous forme morse dans le tableau aa
c        avec les tableaux auxiliaires d'indices ia et ja
c   gg,h,cg: vecteur necessaire au solveur. Ces vecteurs sont expliques
c   dans la subroutine gradcp


      implicit none
c
c declaration des variables
c
      integer i,j,chele,ntn,ord_gr,nte,nti,nnf,nbndele 
      integer lb,INDICE,nelemB,nelemC,nelemA,kmax,nbpoint 
      real*8  K,N1,N2,N3,N4,aire,eps,perm1,poro,poroh,ast   !perm2
      logical ELEMENTBIS

c      logical b         !variables provisoires
c      real*8 z,y        !idem


      parameter (lb=50) !on peut descendre a 30     

      dimension N1(3),N2(3),N3(3),N4(3)
      dimension K(2,2),ELEMENTBIS(5),INDICE(5)
      integer, ALLOCATABLE, DIMENSION(:,:) :: TABele
      integer, ALLOCATABLE, DIMENSION(:) :: VECTI
      integer, ALLOCATABLE, DIMENSION(:) :: VECTF
      integer, ALLOCATABLE, DIMENSION(:) :: ia
      integer, ALLOCATABLE, DIMENSION(:) :: ja
      integer, ALLOCATABLE, DIMENSION(:) :: ib
      integer, ALLOCATABLE, DIMENSION(:) :: jb
      integer, ALLOCATABLE, DIMENSION(:) :: ic
      integer, ALLOCATABLE, DIMENSION(:) :: jc
      integer, ALLOCATABLE, DIMENSION(:) :: VECT1 

      integer, ALLOCATABLE, DIMENSION(:,:) :: MATA
      integer, ALLOCATABLE, DIMENSION(:,:) :: MATB
      integer, ALLOCATABLE, DIMENSION(:,:) :: MATC

      real*8, ALLOCATABLE, DIMENSION(:) :: SMinter 
      real*8, ALLOCATABLE, DIMENSION(:) :: SM
      real*8, ALLOCATABLE, DIMENSION(:) :: X1
      real*8, ALLOCATABLE, DIMENSION(:) :: X2
      real*8, ALLOCATABLE, DIMENSION(:,:) :: G
      real*8, ALLOCATABLE, DIMENSION(:,:) :: COORD
      real*8, ALLOCATABLE, DIMENSION(:) :: CONDB1
      real*8, ALLOCATABLE, DIMENSION(:) :: CONDB2
      real*8, ALLOCATABLE, DIMENSION(:,:) :: droite
      real*8, ALLOCATABLE, DIMENSION(:) :: aa
      real*8, ALLOCATABLE, DIMENSION(:) :: al
      real*8, ALLOCATABLE, DIMENSION(:) :: bb
      real*8, ALLOCATABLE, DIMENSION(:) :: cc
      real*8, ALLOCATABLE, DIMENSION(:) :: gg
      real*8, ALLOCATABLE, DIMENSION(:) :: h
      real*8, ALLOCATABLE, DIMENSION(:) :: cg

      DATA kmax/500/
      DATA eps/1.d-10/
c
c ouverture des fichiers
c                      
      open(11,file='DON/element.dat')
      open(12,file='DON/noeud.dat') 
      open(13,file='DON/grosmail.dat')
      open(14,file='DON/perm.dat')
      open(15,file='DON/nbrmail.dat')  
      open(16,STATUS='SCRATCH',
     @                  FORM='UNFORMATTED',ACCESS='DIRECT',RECL=24) 
      open(17,file='RESULTATS/resultat.dat')

      open(20,file='DIVERS/sys.dat')


      write(*,*)'        LECTURE DU MAILLAGE'
c
c on lit le nombre total de noeud et le nombre total d'element du maillage 
c     
      read(15,*)ntn
      read(15,*)nte
c
c allocation dynamique de memoire des tableaux VECTI, VECTF et TABele 
c      
      ALLOCATE (VECTI(ntn))
      ALLOCATE (VECTF(ntn))
      ALLOCATE (TABele(nte,5))
c
c on verifie que le nombre de point de la maille grossiere est superieur a 2
c
      read(13,*)nbpoint
      if (nbpoint.le.2) then
                 write(*,*)'ERREUR SUR LES DONNEES DE LA GROSSE MAILLE'
                 STOP
      endif
c
c allocation dynamique de memoire des tableaux G, COORD et droite
c
      ALLOCATE (G(nbpoint,2))
      ALLOCATE (COORD(nbpoint,2))
      ALLOCATE (droite(nbpoint,2))
c
c lecture des coordonnees des points qui constituent la grosse maille
c
      do i=1,nbpoint
         read(13,*)(G(i,j),j=1,2)
      enddo
c
c initialisation des variables nti et nnf
c
      nti=1
      nnf=1
c
c calcul des equations des droites qui constituent la maille grossiere
c
      call EQUATIONS(G,COORD,nbpoint,aire)
      write(*,*)'aire1',aire
      aire=0.d0


c
c on parcourt tous les noeuds du maillage que l'on affecte a VECTI ou
c VECTF suivant leur position
c
      do j=1,ntn
         read(12,*)(N1(i),i=1,3)
         call REMPLIR (nbpoint,COORD,G,VECTI,VECTF,droite,
     @                        nti,nnf,N1(2),N1(3),N1(1))
      enddo   
      nti=nti-1
      nnf=nnf-1

c      write(*,*)'ATTENTION les VECTI et les VECTF doivent etre '
c      write(*,*)'ranges dans l'ordre croissant'

      write(*,*)'VECTI' 
      write(*,*)(VECTI(i),i=1,nti)
      write(*,*)'VECTF' 
      write(*,*)(VECTF(i),i=1,nnf)

c
c on rend la memoire des tableaux G, droite et COORD au systeme
c      
      DEALLOCATE(G)
      DEALLOCATE(droite)
      DEALLOCATE(COORD)


      write(*,*)'Rangements des elements du maillage dans un tableaux.'
c
c on range les elements du maillage dans le tableau TABele
c
      REWIND(11)
      do i=1,nte
         read(11,*)(TABele(i,j),j=1,5)
c         write(20,*)(TABele(i,j),j=1,5)
      enddo
      ord_gr=nti*nnf+27
   
      write(*,*)'ord_gr',ord_gr

c      do i=1,ntn
c       read(16,REC=i)z,y,b,j
c       write(20,*)z,y,b,j
c      enddo

      goto 12

c
c allocation dynamique de memoire des tableaux VECT1,ia,ib,ic
c 
c      ALLOCATE (VECT1(ord_gr))
c      ALLOCATE (ia(nti+1)) 
c      ALLOCATE (ic(nnf+1))
c      ALLOCATE (ib(nnf+1))
c
c      write(*,*)'Calcul du nombre total d''elements de la matrice et '
c      write(*,*)'calcul des tableaux ia,ib,ic.' 
c
c calcul des tableaux ia,ib,ic et calcul de la dimension des 
c tableaux aa,bb,cc 
c
c      write(*,*)'66'
c      write(20,*)'66'
c
c      call PRELIMINAIRE (VECTF,VECTI,TABele,nnf,nti,nte,ic,nelemC,VECT1,
c     @                ib,nelemB,ia,nelemA)
c
c      write(*,*)'77'
c
c allocation dynamique de memoire des tableaux cc,jc
c
c      ALLOCATE (cc(nelemC))
c      ALLOCATE (jc(nelemC))
c
c calcul du tableau ic et initialisation du tableau cc
c
c      call CAL_JC (VECT1,jc,nelemC,nnf,cc)
c
c on rend la memoire du tableau VECT1 au systeme
c
c      DEALLOCATE(VECT1)                                                    
c
c
c      goto 15
12    continue
      
      write(*,*)'nouvaut'
      write(*,*)'les tailles sont:',nti*lb,nnf*lb,nnf*lb

      ALLOCATE(MATA(nti,lb))
      ALLOCATE(MATB(nnf,lb))
      ALLOCATE(MATC(nnf,lb))
      ALLOCATE (ia(nti+1))

      write(*,*)'toto1'

      call PRELIMINAIRE21(VECTF,VECTI,TABele,nnf,nti,nte,ia,
     @                         MATA,MATB,MATC,ord_gr,lb)

      write(*,*)'toto2'

      DEALLOCATE(MATA)
      ALLOCATE (VECT1(ord_gr))
      ALLOCATE (ic(nnf+1))
      ALLOCATE (ib(nnf+1))

      write(*,*)'toto3'
      call PRELIMINAIRE22(VECTF,VECTI,TABele,nnf,nti,nte,ic,nelemC,
     @                      VECT1,ib,nelemB,ia,nelemA,MATB,MATC)

      write(*,*)'je que c est ici'
c      PAUSE
      DEALLOCATE(MATB)
      write(*,*)'yyyyyyyyy'

      DEALLOCATE(MATC)
c
c allocation dynamique de memoire des tableaux cc,jc
c
      ALLOCATE (cc(nelemC))
      ALLOCATE (jc(nelemC))
c
c calcul du tableau ic et initialisation du tableau cc
c
      call CAL_JC (VECT1,jc,nelemC,nnf,cc)
c
c on rend la memoire du tableau VECT1 au systeme
c
      DEALLOCATE(VECT1)

15    continue


c
c allocation dynamique de memoire des tableaux aa,bb,ja,jb,al,SMinter
c      
      ALLOCATE (aa(nelemA))
      ALLOCATE (al(nelemA))
      ALLOCATE (ja(nelemA)) 
      ALLOCATE (bb(nelemB))
      ALLOCATE (jb(nelemB))
      ALLOCATE (SMinter(nti))

      write(*,*)'CALCUL DES CONTRIBUTIONS ET ASSEMBLAGE'
c      write(20,*)'CALCUL DES CONTRIBUTIONS ET ASSEMBLAGE'
c
c initialisation de tous les tableaux a zero
c            
      call MISE_A_ZERO(bb,jb,aa,ja,K,SMinter,nti,nelemA,nelemB,poroh)

c      do i=1,nelemB
c         write(20,*)i,'bb=',bb(i),'    jb=',jb(i)
c      enddo
c      write(20,*)
c      do i=1,nelemA
c         write(20,*)i,'aa=',aa(i),'    ja=',ja(i)
c      enddo
c      write(20,*)
c      do i=1,nelemC
c         write(20,*)i,'cc=',cc(i),'    jc=',jc(i)
c      enddo

      
      rewind(14)
      do chele=1,nte
c
c on lit la permeabilite de l'element
c
         read(14,*)perm1      !perm2,poro

c         ast=perm1**2
c         perm1=perm1/ast

         poro=0.1d0
         K(1,1)=perm1
         K(2,2)=perm1         !k(2,2)=perm2
c
c on donne les informations de l'element
c
         call COORD_T_F(TABele,elementbis,INDICE,nbndele,
     @                  N1,N2,N3,N4,nte,chele)
         if (nbndele.eq.5) then   
c
c calcul de la contribution et assemblage pour un element quadrangulaire
c
             call CALCULQ1(ELEMENTBIS,INDICE,bb,ib,jb,aa,ia,ja,SMinter
     @                    ,N1,N2,N3,N4,K,cc,ic,jc,aire,poro,poroh)
                          else               
c
c calcul de la contribution et assemblage pour un element triangulaire
c
             call CALCULP1(ELEMENTBIS,INDICE,bb,ib,jb,aa,ia,ja,SMinter
     @                     ,N1,N2,N3,K,cc,ic,jc,aire,poro,poroh)
         endif
      enddo
      write(*,*)
      write(*,*)'aire2',aire

      write(*,*)'CONSTITUTION DES 3 SYSTEMES LINEAIRES ET RESOLUTION'
      write(20,*)'CONSTITUTION DES 3 SYSTEMES LINEAIRES ET RESOLUTION'
      do i=1,nelemC
         write(20,*)i,'cc=',cc(i),'    jc=',jc(i)
      enddo
c
c on tri le tableau aa
c                 
      do i=1,nti
        call TRI(ja(ia(i)),aa(ia(i)),ia(i+1)-ia(i))
      enddo
c
c on tri le tableau bb
c
      do i=1,nnf
        call TRI(jb(ib(i)),bb(ib(i)),ib(i+1)-ib(i))
      enddo

      do i=1,nelemB
         write(20,*)i,'bb=',bb(i),'    jb=',jb(i)
      enddo
      do i=1,nelemA
         write(20,*)i,'aa=',aa(i),'    ja=',ja(i)
      enddo
c
c  calcul de la factorise de choleski incomplete
c
      call ICholeski(nti,aa,ia,ja,al)

      write(20,*)'factorise de choleski'
      do i=1,nelemA
         write(20,*)'al=',al(i),'    ja=',ja(i)
      enddo
c
c on rend la memoire du tableau TABele au systeme
c
      DEALLOCATE(TABele)
c
c allocation dynamique de memoire des tableaux SM,CONDB1,CONDB2,X1,X2
c
      ALLOCATE (SM(nti))
      ALLOCATE (CONDB1(nnf))
      ALLOCATE (CONDB2(nnf))
      ALLOCATE (X1(nti))
      ALLOCATE (X2(nti))
c
c on initialise a zero le vecteur X1 et on affecte le vecteur SMinter au 
c vecteur SM
c      
      do i=1,nti
         SM(i)=SMinter(i)
c         write(20,*)SM(i) 
         X1(i)=0.d0
      enddo
c
c on affecte les conditions aux limites du probleme local 1 au vecteur SM
c
      call SECOND_MEMB(cc,ic,jc,CONDB1,CONDB2,SM,nnf,
     @                 nti,VECTF,1) 

      write(20,*)
      write(20,*)'SM1'
      do i=1,nti
         write(20,*)i,'  ',SM(i)
      enddo
c
c allocation dynamique de memoire des tableaux gg,h,cg
c           
      ALLOCATE (gg(nti))
      ALLOCATE (h(nti))
      ALLOCATE (cg(nti))
c
c on resout le systeme lineaire relatif au probleme local 1
c
      call GradC(nti,aa,ia,ja,al,SM,X1,gg,h,cg,kmax,eps)
      call ecrire(X1,nti)
      
      write(20,*)'FIN DE LA PREMIERE SOLUTION'      
c
c on initialise a zero le vecteur X2 et on affecte le vecteur SMinter au
c vecteur SM
c
      do i=1,nti
         SM(i)=SMinter(i)
         X2(i)=0.d0
      enddo
c
c on affecte les conditions aux limites du probleme local 2 au vecteur SM
c
      call SECOND_MEMB(cc,ic,jc,CONDB1,CONDB2,SM,nnf,
     @                 nti,VECTF,2) 
c
c on resout le systeme lineaire relatif au probleme local 2
c      
      call GradC(nti,aa,ia,ja,al,SM,X2,gg,h,cg,kmax,eps)
      call ecrire(X2,nti)
c
c on rend la memoire du tableau gg,h,cg
c
      DEALLOCATE (gg)
      DEALLOCATE (h)
      DEALLOCATE (cg)

c      call NORME(VECTI,VECTF,x,nti,nnf)

      write(*,*)'CALCUL DU TENSEUR HOMOGENEISE'

c
c calcul du tenseur homogeneise
c

      call PERMEAEQUI(aa,ia,ja,X1,X2,nti,nnf,aire,CONDB1,CONDB2,
     @               bb,ib,jb,cc,ic,jc,poroh)
c
c fermeture des fichiers
c
      close(11)
      close(12)
      close(13)
      close(14)
      close(15)
      close(16)
      close(17)    
      close(20)
      write(*,*)'nombre d''inconnues du systeme :',nti
      write(*,*)
      write(*,*)'fin de l''application'
      end


c  Fichier 2mb_qua.f
c
c**************************************************************
      real*8 function INTEGRPQ(nbr1,N1,N2,N3,N4)           
c**************************************************************
c
c  Cette fonction calcule l'approximation de l'integrale de
c  (f*fonction de base) sur un quadrilatere.
c  On utilise la formule p.??? du  livre 'PRESENTATION DE LA METHODE
c  DES ELEMENTS FINIS' (T.Touzot) 19??.
c
c  N1,N2,N3,N4: contient les numeros, ainsi que les coordonnees
c               des quatre points qui constituent le quadrilatere.
c  nbr1: numero designant la nbr1ieme fonction de base du
c        quadrilatere.
c 

      implicit none

      integer   nbr1
      real*8    somme1,BASEC,N1,N2,N3,N4,det,a,FONCT,x,y
      dimension N1(*),N2(*),N3(*),N4(*),BASEC(4)

      a=1.d0/SQRT(3.d0)
      
      call FBASEC(BASEC,det,N1,N2,N3,N4,a,a,x,y) 
      somme1=(FONCT(x,y)*BASEC(nbr1))           
      call FBASEC(BASEC,det,N1,N2,N3,N4,-a,a,x,y) 
      somme1=somme1+(FONCT(x,y)*BASEC(nbr1))            
      call FBASEC(BASEC,det,N1,N2,N3,N4,a,-a,x,y) 
      somme1=somme1+(FONCT(x,y)*BASEC(nbr1))             
      call FBASEC(BASEC,det,N1,N2,N3,N4,-a,-a,x,y) 
      somme1=somme1+(FONCT(x,y)*BASEC(nbr1))           
      
      INTEGRPQ=somme1*ABS(det)     

      RETURN
      end


c Fichier 2mb_tri.f
c
c******************************************************************
      real*8 function INTEGRP(det,N1,N2,N3,nbr1)
c******************************************************************
c
c  Cette fonction calcule l'approximation de l'integrale de 
c  (f*fonction de base) sur le triangle.
c  On utilise la formule p.??? du  livre 'PRESENTATION DE LA METHODE 
c  DES ELEMENTS FINIS' (T.Touzot) 19??.
c
c  det: jacobien de la transformation: triangle de reference --> triangle 
c   N1,N2,N3 : tableau contenant le numero de l'element ainsi
c              que les numeros des noeuds qui le constituent
c  nbr1: numero designant la nbr1ieme fonction de base du
c        triangle.
c
c
      implicit none

      integer   nbr1
      real*8    det,BASEP,somme,FONCT,a,b,x,y,N1,N2,N3
      dimension BASEP(3),N1(*),N2(*),N3(*)

      a=(1.d0/6.d0)
      b=(2.d0/3.d0)

      call FBASE(BASEP,N1,N2,N3,a,a,x,y)
      somme=FONCT(x,y)*BASEP(nbr1)              
      call FBASE(BASEP,N1,N2,N3,b,a,x,y)
      somme=somme+FONCT(x,y)*BASEP(nbr1)       
      call FBASE(BASEP,N1,N2,N3,a,b,x,y)
      somme=somme+FONCT(x,y)*BASEP(nbr1)        
      INTEGRP=somme*ABS(det)*a

      RETURN
      end









c  Fichier 2nd_memb.f
c
c********************************************************************
      subroutine SECOND_MEMB(cc,ic,jc,CONDB1,CONDB2,SM,nnf,nti,
     @                       VECTF,numero)
c*********************************************************************
c
c   Cette procedure ajoute aux second memebre les conditions aux limites
c   suivant le probleme local que l'on veut resoudre. Deplus des contions
c   initiales sont sauvegardees dans un tableau en vue du calcul 
c   tenseur homogeneise.
c
c   CONDB1,CONDB2: tableaux contenant les conditions initiales
c                  sur le bord suivant le probleme local
c   SM: vecteur second membre avec conditions aux limites    
c   nnf: nombre de noeud a la frontiere
c   VECTF: tableau contenant les numero des noeuds situe sur la 
c          frontiere du domaine 
c   numero: numero du probleme local resolu
c   cc: tableau des contributions bord_int
c   ic,jc: tableaux d'indices associes au tableau cc
c   nti :nombre total d'inconnues du systeme lineaire
c
      implicit none

      integer   nnf,k1,k2,j,VECTF,numero,ic,jc,kk,nti
      real*8    CONDB1,CONDB2,SM,BORD_PB1,BORD_PB2,cc  
      dimension cc(*),ic(*),jc(*),SM(*) 
      dimension CONDB1(*),CONDB2(*),VECTF(*)
      
      kk=1
c
c probleme local 1
c
      if (numero.eq.1) then 
         do k2=1,nnf
            CONDB1(k2)=BORD_PB1(VECTF(k2))
            do j=1,ic(k2+1)-ic(k2)
               k1=jc(kk)
c            write(20,*)'cc',cc(kk),'     ',kk,'  SM',SM(k1)
                 SM(k1)=-CONDB1(k2)*cc(kk)+SM(k1)
               kk=kk+1
            enddo
         enddo
             else
c
c probleme local 2
c
        if (numero.eq.2) then
          do k2=1,nnf
            CONDB2(k2)=BORD_PB2(VECTF(k2))
            do j=1,ic(k2+1)-ic(k2)
               k1=jc(kk)
                   SM(k1)=-CONDB2(k2)*cc(kk)+SM(k1)
               kk=kk+1
            enddo
          enddo
         endif
      endif

      RETURN
      end





c  Fichier aire1.f
c
c*****************************************************************
      subroutine AIRE_ELE1(aire,N1,N2,N3)
c*****************************************************************
c
c   Cette procedure calcule l'aire du triangle N1,N2,N3 
c
c   N1,N2,N3: contient les numeros, ainsi que les coordonnees des
c                trois points qui constituent le triangle
c   aire: aire du triangle
c
      implicit none
      
      real*8    aire,det,N1,N2,N3,J11,J12,J21,J22 
      dimension N1(*),N2(*),N3(*) 
c
c calcul du jacobien de la transformation
c      
      J11=(N2(2)-N1(2)) 
      J21=(N2(3)-N1(3))
      J12=(N3(2)-N1(2))
      J22=(N3(3)-N1(3))                     
      det=(J11*J22)-(J12*J21)
c
c calcul de l'aire     
c
      aire=(ABS(det))/2.d0

      RETURN
      end                                        






c  Fichier aire2.f
c
c*****************************************************************
      subroutine AIRE_ELE2(aire,N1,N2,N3)
c*****************************************************************
c
c   Cette procedure calcule l'aire du triangle N1,N2,N3 
c
c   N1,N2,N3: contient les numeros, ainsi que les coordonnees des
c                trois points qui constituent le triangle
c   aire: aire du triangle
c
      implicit none
      
      real*8    aire,det,N1,N2,N3,J11,J12,J21,J22 
      dimension N1(*),N2(*),N3(*) 
c
c calcul du jacobien de la transformation
c      
      J11=(N2(2)-N1(2)) 
      J21=(N2(3)-N1(3))
      J12=(N3(2)-N1(2))
      J22=(N3(3)-N1(3))                     
      det=(J11*J22)-(J12*J21)
c
c calcul de l'aire     
c
      aire=(ABS(det))/2.d0

      RETURN
      end                                        






c    Fichier aire.f
c
c**********************************************************************
      subroutine EQUATIONS(G,COORD,nbpoint,aire) 
c**********************************************************************
c
c   Cette procedure calcule les vecteurs directeurs des droites
c   qui forment la grosse maille, ainsi que l'aire de cette derniere.
c
c   G: tableau contenant les coordonnees des points de la 
c      grosse maille
c   COORD: tableaux contenants les valeurs des vecteurs directeurs
c   nbpoint :nombre de point de la grosse maille
c   aire: aire de la grosse maille
c
c
      implicit none

      integer   i,nbpoint,k
      real*8    G,COORD,aire,p,l1,l2,l3,a1
      dimension G(nbpoint,2),COORD(nbpoint,2)

c
c calcul des coordonnees des vecteurs directeurs
c
      do i=1,nbpoint-1 
         COORD(i,1)=G(i+1,1)-G(i,1)
         COORD(i,2)=G(i+1,2)-G(i,2) 
      enddo
      COORD(nbpoint,1)=G(1,1)-G(nbpoint,1)
      COORD(nbpoint,2)=G(1,2)-G(nbpoint,2) 

      do i=1,nbpoint
         do k=1,2
            if (ABS(COORD(i,k)).le.1.d-05) then
                                     COORD(i,k)=0.d0
            endif
         enddo
      enddo
c
c calcul de l'aire de la grosse maille
c
      AIRE=0.d0
      do i=1,nbpoint-2
           l1=SQRT(((G(i+2,1)-G(1,1))**2)+((G(i+2,2)-G(1,2))**2))
           l2=SQRT(((G(i+1,1)-G(1,1))**2)+((G(i+1,2)-G(1,2))**2))
           l3=SQRT(((G(i+2,1)-G(i+1,1))**2)+((G(i+2,2)-G(i+1,2))**2))
           p=(l1+l2+l3)/2.d0
           A1=sqrt(p*(p-l1)*(p-l2)*(p-l3))
           AIRE=AIRE+A1 
      enddo
      
      RETURN
      end


                                               
c  Fichier assemb_a.f
c
c****************************************************************
      subroutine ASSEMBLAGE(aa,ia,ja,i,j,som1)
c****************************************************************
c
c  Cette procedure effectue l'assemblage du tableau aa qui
c  contient les contributions int-int.
c
c  aa: tableau contenant les contributions int-int
c  ia,ja: tableaux d'indices associes au tableau aa
c  i,j: indices ou l'on doit effectuer l'assemblage
c  som1: contribution a assembler
c
c
      implicit none

      integer   ia,ja,i,j,k,l
      real*8    aa,som1
      dimension aa(*),ia(*),ja(*)

      k=0
      if (i.eq.j) then
         k=ia(i+1)-1
         ja(k)=i
                  else
         if (i.gt.j) then
            do l=ia(i),ia(i+1)-2
               if (ja(l).eq.0) then
                               GOTO 4
               endif
               if (ja(l).eq.j) then
                               k=l
                               GOTO 4
               endif
            enddo
4           continue
            if (k.eq.0) then 
                        k=l
                        ja(l)=j
            endif
         endif
      endif
      if (k.ne.0) then
               aa(k)=aa(k)+som1
      endif

      RETURN
      end
c  Fichier assemb_c.f
c
c****************************************************************
      subroutine ASSEMBLAGEbis(cc,ic,jc,i,j,som1)
c****************************************************************
c
c  Cette procedure effectue l'assemblage du tableau cc qui
c  contient les contributions bord-int.
c
c  cc: tablecu contenant les contributions bord-int
c  ic,jc: tableaux d'indices associes au tableau cc 
c  i,j: indices ou l'on doit effectuer l'assemblcge
c  som1: contribution a assembler
c
c
      implicit none

      integer   ic,jc,i,j,k,l,deb
      real*8    cc,som1
      dimension cc(*),ic(*),jc(*)


      deb=ic(i)
      do l=deb,deb+(ic(i+1)-ic(i)-1)
         if (jc(l).eq.j) then
                           k=l
                           goto 5
         endif
      enddo
      write(*,*)'probleme dcns assemb_c.f'
                    stop
5     continue
      cc(k)=cc(k)+som1


      RETURN
      end
c  Fichier base_qua.f
c
c************************************************************
      subroutine FBASEC(BASEC,det,N1,N2,N3,N4,xch,ych,x,y)
c************************************************************
c
c   Cette procedure calcule la valeur des quatres fonctions de bases
c   au point (xch,ych) appartenant au carre de reference, ainsi que
c   le jacobien de l'application: carre de reference --> quadrilatere.
c
c   BASEC: tableau contenant les valeurs des quatres fonctions de 
c          base au point (xch,ych)
c   det: jacobien de la transformation: carre de reference --> quadrilatere
c   N1,N2,N3,N4: contient les numeros, ainsi que les coordonnees des 
c                quatre points qui constituent le quadrilatere
c   xch,ych: coordonnees du point de calcul dans le carre de reference
c   x,y: coordonnees du point dans le quadrilatere
c
c
      implicit none

      real*8    BASEC,xch,ych,det,N1,N2,N3,N4
      real*8    J11,J12,J21,J22,a1,a2,b1,b2,x,y
      dimension BASEC(*),N1(*),N2(*),N3(*),N4(*)
      
      a1=(1.d0+xch)
      a2=(1.d0-xch)
      b1=(1.d0+ych)
      b2=(1.d0-ych)
c
c fonctions de bases
c
      BASEC(1)=(a2*b2)/4.d0
      BASEC(2)=(a1*b2)/4.d0
      BASEC(3)=(a1*b1)/4.d0
      BASEC(4)=(a2*b1)/4.d0
c
c coordonnees reelle du point de calcul
c
      x=(BASEC(1)*N1(2)+BASEC(2)*N2(2)+BASEC(3)*N3(2)+BASEC(4)*N4(2))
      y=(BASEC(1)*N1(3)+BASEC(2)*N2(3)+BASEC(3)*N3(3)+BASEC(4)*N4(3))
c
c calcul du jacobien de la transformation
c
      J11=-N1(2)+N2(2)+N3(2)-N4(2)+ych*(N1(2)-N2(2)+N3(2)-N4(2))
      J11=J11/4.d0

      J12=-N1(3)+N2(3)+N3(3)-N4(3)+ych*(N1(3)-N2(3)+N3(3)-N4(3)) 
      J12=J12/4.d0

      J21=-N1(2)-N2(2)+N3(2)+N4(2)+xch*(N1(2)-N2(2)+N3(2)-N4(2))
      J21=J21/4.d0

      J22=-N1(3)-N2(3)+N3(3)+N4(3)+xch*(N1(3)-N2(3)+N3(3)-N4(3))
      J22=J22/4.d0

      det=(J11*J22)-(J21*J12)
               
      RETURN
      end                                        

c  Fichier base_tri.f
c
c********************************************************************
      subroutine FBASE(BASEP,N1,N2,N3,xch,ych,x,y)
c********************************************************************
c
c  Cette procedure calcule les fonctions de base pour le 
c  triangle de reference.
c
c   BASEP: tableau contenant les valeurs des fonctions de bases
c   N1,N2,N3 : tableau contenant le numero de l'element ainsi
c              que les numeros des noeuds qui le constituent
c   xch,ych: coordonnees du point de calcul dans le triangle de reference 
c   x,y: coordonnees du point dans le triangle
c
c

      implicit none

      real*8    xch,ych,x,y,BASEP,N1,N2,N3
      dimension BASEP(*),N1(*),N2(*),N3(*)
c
c fonctions de bases 
c
      BASEP(1)=(1.d0-xch-ych)
      BASEP(2)=xch
      BASEP(3)=ych
c
c coordonnees reelle du point de calcul
c
      x=(BASEP(1)*N1(2)+BASEP(2)*N2(2)+BASEP(3)*N3(2))
      y=(BASEP(1)*N1(3)+BASEP(2)*N2(3)+BASEP(3)*N3(3))

      RETURN
      end
c  Fichier cond_bor.f
c
c*********************************************************************
      real*8 function BORD_PB1(r)
c*********************************************************************
c 
c  Cette fonction donne les conditions aux bord du probleme local 1
c 
c  r: numero du noeud situe a la frontiere du domaine
c
c
      implicit none

      real*8   x,y 
      integer  r

      read(16,rec=r)x,y
      BORD_PB1=x                    !+(y*0.d0)

c      BORD_PB1=80*x+25.d0*y+50.d0                    !+(y*0.d0)

      RETURN
      end





c*********************************************************************      
      real*8 function BORD_PB2(r)
c*********************************************************************
c 
c  Cette fonction donne les conditions aux bord du probleme local 2
c 
c  r: numero du noeud situe a la frontiere du domaine
c
c

      implicit none

      real*8  x,y
      integer r

      read(16,rec=r)x,y
      BORD_PB2=y                  !+(x*0.d0)


c      BORD_PB2=3*x+68*y+10.d0                  !+(x*0.d0)   

      RETURN
      end



c  Fichier ecr_vect.f
c
c********************************************************************
        subroutine ecrire(b,n)
c********************************************************************
c
c    Ce sous-programme ecrit dans un fichier et/ou a l'ecran le 
c    vecteur b en colonne.
c  
c    b: vecteur
c    n: dimension du vecteur b
c

        implicit none 

        integer i,n
        real*8  b(*)

        write(20,*)
        do i=1,n
          write(20,*) i,b(i)
        enddo

        RETURN
        end



c   Fichier essai.f
c
c*************************************************************************
       subroutine  PERMEQ2(Y1,Y2,Y3)  
c************************************************************************

       implicit none

       real*8 det,J,Y1,Y2,Y3,Kxx,Kxy,Kyy 
       real*8 a11,a12,a13,a21,a22,a23,a31,a32,a33
       real*8 alpha1,beta1,gamma1,alpha2,beta2,gamma2

       dimension J(3,3)

       write(20,*)Y1,Y2,Y3



       alpha1=80.d0
       beta1=25.d0
c       gamma1=1.d0

       alpha2=3.d0
       beta2=68.d0
c       gamma2=1.d0


       J(1,1)=alpha1**2    
       J(2,1)=alpha1*alpha2 
       J(3,1)=alpha2**2  

       J(1,2)=2.d0*alpha1*beta1
       J(2,2)=alpha1*beta2+alpha2*beta1
       J(3,2)=2.d0*alpha2*beta2

       J(1,3)=beta1**2
       J(2,3)=beta1*beta2
       J(3,3)=beta2**2

       
       a11=J(2,2)*J(3,3)-J(3,2)*J(2,3)
       a21=J(3,1)*J(2,3)-J(2,1)*J(3,3)
       a31=J(2,1)*J(3,2)-J(3,1)*J(2,2)

       a12=J(1,3)*J(3,2)-J(1,2)*J(3,3)
       a22=J(1,1)*J(3,3)-J(1,3)*J(3,1)
       a32=J(1,2)*J(3,1)-J(3,2)*J(1,1)

       a13=J(1,2)*J(2,3)-J(1,3)*J(2,2)
       a23=J(2,1)*J(1,3)-J(2,3)*J(1,1)
       a33=J(1,1)*J(2,2)-J(1,2)*J(2,1)

       det=J(1,1)*a11+J(1,2)*a21+J(1,3)*a31      

       det=1.d0/det

       a11=a11*det
       a21=a21*det
       a31=a31*det

       a12=a12*det
       a22=a22*det
       a32=a32*det

       a13=a13*det
       a23=a23*det
       a33=a33*det
 
       Kxx=a11*Y1+a12*Y2+a13*Y3
       Kxy=a21*Y1+a22*Y2+a23*Y3
       Kyy=a31*Y1+a32*Y2+a33*Y3

       write(*,19)
       write(*,20)Kxx,Kxy,Kyy


19    FORMAT(5x,'Kxx',12x,'Kxy',12x,'Kyy')
20    FORMAT(4(E11.4,4x))



       RETURN
       end

c  Fichier fonct_2m.f
c
c*************************************************************************
      real*8 function FONCT(x,y)
c*************************************************************************
c
c  Cette fonction donne la valeur de la fonction f au
c  point (x,y).
c
c  x,y : coordonnee du point de calcul
c
c
      implicit none

      real*8 x,y,pi

      DATA pi/3.1415926000000/
      FONCT=((x+y)*0.d0)

c       FONCT=-(y-4.d0*x*y-y**2+4.d0*x*y**2-2.d0*y**2+2.d0*y**3)
c     @       -(x-4.d0*y*x-x**2+4.d0*y*x**2-2.d0*x**2+2.d0*x**3)
c       FONCT=2.d0*x*(1.d0-x)+2.d0*y*(1.d0-y)
c      FONCT=-3.d0*(x**2)/2.d0
c       FONCT=DSIN(x*y)*(x**2+y**2)
c        FONCT=10.d0*pi**2*DSIN(pi*x)*DSIN(pi*y)

      RETURN
      end
C Fichier gradcp.f
C
c********************************************************************
c            FICHIERS NECESSAIRES A LA SUBROUTINE gradc
c********************************************************************

c      INCLUDE'./prod_mat.f'
c      INCLUDE'./resol_sys.f'

C***********************************************************************
      SUBROUTINE GradC(n,aa,ia,ja,al,b,x,g,h,cg,kMax,eps)
C***********************************************************************
c
C Ce sous-programme resout le systeme lineaire Ax=b par
C gradient conjugue preconditionne, avec au plus kMax 
C iterations. Le test d"arret est: |Ax^k-b|<eps|Ax^0-b|.
C La matrice A du systeme lineaire, de taille n*n
C est stockee sous forme Morse dans le tableau aa;
C ce stockage necessite aussi les tableaux auxiliaires
C d"indices ia et ja.
C La matrice C de preconditionnement est la factorisation
C de Choleski incomplete de A (C=L^* (L^*)^{T})  et la
C matrice L^* est stockee sous forme Morse dans le 
C tableau al, avec les memes tableaux d"indices ia et 
C ja que A.
C
C  aa: matrice A du systeme stockee sous forme Morse avec 
C      comme tableau d'indices ia et ja
C  al: factorisee de Choleski incomplete de A avec 
C      comme tableau d'indices ia et ja 
C  b: second membre du systeme lineaire
C  kMax: nombre maximal d"iterations
C  eps: precision du calcul
C  x:tableau contenant les valeurs de la solution 
C  gcg:scalaire contenant le carre de la norme de
C      g (en pratique ce nombre doit decroitre au 
C         cours des iterations).
C  g, cg: tableaux contenant les valeurs de g^n et
C         de C g^n respectivement;
C  h: tableau contenant la direction de descente h^n
C
C
      implicit none
      INTEGER n,ia,ja,kmax,i,k
      REAL*8  aa,al,b,x,g,h,cg,gcg,hah,ro,gamma,eps

      DIMENSION x(*),b(*),g(*),h(*),cg(*)
      DIMENSION aa(*),ja(*),ia(*),al(*)

      DO  k=1, kMax

        IF (k.EQ.1) THEN
          CALL IAmul(n,aa,ia,ja,x,h)
          DO  i=1,n
            cg(i)=b(i)-h(i)
            g(i)=cg(i)
          enddo
          CALL ISolvSys(n,g,al,ia,ja)
          gcg=0.d0
          DO  i=1,n
            gcg=gcg + g(i)*cg(i)
            h(i)=g(i)
          enddo
        ENDIF

        CALL IAmul(n,aa,ia,ja,h,b)
        hAh=0.d0
        DO i=1,n		
          hAh=hAh+h(i)*b(i)
        enddo
        ro=gcg/hAh
        DO  i=1,n
          cg(i) = cg(i) -ro*b(i)
        enddo
        CALL ISolvSys(n,b,al,ia,ja)
        DO  i=1,n
          x(i)=x(i)+ro*h(i)
          g(i)=g(i)- ro*b(i)
        enddo
        gamma=gcg
        gcg=0.d0
        DO  i=1,n
          gcg=gcg + g(i)*cg(i)
        enddo

c        IF(k.EQ.1) eps=eps*gcg


        IF (gcg.LT.eps) THEN
            write(*,*)'nombre d''iteration:',k,'  gcg=',gcg             
                         RETURN
        endif

        gamma=gcg/gamma
        DO  i=1,n
          h(i)=g(i)+gamma*h(i)
        enddo

      enddo

      write(*,*)'le solveur gradiant conjugue ne converge pas.' 
      write(17,*)'le solveur gradiant conjugue ne converge pas.'
      STOP

c      RETURN
      END

c  Fichier grad_qua.f
c
c***************************************************************
      subroutine GRADIANT(GRAD,det,N1,N2,N3,N4,xch,ych)
c***************************************************************
c
c   Cette procedure calcule le gradiant des fonctions de base 
c   au point (xch,yxch) pour le carre de reference
c
c   GRAD: tableau contenant les valeurs des gradiants des 
c         fonctions de bases  
c   det: jacobien de la transformation: carre de reference --> quadrilatere 
c   N1,N2,N3,N4: contient les numeros, ainsi que les coordonnees des
c                quatre points qui constituent le quadrilatere
c   xch,ych: coordonnees du point de calcul
c
c

      implicit none

      real*8    GRAD,xch,ych,det,N1,N2,N3,N4,J11,J12,J21,J22
      dimension GRAD(4,*),N1(*),N2(*),N3(*),N4(*) 
c
c calcul du jacobien de la transformation
c
      J11=-N1(2)+N2(2)+N3(2)-N4(2)+ych*(N1(2)-N2(2)+N3(2)-N4(2))
      J11=J11/4.d0

      J12=-N1(3)+N2(3)+N3(3)-N4(3)+ych*(N1(3)-N2(3)+N3(3)-N4(3)) 
      J12=J12/4.d0

      J21=-N1(2)-N2(2)+N3(2)+N4(2)+xch*(N1(2)-N2(2)+N3(2)-N4(2))
      J21=J21/4.d0

      J22=-N1(3)-N2(3)+N3(3)+N4(3)+xch*(N1(3)-N2(3)+N3(3)-N4(3))
      J22=J22/4.d0

      det=(J11*J22)-(J21*J12)
c
c calcul du gradiant               
c
      GRAD(1,1)=(J22/det)*((ych-1)/4.d0)-(J12/det)*((xch-1)/4.d0)
      GRAD(1,2)=(-J21/det)*((ych-1)/4.d0)+(J11/det)*((xch-1)/4.d0)
      
      GRAD(2,1)=(J22/det)*((1-ych)/4.d0)-(J12/det)*((-xch-1)/4.d0)
      GRAD(2,2)=(-J21/det)*((1-ych)/4.d0)+(J11/det)*((-xch-1)/4.d0) 

      GRAD(3,1)=(J22/det)*((ych+1)/4.d0)-(J12/det)*((xch+1)/4.d0)
      GRAD(3,2)=(-J21/det)*((ych+1)/4.d0)+(J11/det)*((xch+1)/4.d0) 
      
      GRAD(4,1)=(J22/det)*((-ych-1)/4.d0)-(J12/det)*((1-xch)/4.d0)
      GRAD(4,2)=(-J21/det)*((-ych-1)/4.d0)+(J11/det)*((1-xch)/4.d0) 
      
      RETURN
      end                                        
c  Fichier grad_tri.f
c
c*****************************************************************
      subroutine GRADIANTP(GRADP,det,N1,N2,N3)
c*****************************************************************
c
c   Cette procedure calcule le gradiant des differentes fonctions 
c   de base pour le triangle .
c
c   N1,N2,N3: contient les numeros, ainsi que les coordonnees des
c                trois points qui constituent le triangle
c   GRADP: tableau contenant les valeurs des  gradiants
c         des fonctions de base
c   det: jacobien de la transformation: triangle de reference --> triangle
c
c
      implicit none
      
      real*8    GRADP,det,N1,N2,N3,J11,J12,J21,J22 
      dimension GRADP(3,*),N1(*),N2(*),N3(*) 
c
c calcul du jacobien de la transformation
c      
      J11=(N2(2)-N1(2)) 
      J21=(N2(3)-N1(3))
      J12=(N3(2)-N1(2))
      J22=(N3(3)-N1(3))                     
      det=(J11*J22)-(J12*J21)
c
c calcul du gradiant     
c
      GRADP(1,1)=(J21-J22)/det
      GRADP(1,2)=(J12-J11)/det
      GRADP(2,1)=(J22/det)
      GRADP(2,2)=-(J12/det)
      GRADP(3,1)=-(J21/det)
      GRADP(3,2)=(J11/det)

      RETURN
      end                                        






C Fichier icholes.f
C
C******************************************************************
      SUBROUTINE ICholeski(ns,aa,ia,ja,al)
c******************************************************************
c
c
C Ce sous-programme effectue la factorisation de
C Choleski incomplete de la matrice A stockee sous
C forme Morse dans le tableau aa avec les tableaux
C auxiliaires d"indices ia et ja.
C ia(k): indice dans le tableau aa du premier element
C   de la i eme ligne de A stocke dans aa,
C ja(k): indice j de colonne de l"element Aij stocke
C   dans le tableau aa a l"indice k.
C Le resultat est stocke dans le tableau al, qui 
C utilise les memes tableaux auxiliaires d"indices
C ia et ja que aa.
C Le calcul de la factorisation incomplete se fait
C ligne par ligne en terminant par le calcul de 
C l"element diagonal.
C
C
      implicit none
      INTEGER   ia,ja,ns,k,jljk,jlik,ik,jk,i,j
      real*8    aa,al,alii,alij
      DIMENSION aa(*),al(*),ia(*),ja(*)
C
      DO  i=1,ns
        DO  k=ia(i),ia(i+1)-2
          j=ja(k)
          alij=aa(k)
          ik=ia(i)
          jk=ia(j)
35	      IF(ik.LE.(ia(i+1)-2)) THEN
            jlik=ja(ik)
36	        IF(jk.LE.(ia(j+1)-2)) THEN
              jljk=ja(jk)
              IF(jlik.GT.jljk)THEN
                jk=jk+1
                GOTO 36
              ELSEIF(jlik.LT.jljk)THEN
                  ik=ik+1
                  GOTO 35
              ELSE
                alij=alij-(al(ik)*al(jk))
                ik=ik+1
                jk=jk+1
                GOTO 35
              ENDIF
            ENDIF
          ENDIF
          al(k)=alij/al(ia(j+1)-1)
        ENDDO 
        alii=aa(ia(i+1)-1)
        DO  k=ia(i), ia(i+1)-2
          alii=alii-(al(k)*al(k))
        enddo
          al(ia(i+1)-1)=dsqrt(dabs(alii))
      enddo

      RETURN
      END
c Fichier info_ele.f
c
c****************************************************************
      subroutine COORD_T_F(TABele,elementbis,INDICE,nbndele,
     @                     N1,N2,N3,N4,nte,k)
c****************************************************************
c
c   Cette procedure donne les informations necessaires sur chaque
c   elements de la triangulation.
c
c   TABele: tableau contenant le numero de l'element, ainsi que les
c           numeros des noeuds qui constituent l'element
c   ELEMENTBIS: tableau qui permet de dire si un noeud de l'element est
c              sur le bord ou non.
c   INDICE: tableau donnant le rang d'un noeud appartenant a VECTI ou VECTF
c   nbnele: nombre entier qui differencie le triangle du quadrilatere
c   N1,N2,N3,N4: contient les numeros, ainsi que les coordonnees des
c                quatre points qui constituent le quadrilatere
c   nte: nombre total d'element
c   k: numero de l'element sur lequel on veut les informations
c
c
      implicit none

      integer   nbndele,TABele,INDICE,k,nte
      real*8    N1,N2,N3,N4
      logical   elementbis
      dimension N1(*),N2(*),N3(*),N4(*)
      dimension TABele(nte,*),elementbis(*),INDICE(*)
c
c on regarde si on a affaire a un triangle ou un quadrialatere
c
      if (TABele(k,5).eq.0) then
                              nbndele=4
                            else
                              nbndele=5
      endif
c
c on lit les informations dans le fichier NoeudV.dat
c      
      read(16,REC=TABele(k,2))N1(2),N1(3),ELEMENTBIS(2),INDICE(2)
      read(16,REC=TABele(k,3))N2(2),N2(3),ELEMENTBIS(3),INDICE(3)
      read(16,REC=TABele(k,4))N3(2),N3(3),ELEMENTBIS(4),INDICE(4)
      if (nbndele.eq.5) then
      read(16,REC=TABele(k,5))N4(2),N4(3),ELEMENTBIS(5),INDICE(5)
      endif

      RETURN
      end

                                               



c    Fichier jc.f
c
c************************************************************************
      subroutine CAL_JC (VECT1,jc,nelemC,nnf,cc)
c***********************************************************************
c
c Cette procedure initialise le tableau cc et calcule le tableau
c jc final
c
c   nelemC :dimension du tableau cc
c   nnf :nombre de noeud situes sur la frontiere
c   cc: tableau contenant les contributions bord-int
c   jc: tableau d'indice associe a cc    
c   VECT1: vecteur contenant les valeurs de jc non nulle
c
      implicit none 
 
      integer   i,VECT1,nelemC,nnf,jc
      real*8    cc
      dimension VECT1(*),jc(*),cc(*)

      do i=1,nelemC
         if (VECT1(i).ne.0) then
                              jc(i)=VECT1(i)
                            else
                              jc(i)=nnf+1
         endif
         write(20,*)i,'jc=',jc(i)
         cc(i)=0.d0
      enddo

      RETURN
      end


c  Fichier mise_a_0.f
c
c******************************************************************
      subroutine  MISE_A_ZERO(bb,jb,aa,ja,K,SMinter,nti,
     @                        nelemA,nelemB,poroh)
c*****************************************************************
c
c  Cette procedure initialise les tableaux bb,jb,aa,ja,SMinter
c  a zero, ainsi que la partie non diagonale du tenseur K.
c
c  bb: tableau contenant les contributions bord-bord
c  aa: tableau contenant les contributions int-int (c'est la matrice A
c      du systeme lineaire)
c  ja(k): indice j de colonne de l"element Aij stocke
c         dans le tableau aa a l"indice k
c  jb: idem que ja pour le tableau bb
c  K: tableau contenant le tenseur des permeabilites
c  SMinter: vecteur contenant le second membre
c           du systeme lineaire sans les conditions aux limites 
c  nti: nombre total d'inconnues du systeme lineaire
c  nelemA: dimension du tableau aa
c  nelemB: dimension du tableau bb
c
      implicit none

      integer   nti,i,ja,nelemA,nelemB,jb
      real*8    SMinter,aa,K,bb,poroh
      dimension SMinter(*),aa(*),ja(*),K(2,*),bb(*),jb(*)


      do i=1,nti      
         SMinter(i)=0.d0
      enddo


      do i=1,nelemA
         aa(i)=0.d0
         ja(i)=0
      enddo

      do i=1,nelemB
         bb(i)=0.d0
         jb(i)=0
      enddo

      K(1,2)=0.d0
      K(2,1)=K(1,2)

      poroh=0.d0
      
      RETURN
      end


 



c  Fichier nfr_nint.f
c
c********************************************************************
       subroutine REMPLIR (nbpoint,COORD,G,VECTI,VECTF,droite,
     @                             indiceI,indiceF,x,y,nbr1)
c*********************************************************************
c
c   Cette procedure remplie le tableau VECTF des numeros des noeuds
c   situe sur la frontiere et VECTI des noeuds situe a l'interieur
c   du domaine. On ecrit dans le fichier noeudV.dat les coordonnees
c   de chaque noeuds, sa position par rapport a la frontiere et son 
c   rang dans VECTI ou VECTF.
c
c   G: tableau contenant les coordonnees des points de la
c      grosse maille
c   COORD: tableaux contenants les valeurs des vecteurs directeurs
c   nbpoint :nombre de point de la grosse maille
c   VECTI: vecteur contenant les numeros des noeuds situes a l'interieur
c          de la grosse maille
c   VECTF: vecteur contenant les numeros des noeuds situes sur la frontiere
c          de la grosse maille
c   droite: tableau des equations des droites   
c   indiceI: indice comptant le nombre de noeuds situes a l'interieur
c   indiceF: indice comptant le nombre de noeuds situes sur la frontiere 
c   x,y: coordonnees du noeud
c   nbr1: numero du noeud
c

       implicit none

       integer   i,indiceI,indiceF,VECTI,VECTF,nbpoint,entier
       real*8    x,y,G,droite,nbr1,dif,COORD
       dimension G(nbpoint,*),VECTI(*),VECTF(*)
       dimension droite(nbpoint,*),COORD(nbpoint,*)

c
c calcul des equations des droites
c
       do i=1,nbpoint
          if ((coord(i,1).eq.0.d0).or.(coord(i,2).eq.0.d0))  then
                  if (coord(i,1).eq.0.d0) then
                      droite(i,1)=x
                      droite(i,2)=G(i,1)          !  ON PEUT
                                   else           !  AMELIORER
                      droite(i,1)=y               !  CETTE PARTIE
                      droite(i,2)=G(i,2) 
                   endif     
                                               else
                  droite(i,1)=(x-G(i,1))/coord(i,1)
                  droite(i,2)=(y-G(i,2))/coord(i,2) 
           endif        
       enddo
       i=1
c
c rangement des numeros des noeuds
c
c       write(*,*)
c       write(*,*)'nbr',nbr1
       do while (i.le.nbpoint)
          dif=ABS(droite(i,1)-droite(i,2))
c          write(*,*)'dif',dif
          if (dif.le.1.d-03) then
                                VECTF(indiceF)=nbr1
                                entier=nbr1
                  write(16,REC=entier)x,y,.false.,indiceF 
                                indiceF=indiceF+1 
                                RETURN
                              else
                                i=i+1
          endif
       enddo
       VECTI(indiceI)=nbr1
       entier=nbr1
       write(16,REC=entier)x,y,.true.,indiceI
       indiceI=indiceI+1 

        RETURN
        end




                                               
c   Fichier norme.f
c
c**********************************************************************
       subroutine NORME(VECTI,VECTF,x,nti,nnf)
c**********************************************************************
c
c  Cette procedure calcule l'erreur relative entre la solution
c  exacte et la solution approchee.
c  Cette procedure est utilise pour verifier la solution element fini
c  lorsque l'on connait la solution exacte
c
c
c   VECTI: vecteur contenant les numeros des noeuds situes a l'interieur
c          de la grosse maille
c   VECTF: vecteur contenant les numeros des noeuds situes sur la frontiere
c          de la grosse maille
c   x: solution du probleme local
c   nti :nombre total d'inconnues du systeme lineaire
c   nnf :nombre de noeud situes sur la frontiere



      implicit none

      integer nti,nnf,i,VECTI,VECTF
      real*8 x,titi,Ninf,toto,aux,BORD_PB1
      dimension VECTI(*),VECTF(*),x(*)

c
c Norme infini
c
      Ninf=0.d0
      write(20,*)'comparaison a la solution exacte '
      do i=1,nti
         aux=BORD_PB1(VECTI(i))
         write(20,*)i,' ',aux,' ',x(i)
         toto=ABS(aux-x(i))
         if (toto.gt.Ninf) then
                             Ninf=toto
         endif
      enddo
      write(20,*)
        write(20,*)'Norme infinie =',Ninf
      write(20,*)
c
c Erreur relative L2
c
      Ninf=0.d0
      titi=0.d0
      write(20,*)'Norme L2 '
      do i=1,nti
         aux=BORD_PB1(VECTI(i))
         write(20,*)i,' ',aux,' ',x(i)
         toto=(aux-x(i))*(aux-x(i))
         titi=titi+aux**2
         Ninf=Ninf+toto
      enddo
      write(20,*)
      write(20,*)'Erreur relative pour la Norme L2 ',
     @            DSQRT(Ninf)/DSQRT(titi)
      write(20,*)

      RETURN
      end
c  Fichier permequi.f
c
c         INCLUDE'essai.f'



c********************************************************************
        subroutine PERMEAEQUI(aa,ia,ja,X1,X2,nti,nnf,aire,CONDB1,
     @                       CONDB2,bb,ib,jb,cc,ic,jc,poroh)
c*********************************************************************
c
c       Cette procedure calcule le tenseur homogeneise a partir  
c       des 2 solutions des problemes locaux.    
c
c   aa: tableau contenant les contributions int-int (c'est la matrice A
c       du systeme lineaire)
c   ia(k): indice dans le tableau aa du premier element
c        de la i eme ligne de A stocke dans aa,
c   ja(k): indice j de colonne de l"element Aij stocke
c          dans le tableau aa a l"indice k
c   bb: tableau contenant les contributions bord-bord
c   cc: tableau contenant les contributions bord-int
c   ib,jb: idem que ia et ja pour le tableau bb
c   ic,jc: idem que ia et ja pour le tableau cc
c   X1: solution du probleme local 1
c   X2 :solution du probleme local 2
c   CONDB1,CONDB2: tableaux contenant les conditions aux limites
c                  sur le bord suivant le probleme local
c   nti :nombre total d'inconnues du systeme lineaire
c   nnf :nombre de noeud situes sur la frontiere
c   aire :aire de la grosse maille
c
        implicit none

        integer   i,j,nti,nnf,l,entier,k,ia,ja,ib,jb,ic,jc,m,kk
        real*8    X1,X2,somme1,somme2,somme3,aire,poroh
        real*8    K11,K22,K12,CONDB1,CONDB2,aa,bb,cc,ast
        dimension X2(*),X1(*),aa(*),ia(*),ja(*),bb(*),ib(*),jb(*)
        dimension cc(*),ic(*),jc(*),CONDB1(*),CONDB2(*)
c
c initialisation
c
        somme1=0.d0
        somme2=0.d0
        somme3=0.d0

       write(20,*)
       write(20,*)'somme1',somme1
       write(20,*)'somme2',somme2
       write(20,*)'somme3',somme3
c
c contribution int-int
c
        m=1
        l=1
        do i=1,nti
           do k=1,(ia(m+1)-ia(m))
              j=ja(l)
           write(20,*)X1(i),X1(j),aa(l)
           write(20,*)X2(i),X2(j),aa(l)
              somme1=somme1+(X1(i)*X2(j)*aa(l))
              somme2=somme2+(X2(i)*X2(j)*aa(l))
              somme3=somme3+(X1(i)*X1(j)*aa(l))
              l=l+1
           enddo
           m=m+1
        enddo

       write(20,*)
       write(20,*)'somme1',somme1
       write(20,*)'somme2',somme2
       write(20,*)'somme3',somme3
      
        m=2
        l=2
        do i=2,nti
           do k=1,(ia(m+1)-ia(m)-1)
              j=ja(l)
              somme1=somme1+(X1(j)*X2(i)*aa(l))
              somme2=somme2+(X2(j)*X2(i)*aa(l))
              somme3=somme3+(X1(j)*X1(i)*aa(l))
              l=l+1
           enddo
           l=l+1
           m=m+1
        enddo
c
c contribution bord-int
c

       write(20,*)
       write(20,*)'somme1',somme1
       write(20,*)'somme2',somme2
       write(20,*)'somme3',somme3

       kk=1
       do i=1,nnf
          do j=1,ic(i+1)-ic(i)
              entier=jc(kk)
                somme1=somme1+(CONDB2(i)*cc(kk)*X1(entier))
                somme2=somme2+(CONDB2(i)*cc(kk)*X2(entier))
                somme3=somme3+(CONDB1(i)*cc(kk)*X1(entier))
              kk=kk+1
          enddo
       enddo

       write(20,*)
       write(20,*)'somme1',somme1
       write(20,*)'somme2',somme2
       write(20,*)'somme3',somme3


       kk=1
       do i=1,nnf
          do j=1,ic(i+1)-ic(i)
              entier=jc(kk)
                somme1=somme1+(CONDB1(i)*cc(kk)*X2(entier))
                somme2=somme2+(CONDB2(i)*cc(kk)*X2(entier))
                somme3=somme3+(CONDB1(i)*cc(kk)*X1(entier))
              kk=kk+1
          enddo
       enddo

c
c contribution bord-bord
c
       write(20,*)
       write(20,*)'somme1',somme1
       write(20,*)'somme2',somme2
       write(20,*)'somme3',somme3

        m=1
        l=1
        do i=1,nnf
           do k=1,(ib(m+1)-ib(m))
              j=jb(l)
              somme1=somme1+(CONDB1(i)*CONDB2(j)*bb(l))
              somme2=somme2+(CONDB2(i)*CONDB2(j)*bb(l))
              somme3=somme3+(CONDB1(i)*CONDB1(j)*bb(l))
              l=l+1
           enddo
           m=m+1
        enddo

       write(20,*)
       write(20,*)'somme1',somme1
       write(20,*)'somme2',somme2
       write(20,*)'somme3',somme3

        m=2
        l=2
        do i=2,nnf
           do k=1,(ib(m+1)-ib(m)-1)
              j=jb(l)
              somme1=somme1+(CONDB1(j)*CONDB2(i)*bb(l))
              somme2=somme2+(CONDB2(j)*CONDB2(i)*bb(l))
              somme3=somme3+(CONDB1(j)*CONDB1(i)*bb(l))
              l=l+1
           enddo
           l=l+1
           m=m+1
        enddo

       write(20,*)
       write(20,*)'somme1',somme1
       write(20,*)'somme2',somme2
       write(20,*)'somme3',somme3
c
c division par l'aire le la grosse maille
c
      
      K12=somme1/aire
      K22=somme2/aire
      K11=somme3/aire
      poroh=poroh/aire


      if (ABS(K11).le.1.d-05) then
                            K11=0.d0
      endif
      if (ABS(K12).le.1.d-05) then
                            K12=0.d0
      endif
      if (ABS(K22).le.1.d-05) then
                            K22=0.d0
      endif
      write(*,*)
      write(*,19) 
      write(*,20)K11,K12,K22,poroh

c      write(17,*)
      write(17,*)K11,K12,K22         !,poroh


19    FORMAT(5x,'Kxx',12x,'Kxy',12x,'Kyy',12x,'PORO')
20    FORMAT(4(E11.4,4x))

c     c'est un essai
       write(*,*)
       
c       call PERMEQ2(K11,K12,K22)

c     c''est pour trouver un minorant
c      ast=(K11*K22)-(K12**2)
c      K11=K11/ast
c      K12=K12/ast
c      K22=K22/ast
c      write(*,20)K11,K12,K22

      RETURN
      end
                                             
c   Fichier perm_Kx.f
c
c
c*************************************************************************
      real*8 function K11t(x,y)
c*************************************************************************
c
      implicit none
      real*8 x,y

c      K11t=1.d0         !x+y
      K11t=x+y

      RETURN
      end

c*************************************************************************
      real*8 function K22t(x,y)
c*************************************************************************
c
      implicit none
      real*8 x,y

c      K22t=1.d0           !x+y
      K22t=x+y

      RETURN
      end

c*************************************************************************
      real*8 function K12t(x,y)
c*************************************************************************
c
      implicit none
      real*8 x,y

      K12t=0.d0       !x+y

      RETURN
      end
c      Fichier prelimi1.f
c
c******************************************************************
      subroutine PRELIMINAIRE(VECTF,VECTI,TABele,nnf,nti,nte,ic,nelemC,
     @                      VECT1,ib,nelemB,ia,nelemA)    
c*****************************************************************
c
c  Cette procedure affecte aux tableaux ia,ib,ic les valeurs definitives
c  en vue de l'assemblage des tableaux aa,bb et cc. Deplus les dimensions 
c  des tableaux aa,bb et cc sont calcules.
c
c  VECTF: tableau contenant les numero des noeuds situe sur la
c         frontiere du domaine
c  VECTI: tableau contenant les numero des noeuds situe a 
c         l'interieur du domaine
c  TABele: tableaux contenant les numeros des noeuds qui constituent 
c          les elements
c  nti: nombre total d'inconnue du systeme lineaire
c  nte: nombre total d'elements
c  ia(k): indice dans le tableau aa du premier element
c         de la i eme ligne de A stocke dans aa
c  ib,ic: idem que ia pour les tableaux bb,cc
c  nelemA: dimension du tableau aa
c  nelemB: dimension du tableau bb
c  nelemC: dimension du tableau cc
c  VECT1: tableau d'entiers en vue de calculer le tableau jc.
c  nnf :nombre de noeud situes sur la frontiere
c
c
      implicit none

      integer   nte,i,j,entIF,VECTF,VECT1,ib,nelemB,deb,pos1,pos2
      integer   lb,k,INDICE,nbndele,l,entIV,ind,VECTI,nti,nelemA
      integer   tt,ic,nelemC,nnf,TABele,a,b,ivois,ia,m,pos,nb
      real*8    N1,N2,N3,N4
      logical   elementbis
      dimension VECTF(*),INDICE(5),elementbis(5),ib(*),ia(*),VECTI(*)
      dimension N1(3),N2(3),N3(3),N4(3)
      dimension ic(*),TABele(nte,*),VECT1(*) 

      integer, ALLOCATABLE, DIMENSION(:,:) :: MATA 
      integer, ALLOCATABLE, DIMENSION(:,:) :: MATB
      integer, ALLOCATABLE, DIMENSION(:,:) :: MATC
c
c allocation dynamique de memoire des tableaux intermediaires
c MATA, MATB et MATC 
c

      write(20,*)'uuuuu0'

      lb=nti+27

      ALLOCATE(MATA(nti,lb))
      ALLOCATE(MATB(nnf,lb))
      ALLOCATE(MATC(nnf,lb))

      write(20,*)'uuuuu1'

      do i=1,nti
         do j=1,lb
            MATA(i,j)=-1
         enddo
      enddo

      do i=1,nnf
         do j=1,lb
            MATB(i,j)=-1
            MATC(i,j)=-1
         enddo
      enddo

      nb=1
      ind=1
      do k=1,nte
c         write(*,*) 
c         write(20,*)'element',k
         call COORD_T_F(TABele,elementbis,INDICE,nbndele,
     @                        N1,N2,N3,N4,nte,k)        
c         write(20,*)(TABele(k,j),j=1,5)
c         write(20,*)(elementbis(j),j=1,5)
c         write(*,*)'nbndele',nbndele
         do j=2,(nbndele-1)
c         write(*,*)'j',j
                deb=j+1
                pos1=INDICE(j)
                entIF=TABele(k,j)
                if (elementbis(j)) then
                    do m=deb,nbndele
                       entIV=TABele(k,m)
                       pos2=INDICE(m)
                       if (elementbis(m)) then
                           if (entIV.lt.entIF) then
                           call RANGEMENT(MATA,pos1,entIF,entIV,lb,nti) 
                                               else
                           call RANGEMENT(MATA,pos2,entIF,entIV,lb,nti)
                           endif
                                          else
                           call RANGEMENT(MATC,pos2,entIF,entIV,lb,nnf)
                           nb=nb+1
                       endif
                    enddo
                                    else
                    do m=deb,nbndele
                       entIV=TABele(k,m)
                       pos2=INDICE(m)
                       if (.not.(elementbis(m))) then
                           if (entIV.lt.entIF) then
                           call RANGEMENT(MATB,pos1,entIF,entIV,lb,nnf)
                                               else
                           call RANGEMENT(MATB,pos2,entIF,entIV,lb,nnf)
                           endif
                                                 else
                           call RANGEMENT(MATC,pos1,entIF,entIV,lb,nnf)
                           nb=nb+1
                       endif
                    enddo
                endif
           enddo 
       enddo      
                    
       do i=1,nti
          l=1
          do while (MATA(i,l).ne.-1) 
             l=l+1
          enddo
          ia(i)=l-1
       enddo

       DEALLOCATE(MATA)


        write(*,*)'nb=',nb
c       nb=nb+nnf
c       ALLOCATE(VECT1(nb))

        write(20,*)'ord_gr=',nb+nnf

        write(*,*)'ord_gr=',nb+nnf

       do i=1,nnf
          write(20,*)
          write(20,*)'MATB',(MATB(i,tt),tt=1,10)
       enddo 

       do i=1,nnf
          write(20,*)
          write(20,*)'MATC',(MATC(i,tt),tt=1,10)
       enddo
 
c       PAUSE 

       do i=1,nnf
          write(20,*)
          write(20,*)'MATC',(MATC(i,tt),tt=1,10)
          l=1
          j=1
          do while (MATB(i,l).ne.-1)
             l=l+1
          enddo
          do while (MATC(i,j).ne.-1)
             nb=MATC(i,j)-VECTF(i)
             pos=1
             do while ((nb.ne.VECTI(pos)).and.(pos.le.nti)) 
                pos=pos+1
             enddo
             VECT1(ind)=pos
             ind=ind+1
             j=j+1
          enddo
          ib(i)=l-1
          write(20,*)'ib',ib(i)
          ic(i)=j-1
          if (ic(i).eq.0) then
             VECT1(ind)=0
             ind=ind+1
             ic(i)=1
          endif
       enddo

       write(20,*)'VECT1',(VECT1(a),a=1,ind+3)
      PAUSE

c      DEALLOCATE(MATA)
      DEALLOCATE(MATB)


      DEALLOCATE(MATC)

      write(20,*)'ic                        prelimi1111111 '

c      pause
      do i=1,nnf
          write(20,*)'ic',ic(i),'   i',i
      enddo
      write(20,*)'ic final'


c      PAUSE
c
c calcul du tableau ic final ainsi que nelemC
c
      a=ic(1)
      ic(1)=1
      write(20,*)ic(1)
      do i=1,nnf-1
         b=ic(i+1)
         ic(i+1)=ic(i)+a
         write(20,*)ic(i)
         a=b
      enddo
      ic(nnf+1)=ic(nnf)+a
      nelemC=ic(nnf)+a-1

      write(20,*)'ic final'
      do i=1,nnf
          write(20,*)ic(i)
      enddo

      write(20,*)'nelemc',nelemC


      do i=1,nnf
         write(20,*)'ib',ib(i)
      enddo


      write(20,*)
      write(20,*)'ib final'
c
c calcul du tableau ib final ainsi que nelemB
c
      nelemB=0
      do i=1,nnf
         ivois=ib(i)
         ib(i)=nelemB+1
         write(20,*)ib(i)
         nelemB=nelemB+ivois+1
      enddo
      ib(nnf+1)=nelemB+1
      write(20,*)ib(nnf+1)
      write(20,*)'nelemb',nelemB


      do i=1,nti
         write(20,*)'ia',ia(i)
      enddo


      write(20,*)
      write(20,*)'ia final'
c
c calcul du tableau ia final ainsi que nelemA
c
      nelemA=0
      do i=1,nti
         ivois=ia(i)
         ia(i)=nelemA+1
         write(20,*)ia(i)
         nelemA=nelemA+ivois+1
      enddo
      ia(nti+1)=nelemA+1
      write(20,*)ia(nti+1)
      write(20,*)'nelema',nelemA


      RETURN
      end



c*****************************************************************
       subroutine RANGEMENT(MATA,pos,entIF,entIV,nti,borne)
c*****************************************************************
c
c
       implicit none
       integer MATA,l,nti,pos,entIF,entIV,borne

       dimension MATA(borne,*)


       l=1
       do while (l.le.nti)
          if (MATA(pos,l).ne.entIF+entIV) then      
              if (MATA(pos,l).eq.-1) then
                  MATA(pos,l)=entIF+entIV
c                  write(*,*)'MAT',(MATA(pos,k),k=1,l)
                  RETURN
                                     else
                  l=l+1
c               write(*,*)'l=',l
              endif
                                          else
              RETURN
          endif
       enddo
       
       RETURN
       end

c      Fichier prelimi1.f
c
c******************************************************************
      subroutine PRELIMINAIRE21(VECTF,VECTI,TABele,nnf,nti,nte,ia,
     @                         MATA,MATB,MATC,ord_gr,lb)    
c*****************************************************************
c
c  Cette procedure affecte aux tableaux ia,ib,ic les valeurs definitives
c  en vue de l'assemblage des tableaux aa,bb et cc. Deplus les dimensions 
c  des tableaux aa,bb et cc sont calcules.
c
c  VECTF: tableau contenant les numero des noeuds situe sur la
c         frontiere du domaine
c  VECTI: tableau contenant les numero des noeuds situe a 
c         l'interieur du domaine
c  TABele: tableaux contenant les numeros des noeuds qui constituent 
c          les elements
c  nti: nombre total d'inconnue du systeme lineaire
c  nte: nombre total d'elements
c  ia(k): indice dans le tableau aa du premier element
c         de la i eme ligne de A stocke dans aa
c  ib,ic: idem que ia pour les tableaux bb,cc
c  nelemA: dimension du tableau aa
c  nelemB: dimension du tableau bb
c  nelemC: dimension du tableau cc
c  VECT1: tableau d'entiers en vue de calculer le tableau jc.
c  nnf :nombre de noeud situes sur la frontiere
c
c
      implicit none

      integer   nte,i,j,entIF,VECTF,deb,pos1,pos2
      integer   lb,k,INDICE,nbndele,l,entIV,ind,VECTI,nti
      integer   nnf,TABele,ia,m,nb,iii,jjj
      integer   ord_gr,MATA,MATB,MATC
      real*8    N1,N2,N3,N4
      logical   elementbis
      dimension MATA(nti,*),MATB(nnf,*),MATC(nnf,*)
      dimension VECTF(*),INDICE(5),elementbis(5),ia(*),VECTI(*)
      dimension N1(3),N2(3),N3(3),N4(3)
      dimension TABele(nte,*) 

c      integer, ALLOCATABLE, DIMENSION(:,:) :: MATA 
c      integer, ALLOCATABLE, DIMENSION(:,:) :: MATB
c      integer, ALLOCATABLE, DIMENSION(:,:) :: MATC
c
c allocation dynamique de memoire des tableaux intermediaires
c MATA, MATB et MATC 
c

      write(20,*)'uuuuu0'

c      lb=nti+27

c      ALLOCATE(MATA(nti,lb))
c      ALLOCATE(MATB(nnf,lb))
c      ALLOCATE(MATC(nnf,lb))

      write(20,*)'uuuuu1',lb,nnf,nti

      do i=1,nti
         do j=1,lb
            MATA(i,j)=-1
         enddo
      enddo

      do i=1,nnf
         do j=1,lb
            MATB(i,j)=-1
            MATC(i,j)=-1
         enddo
      enddo

c      do i=1,nti
c           write(20,*)'MATA',(MATA(i,kkk),kkk=1,100)
c      enddo

c      do i=1,nnf
c           write(20,*)'MATB',(MATB(i,kkk),kkk=1,100)
c      enddo

c      do i=1,nnf
c           write(20,*)'MATC',(MATC(i,kkk),kkk=1,100)
c      enddo
c      pause

      nb=1
      ind=1
      do k=1,nte
         write(20,*) 
         write(20,*)'element',k
         call COORD_T_F(TABele,elementbis,INDICE,nbndele,
     @                        N1,N2,N3,N4,nte,k)        
         write(20,*)(TABele(k,j),j=1,5)
         write(20,*)(elementbis(j),j=1,5)
         write(20,*)'nbndele',nbndele
         do j=2,(nbndele-1)
c         write(*,*)'j',j
                deb=j+1
                pos1=INDICE(j)
                entIF=TABele(k,j)
                if (elementbis(j)) then
                    do m=deb,nbndele
                       entIV=TABele(k,m)
                       pos2=INDICE(m)
                       if (elementbis(m)) then
                           if (entIV.lt.entIF) then
c                 write(20,*)'MATA'
                           call RANGEMENT(MATA,pos1,entIF,entIV,lb,nti) 
                                               else
c                 write(20,*)'MATA'
                           call RANGEMENT(MATA,pos2,entIF,entIV,lb,nti)
                           endif
                                          else
c                 write(20,*)'MATC'
                           call RANGEMENT(MATC,pos2,entIF,entIV,lb,nnf)
                           nb=nb+1
                       endif
                    enddo
                                    else
                    do m=deb,nbndele
                       entIV=TABele(k,m)
                       pos2=INDICE(m)
                       if (.not.(elementbis(m))) then
                           if (entIV.lt.entIF) then
c                 write(20,*)'MATB'
                           call RANGEMENT(MATB,pos1,entIF,entIV,lb,nnf)
c                 write(20,*)'MAT',(MATB(pos1,kkk),kkk=1,100)
                                               else
c                 write(20,*)'MATB'
                           call RANGEMENT(MATB,pos2,entIF,entIV,lb,nnf)
c                 write(20,*)'MAT',(MATB(pos2,kkk),kkk=1,100)
                           endif
                                                 else
c                 write(20,*)'MATC'
                           call RANGEMENT(MATC,pos1,entIF,entIV,lb,nnf)
                           nb=nb+1
                       endif
                    enddo
                endif
           enddo 
       enddo      

                write(20,*)'***********************'
                do iii=1,nti
                  write(20,*)'MATA',(MATA(iii,jjj),jjj=1,lb)
                enddo
                   write(20,*)
                do iii=1,nnf
                   write(20,*)'MATB',(MATB(iii,jjj),jjj=1,lb)
                enddo
                   write(20,*)
                do iii=1,nnf
                   write(20,*)'MATC',(MATC(iii,jjj),jjj=1,lb)
                enddo
                write(20,*)'***********************'

                    
       do i=1,nti
          l=1
          do while (MATA(i,l).ne.-1) 
             l=l+1
          enddo
          ia(i)=l-1
       enddo

       ord_gr=nb+nnf+100

       write(*,*)'nouvel ord-gr',ord_gr 

       RETURN
       end





c*****************************************************************
c       subroutine RANGEMENT(MATA,pos,entIF,entIV,lb,borne)
c*****************************************************************
c
c
c       implicit none
c       integer MATA,l,lb,pos,entIF,entIV,borne,k
c
c       dimension MATA(borne,*)
c
c       write(20,*)'entIF,entIV,pos',entIF,' ',entIV,' ',pos
c       l=1
c       do while (l.le.lb)
c          if (MATA(pos,l).ne.entIF+entIV) then      
c              if (MATA(pos,l).eq.-1) then
c                  MATA(pos,l)=entIF+entIV
c                  write(20,*)'MAT',(MATA(pos,k),k=1,100)
c                  RETURN
c                                     else
c                  l=l+1
c               write(*,*)'l=',l
c              endif
c                                          else
c              RETURN
c          endif
c       enddo
c       
c       RETURN
c       end


c      Fichier prelimi1.f
c
c******************************************************************
      subroutine PRELIMINAIRE22(VECTF,VECTI,TABele,nnf,nti,nte,ic,
     @               nelemC,VECT1,ib,nelemB,ia,nelemA,MATB,MATC)    
c*****************************************************************
c
c  Cette procedure affecte aux tableaux ia,ib,ic les valeurs definitives
c  en vue de l'assemblage des tableaux aa,bb et cc. Deplus les dimensions 
c  des tableaux aa,bb et cc sont calcules.
c
c  VECTF: tableau contenant les numero des noeuds situe sur la
c         frontiere du domaine
c  VECTI: tableau contenant les numero des noeuds situe a 
c         l'interieur du domaine
c  TABele: tableaux contenant les numeros des noeuds qui constituent 
c          les elements
c  nti: nombre total d'inconnue du systeme lineaire
c  nte: nombre total d'elements
c  ia(k): indice dans le tableau aa du premier element
c         de la i eme ligne de A stocke dans aa
c  ib,ic: idem que ia pour les tableaux bb,cc
c  nelemA: dimension du tableau aa
c  nelemB: dimension du tableau bb
c  nelemC: dimension du tableau cc
c  VECT1: tableau d'entiers en vue de calculer le tableau jc.
c  nnf :nombre de noeud situes sur la frontiere
c
c
      implicit none

      integer   nte,i,j,VECTF,VECT1,ib,nelemB
      integer   tt,l,ind,VECTI,nti,nelemA
      integer   ic,nelemC,nnf,TABele,a,b,ivois,ia,pos,nb
      integer   MATB,MATC
c      real*8    N1,N2,N3,N4
      dimension MATB(nnf,*),MATC(nnf,*)
      dimension VECTF(*),ib(*),ia(*),VECTI(*)
c      dimension N1(3),N2(3),N3(3),N4(3)
      dimension ic(*),TABele(nte,*),VECT1(*) 

c       do i=1,nnf
c          write(20,*)
c          write(20,*)'MATB',(MATB(i,tt),tt=1,10)
c       enddo 

c       do i=1,nnf
c          write(20,*)
c          write(20,*)'MATC',(MATC(i,tt),tt=1,10)
c       enddo
 
c       PAUSE 

       ind=1
       do i=1,nnf
c          write(20,*)
c          write(20,*)'MATC',(MATC(i,tt),tt=1,10)
          l=1
          j=1
          do while (MATB(i,l).ne.-1)
             l=l+1
          enddo
          do while (MATC(i,j).ne.-1)
             nb=MATC(i,j)-VECTF(i)
             pos=1
             do while ((nb.ne.VECTI(pos)).and.(pos.le.nti)) 
                pos=pos+1
             enddo
             VECT1(ind)=pos
             ind=ind+1
             j=j+1
          enddo
          ib(i)=l-1
c          write(20,*)'ib',ib(i)
          ic(i)=j-1
          if (ic(i).eq.0) then
             VECT1(ind)=0
             ind=ind+1
             ic(i)=1
          endif
       enddo

c       write(20,*)'VECT1',(VECT1(a),a=1,ind+3)
c      PAUSE

c      DEALLOCATE(MATA)
c      DEALLOCATE(MATB)


c      DEALLOCATE(MATC)

      write(20,*)'ic                        prelimi1111111 '

c      pause
c      do i=1,nnf
c          write(20,*)'ic',ic(i),'   i',i
c      enddo
c      write(20,*)'ic final'


c      PAUSE
c
c calcul du tableau ic final ainsi que nelemC
c
      a=ic(1)
      ic(1)=1
c      write(20,*)ic(1)
      do i=1,nnf-1
         b=ic(i+1)
         ic(i+1)=ic(i)+a
c         write(20,*)ic(i)
         a=b
      enddo
      ic(nnf+1)=ic(nnf)+a
      nelemC=ic(nnf)+a-1

      write(20,*)'ic final'
      do i=1,nnf
          write(20,*)ic(i)
      enddo

      write(20,*)'nelemc',nelemC
      write(*,*)'le veritable ord_gr',nelemC

c      do i=1,nnf
c         write(20,*)'ib',ib(i)
c      enddo


      write(20,*)
      write(20,*)'ib final'
c
c calcul du tableau ib final ainsi que nelemB
c
      nelemB=0
      do i=1,nnf
         ivois=ib(i)
         ib(i)=nelemB+1
         write(20,*)ib(i)
         nelemB=nelemB+ivois+1
      enddo
      ib(nnf+1)=nelemB+1
      write(20,*)ib(nnf+1)
      write(20,*)'nelemb',nelemB


      do i=1,nti
c         write(20,*)'ia',ia(i)
      enddo


      write(20,*)
      write(20,*)'ia final'
c
c calcul du tableau ia final ainsi que nelemA
c
      nelemA=0
      do i=1,nti
         ivois=ia(i)
         ia(i)=nelemA+1
         write(20,*)ia(i)
         nelemA=nelemA+ivois+1
      enddo
      ia(nti+1)=nelemA+1
      write(20,*)ia(nti+1)
      write(20,*)'nelema',nelemA


      RETURN
      end



c      Fichier prelimi.f
c
c******************************************************************
c      subroutine PRELIMINAIRE(VECTF,VECTI,TABele,nnf,nti,nte,ic,nelemC,
c     @                      VECT1,ib,nelemB,ia,nelemA)    
c*****************************************************************
c
c  Cette procedure affecte aux tableaux ia,ib,ic les valeurs definitives
c  en vue de l'assemblage des tableaux aa,bb et cc. Deplus les dimensions 
c  des tableaux aa,bb et cc sont calcules.
c
c  VECTF: tableau contenant les numero des noeuds situe sur la
c         frontiere du domaine
c  VECTI: tableau contenant les numero des noeuds situe a 
c         l'interieur du domaine
c  TABele: tableaux contenant les numeros des noeuds qui constituent 
c          les elements
c  nti: nombre total d'inconnue du systeme lineaire
c  nte: nombre total d'elements
c  ia(k): indice dans le tableau aa du premier element
c         de la i eme ligne de A stocke dans aa
c  ib,ic: idem que ia pour les tableaux bb,cc
c  nelemA: dimension du tableau aa
c  nelemB: dimension du tableau bb
c  nelemC: dimension du tableau cc
c  VECT1: tableau d'entiers en vue de calculer le tableau jc.
c  nnf :nombre de noeud situes sur la frontiere
c
c
c      implicit none
c
c      integer   nte,i,j,entF,entI,VECTF,VECT1,ib,nelemB
c      integer   k,INDICE,nbndele,l,ent1,ll1,ti,ind,ll2,VECTI,nti,nelemA
c      integer   ic,nelemC,nnf,TABele,a,b,ivois,ll3,ia,r,s
c      real*8    N1,N2,N3,N4
c      logical   elementbis,bolF,bolI
c      dimension VECTF(*),INDICE(5),elementbis(5),ib(*),ia(*),VECTI(*)
c      dimension N1(3),N2(3),N3(3),N4(3)
c      dimension ic(*),TABele(nte,*),VECT1(*) 
c
c      integer, ALLOCATABLE, DIMENSION(:) :: VECT 
c      integer, ALLOCATABLE, DIMENSION(:) :: VECT2
c      integer, ALLOCATABLE, DIMENSION(:) :: VECT3
c
c allocation dynamique de memoire des tableaux intermediaires
c VECT, VECT2 et VECT3
c
c      ALLOCATE(VECT(nnf))
c      ALLOCATE(VECT2(nnf))
c      ALLOCATE(VECT3(nti))
c      bolF=.true.
c      bolI=.true.
c      r=1
c      s=1
c      ind=1
c      do while (bolI.or.bolF)
c         ll1=1
c         ll2=1
c         ll3=1
c         do ti=1,nnf
c            VECT(ti)=-1
c            VECT2(ti)=-1
c         enddo
c         do ti=1,nti
c            VECT3(ti)=-1
c         enddo
c         entF=VECTF(r)
c         entI=VECTI(s)
c         do k=1,nte
c            if (bolF.and.((TABele(k,2).eq.entF).or.(TABele(k,3).eq.entF)
c     @          .or.(TABele(k,4).eq.entF).or.(TABele(k,5).eq.entF)))then
c                call COORD_T_F(TABele,elementbis,INDICE,nbndele,
c     @                        N1,N2,N3,N4,nte,k)
c                          do j=2,nbndele            
c                             if (elementbis(j)) then
c                                l=1
c                                ent1=TABele(k,j)
c                              if (ent1.ne.0) then
c                                do while (l.le.nnf) 
c                                   if (VECT(l).ne.ent1+entF) then
c                                      if (VECT(l).eq.-1) then
c                                                    VECT(ll1)=ent1+entF
c                                                    VECT1(ind)=INDICE(j)
c                                                    ind=ind+1
c                                                    ll1=ll1+1
c                                                        else
c                                                     l=l+1
c                                      endif
c                                                     else
c                                                     GOTO 10
c                                   endif
c                                enddo
c                              endif
c                                                else
c                                l=1
c                                ent1=TABele(k,j)
c                              if((ent1.lt.entF).and.(ent1.ne.0)) then
c                                do while (l.le.nnf) 
c                                   if (VECT2(l).ne.ent1+entF) then
c                                      if (VECT2(l).eq.-1) then
c                                                    VECT2(ll2)=ent1+entF
c                                                    ll2=ll2+1
c                                                        else
c                                                     l=l+1
c                                      endif
c                                                     else
c                                                     GOTO 10
c                                   endif
c                                enddo
c                              endif
c                             endif
c10                           continue
c                          enddo
c            endif
c            if(bolI.and.((TABele(k,2).eq.entI).or.(TABele(k,3).eq.entI)
c     @          .or.(TABele(k,4).eq.entI).or.(TABele(k,5).eq.entI)))then
c                call COORD_T_F(TABele,elementbis,INDICE,nbndele,
c     @                        N1,N2,N3,N4,nte,k)
c                          do j=2,nbndele
c                             if (elementbis(j)) then
c                                l=1
c                                ent1=TABele(k,j)
c                              if ((ent1.lt.entI).and.(ent1.ne.0)) then
c                                do while (l.le.nti) 
c                                   if (VECT3(l).ne.ent1+entI) then
c                                      if (VECT3(l).eq.-1) then
c                                                    VECT3(ll3)=ent1+entI
c                                                     ll3=ll3+1
c                                                        else
c                                                     l=l+1
c                                      endif
c                                                     else
c                                                     GOTO 11
c                                   endif
c                                enddo
c                              endif
c                             endif
c11                           continue
c                          enddo
c            endif
c         enddo
c          if (r.le.nnf) then
c                    write(20,*)'VECT',(VECT(a),a=1,ll1+1)
c                    ic(r)=ll1-1
c                    ib(r)=ll2-1
cc                    write(20,*)'VECT1',(VECT1(a),a=1,ll1+1)
c                    if (ic(r).eq.0) then
c                           VECT1(ind)=0
c                           ind=ind+1
c                           ic(r)=1
c                    endif
c          endif
c          if (s.le.nti) then
c                    ia(s)=ll3-1
c          endif
c         r=r+1
c         if (r.gt.nnf) then
c                         bolF=.false.
c         endif
c         s=s+1
c         if (s.gt.nti) then
c                         bolI=.false.
c         endif
c      enddo
c      write(20,*)'VECT1',(VECT1(a),a=1,ind+3)
c
c      do i=1,nnf
c         write(*,*)'ic',ic(i),'         i=',i
c         write(20,*)'ic',ic(i),'         i=',i
c      enddo
cc
cc on rend la memoire des tableaux VECT, VECT2 et VECT3 au systeme
cc
c      DEALLOCATE(VECT)
c      DEALLOCATE(VECT2)
c      DEALLOCATE(VECT3)
c
c      write(20,*)'nnf',nnf
c      write(20,*)ic(1)
c      write(20,*)'ic final'
c
cc
cc calcul du tableau ic final ainsi que nelemC
cc    
c      a=ic(1)
c      ic(1)=1    
cc      write(20,*)ic(1)
c      do i=1,nnf-1
c         b=ic(i+1)
c         ic(i+1)=ic(i)+a
c         a=b
c      enddo                          
c      ic(nnf+1)=ic(nnf)+a
c      nelemC=ic(nnf)+a-1
c
c          
c      write(20,*)'ic final'
c      do i=1,nnf
c          write(20,*)ic(i)
c      enddo
c
c      write(20,*)'nelemc',nelemC
c
c
c      do i=1,nnf
c         write(20,*)'ib',ib(i)
c      enddo
c
c    
c      write(20,*)
c      write(20,*)'ib final'
cc
cc calcul du tableau ib final ainsi que nelemB
cc
c      nelemB=0
c      do i=1,nnf
c         ivois=ib(i)
c         ib(i)=nelemB+1
c         write(20,*)ib(i)
c         nelemB=nelemB+ivois+1
c      enddo
c      ib(nnf+1)=nelemB+1
c      write(20,*)ib(nnf+1)
c      write(20,*)'nelemb',nelemB
c
c
c      do i=1,nti
c         write(20,*)'ia',ia(i)
c      enddo
c
c      
c      write(20,*)
c      write(20,*)'ia final'
cc
cc calcul du tableau ia final ainsi que nelemA
cc
cc      nelemA=0
c      do i=1,nti
c         ivois=ia(i)
c         ia(i)=nelemA+1
c         write(20,*)ia(i)
c         nelemA=nelemA+ivois+1
c      enddo
c      ia(nti+1)=nelemA+1
c      write(20,*)ia(nti+1)
c      write(20,*)'nelema',nelemA
c
c
c      RETURN
c      end





C Fichier prod_mat.f
c
C
c******************************************************************
      SUBROUTINE IAmul(ns,aa,ia,ja,x,b)
c******************************************************************
c
C Ce sous-programme calcule le produit de la matrice
C A, stockee sous forme Morse dans les tableaux 
C (aa,ia,ja), par le vecteur x; le resultat est dans le
C tableau b.
C Le calcul se fait d"abord en calculant la contribution
C associee a la partie triangulaire inferieure de la 
C matrice (qui est celle stockee dans le tableau aa),
C puis celle associee a la partie triangulaire superieure
C stricte.
C
C
      implicit none
      INTEGER ns,i,k,jak,ia,ja
      REAL*8  aa,x,xi,b,somme

      DIMENSION x(*),b(*)
      DIMENSION aa(*),ia(*),ja(*)

      DO  i=1,ns
        somme=0.d0
        DO  k=ia(i),ia(i+1)-1
          somme=somme+aa(k)*x(ja(k))
        enddo
        b(i)=somme
      enddo
      DO  i=1,ns
        xi=x(i)
        DO  k=ia(i),ia(i+1)-2
          jak=ja(k)
          b(jak)=b(jak)+aa(k)*xi
        enddo
      enddo

      RETURN
      END


c  Fichier prod_qua.f
c
c*******************************************************     
      real*8 function PRODUIT (K,GRAD,nbr1,nbr2)
c*******************************************************
c
c   Cette fonction calcule le produit scalaire de [K(x)GRAD(u).GRAD(v)],
c   u et v designant deux fonctions de base quelconque du
c   quadrilatere.  
c
c   K: tableau contenant le tenseur des permeabilites
c   GRAD: tableau contenant les valeurs des  gradiants
c         des fonctions de base
c   nbr1,nbr2: numero designant la nbr1ieme et la nbr2ieme
c              fonction de base
c    


      implicit none
      integer   nbr1,nbr2   
      real*8    K,GRAD
      dimension K(2,*),GRAD(4,*)
c
c produit scalaire
c
      PRODUIT=K(1,1)*GRAD(nbr1,1)*GRAD(nbr2,1)+
     @        K(1,2)*GRAD(nbr1,2)*GRAD(nbr2,1)+
     @        K(2,1)*GRAD(nbr1,1)*GRAD(nbr2,2)+
     @        K(2,2)*GRAD(nbr1,2)*GRAD(nbr2,2)
      
      RETURN
      end
                         

                         

c Fichier prod_tri.f
c
c******************************************************************
      real*8 function PRODUITP (K,GRADP,nbr1,nbr2)
c******************************************************************
c
c  Cette fonction calcule le produit scalaire (K(x)GRAD(u).GRAD(v))
c  pour un triangle.
c
c   K: tableau contenant le tenseur des permeabilites
c   GRADP: tableau contenant les valeurs des  gradiants
c         des fonctions de base
c   nbr1,nbr2: numero designant la nbr1ieme et la nbr2ieme
c              fonction de base

      implicit none

      integer   nbr1,nbr2     
      real*8    K,GRADP
      dimension GRADP(3,*),K(2,*)

c
c produit scalaire
c
      PRODUITP=K(1,1)*GRADP(nbr1,1)*GRADP(nbr2,1)+
     @         K(1,2)*GRADP(nbr1,2)*GRADP(nbr2,1)+
     @         K(2,1)*GRADP(nbr1,1)*GRADP(nbr2,2)+
     @         K(2,2)*GRADP(nbr1,2)*GRADP(nbr2,2)


      RETURN
      end


c Fichier quadrila.f
c
c*********************************************************************
c                 FICHIERS NECESSAIRES A LA SUBROUTINE CALCULQ1
c*********************************************************************

c      INCLUDE'./grad_qua.f'
c      INCLUDE'./prod_qua.f'
c      INCLUDE'./rige_qua.f'
c      INCLUDE'./base_qua.f'
c      INCLUDE'./2mb_qua.f'
c      INCLUDE'./aire2.f'

c*******************************************************************
      subroutine CALCULQ1(ELEMENTBIS,INDICE,bb,ib,jb,aa,ia,ja,SMinter
     @                    ,N1,N2,N3,N4,K,cc,ic,jc,aire,poro,poroh)
c*******************************************************************
c
c  Cette procedure calcule et assemble les differentes contributions amenees
c  par un element de type quadrilatere dans le tableau aa. Deplus 
c  certains resultats necessaire en vue du calcul du tenseur homogeneise
c  sont sauvegardes dans les tableaux bb et cc.
c
c  ELEMENTBIS: tableau qui permet de dire si un noeud de l'element est 
c              sur le bord ou non.
c  INDICE:  tableau donnant le rang d'un noeud appartenant a VECTI ou VECTF
c  bb: tableau contenant les contributions bord-bord
c  aa: tableau contenant les contributions int-int (c'est la matrice A
c      du systeme lineaire) 
c  cc: tableau contenant les contributions bord-int
c  ia(k): indice dans le tableau aa du premier element
c        de la i eme ligne de A stocke dans aa,
c  ja(k): indice j de colonne de l"element Aij stocke
c         dans le tableau aa a l"indice k
c  ib,jb: idem que ia et ja pour le tableau bb
c  ic,jc: idem que ia et ja pour le tableau cc
c  SMinter:vecteur contenant le second membre du systeme lineaire
c            sans les conditions aux limites
c   N1,N2,N3,N4: contient les numeros, ainsi que les coordonnees des
c                quatre points qui constituent le quadrilatere
c   K: tableau contenant le tenseur des permeabilites
c
c
      implicit none           

      integer   ele,l,nbr1,INDICE,ia,ja,ib,jb,ic,jc,nbr2,k1,k2,ent  
      real*8    K,N1,N2,N3,N4,GRAD,det,aa,bb,cc,SMinter,aire0,aire1 
      real*8    INTEGRALE,som1,INTEGRPQ,aire,poro,poroh
      logical   ELEMENTBIS,log1,log2
      dimension N1(*),N2(*),N3(*),N4(*),K(2,*),GRAD(4,2)  
      dimension ELEMENTBIS(*),INDICE(*),bb(*),ib(*),jb(*)  
      dimension aa(*),ia(*),ja(*),SMinter(*),cc(*),ic(*),jc(*)  
     
      call AIRE_ELE2(aire0,N1,N2,N3)
      call AIRE_ELE2(aire1,N3,N4,N1)
      aire=aire+(aire0+aire1)
      poroh=poroh+(poro*(aire0+aire1))

 
       do ele=2,5
         nbr1=ele-1
         log2=.not.ELEMENTBIS(ele)
         do l=ele,5
            k1=INDICE(ele)
            k2=INDICE(l)
            nbr2=l-1
            log1=.not.ELEMENTBIS(l)
c
c calcul de la contribution
c
            som1=INTEGRALE(GRAD,N1,N2,N3,N4,K,det,nbr1,nbr2)
c
c assemblage
c            
            if (log1.and.log2) then     
                  if (k2.ge.k1) then                         
                           ent=k1
                           k1=k2
                           k2=ent
                  endif
                  call ASSEMBLAGE(bb,ib,jb,k1,k2,som1)
                               else
    
            if (ELEMENTBIS(ele).and.ELEMENTBIS(l)) then
                  if (k2.ge.k1) then                         
                           ent=k1
                           k1=k2
                           k2=ent
                  endif
                  call ASSEMBLAGE(aa,ia,ja,k1,k2,som1)
                  else

            log1=.not.ELEMENTBIS(l)
        if (ELEMENTBIS(ele).and.log1) then
                   call ASSEMBLAGEbis(cc,ic,jc,k2,k1,som1)
                else

          if (ELEMENTBIS(l)) then               
                     k1=INDICE(l)
                     K2=INDICE(ele)
                     call ASSEMBLAGEbis(cc,ic,jc,k2,k1,som1) 
            endif
           endif
         endif
        endif
       enddo
c
c calcul de la contribution du second membre et assemblage
c
       if (ELEMENTBIS(ele)) then
            k1=INDICE(ele)
            SMinter(k1)=SMinter(k1)+INTEGRPQ(nbr1,N1,N2,N3,N4)
       endif
      enddo

      RETURN
      end
         
c Fichier qu_compl.f
c
c*********************************************************************
c                 FICHIERS NECESSAIRES A LA SUBROUTINE CALCULQ1
c*********************************************************************

c      INCLUDE'BIB/grad_qua.f'
c      INCLUDE'BIB/prod_qua.f'
c      INCLUDE'BIB/rige2_qu.f'
c      INCLUDE'BIB/base_qua.f'
c      INCLUDE'BIB/2mb_qua.f'
c      INCLUDE'BIB/perm_Kx.f'

C  Fichier resol_sys.f
C
C******************************************************************
            SUBROUTINE ISolvSys(ns,z,al,il,jl)
C******************************************************************
c
C Ce sous-programme resout le systeme lineaire
C LL^T z = r, ou L est une matrice triangulaire
C inferieure de taille ns x ns, stockee sous forme
C Morse dans les tableaux (al,il,jl). En entree, le 
C tableau z contient le second membre du systeme; 
C en sortie, il contient la solution z.
C La resolution se fait en 2 etapes: une etape de 
C descente, suivie d'une etape de remontee.
C
C

      implicit none
      INTEGER ns,i,i1,k,jlk,il,jl
      REAL*8  al,z,somme
      DIMENSION z(*),al(*),il(*),jl(*)

      DO  i=1,ns
        somme=z(i)
        DO  k=il(i), il(i+1)-2
          somme=somme-al(k)*z(jl(k))
        enddo
        z(i)=somme/al(il(i+1)-1)
      enddo
      DO  i=1,ns
        z(i) = z(i)/al(il(i+1)-1)
      enddo
      DO  i1=1,ns
        i=ns+1-i1
        DO  k=il(i), il(i+1)-2
          jlk=jl(k)
          z(jlk) = z(jlk) - al(k)*z(i)/al(il(jlk+1)-1)
        enddo
      enddo

      RETURN
      END
c  Fichier  rige2_qu.f
c
c*****************************************************************
      real*8 function INTEGRALE(GRAD,N1,N2,N3,N4,K,det,nbr1,nbr2)
c*****************************************************************
c
c   Cette fonction calcule l'integrale de K(x)GRAD(u).GRAD(v) sur
c   un quadrilatere. u et v designant deux fonctions de base quelconque
c   du quadrilatere.
c
c   N1,N2,N3,N4: contient les numeros, ainsi que les coordonnees des
c                quatre points qui constituent le quadrilatere
c   K: tableau contenant le tenseur des permeabilites
c   GRAD: tableau contenant les valeurs des  gradiants
c         des fonctions de base
c   nbr1,nbr2: numero designant la nbr1ieme et la nbr2ieme
c              fonction de base
c   det: jacobien de la transformation: carre de reference --> quadrilatere  
c
c

      implicit none

      integer   nbr1,nbr2          
      real*8    somme,GRAD,N1,N2,N3,N4,det,K,PRODUIT,point
      dimension N1(*),N2(*),N3(*),N4(*),K(2,*),GRAD(4,*)

      point=1.d0/SQRT(3.d0)


      write(20,*)
      call GRADIANT(GRAD,det,N1,N2,N3,N4,point,point) 
      call VAL_TENS(K,N1,N2,N3,N4,point,point)
      somme=PRODUIT(K,GRAD,nbr1,nbr2)*ABS(det)
      call GRADIANT(GRAD,det,N1,N2,N3,N4,-point,point) 
      call VAL_TENS(K,N1,N2,N3,N4,-point,point)
      somme=somme+PRODUIT(K,GRAD,nbr1,nbr2)*ABS(det)
      call GRADIANT(GRAD,det,N1,N2,N3,N4,-point,-point) 
      call VAL_TENS(K,N1,N2,N3,N4,-point,point)
      somme=somme+PRODUIT(K,GRAD,nbr1,nbr2)*ABS(det)
      call GRADIANT(GRAD,det,N1,N2,N3,N4,point,-point) 
      call VAL_TENS(K,N1,N2,N3,N4,point,-point)
      somme=somme+PRODUIT(K,GRAD,nbr1,nbr2)*ABS(det)
      INTEGRALE=somme    
      
      RETURN
      end


c*****************************************************************
      subroutine VAL_TENS(K,N1,N2,N3,N4,xch,ych)    
c*****************************************************************
c
c
      implicit none

      real*8    BASEC,xch,ych,N1,N2,N3,N4
      real*8    a1,a2,b1,b2,x,K,y,K11t,K22t,K12t
      dimension BASEC(4),N1(*),N2(*),N3(*),N4(*),K(2,*)

      a1=(1.d0+xch)
      a2=(1.d0-xch)
      b1=(1.d0+ych)
      b2=(1.d0-ych)

      BASEC(1)=(a2*b2)/4.d0
      BASEC(2)=(a1*b2)/4.d0
      BASEC(3)=(a1*b1)/4.d0
      BASEC(4)=(a2*b1)/4.d0

      x=(BASEC(1)*N1(2)+BASEC(2)*N2(2)+BASEC(3)*N3(2)+BASEC(4)*N4(2))
      y=(BASEC(1)*N1(3)+BASEC(2)*N2(3)+BASEC(3)*N3(3)+BASEC(4)*N4(3))

      K(1,1)=K11t(x,y)
      K(2,2)=K22t(x,y)
      K(1,2)=K12t(x,y)
      K(2,1)=K(1,2)

      RETURN
      end








c  Fichier  rige2_tr.f
c
c*****************************************************************
      real*8 function INTEGRALEP(GRADP,N1,N2,N3,K,det,nbr1,nbr2)
c*****************************************************************
c
c   Cette fonction calcule l'integrale de K(x)GRAD(u).GRAD(v) sur
c   un quadrilatere. u et v designant deux fonctions de base quelconque
c   du quadrilatere.
c
c   N1,N2,N3: contient les numeros, ainsi que les coordonnees des
c             trois points qui constituent le triangle
c   K: tableau contenant le tenseur des permeabilites
c   GRAD: tableau contenant les valeurs des  gradiants
c         des fonctions de base
c   nbr1,nbr2: numero designant la nbr1ieme et la nbr2ieme
c              fonction de base
c   det: jacobien de la transformation: triangle de reference --> triangle  
c
c

      implicit none

      integer   nbr1,nbr2          
      real*8    somme,GRADP,N1,N2,N3,det,K,PRODUITP,a,b
      dimension N1(*),N2(*),N3(*),K(2,*),GRADP(3,*)

      a=(1.d0/6.d0)
      b=(2.d0/3.d0)


      write(20,*)
      call VAL_TENST(K,N1,N2,N3,a,a)
      somme=PRODUITP(K,GRADP,nbr1,nbr2)*ABS(det)*a
      call VAL_TENST(K,N1,N2,N3,b,a)
      somme=somme+PRODUITP(K,GRADP,nbr1,nbr2)*ABS(det)*a
      call VAL_TENST(K,N1,N2,N3,a,b)
      somme=somme+PRODUITP(K,GRADP,nbr1,nbr2)*ABS(det)*a
      INTEGRALEP=somme    
      
      RETURN
      end


c*****************************************************************
      subroutine VAL_TENST(K,N1,N2,N3,xch,ych)    
c*****************************************************************
c
c
      implicit none

      real*8    BASEP,xch,ych,N1,N2,N3
      real*8    x,K,y,K11t,K22t,K12t
      dimension BASEP(3),N1(*),N2(*),N3(*),K(2,*)


      BASEP(1)=(1.d0-xch-ych)
      BASEP(2)=xch
      BASEP(3)=ych

      x=(BASEP(1)*N1(2)+BASEP(2)*N2(2)+BASEP(3)*N3(2))
      y=(BASEP(1)*N1(3)+BASEP(2)*N2(3)+BASEP(3)*N3(3))

      K(1,1)=K11t(x,y)
      K(2,2)=K22t(x,y)
      K(1,2)=K12t(x,y)
      K(2,1)=K(1,2)

      RETURN
      end








c  Fichier  rige_qua.f
c
c*****************************************************************
c      real*8 function INTEGRALE(GRAD,N1,N2,N3,N4,K,det,nbr1,nbr2)
c*****************************************************************
c
c   Cette fonction calcule l'approxiamtion de l'integrale de 
c   [K(x)GRAD(u).GRAD(v)] sur un quadrilatere.
c    u et v designant deux fonctions de base quelconque
c   du quadrilatere.
c   On utilise la formule p.??? du  livre 'PRESENTATION DE LA METHODE
c   DES ELEMENTS FINIS' (T.Touzot) 19??.
c
c   N1,N2,N3,N4: contient les numeros, ainsi que les coordonnees des
c                quatre points qui constituent le quadrilatere
c   K: tableau contenant le tenseur des permeabilites
c   GRAD: tableau contenant les valeurs des  gradiants
c         des fonctions de base
c   nbr1,nbr2: numero designant la nbr1ieme et la nbr2ieme
c              fonction de base
c   det: jacobien de la transformation: carre de reference --> quadrilatere  
c
c
c
c      implicit none
c
c      integer   nbr1,nbr2          
c      real*8    somme,GRAD,N1,N2,N3,N4,det,K,PRODUIT,point
c      dimension N1(*),N2(*),N3(*),N4(*),K(2,*),GRAD(4,*)
c
c      point=1.d0/SQRT(3.d0)
c
c      call GRADIANT(GRAD,det,N1,N2,N3,N4,point,point) 
c      somme=PRODUIT(K,GRAD,nbr1,nbr2)                   
c      call GRADIANT(GRAD,det,N1,N2,N3,N4,-point,point) 
c      somme=somme+PRODUIT(K,GRAD,nbr1,nbr2)             
c      call GRADIANT(GRAD,det,N1,N2,N3,N4,-point,-point) 
c      somme=somme+PRODUIT(K,GRAD,nbr1,nbr2)            
c      call GRADIANT(GRAD,det,N1,N2,N3,N4,point,-point) 
c      somme=somme+PRODUIT(K,GRAD,nbr1,nbr2)             
c      
c      INTEGRALE=somme*ABS(det)    
c      
c      RETURN
c      end
c  Fichier triangle.f
c
c********************************************************************
c            FICHIERS NECESSAIRES A LA SUBROUTINE CALCULP1 
c********************************************************************

c      INCLUDE'./grad_tri.f'
c      INCLUDE'./prod_tri.f'   
c      INCLUDE'./base_tri.f'
c      INCLUDE'./2mb_tri.f'
c      INCLUDE'./aire1.f'

c********************************************************************
      subroutine CALCULP1(ELEMENTBIS,INDICE,bb,ib,jb,aa,ia,ja,SMinter
     @               ,N1,N2,N3,K,cc,ic,jc,aire,poro,poroh)
c********************************************************************
c
c  Cette procedure calcule les differentes contributions amenees par
c  un element triangulaire, et effectue l'assemblage.
c  On sauvegarde egalement certains resultats necessaire en vue du
c  calcul du tenseur homogeneise dans les tableaux bb et cc.
c
c  ELEMENTBIS: tableau qui permet de dire si un noeud de l'element est
c              sur le bord ou non.
c  INDICE:  tableau donnant le rang d'un noeud appartenant a VECTI ou VECTF
c  bb: tableau contenant les contributions bord-bord
c  aa: tableau contenant les contributions int-int (c'est la matrice A
c      du systeme lineaire)
c  cc: tableau contenant les contributions bord-int
c  ia(k): indice dans le tableau aa du premier element
c        de la i eme ligne de A stocke dans aa,
c  ja(k): indice j de colonne de l"element Aij stocke
c         dans le tableau aa a l"indice k
c  ib,jb: idem que ia et ja pour le tableau bb
c  ic,jc: idem que ia et ja pour le tableau cc
c  SMinter:vecteur contenant le second membre du systeme lineaire
c            sans les conditions aux limites 
c   N1,N2,N3: contient les numeros, ainsi que les coordonnees des
c             trois points qui constituent le triangle
c   K: tableau contenant le tenseur des permeabilites
c

      implicit none

      integer   INDICE,ele,l,nbr1,ia,ja,ib,jb,ic,jc,nbr2,k1,k2,ent
      real*8    K,N1,N2,N3,det,det_aire,GRADP,aire,poro,poroh 
      real*8    som1,PRODUITP,aa,bb,cc,SMinter,INTEGRP,aire0  
      logical   ELEMENTBIS,log1,log2    
      dimension N1(*),N2(*),N3(*),K(2,*),GRADP(3,2)
      dimension ELEMENTBIS(*),INDICE(*),bb(*),ib(*),jb(*)
      dimension aa(*),ia(*),ja(*),SMinter(*),cc(*),ic(*),jc(*)

      call AIRE_ELE1(aire0,N1,N2,N3) 
      aire=aire+aire0
      poroh=poroh+(poro*aire0)

      call GRADIANTP(GRADP,det,N1,N2,N3)
      det_aire=ABS(det)*0.5d0

      do ele=2,4
         nbr1=ele-1 
         log2=.not.ELEMENTBIS(ele)
         do l=ele,4
            k1=INDICE(ele) 
            k2=INDICE(l)
            nbr2=l-1
            log1=.not.ELEMENTBIS(l) 
c
c calcul de la contribution
c
            som1=PRODUITP(K,GRADP,nbr1,nbr2)*det_aire
            write(20,*)'prod',PRODUITP(K,GRADP,nbr1,nbr2)            
            write(20,*)'som1',som1
c
c assemblage
c            
            if (log1.and.log2) then
                     if (k2.gt.k1) then
                               ent=k1
                               k1=k2
                               k2=ent
                     endif
                     call ASSEMBLAGE(bb,ib,jb,k1,k2,som1)
            else
            
            if (ELEMENTBIS(ele).and.ELEMENTBIS(l)) then
                     if (k2.gt.k1) then
                               ent=k1
                               k1=k2
                               k2=ent
                     endif
                     call ASSEMBLAGE(aa,ia,ja,k1,k2,som1)
            else

            log1=.not.ELEMENTBIS(l)
        if (ELEMENTBIS(ele).and.log1) then
                     call ASSEMBLAGEbis(cc,ic,jc,k2,k1,som1)
                else

          if (ELEMENTBIS(l)) then 
                     k1=INDICE(l)
                     k2=INDICE(ele)
                     call ASSEMBLAGEbis(cc,ic,jc,k2,k1,som1)
           endif
          endif
         endif
        endif
       enddo
c
c calcul de la contribution du second membre et assemblage
c
       if (ELEMENTBIS(ele)) then
           k1=INDICE(ele)
           SMinter(k1)=SMinter(k1)+INTEGRP(det,N1,N2,N3,nbr1)
       endif
      enddo

      RETURN
      end





C Fichier tri.f
c
c*******************************************************************
      SUBROUTINE Tri(criter,record,n)
c*******************************************************************
c
C   Il s'agit de ranger les elements du tableau criter
C   par ordre croissant d"indices et de modifier 
C   simultanement les elements du tableau record.
C
c   criter,record: tableaux
c   n: dimension des tableaux criter et record


      implicit none
      integer n
      INTEGER crit,criter(n),l,r,j,i  
      real*8  record(n),rec

      IF(n.le.1) RETURN
      l=n/2+1
      r=n
2     IF(l.gt.1)THEN 
        l=l-1
        rec=record(l)
        crit=criter(l)
      ELSE
        rec=record(r)
        crit=criter(r)
        record(r)=record(1)
        criter(r)=criter(1)
        r=r-1
        IF(r.eq.1) THEN
          record(1)=rec
          criter(1)=crit
          RETURN
        ENDIF
      ENDIF
3     j=l
4     i=j
      j=2*j
      IF(j-r.lt.0)THEN
        GOTO 5
      ELSEIF(j.eq.r)THEN
        GOTO 6
      ELSE
        GOTO 8
      ENDIF
5     IF(criter(j).lt.criter(j+1))j=j+1
6     IF(crit.ge.criter(j))GOTO 8
      record(i)=record(j)
      criter(i)=criter(j)
      GOTO 4
8     record(i)=rec
      criter(i)=crit
      GOTO 2

c      RETURN
      END
