/*
 * SimulatorGUI class: JHomogenizer
 *
 */

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import java.io.*;
import java.util.Observer;
import java.util.Observable;
import java.util.StringTokenizer;
import java.util.Vector;

/*
 * The basic frame class
 */
public class JHomogenizer extends Frame implements Observer, ActionListener,
   ItemListener, Runnable {

  // constructor for the class
  public JHomogenizer() {
    super("Homogenization GUI");

    // set the application frame
    applicationFrame = (Frame) this;
    isApplicationFrameSet = true;

    // get the os information and toolkit
    runtime = Runtime.getRuntime();
    toolkit = Toolkit.getDefaultToolkit();

    // get the OS file separator
    fileSeparator = File.separator;

    // allow the object to get events without a listener if necessary
    enableEvents(AWTEvent.WINDOW_EVENT_MASK);

    // start by getting the screen size and resolution - these are used to set
    // the size of the pieces of the GUI.
    screensize = toolkit.getScreenSize();
    screensize.width = 98 * screensize.width / 100;
    screensize.height = 98 * screensize.height / 100;
    screenresolution = toolkit.getScreenResolution();
    int sw = screensize.width / 2;
    if(sw < 500) sw = 500;
    int sh = screensize.height;
    if(sh < 800) sh = 800;

    // build the interface for user input and output
    buildPanel();

    // set up the frame to show the progress of the homogenization
    progressFrame = new Frame("Graphical Progress Panel");
      progressPanel = new Panel();
      progressPanel.setLayout(new BorderLayout());
      progressPanel.setBackground(Color.blue);
      progressPanel.setSize(490,530);
      progressDismissButton = new Button("Dismiss Progress Panel");
      progressDismissButton.addActionListener(this);
    progressFrame.add(progressPanel);
    progressFrame.setSize(485,540);

    // build a map object to contain the data for the present case
    mapObjectVector = new Vector();
    try {
      currentMapObject = new MapObject();
      workingMapObject = currentMapObject;
      getWorkingMapData();
    } catch(BadArraySizeException bse) {
    }

  }


  //
  ///////////////////////////////////////////////////////////////////////////
  //                                                                       //
  //   I. Graphical User Interface Methods                                 //
  //                                                                       //
  //     - buildPanel()                                                    //
  //     - toggleMapEditor()                                               //
  //     - toggleHomogenizationOutput()                                    //
  //     - printMessage()                                                  //
  //     - askQuestion()                                                   //
  //     - actionPerformed()                                               //
  //     - itemStateChanged()                                              //
  //     - getSelectedLevel0MapObject()                                    //
  //     - getSelectedLevel1MapObject()                                    //
  //     - getSelectedLevel2MapObject()                                    //
  //     - workingWorkingWorking()                                         //
  //                                                                       //
  ///////////////////////////////////////////////////////////////////////////
  //

  public void buildPanel() {

    homogenizationPanel = new Panel();
    homogenizationPanel.setLayout(new BorderLayout());
    homogenizationPanel.setBackground(panelColor);

      titleLabel = new Label("Interactive Homogenization Tool", Label.CENTER);
      titleLabel.setBackground(panelColor);

      displayPanel = new Panel();
      displayPanel.setLayout(new BorderLayout());
      displayPanel.setBackground(panelColor);

        inputPanel = new Panel();
        inputPanel.setLayout(new BorderLayout());
        inputPanel.setBackground(panelColor);

          inputLabel = new Label("Homogenization Input", Label.LEFT);
          inputLabel.setBackground(labelColor);

          inputDisplayPanel = new Panel();
          inputDisplayPanel.setLayout(new GridLayout(1,0));

            methodInputPanel = new Panel();
            methodInputPanel.setLayout(new BorderLayout());
            methodInputPanel.setBackground(panelColor);
              methodLabel = new Label("Homogenization Method List", Label.LEFT);
              methodLabel.setBackground(labelColor);
              methodList = new List(5);
              methodList.add(setmethods);
              if(methods != null)
                 for(int i=0; i<methods.length; i++) methodList.add(methods[i]);

            mapObjectPanel = new Panel();
            mapObjectPanel.setLayout(new BorderLayout());
              mapObjectLabel = new Label("Working Map Lists");
              mapObjectLabel.setBackground(labelColor);
              mapObjectListPanel = new Panel();
              mapObjectListPanel.setLayout(new GridLayout(0,1));
                currentMapObjectPanel = new Panel();
                currentMapObjectPanel.setLayout(new BorderLayout());
                  currentMapObjectLabel = new Label("Original Maps",Label.LEFT);
                  currentMapObjectLabel.setBackground(labelColor);
                  level0MapObjectList = new List(5);
                  level0MapObjectList.addItemListener(this);
                level1MapObjectPanel = new Panel();
                level1MapObjectPanel.setLayout(new BorderLayout());
                  level1MapObjectLabel = new Label("Level 1 Maps", Label.LEFT);
                  level1MapObjectLabel.setBackground(labelColor);
                  level1MapObjectList = new List(5);
                  level1MapObjectList.addItemListener(this);
                level2MapObjectPanel = new Panel();
                level2MapObjectPanel.setLayout(new BorderLayout());
                  level2MapObjectLabel = new Label("Level 2 Maps", Label.LEFT);
                  level2MapObjectLabel.setBackground(labelColor);
                  level2MapObjectList = new List(5);
                  level2MapObjectList.addItemListener(this);

          homogenizationInputPanel = new Panel();
          homogenizationInputPanel.setLayout(new BorderLayout());
          homogenizationInputPanel.setBackground(panelColor);

            homogenizationInputLabel = new Label("Homogenization Input Fields",
                                                 Label.LEFT);
            homogenizationInputLabel.setBackground(labelColor);

            hiPanel = new Panel();
            hiPanel.setLayout(new GridLayout(0,1));
            hiPanel.setBackground(panelColor);

              hetInputPanel = new Panel();
              hetInputPanel.setLayout(new GridLayout(3,3));
              hetInputPanel.setBackground(panelColor);
                hetGridLabel = new Label("Heterogeneous Grid Input",
                                         Label.CENTER);
                hetGridLabel.setBackground(labelColor);
                xhetGridLabel = new Label("X Direction", Label.CENTER);
                xhetGridLabel.setBackground(labelColor);
                xhetGridTextField = new TextField("0", 10);
                  xhetGridTextField.setBackground(backGroundColor);
                  xhetGridTextField.setForeground(foreGroundColor);
                xhetGridTextField.setEditable(false);
                nxdat = 0;
                yhetGridLabel = new Label("Y Direction", Label.CENTER);
                yhetGridLabel.setBackground(labelColor);
                yhetGridTextField = new TextField("0", 10);
                  yhetGridTextField.setBackground(backGroundColor);
                  yhetGridTextField.setForeground(foreGroundColor);
                yhetGridTextField.setEditable(false);
                nydat = 0;
                zhetGridLabel = new Label("Z Direction", Label.CENTER);
                zhetGridLabel.setBackground(labelColor);
                zhetGridTextField = new TextField("0", 10);
                  zhetGridTextField.setBackground(backGroundColor);
                  zhetGridTextField.setForeground(foreGroundColor);
                zhetGridTextField.setEditable(false);
                nydat = 0;
                nzdat = 0;

              homInputPanel = new Panel();
              homInputPanel.setLayout(new GridLayout(3,3));
              homInputPanel.setBackground(panelColor);
                homGridLabel = new Label("Homogeneous Grid Input",
                                         Label.CENTER);
                homGridLabel.setBackground(labelColor);
                xhomGridLabel = new Label("X Direction", Label.CENTER);
                xhomGridLabel.setBackground(labelColor);
                xhomGridTextField = new TextField("1", 10);
                  xhomGridTextField.setBackground(backGroundColor);
                  xhomGridTextField.setForeground(foreGroundColor);
                nxhom = 1;
                yhomGridLabel = new Label("Y Direction", Label.CENTER);
                yhomGridLabel.setBackground(labelColor);
                yhomGridTextField = new TextField("1", 10);
                  yhomGridTextField.setBackground(backGroundColor);
                  yhomGridTextField.setForeground(foreGroundColor);
                nyhom = 1;
                zhomGridLabel = new Label("Z Direction", Label.CENTER);
                zhomGridLabel.setBackground(labelColor);
                zhomGridTextField = new TextField("1", 10);
                  zhomGridTextField.setBackground(backGroundColor);
                  zhomGridTextField.setForeground(foreGroundColor);
                nzhom = 1;

              compRefInputPanel = new Panel();
              compRefInputPanel.setBackground(panelColor);
              compRefInputPanel.setLayout(new GridLayout(3,3));
                compRefGridLabel = new Label("Compputational Refinement Input",
                                             Label.CENTER);
                compRefGridLabel.setBackground(labelColor);
                xcompRefGridLabel = new Label("X Direction", Label.CENTER);
                xcompRefGridLabel.setBackground(labelColor);
                xcompRefGridTextField = new TextField("1", 10);
                  xcompRefGridTextField.setBackground(backGroundColor);
                  xcompRefGridTextField.setForeground(foreGroundColor);
                nxref = 1;
                ycompRefGridLabel = new Label("Y Direction", Label.CENTER);
                ycompRefGridLabel.setBackground(labelColor);
                ycompRefGridTextField = new TextField("1", 10);
                  ycompRefGridTextField.setBackground(backGroundColor);
                  ycompRefGridTextField.setForeground(foreGroundColor);
                nyref = 1;
                zcompRefGridLabel = new Label("Z Direction", Label.CENTER);
                zcompRefGridLabel.setBackground(labelColor);
                zcompRefGridTextField = new TextField("1", 10);
                  zcompRefGridTextField.setBackground(backGroundColor);
                  zcompRefGridTextField.setForeground(foreGroundColor);
                nzref = 1;

        outputPanel = new Panel();
        outputPanel.setLayout(new BorderLayout());
        outputPanel.setBackground(panelColor);
          outputLabel = new Label("Homogenization Text Output");
          outputLabel.setBackground(labelColor);
          outputTextArea = new TextArea("Homogenization Output Appears Here",
                                        20, 80,
                                        TextArea.SCROLLBARS_VERTICAL_ONLY);
          outputTextArea.setBackground(backGroundColor);
          outputTextArea.setForeground(foreGroundColor);
          textOutputShowing = true;
          graphicsOutputArea = new ScrollPane();
            flowImageCanvas = new ImageCanvas(imageWidth, imageHeight);

      // put in the control buttons
      bottomPanel = new Panel();
      bottomPanel.setLayout(new BorderLayout());
        centerbottomPanel = new Panel();
        centerbottomPanel.setLayout(new GridLayout(0,1));
          datacontrols = new Panel();
          datacontrols.setLayout(new GridLayout(1,0));
          datacontrols.setBackground(panelColor);
            mapEditorButton = new Button("Show MapEditor");
            mapEditorButton.addActionListener(this);
            getAllDataButton = new Button("Get All Map Data");
            getAllDataButton.addActionListener(this);
            getSubregionButton = new Button("Get Map Subregion");
            getSubregionButton.addActionListener(this);
            displayValuesButton = new Button("Display Values");
            displayValuesButton.addActionListener(this);
            homogenizeButton = new Button("Homogenize!");
            homogenizeButton.addActionListener(this);
            batchHomogenizeButton = new Button("Batch Homog.");
            batchHomogenizeButton.addActionListener(this);
          simulationPanel = new Panel();
          simulationPanel.setLayout(new GridLayout(1,0));
            esbPanel = new Panel();
            esbPanel.setLayout(new GridLayout(1,0));
              ellipticSolverButton = new Button("Elliptic Solver");
              ellipticSolverButton.addActionListener(this);
              ellipticBoundaryConditionsButton= new Button("BC's");
              ellipticBoundaryConditionsButton.addActionListener(this);
            esbPanel.add(ellipticSolverButton);
            esbPanel.add(ellipticBoundaryConditionsButton);
            fsbPanel = new Panel();
            fsbPanel.setLayout(new GridLayout(1,0));
              flowSolverButton = new Button("Start Flow Solver");
              flowSolverButton.addActionListener(this);
                flowSolverStepButton = new Button("Next Step");
                flowSolverStepButton.addActionListener(this);
                flowSolverStepTextField = new TextField("10",5);
                flowSolverQuitButton = new Button("Stop");
                flowSolverQuitButton.addActionListener(this);
            fsbPanel.add(flowSolverButton);
            seibPanel = new Panel();
            seibPanel.setLayout(new GridLayout(1,0));
              showEllipticImageButton = new Button("Show Elliptic Image");
              showEllipticImageButton.addActionListener(this);
            seibPanel.add(showEllipticImageButton);
            eibPanel = new Panel();
            eibPanel.setLayout(new GridLayout(1,0));
              ellipticImageButton = new Button("Create Elliptic Image");
              ellipticImageButton.addActionListener(this);
            eibPanel.add(ellipticImageButton);
            fibPanel = new Panel();
            fibPanel.setLayout(new GridLayout(1,0));
              fluxImageButton = new Button("Create Elliptic Flux Image");
              fluxImageButton.addActionListener(this);
            fibPanel.add(fluxImageButton);
          statusPanel = new Panel();
          statusPanel.setLayout(new BorderLayout());
            statusLineLabel = new Label("Status:");
            statusLine = new TextField(10);
              statusLine.setBackground(backGroundColor);
              statusLine.setForeground(foreGroundColor);
          controls = new Panel();
          controls.setLayout(new GridLayout(1,0));
          controls.setBackground(panelColor);
            loadMapButton = new Button("Load Map(s)");
            loadMapButton.addActionListener(this);
            importMapButton = new Button("Import Map");
            importMapButton.addActionListener(this);
            printOutputAreaButton = new Button("Print Output Area");
            printOutputAreaButton.addActionListener(this);
            saveMapButton = new Button("Save Map(s)");
            saveMapButton.addActionListener(this);
            exportMapButton = new Button("Export Map");
            exportMapButton.addActionListener(this);
            exitButton = new Button("Exit");
            exitButton.addActionListener(this);
        rightbottomPanel = new Panel();
        rightbottomPanel.setLayout(new BorderLayout());
          graphicsComparisonLabel = new Label("Maps to Compare:");
          graphicsComparisonList = new List(3, false);
          graphicsComparisonList.addItemListener(this);
          graphicsComparisonButton = new Button("Show Comparison");
          graphicsComparisonButton.addActionListener(this);
 
    // slap everything together
    add(homogenizationPanel);
      homogenizationPanel.add("North", titleLabel);
      homogenizationPanel.add("Center", displayPanel);
        displayPanel.add("North", inputPanel);
          inputPanel.add("North", inputLabel);
          inputPanel.add("Center", inputDisplayPanel);

          inputDisplayPanel.add(methodInputPanel);
            methodInputPanel.add("North", methodLabel);
            methodInputPanel.add("Center", methodList);
          inputDisplayPanel.add(mapObjectPanel);
            mapObjectPanel.add("North", mapObjectLabel);
            mapObjectPanel.add("Center", mapObjectListPanel);
              mapObjectListPanel.add(currentMapObjectPanel);
                currentMapObjectPanel.add("North", currentMapObjectLabel);
                currentMapObjectPanel.add("Center", level0MapObjectList);
              mapObjectListPanel.add(level1MapObjectPanel);
                level1MapObjectPanel.add("North", level1MapObjectLabel);
                level1MapObjectPanel.add("Center", level1MapObjectList);
              mapObjectListPanel.add(level2MapObjectPanel);
                level2MapObjectPanel.add("North", level2MapObjectLabel);
                level2MapObjectPanel.add("Center", level2MapObjectList);
          inputDisplayPanel.add(homogenizationInputPanel);
            homogenizationInputPanel.add("North", homogenizationInputLabel);
            homogenizationInputPanel.add("Center", hiPanel);
              hiPanel.add(hetInputPanel);
                hetInputPanel.add(new Label("Het. Input", Label.LEFT));
                hetInputPanel.add(new Label(" "));
                hetInputPanel.add(new Label(" "));
                hetInputPanel.add(xhetGridLabel);
                hetInputPanel.add(yhetGridLabel);
                hetInputPanel.add(zhetGridLabel);
                hetInputPanel.add(xhetGridTextField);
                hetInputPanel.add(yhetGridTextField);
                hetInputPanel.add(zhetGridTextField);
              hiPanel.add(homInputPanel);
                homInputPanel.add(new Label("Homog. Input", Label.LEFT));
                homInputPanel.add(new Label(" "));
                homInputPanel.add(new Label(" "));
                homInputPanel.add(xhomGridLabel);
                homInputPanel.add(yhomGridLabel);
                homInputPanel.add(zhomGridLabel);
                homInputPanel.add(xhomGridTextField);
                homInputPanel.add(yhomGridTextField);
                homInputPanel.add(zhomGridTextField);
              hiPanel.add(compRefInputPanel);
               compRefInputPanel.add(new Label("Comp. Ref. Input", Label.LEFT));
                compRefInputPanel.add(new Label(" "));
                compRefInputPanel.add(new Label(" "));
                compRefInputPanel.add(xcompRefGridLabel);
                compRefInputPanel.add(ycompRefGridLabel);
                compRefInputPanel.add(zcompRefGridLabel);
                compRefInputPanel.add(xcompRefGridTextField);
                compRefInputPanel.add(ycompRefGridTextField);
                compRefInputPanel.add(zcompRefGridTextField);
        displayPanel.add("Center", outputPanel);
          outputPanel.add("North", outputLabel);
          outputPanel.add("Center", outputTextArea);
            graphicsOutputArea.add(flowImageCanvas);
      homogenizationPanel.add("South", bottomPanel);
        bottomPanel.add("Center", centerbottomPanel);
        bottomPanel.add("East", rightbottomPanel);
          centerbottomPanel.add(datacontrols);
            datacontrols.add(mapEditorButton);
            datacontrols.add(getAllDataButton);
            datacontrols.add(getSubregionButton);
            datacontrols.add(displayValuesButton);
            datacontrols.add(homogenizeButton);
            datacontrols.add(batchHomogenizeButton);
          centerbottomPanel.add(simulationPanel);
            simulationPanel.add(esbPanel);
            simulationPanel.add(fsbPanel);
            simulationPanel.add(seibPanel);
            simulationPanel.add(eibPanel);
            simulationPanel.add(fibPanel);
          centerbottomPanel.add(statusPanel);
            statusPanel.add("West", statusLineLabel);
            statusPanel.add("Center", statusLine);
          centerbottomPanel.add(controls);
            controls.add(loadMapButton);
            controls.add(importMapButton);
            controls.add(printOutputAreaButton);
            controls.add(saveMapButton);
            controls.add(exportMapButton);
            controls.add(exitButton);
          rightbottomPanel.add("North", graphicsComparisonLabel);
          rightbottomPanel.add("Center", graphicsComparisonList);
          rightbottomPanel.add("South", graphicsComparisonButton);

  }

  public void buildMapLists() {

    // make sure everything is defined
    if(mapObjectVector == null) return;
    int n0 = mapObjectVector.size();
    if(n0 <= 0) return;
    
    // do the level 0 maps associated with the current map object
    if(level0MapObjectList == null) return;
    if(level0MapObjectList.getItemCount()>0) level0MapObjectList.removeAll();
    for(int i=0; i<n0; i++) {
      MapObject mo = (MapObject) mapObjectVector.get(i);
      level0MapObjectList.add(mo.getMapName());
    }
    if(currentMapIndex > -1) level0MapObjectList.select(currentMapIndex);

    // do the level 1 maps associated with the current map object
    if(level1MapObjectList == null) return;
    if(level1MapObjectList.getItemCount()>0) level1MapObjectList.removeAll();
    // look for the level 1 maps from the current main map
    int n1 = 0;
    if(currentMapObject != null)
                   n1 = currentMapObject.getNumberOfAssociatedMapObjects();
    for(int i=0; i<n1; i++) {
      MapObject amo = currentMapObject.getAssociatedMap(i);
      level1MapObjectList.add(amo.getMapName());
    }
    if(currentLevel1MapIndex > -1) {
      String s = level1MapObjectList.getSelectedItem();
      if(!testForEllipticFilename(s))
                     level1MapObjectList.select(currentLevel1MapIndex);
    }

    // do the level 1 maps associated with the current map object
    if(level2MapObjectList == null) return;
    if(level2MapObjectList.getItemCount()>0) level2MapObjectList.removeAll();
    // look for the level 2 maps from the current main map
    int n2 = 0;
    if(currentLevel2MapObject != null)
               n2 = currentLevel2MapObject.getNumberOfAssociatedMapObjects();
    for(int i=0; i<n2; i++) {
      MapObject amo = currentLevel1MapObject.getAssociatedMap(i);
      level2MapObjectList.add(amo.getMapName());
    }
    if(currentLevel2MapIndex > -1) {
      String s = level2MapObjectList.getSelectedItem();
      if(!testForEllipticFilename(s))
                     level2MapObjectList.select(currentLevel2MapIndex);
    }

  }

  public boolean testForEllipticFilename(String name) {
    boolean test = false;
    if(name == null) return test;
    String s = name.trim();
    if(name.equals("ellipsol")) test = true;
    if(name.equals("ellipxflux")) test = true;
    if(name.equals("ellipyflux")) test = true;
    if(name.equals("ellipzflux")) test = true;
    return test;
  }

  public void toggleMapEditor() {
    if(workingMapObject != null) {
      String mn = workingMapObject.getMapName();
      if(testForEllipticFilename(mn)) {
        printMessage("Current choice is an elliptic solution map - not valid");
        return;
      }
    }
    if(mapEditorShowing) {
      // reset the action button label
      mapEditorButton.setLabel("Show MapEditor");
      // remove the current values popuup associated with the mapeditor
      mapEditor.removeCurrentValuesFrame();
      // hide the mapEditor
      mapEditorFrame.setVisible(false);
      // kill the mapEditor
      mapEditor = null;
      System.gc();
      // let the code know that the mapEditor is gone
      mapEditorShowing = false;
    } else {
      // set up some dimensions
      int sw = screensize.width / 2;
      if(sw < 500) sw = 500;
      int sh = screensize.height;
      if(sh < 800) sh = 800;
      // build the MapEditor to look at things
      mapEditorFrame = new Frame("Homogenization MapEditor");
      mapEditorFrame.setVisible(false);
        controlMask = new boolean [4];
        controlMask[0] = true;
        controlMask[1] = true;
        controlMask[2] = false;
        controlMask[3] = false;
        mapEditor = new MapEditor(mapEditorFrame, sw, sh, 100, 0, controlMask);
        mapEditor.disableExitButton();
      mapEditorFrame.add(mapEditor);
      mapEditorFrame.setSize(mapEditor.getPreferredSize());
      // reset the action button label
      mapEditorButton.setLabel("Kill MapEditor");
      // show the frame at the location
      mapEditorFrame.setLocation(screensize.width - sw, 0);
      mapEditorFrame.show(true);
      mapEditor.popupCurrentValuesFrame();
      if(data != null) {
        mapEditor.setData(data, nxdat, nydat, nzdat);
      } else {
      }
      // let the code know that the mapEditor is up and running
      mapEditorShowing = true;
    }
  }

  private void printMessage(String s) {
    if(isApplicationFrameSet) {
      if(warningDialog == null) {
        warningDialog = new WarningDialog(applicationFrame,
                                          "Error Message Dialog",
                                          s, dialogColor);
      } else {
        warningDialog.setMessage(s);
      }
    }
  }

  private boolean askQuestion(String s) {
    boolean test = false;
    if(isApplicationFrameSet) {
      if(okpd == null) {
        okpd = new OKPromptDialog(applicationFrame,
                                  "User Prompt", s, dialogColor);
      } else {
        okpd.setMessage(s);
      }
      while(okpd.isShowing()) {}
      test = okpd.getState();
      okpd.dispose();
    }
    return test;
  }

  public void actionPerformed(ActionEvent e) {
    Object source = e.getSource();

    if(source.equals(loadMapButton)) loadMap();
    if(source.equals(importMapButton)) importMap();
    if(source.equals(printOutputAreaButton)) printOutputArea();
    if(source.equals(mapEditorButton)) toggleMapEditor();
    if(source.equals(getAllDataButton)) getAllData();
    if(source.equals(getSubregionButton)) getSubregionData();
    if(source.equals(displayValuesButton))
               displayValues(data, nxdat, nydat, nzdat, 0);
    if(source.equals(homogenizeButton))
               if(homogenize(true,true,true)) keepHomogenizationResult();
    if(source.equals(batchHomogenizeButton)) batchHomogenize(true,true,true);

    if(source.equals(ellipticSolverButton))
            if(ellipticSolution(true, true, true)) keepEllipticSolutionResult();
    if(source.equals(ellipticBoundaryConditionsButton))
           setEllipticProblemData();
    if(source.equals(showEllipticImageButton)) {
      if(testForEllipticSolution()) {
        if(!ellipticSolutionShowing) {
          showEllipticSolutionImage();
          ellipticSolutionShowing = true;
          showEllipticImageButton.setLabel("Kill Elliptic Image");
        } else {
          ellipticSolutionFrame.setVisible(false);
          ellipticSolutionFrame.dispose();
          ellipticSolutionShowing = false;
          showEllipticImageButton.setLabel("Show Elliptic Image");
          flowSolutionQuit();
        }
      }
    }
    if(source.equals(ellipticImageButton)) {
      if(testForEllipticSolution()) createEllipticSolutionImage();
    }
    if(source.equals(fluxImageButton)) {
      if(testForEllipticSolution()) createEllipticFluxImage();
    }

    if(source.equals(flowSolverButton)) {
      if(testForEllipticSolution()) {
        if(testForEllipticSolution()) flowSolution();
      }
    }
    if(source.equals(flowSolverStepButton)) flowSolutionStep();
    if(source.equals(flowSolverQuitButton)) flowSolutionQuit();

    if(source.equals(saveMapButton)) saveMap();
    if(source.equals(exportMapButton)) exportMap();
    if(source.equals(exitButton)) exit();

    if(source.equals(progressDismissButton)) dismissProgressFrame();

    if(source.equals(graphicsComparisonButton)) doComparison();

  }

  public void itemStateChanged(ItemEvent e) {
    Object source = e.getSource();
    if(source.equals(level0MapObjectList)) getSelectedLevel0MapObject();
    if(source.equals(level1MapObjectList)) getSelectedLevel1MapObject();
    if(source.equals(level2MapObjectList)) getSelectedLevel2MapObject();
    if(source.equals(graphicsComparisonList)) {
      int index = graphicsComparisonList.getSelectedIndex();
      graphicsComparisonList.remove(index);
      comparisonMapVector.removeElementAt(index);
    }
  }

  public void update(Observable obs, Object obj) {
  }

  public void getSelectedLevel0MapObject() {
    boolean found = false;

    // make sure that the stuff exists and try to get the appropriate map object
    if(level0MapObjectList == null) return;
    String s = level0MapObjectList.getSelectedItem();
    if(s == null) return;
    if(testForEllipticFilename(s)) return;

    if(mapObjectVector == null) return;

    int n = mapObjectVector.size();
    for(int i=0; i<n; i++) {
      MapObject mo = (MapObject) mapObjectVector.get(i);
      String name = mo.getMapName();
      if(s.equals(name)) {
          currentMapName = name;
          currentMapObject = mo;
          currentMapIndex = i;
          currentLevel1MapIndex = -1;
          currentLevel2MapIndex = -1;
          workingMapObject = currentMapObject;
          workingMapObjectLevel = 0;
          workingMapObjectIndex = i;
          found = true;
      }
    }

    // if the map objects exists set up the rest of the information
    if(found) {
      workingWorkingWorking(true,"Changing To Level 0 Map: "
                                 + currentMapName);
      // get the data and display it
      getWorkingMapData();
      buildMapLists();
      workingWorkingWorking(false,"New Level 0 Map Now Current: "
                                 + currentMapName);
    } else {
      printMessage("Selected Map Not Found");
    }

  }

  public void getSelectedLevel1MapObject() {
    boolean found = false;

    // make sure that the stuff exists and try to get the appropriate map object
    if(level1MapObjectList == null) return;
    String s = level1MapObjectList.getSelectedItem();
    if(s == null) return;
    int index = level1MapObjectList.getSelectedIndex();
    if(testForEllipticFilename(s)) {
      level1MapObjectList.deselect(index);
      return;
    }

    if(currentMapObject == null) return;

    Vector v1 = currentMapObject.getAssociatedMaps();
    if(v1 == null) return;

    int n = v1.size();
    for(int i=0; i<n; i++) {
      MapObject mo1 = (MapObject) v1.get(i);
      String name = mo1.getMapName();
      if(s.equals(name)) {
        currentLevel1MapName = name;
        currentLevel1MapObject = mo1;
        currentLevel1MapIndex = i;
        currentLevel2MapIndex = -1;
        workingMapObject = currentLevel1MapObject;
        workingMapObjectLevel = 1;
        workingMapObjectIndex = i;
        found = true;
      }
    }

    if(found) {
      workingWorkingWorking(true,"Changing To Level 1 Map: "
                                 + currentLevel1MapName);
      // get the data and display it
      getWorkingMapData();
      buildMapLists();
      workingWorkingWorking(false,"New Level 1 Map Now Current: "
                                 + currentLevel1MapName);
    } else {
      printMessage("Selected Level 1 Map Not Found");
    }

  }

  public void getSelectedLevel2MapObject() {
    boolean found = false;

    // make sure that the stuff exists and try to get the appropriate map object
    if(level2MapObjectList == null) return;
    String s = level2MapObjectList.getSelectedItem();
    if(s == null) return;
    int index = level2MapObjectList.getSelectedIndex();
    if(testForEllipticFilename(s)) {
      level2MapObjectList.deselect(index);
      return;
    }

    if(currentLevel1MapObject == null) return;

    Vector v1 = currentLevel1MapObject.getAssociatedMaps();
    if(v1 == null) return;

    int n = v1.size();
    for(int i=0; i<n; i++) {
      MapObject mo1 = (MapObject) v1.get(i);
      String name = mo1.getMapName();
      if(s.equals(name)) {
        currentLevel2MapName = name;
        currentLevel2MapObject = mo1;
        currentLevel2MapIndex = i;
        workingMapObject = currentLevel1MapObject;
        workingMapObjectLevel = 2;
        workingMapObjectIndex = i;
        found = true;
      }
    }

    if(found) {
      workingWorkingWorking(true,"Changing To Level 2 Map: "
                                 + currentLevel2MapName);
      // get the data and display it
      getWorkingMapData();
      buildMapLists();
      workingWorkingWorking(false,"New Level 2 Map Now Current: "
                                 + currentLevel2MapName);
    } else {
      printMessage("Selected Level 2 Map Not Found");
    }

  }

  private void workingWorkingWorking(boolean b, String s) {
    working = b;
    workingString = s;
    if(working) {
      if(workingThread == null) {
        workingThread = new Thread(this);
        workingThread.start();
      }
    }
  }

  public void run() {
    int cnt = 0;
    if(workingString == null) workingString = "working ";
    String os = workingString;
    statusLine.setText(os);
    while(working) {
      cnt++;
      os = os + "+";
      if(cnt == 40) {
        os = workingString;
        cnt = 0;
      }
      statusLine.setText(os);
      try {
        Thread.sleep(500);
      } catch(InterruptedException ie) {
      }
    }
    workingThread = null;
    statusLine.setText(workingString);
    // this resets the string to a reasonable value
    workingString = "working";
  }

  //
  ///////////////////////////////////////////////////////////////////////////
  //                                                                       //
  //  II. Variable Access Methods                                          //
  //                                                                       //
  //     - getInput()                                                      //
  //     - testInput()                                                     //
  //     - getAllData()                                                    //
  //     - getSubregionData()                                              //
  //     - getWorkingMapData()                                             //
  //                                                                       //
  ///////////////////////////////////////////////////////////////////////////
  //

  public void getInput() {
    String s;
    // get the method index
    homogenizationMethod = methodList.getSelectedIndex();
    // get the grid sizes for the heterogeneous field
    s = xhetGridTextField.getText();
    try{Integer intval = Integer.valueOf(s);
      nxdat = intval.intValue();
    } catch (NumberFormatException nfe) {
      printMessage("Number Format Exception for nxdat");
    }
    s = yhetGridTextField.getText();
    try{Integer intval = Integer.valueOf(s);
      nydat = intval.intValue();
    } catch (NumberFormatException nfe) {
      printMessage("Number Format Exception for nydat");
    }
    s = zhetGridTextField.getText();
    try{Integer intval = Integer.valueOf(s);
      nzdat = intval.intValue();
    } catch (NumberFormatException nfe) {
      printMessage("Number Format Exception for nzdat");
    }
    // get the grid sizes for the homogenized field
    s = xhomGridTextField.getText();
    try{Integer intval = Integer.valueOf(s);
      nxhom = intval.intValue();
    } catch (NumberFormatException nfe) {
      printMessage("Number Format Exception for nxhom");
    }
    s = yhomGridTextField.getText();
    try{Integer intval = Integer.valueOf(s);
      nyhom = intval.intValue();
    } catch (NumberFormatException nfe) {
      printMessage("Number Format Exception for nyhom");
    }
    s = zhomGridTextField.getText();
    try{Integer intval = Integer.valueOf(s);
      nzhom = intval.intValue();
    } catch (NumberFormatException nfe) {
      printMessage("Number Format Exception for nzhom");
    }
    // get the computational refinement
    s = xcompRefGridTextField.getText();
    try{Integer intval = Integer.valueOf(s);
      nxref = intval.intValue();
    } catch (NumberFormatException nfe) {
      printMessage("Number Format Exception for nxref");
    }
    s = ycompRefGridTextField.getText();
    try{Integer intval = Integer.valueOf(s);
      nyref = intval.intValue();
    } catch (NumberFormatException nfe) {
      printMessage("Number Format Exception for nyref");
    }
    s = zcompRefGridTextField.getText();
    try{Integer intval = Integer.valueOf(s);
      nzref = intval.intValue();
    } catch (NumberFormatException nfe) {
      printMessage("Number Format Exception for nzref");
    }
    // compute the total number of nodes
    nxnyhom = nxhom * nyhom;
    nnhom = nxnyhom * nzhom;
  }

  public boolean testInput() {
    int modval;
    // test to make sure that the number homogenized blocks divides the number
    // of heterogeneous blocks
    modval = nxdat / nxhom;
    if(modval*nxhom != nxdat) {
      printMessage("No. homog. blocks does not divide no. of het. blocks - x");
      return false;
    }
    modval = nydat / nyhom;
    if(modval*nyhom != nydat) {
      printMessage("No. homog. blocks does not divide no. of het. blocks - y");
      return false;
    }
    modval = nzdat / nzhom;
    if(modval*nzhom != nzdat) {
      printMessage("No. homog. blocks does not divide no. of het. blocks - z");
      return false;
    }
    return true;

  }

  public void getAllData() {
    Double [][] getData;
    // make sure the mapEditor exists
    if(mapEditor == null) {
      printMessage("MapEditor is not up and running:");
      return;
    }

    workingWorkingWorking(true, "Getting All Data");
    // get the data and the number of blocks in each direction
    getData = mapEditor.getMapValues();
    if(getData != null) {
      dataSetLoaded = true;
    } else {
      dataSetLoaded = false;
    }
    nxdat = mapEditor.getNumberOfBlocksX();
    nydat = mapEditor.getNumberOfBlocksY();
    nzdat = mapEditor.getNumberOfBlocksZ();
    nxnydat = nxdat * nydat;
    nndat = nxnydat * nzdat;

    // see if the map is the same as the current map
    if(mapObjectVector != null) {
      MapObject mo1;
      try {
        mo1 = new MapObject(currentMapName, getData, nxdat, nydat, nzdat);
      } catch(BadArraySizeException bse) {
        workingWorkingWorking(false, "Get All Data Failed");
        printMessage("BadArraySizeException Encountered");
        return;
      }
      int n = mapObjectVector.size();
      for(int i=0; i<n; i++) {
        MapObject mo2 = (MapObject) mapObjectVector.get(i);
        if(mo2.equals(mo1)) {
          workingWorkingWorking(false, "Get All Data Failed");
          printMessage("Map Already Exists in this Session");
          return;
        }
      }
    }

    // pop up a dialog to get a name for the data set
    MapNameInputDialog mnid = new MapNameInputDialog(this);
    mnid.show();
    String mapNameString;
    if(mnid.getState()) {
      mapNameString = mnid.getMapName();
      if(mapNameString == null) {
        workingWorkingWorking(false, "Get All Data Failed");
        printMessage("Name Input Error - Name Is Null");
        return;
      }
    } else {
      workingWorkingWorking(false, "Get All Data Failed");
      printMessage("Name Input Error - Name Input Aborted");
      return;
    }
    if(mapNameConflict(mapNameString)) {
      workingWorkingWorking(false, "Get All Data Failed");
      printMessage("Map File Already Loaded or Name is null");
      return;
    }
    if(mapNameConflict(mapNameString + ".map")) {
      workingWorkingWorking(false, "Get All Data Failed");
      printMessage("Map File Already Loaded or Name is null");
      return;
    }
    currentMapName = mapNameString;
    // build a map object for storage purposes and later use
    try {
      currentMapObject = new MapObject(currentMapName,
                                       getData, nxdat, nydat, nzdat);
      workingMapObject = currentMapObject;
      getWorkingMapData();
    } catch(BadArraySizeException bse) {
      workingWorkingWorking(false, "Get All Data Failed");
      printMessage("BadArraySizeException Encountered");
      return;
    }
    currentMapObject.setSaved(false);
    mapObjectVector.addElement(currentMapObject);
    currentMapIndex = mapObjectVector.size() - 1;
    currentLevel1MapIndex = -1;
    currentLevel2MapIndex = -1;
    // finally, select this as the current map in the list
    workingMapObjectLevel = 0;
    workingMapObjectIndex = mapObjectVector.size() - 1;
    buildMapLists();

    // build the arrays and set the tensor values
    kxx = new Double [nndat];
    kxy = new Double [nndat];
    kxz = new Double [nndat];
    kyx = new Double [nndat];
    kyy = new Double [nndat];
    kyz = new Double [nndat];
    kzx = new Double [nndat];
    kzy = new Double [nndat];
    kzz = new Double [nndat];
    for (int i=0; i<nndat; i++) {
       kxx[i] = getData[i][0]; 
       kxy[i] = getData[i][1];
       kxz[i] = getData[i][2];
       kyx[i] = getData[i][3];
       kyy[i] = getData[i][4]; 
       kyz[i] = getData[i][5];
       kzx[i] = getData[i][6];
       kzy[i] = getData[i][7];
       kzz[i] = getData[i][8]; 
    }
    // set the text fields in the gui
    xhetGridTextField.setText(new Integer(nxdat).toString());
    yhetGridTextField.setText(new Integer(nydat).toString());
    zhetGridTextField.setText(new Integer(nzdat).toString());
    // finally, reset the homogenization flag so that the application knows to
    // do this to the new data set just loaded.
    homogenizationDone = false;

    workingWorkingWorking(false, "Got All Data");

  }

  public void getSubregionData() {
    Double [][] getData;
    // make sure the mapEditor exists
    if(mapEditor == null) {
      printMessage("MapEditor is not up and running:");
      return;
    }

    workingWorkingWorking(true, "Getting Subregion Data");
    // get the data and the number of blocks in each direction
    getData = mapEditor.getSubregionValues();
    if(getData != null) {
      dataSetLoaded = true;
    } else {
      dataSetLoaded = false;
    }
    // test to see if the data set has anything in it - the subregion may not
    // have been created yet.
    if(getData == null) {
      workingWorkingWorking(true, "Get Subregion Data Failed");
      printMessage("No Current Subregion Available");
      return;
    }
    nxdat = mapEditor.getNumberOfSubregionBlocksX();
    nydat = mapEditor.getNumberOfSubregionBlocksY();
    nzdat = mapEditor.getNumberOfSubregionBlocksZ();
    nndat = nxdat * nydat * nzdat;

    // see if the map is the same as the current map
    if(mapObjectVector != null) {
      MapObject mo1;
      try {
        mo1 = new MapObject(currentMapName, getData, nxdat, nydat, nzdat);
      } catch(BadArraySizeException bse) {
        workingWorkingWorking(true, "Get Subregion Data Failed");
        printMessage("BadArraySizeException Encountered");
        return;
      }
      int n = mapObjectVector.size();
      for(int i=0; i<n; i++) {
        MapObject mo2 = (MapObject) mapObjectVector.get(i);
        if(mo2.equals(mo1)) {
          workingWorkingWorking(true, "Get Subregion Data Failed");
          printMessage("Map Already Exists in this Session");
          return;
        }
      }
    }

    // pop up a dialog to get a name for the data set
    MapNameInputDialog mnid = new MapNameInputDialog(this);
    mnid.show();
    String mapNameString;
    if(mnid.getState()) {
      mapNameString = mnid.getMapName();
      if(mapNameString == null) {
        workingWorkingWorking(true, "Get Subregion Data Failed");
        printMessage("Name Input Error - Name Is Null");
        return;
      }
    } else {
      workingWorkingWorking(true, "Get Subregion Data Failed");
      printMessage("Name Input Error - Name Input Aborted");
      return;
    }
    if(mapNameConflict(mapNameString)) {
      workingWorkingWorking(true, "Get Subregion Data Failed");
      printMessage("Map File Already Loaded or Name is null");
      return;
    }
    if(mapNameConflict(mapNameString + ".map")) {
      workingWorkingWorking(true, "Get Subregion Data Failed");
      printMessage("Map File Already Loaded or Name is null");
      return;
    }
    currentMapName = mapNameString;
    // build a map object for storage purposes and later use
    try {
      currentMapObject = new MapObject(currentMapName,
                                       getData, nxdat, nydat, nzdat);
      workingMapObject = currentMapObject;
      getWorkingMapData();
    } catch(BadArraySizeException bse) {
      workingWorkingWorking(true, "Get Subregion Data Failed");
      printMessage("BadArraySizeException Encountered");
      return;
    }
    currentMapObject.setSaved(false);
    mapObjectVector.addElement(currentMapObject);
    currentMapIndex = mapObjectVector.size() - 1;
    currentLevel1MapIndex = -1;
    currentLevel2MapIndex = -1;
    // finally, select this as the current map in the list
    workingMapObjectLevel = 0;
    workingMapObjectIndex = mapObjectVector.size() - 1;
    buildMapLists();

    // build the arrays and set the tensor values
    kxx = new Double [nndat];
    kxy = new Double [nndat];
    kxz = new Double [nndat];
    kyx = new Double [nndat];
    kyy = new Double [nndat];
    kyz = new Double [nndat];
    kzx = new Double [nndat];
    kzy = new Double [nndat];
    kzz = new Double [nndat];
    for (int i=0; i<nndat; i++) {
       kxx[i] = getData[i][0]; 
       kxy[i] = getData[i][1];
       kxz[i] = getData[i][2];
       kyx[i] = getData[i][3];
       kyy[i] = getData[i][4]; 
       kyz[i] = getData[i][5];
       kzx[i] = getData[i][6];
       kzy[i] = getData[i][7];
       kzz[i] = getData[i][8]; 
    }
    // set the text fields in the gui
    xhetGridTextField.setText(new Integer(nxdat).toString());
    yhetGridTextField.setText(new Integer(nydat).toString());
    zhetGridTextField.setText(new Integer(nzdat).toString());
    // finally, reset the homogenization flag so that the application knows to
    // do this to the new data set just loaded.
    homogenizationDone = false;

    workingWorkingWorking(false, "Got Subregion Data");

  }

  public boolean getWorkingMapData() {
    if(workingMapObject == null) return false;
    nxdat = workingMapObject.getNumberOfXBlocks();
    if(nxdat <= 0) return false;
    nydat = workingMapObject.getNumberOfYBlocks();
    if(nydat <= 0) return false;
    nzdat = workingMapObject.getNumberOfZBlocks();
    if(nzdat <= 0) return false;
    xhetGridTextField.setText(new Integer(nxdat).toString());
    yhetGridTextField.setText(new Integer(nydat).toString());
    zhetGridTextField.setText(new Integer(nzdat).toString());
    data = workingMapObject.getData();
    if(data != null) {
      dataSetLoaded = true;
    } else {
      dataSetLoaded = false;
    }
    return true;
  }

  //
  ///////////////////////////////////////////////////////////////////////////
  //                                                                       //
  // III. Variable Setting Methods                                         //
  //                                                                       //
  ///////////////////////////////////////////////////////////////////////////
  //

  //
  ///////////////////////////////////////////////////////////////////////////
  //                                                                       //
  //  IV. Variable Manipulation Methods                                    //
  //                                                                       //
  //     - homogenize()                                                    //
  //     - batchHomogenize()                                               //
  //     - setHomogenizationMask()                                         //
  //     - testHomogenizationMask()                                        //
  //                                                                       //
  ///////////////////////////////////////////////////////////////////////////
  //

  public boolean homogenize(boolean showOut, boolean showError,
                         boolean showStatus) {

    // try to get the data from the working map object
    if(!getWorkingMapData()) {
      dataSetLoaded = false;
      if(showStatus) workingWorkingWorking(false,
                                           "Homogenization Failed: No Data");
      if(showError)
           printMessage("Null Data: Get Data From the MapEditor or Load A Map");
      return false;
    } 

    if(showStatus) workingWorkingWorking(true, "Homogenization In Progress");

    int j2het, k2het;
    int j2dat, k2dat;
    int indhet, inddat;
    int index = 0;
    AverageObject [] averageObject = new AverageObject[maxThreads];
    boolean [] maskVector = new boolean[maxThreads];
    Double [] output;
    boolean homogenizeSucceeded = true;

    // get the input paramters from the text areas in the gui
    getInput();
    if(homogenizationMethod < 0) {
      if(showStatus) workingWorkingWorking(false,
                      "Homogenization Failed: No Homogenization Method Chosen");
      if(showError) printMessage("Choose a Homogenization Method");
      return false;
    }

    // test the input
    if(!testInput()) {
      if(showStatus) workingWorkingWorking(false,
                                           "Homogenization Failed: Bad Input");
      if(showError) printMessage("Something wrong in the homogenization input");
      return false;
    }

    // clear the output text area
    if(showOut) outputTextArea.setText("Homogenized Process Started.......\n");

    // initialize the progress frame
    //initializeProgressFrame();

    // compute some integers
    int nxhet = nxdat / nxhom; 
    int nyhet = nydat / nyhom; 
    int nzhet = nzdat / nzhom; 
    int nxnyhet = nxhet * nyhet;
    int nnhet = nxnyhet * nzhet;

    // set up the storage for the averaged values
    hdata = new Double[nnhom][9];
    doneVector = new boolean[nnhom];
    for(int i=0; i<nnhom; i++) { 
      doneVector[i] = false;
      for(int l=0; l<9; l++) hdata[i][l] = null;
    }

    // the homogenization mask
    homogenizationMask = new int [nnhom];
    setHomogenizationMask();

    // test the homogenization mask to make sure that appropriate methods have
    // been set
    if(!testHomogenizationMask()) {
      if(showStatus) workingWorkingWorking(false,
                      "Homogenization Failed: Homogenization Mask Wrong");
      return false;
    }

    // set the mask vector
    for(int i=0; i<maxThreads; i++) maskVector[i] = true;

    // set the flag to indicate that there are more cells to homogenize
    moreHomogenizationCells = true;

    // set the thread counter
    int numberOfThreads = 0;

    // loop over all the homogenized blocks
    for (int l=1; l<=nnhom; l++) {
      Double [][] d = new Double[nnhet][9];

      // get the length of the vector of threads
      // compute the next index
      int k = ( l - 1 ) / nxnyhom + 1;
      int j = ( l - ( k - 1 ) * nxnyhom - 1 ) / nxhom + 1;
      int i = l - ( j - 1 ) * nxhom - ( k - 1 ) * nxnyhom;

      // update the text area
      if(showOut) {
        String s = "\nStarting block: " + l
                 + "   i = " + i
                 + "   j = " + j
                 + "   k = " + k
                 + " .............";
        outputTextArea.append(s);
        outputTextArea.setCaretPosition(10000000);
      }

      int k2 = ( k - 1 ) * nxnydat;
      int j2 = ( j - 1 ) * nxdat;

      // loop over the set of heterogeneous cells in the homogenization block
      // and stuff the data into a temporary array
      for (int kk=1; kk<=nzhet; kk++) {
        k2het = ( kk - 1 ) * nxnyhet;
        k2dat = ( kk - 1 + ( k - 1 ) * nzhet ) * nxdat * nydat;
        for (int jj=1; jj<=nyhet; jj++) {
          j2het = ( jj - 1 ) * nxhet + k2het;
          j2dat = ( jj - 1 + ( j - 1 ) * nyhet ) * nxdat + k2dat;
          for (int ii=1; ii<=nxhet; ii++) {
             indhet = ii + j2het - 1;
             inddat = ii + ( i - 1 ) * nxhet + j2dat - 1;
             for(int ll=0; ll<9; ll++) d[indhet][ll] = data[inddat][ll];
          }
        }
      }

      // get the next available index in the average object array
      for(int ii=0;ii<maxThreads;ii++) {
        if(maskVector[ii]) {
          index = ii;
          maskVector[ii] = false;
          break;
        }
      }

//
// STEP II:
//
//    Add your method name to the following. You need to make sure that the
//    methods appear in the same order as the methods list in STEP I. It should
//    be pretty clear how to do this. Keep in mind that the 0 choice allows the
//    user to set the blocks one at a time using any of the methods listed
//    below.
//
//    Once you are through with this step the method should be available for use
//    in the interface including using the method in combination with other
//    methods.
//
      // build the next available
      try {
        averageObject[index] = null;
        // switch for the method
        //switch(homogenizationMask[index]) {
        switch(homogenizationMask[l-1]) {
          case 1:
            averageObject[index] =
                new ArithmeticAverage(d, nxhet, nyhet, nzhet, l-1);
            break;
          case 2:
            averageObject[index] =
                new HarmonicAverage(d, nxhet, nyhet, nzhet, l-1);
            break;
          case 3:
            averageObject[index] =
                new GeometricAverage(d, nxhet, nyhet, nzhet, l-1);
            break;
          case 4:
            averageObject[index] =
                new PeriodicWaveletAverage2D(d, nxhet, nyhet, nzhet, l-1);
            break;
          case 5:
            averageObject[index] =
                new THLBCodeAverage(d, nxhet, nyhet, nzhet, l-1);
            break;
          case 6:
            averageObject[index] =
                new HomCodeAverage(d, nxhet, nyhet, nzhet, l-1);
            break;
          case 7:
            averageObject[index] =
                new BVidalFracCodePermAverage(d, nxhet, nyhet, nzhet, l-1);
            break;
          case 8:
            averageObject[index] =
                new BVidalFracCodeDispAverage(d, nxhet, nyhet, nzhet, l-1);
            break;
          case 9:
            averageObject[index] =
                new AperiodicWaveletAverage2D(d, nxhet, nyhet, nzhet, l-1);
            break;
          default:
            if(showStatus) workingWorkingWorking(false,
                    "Homogenization Failed: Invalid Homogenization Method");
            if(showError)
                printMessage("Invalid homogenization method in block " + index);
            // since the homogenization is not done jump out of the routine
            return false;
        } 
        numberOfThreads = numberOfThreads + 1;
      } catch(BadArraySizeException base) {
        System.out.println("Bad Array Size Exception");
      }

      // update the text area with the homogenization type
      if(showOut) {
        if(homogenizationMask[index] > 0
             && homogenizationMask[index] < methods.length) {
          String s = "\n    Homogenization Method: "
                   + methods[homogenizationMask[index]-1]
                   + " .............";
          outputTextArea.append(s);
          outputTextArea.setCaretPosition(10000000);
        }
      }

      // sleep the process to put a little time between the images
      while(numberOfThreads >= maxThreads) {
        try {
          Thread.sleep(100);
          for (int ii=0; ii<maxThreads; ii++) {
            if(averageObject[ii] != null) {
              if(averageObject[ii].getStatus()) {
                int tag = averageObject[ii].getTag();
                output = averageObject[ii].getDoubleValue();
                for(int ll=0; ll<9; ll++) hdata[tag][ll] = output[ll];
                if(!averageObject[ii].isValueOK()) homogenizeSucceeded = false;
                maskVector[ii] = true;
                doneVector[tag] = true;
                //updateProgressFrame();
                numberOfThreads = numberOfThreads - 1;
              }
            } else {
              maskVector[ii] = true;
              numberOfThreads = numberOfThreads - 1;
            }
          }
        } catch(InterruptedException ie) {
          break;
        }
      }

      try {
        Thread.sleep(1000);
      } catch(InterruptedException ie) {
      }

    } // end of the homogenization loop

    // finally, we have to wait until all the results have been computed
    while(numberOfThreads > 0) {
      try {
        Thread.sleep(100);
        for (int ii=0; ii<maxThreads; ii++) {
          if(averageObject[ii] != null) {
            if(averageObject[ii].getStatus()) {
              int tag = averageObject[ii].getTag();
              output = averageObject[ii].getDoubleValue();
              for(int ll=0; ll<9; ll++) hdata[tag][ll] = output[ll];
              if(!averageObject[ii].isValueOK()) homogenizeSucceeded = false;
              maskVector[ii] = true;
              doneVector[tag] = true;
              //updateProgressFrame();
              numberOfThreads = numberOfThreads - 1;
            }
          }
        }

      } catch(InterruptedException ie) {
        break;
      }

      try {
        Thread.sleep(100);
      } catch(InterruptedException ie) {
      }

    }

    // reset the homogenization done flag
    homogenizationDone = true;

    // dismiss the frame when the processes are done
    //dismissProgressFrame();

    // print a message
    if(showStatus) workingWorkingWorking(false, "Homogenization Done");

    // finally, test to see if the homogenization was completed correctly
    if(!homogenizeSucceeded) {
      workingWorkingWorking(false, "Homogenization Failed");
      return false;
    }

    return true;

  }

  public void keepHomogenizationResult() {

    // set up the associated map object with the original
    String aname = null;
    MapNameInputDialog mnid = new MapNameInputDialog(this);
    mnid.show();
    if(mnid.getState()) {
      aname = mnid.getMapName();
      if(aname == null) {
        workingWorkingWorking(false,
                    "Homogenization Failed: No Map Name Specified");
        printMessage("Name Input Error - Name Is Null");
        return;
      }
    } else {
      workingWorkingWorking(false, "Homogenization Failed: Name Input Error");
      printMessage("Name Input Error - Name Input Aborted");
      return;
    }
    // build a map object for storage purposes and later use
    try {
      MapObject mo = new MapObject(aname, hdata, nxhom, nyhom, nzhom);
      workingMapObject.addAssociatedMap(mo);
    } catch(BadArraySizeException bse) {
      workingWorkingWorking(false, "Homogenization Failed: Bad Array Size");
      printMessage("BadArraySizeException Encountered");
      return;
    }
    // finally, select this as the working map in the list
    buildMapLists();

  }

  public void keepEllipticSolutionResult() {
    if(workingMapObject == null) return;
    String mname = workingMapObject.getMapName();
    String dirname = "ellipsol";
    String esname = "ellipsol.map";
    int nxv = workingMapObject.getNumberOfXBlocks();
    int nyv = workingMapObject.getNumberOfYBlocks();
    int nzv = workingMapObject.getNumberOfZBlocks();
    try {
      MapObject mo = new MapObject(dirname, ellipsol, nxv, nyv, nzv);
      workingMapObject.addAssociatedMap(mo);
      buildMapLists();
    } catch(BadArraySizeException bse) {
      workingWorkingWorking(false,"keepEllipticSolution Failed: Ellipsol Save");
      printMessage("BadArraySizeException Encountered");
      return;
    }
    dirname = "ellipxflux";
    esname = "ellipxflux.map";
    try {
      MapObject mo = new MapObject(dirname, ellipxflux, nxv+1, nyv, nzv);
      workingMapObject.addAssociatedMap(mo);
      buildMapLists();
    } catch(BadArraySizeException bse) {
      workingWorkingWorking(false,"keepEllipticSolution Failed: Gradient Save");
      printMessage("BadArraySizeException Encountered");
      return;
    }
    dirname = "ellipyflux";
    esname = "ellipyflux.map";
    try {
      MapObject mo = new MapObject(dirname, ellipyflux, nxv, nyv+1, nzv);
      workingMapObject.addAssociatedMap(mo);
      buildMapLists();
    } catch(BadArraySizeException bse) {
      workingWorkingWorking(false,"keepEllipticSolution Failed: Gradient Save");
      printMessage("BadArraySizeException Encountered");
      return;
    }
    dirname = "ellipzflux";
    esname = "ellipzflux.map";
    try {
      MapObject mo = new MapObject(dirname, ellipzflux, nxv, nyv, nzv+1);
      workingMapObject.addAssociatedMap(mo);
      buildMapLists();
    } catch(BadArraySizeException bse) {
      workingWorkingWorking(false,"keepEllipticSolution Failed: Gradient Save");
      printMessage("BadArraySizeException Encountered");
      return;
    }
  }

  public void setEllipticProblemData() {
    EllipticProblemDataInputDialog bcid
                = new EllipticProblemDataInputDialog(this, 0, 0);
    bcid.setEllipticProblemData(bcType);
    bcid.show();
    if(bcid.getState()) {
      bcType = bcid.getEllipticProblemData();
    }
  }

  public void batchHomogenize(boolean showOut, boolean showError,
                              boolean showStatus) {
    MapObject mo;

    // print out some loading instructions
    outputBatchInstructions();
    if(showStatus) workingWorkingWorking(true,"Batch Homogenization Working");

    // get the input paramters from the text areas in the gui and set up the
    // homogenization mask
    getInput();
    homogenizationMask = new int [nnhom];
    setHomogenizationMask();

    // check to see if the method is set
    if(homogenizationMethod < 0 || homogenizationMethod > nmethods) {
      if(showStatus) workingWorkingWorking(false,
                  "Batch Homogenization Failed: no homogenization method set ");
      return;
    }

    // test the homogenization mask to make sure that appropriate methods have
    // been set
    if(!testHomogenizationMask()) {
      if(showStatus) workingWorkingWorking(false,
                      "Batch Homogenization Failed: Homogenization Mask Wrong");
      return;
    }

    // first get the map file to use
    FileDialog fd = new FileDialog(this);
    fd.show();
    String dirString = fd.getDirectory();
    if(dirString == null) {
      if(showStatus) workingWorkingWorking(false,
                      "Batch Homogenization Failed: null directory string");
      if(showError)
                  printMessage("Null Directory String In Batch Homogenization");
      return;
    }
    // test the folder chosen by the user
    File f = new File(dirString);
    if(!f.exists()) {
      if(showStatus) workingWorkingWorking(false,
                            "Batch Homogenization Failed: null directory");
      if(showError) printMessage("Null Directory In Batch Homogenization");
      return;
    }
    if(!f.isDirectory()) {
      if(showStatus) workingWorkingWorking(false,
                            "Batch Homogenization Failed: not a directory");
      if(showError)
                printMessage("File Is Not A Directory In Batch Homogenization");
      return;
    }

    // get a listing of the directory
    String [] files = f.list();
    if(files == null) {
      if(showStatus) workingWorkingWorking(false,
                     "Batch Homogenization Failed: no files in directory");
      if(showError)
                  printMessage("No Files In Directory In Batch Homogenization");
      return;
    } 
    int nf = files.length;
    if(nf <= 0) {
      if(showStatus) workingWorkingWorking(false,
                     "Batch Homogenization Failed: no files in directory");
      if(showError)
                  printMessage("No Files In Directory In Batch Homogenization");
      return;
    } 

    // set up a map object
    try {
      mo = new MapObject();
    } catch(BadArraySizeException bse) {
      if(showStatus) workingWorkingWorking(false,
                     "Batch Homogenization Failed: mapObject creation failed");
      if(showError)
             printMessage("Batch Homogeniation Failed: " + bse.toString());
      return;
    }
    mo.setMapName("batchMapObject");

    // loop over the files trying to homogenize - the files for which the
    // homogenization fails will be indicated
    for(int i=0; i<nf; i++) {
      String os = "";
      if(showStatus) workingWorkingWorking(true,
                              "Batch Homogenization On: " + files[i]);
      if(!mo.loadMap(dirString + fileSeparator + files[i])) {
        os = files[i] + ": Homogenization Failed\n";
      } else {
        // set up the current map information in the application
        workingMapObject = mo;
        getWorkingMapData();
        if(homogenize(false,false,false)) {
          if(hdata != null) {
            os = files[i]
               + ": Homogenization Values:  \n";
            if(hdata[0][0] != null) {
               os = os
                  + "     [ "
                  + digitFilter(hdata[0][0].doubleValue(), 4)
                  + "     ";
            } else {
               os = os
                  + "     [ "
                  + "null"
                  + "     ";
            }
            if(hdata[0][1] != null) {
               os = os
                  + digitFilter(hdata[0][1].doubleValue(), 4) + "     ";
            } else {
               os = os
                  + "null"
                  + "     ";
            }
            if(hdata[0][2] != null) {
               os = os
                  + digitFilter(hdata[0][2].doubleValue(), 4) + " ]\n";
            } else {
               os = os
                  + "null" + " ]\n";
            }
            if(hdata[0][3] != null) {
               os = os
                  + "     [ "
                  + digitFilter(hdata[0][3].doubleValue(), 4)
                  + "     ";
            } else {
               os = os
                  + "     [ "
                  + "null"
                  + "     ";
            }
            if(hdata[0][4] != null) {
               os = os
                  + digitFilter(hdata[0][4].doubleValue(), 4) + "     ";
            } else {
               os = os
                  + "null"
                  + "     ";
            }
            if(hdata[0][5] != null) {
               os = os
                  + digitFilter(hdata[0][5].doubleValue(), 4) + " ]\n";
            } else {
               os = os
                  + "null" + " ]\n";
            }
            if(hdata[0][6] != null) {
               os = os
                  + "     [ "
                  + digitFilter(hdata[0][6].doubleValue(), 4)
                  + "     ";
            } else {
               os = os
                  + "     [ "
                  + "null"
                  + "     ";
            }
            if(hdata[0][7] != null) {
               os = os
                  + digitFilter(hdata[0][7].doubleValue(), 4) + "     ";
            } else {
               os = os
                  + "null"
                  + "     ";
            }
            if(hdata[0][8] != null) {
               os = os
                  + digitFilter(hdata[0][8].doubleValue(), 4) + " ]\n";
            } else {
               os = os
                  + "null" + " ]\n";
            }
          } else {
            os = files[i] + ": Homogenization Values:  output null"
               + "\n";
          }
        } else {
          os = files[i] + ": Homogenization Values:  homogenization failed"
             + "\n";
        }
        if(showOut) {
          outputTextArea.append(os);
          outputTextArea.setCaretPosition(10000000);
        }
      }
    }
    if(showStatus) workingWorkingWorking(false,
                                        "Done With Batch Homogenization");

  }

  public void setHomogenizationMask() {

    nmethods = methods.length;
    homogenizationMask = new int [nnhom];

    // illegal homogenization method
    if(homogenizationMethod < 0 || homogenizationMethod > nmethods) {
      for(int i=0; i<nnhom; i++) homogenizationMask[i] = -1;
      return;
    }

    // constant homogenization method
    if(homogenizationMethod > 0 && homogenizationMethod <= nmethods) {
      for(int i=0; i<nnhom; i++) homogenizationMask[i] = homogenizationMethod;
    }

    // set the homogenization block by block
    if(homogenizationMethod == 0) {
      // set up an array that drops the first method
      HomogenizationMaskDialog hmd
               = new HomogenizationMaskDialog(applicationFrame,
                                              homogenizationMask,
                                              methods,
                                              DEFAULT_METHOD,
                                              nxhom,
                                              nyhom,
                                              nzhom);
      hmd.show();
      if(hmd.getState()) {
        homogenizationMask = hmd.getHomogenizationMask();
        // this line increments the method up by one
        for(int i=0; i<nnhom; i++) homogenizationMask[i]++;
      }
      hmd.setVisible(false);
      hmd = null;
      return;
    }

  }

  public boolean testHomogenizationMask() {
    // test for a null mask
    if(homogenizationMask == null) {
      printMessage("homogenizationMask is null - no mask set");
      return false;
    }
    // test the mask for the appropriate size
    int n = homogenizationMask.length;
    if(n != nnhom) {
      printMessage("homogenizationMask is the wrong size n = "
                  + n + "  nnhom = " + nnhom);
      return false;
    }
    // loop over the mask testing for each block
    for(int i=0; i<nnhom; i++) {
      if(homogenizationMask[i] < 1) {
        printMessage("homogenization mask value too small:  i = "
                    + "   value = " + homogenizationMask[i]);
        return false;
      }
      if(homogenizationMask[i] > nmethods) {
        printMessage("homogenization mask value too large:  i = "
                    + "   value = " + homogenizationMask[i]);
        return false;
      }
    }
    return true;
  }

  public boolean ellipticSolution(boolean showOut, boolean showError,
                               boolean showStatus) {

    boolean cp = true;
    // test to see if the solution has already been computed
    MapObject emo = workingMapObject.getAssociatedMap("ellipsol");
    if(emo != null) {
      cp = false;
      ellipsol = emo.getData();
      if(ellipsol == null) cp = true;
      // try to load the flux
      MapObject gmo = workingMapObject.getAssociatedMap("ellipxflux");
      if(gmo != null) ellipxflux = gmo.getData();
      gmo = workingMapObject.getAssociatedMap("ellipyflux");
      if(gmo != null) ellipyflux = gmo.getData();
      gmo = workingMapObject.getAssociatedMap("ellipzflux");
      if(gmo != null) ellipzflux = gmo.getData();
      int nnp = workingMapObject.getTotalBlocks();
      flow = new Double[nnp][1];
      for(int i=0; i<nnp; i++) flow[i][0] = new Double(0.0);
    }
    if(!cp) cp = askQuestion("Solution has been computed: Recompute?");
    if(!cp) workingWorkingWorking(false, "No Elliptic Solution Computed");

    // flush the text in the output area
    outputTextArea.setText("\n\n Elliptic Solution Process Output:\n\n");

    // make sure the data set is setup
    if(!getWorkingMapData()) {
      if(showStatus) workingWorkingWorking(false,
                                           "Elliptic Solution Failed: No Data");
      if(showError) printMessage("Elliptic Solution Failed: No Data");
      return false;
    } 

    // boundary condition information for the solution
    PointSources pointSources = null;
    String fbfxl = "0.0";
    String fbfxr = "0.0";
    String fbfyl = "0.0";
    String fbfyr = "0.0";
    String fbfzl = "0.0";
    String fbfzr = "0.0";

    // start things up
    workingWorkingWorking(true, " Computing Solution of Elliptic Problem");

    // set up the grid information
    int nxv = workingMapObject.getNumberOfXBlocks();
    int nyv = workingMapObject.getNumberOfYBlocks();
    int nzv = workingMapObject.getNumberOfZBlocks();
    double xa = workingMapObject.getxLeft();
    double xb = workingMapObject.getxRight();
    double ya = workingMapObject.getyLeft();
    double yb = workingMapObject.getyRight();
    double za = workingMapObject.getzLeft();
    double zb = workingMapObject.getzRight();
    double [] xg = workingMapObject.getxGrid();
    double [] yg = workingMapObject.getyGrid();
    double [] zg = workingMapObject.getzGrid();
    getSelectedLevel1MapObject();
    int nxr = 1;
    int nyr = 1;
    int nzr = 1;
    nzr = nzr / nzv;
    if(currentLevel1MapObject != null) {
      nxr = currentLevel1MapObject.getNumberOfXBlocks();
      nxr = nxr / nxv;
      nyr = currentLevel1MapObject.getNumberOfYBlocks();
      nyr = nyr / nyv;
      nzr = currentLevel1MapObject.getNumberOfZBlocks();
      nzr = nzr / nzv;
    }

    // set the forcing function if we have a point source deal
    String ftype = EllipticRectMFEObject.SYMBOLIC_FUNCTION_DEFINITION;
    if(bcType.equals(EllipticProblemDataInputDialog.POINT_SOURCE)) {
      pointSources = new PointSources();
      pointSources.addSource(xg[0], yg[0], zg[0], 1.0);
      pointSources.addSource(xg[nxv], yg[nyv], zg[nzv], -1.0);
      ftype = EllipticRectMFEObject.POINT_SOURCE_FUNCTION_DEFINITION;
      fbfxl = "0.0";
      fbfxr = "0.0";
      fbfyl = "0.0";
      fbfyr = "0.0";
      fbfzl = "0.0";
      fbfzr = "0.0";
    } else if(bcType.equals(EllipticProblemDataInputDialog.X_FLUX)) {
      pointSources = null;
      ftype = EllipticRectMFEObject.POINT_SOURCE_FUNCTION_DEFINITION;
      fbfxl = "1.0";
      fbfxr = "1.0";
      fbfyl = "0.0";
      fbfyr = "0.0";
      fbfzl = "0.0";
      fbfzr = "0.0";
    } else if(bcType.equals(EllipticProblemDataInputDialog.Y_FLUX)) {
      pointSources = null;
      ftype = EllipticRectMFEObject.POINT_SOURCE_FUNCTION_DEFINITION;
      fbfxl = "0.0";
      fbfxr = "0.0";
      fbfyl = "1.0";
      fbfyr = "1.0";
      fbfzl = "0.0";
      fbfzr = "0.0";
    } else if(bcType.equals(EllipticProblemDataInputDialog.Z_FLUX)) {
      pointSources = null;
      ftype = EllipticRectMFEObject.POINT_SOURCE_FUNCTION_DEFINITION;
      fbfxl = "0.0";
      fbfxr = "0.0";
      fbfyl = "0.0";
      fbfyr = "0.0";
      fbfzl = "1.0";
      fbfzr = "1.0";
    }

    // get the data
    int nnp = workingMapObject.getTotalBlocks();
    int nvelx = ( nxv + 1 ) * nyv * nzv;
    int nvely = nxv * ( nyv + 1 ) * nzv;
    int nvelz = nxv * nyv * ( nzv + 1 );

    Double [][] AS = workingMapObject.getData();
    double [] as = new double [9*nnp];
    // pull off the tensor
    int shft = 0;
    for(int l=0; l<9; l++) {
      for(int i=0; i<nnp; i++) as[i+shft] = AS[i][l].doubleValue();
      shft = shft + nnp;
    }

    // build the elliptic solver
    try {
      ermfe = new EllipticRectMFEObject(
                         EllipticRectMFEObject.NUMERIC_FUNCTION_DEFINITION,
                         null,
                         as,
                         ftype,
                         "0.0",
                         null,
                         pointSources,
                         EllipticRectMFEObject.SYMBOLIC_FUNCTION_DEFINITION,
                         EllipticRectMFEObject.NEUMANN_BOUNDARY	,
                         fbfxl, fbfxr, fbfyl, fbfyr, fbfzl, fbfzr,
                         null, null, null, null, null, null,
                         null, null, null,
                         null, null, null,
                         null, null, null,
                         xa, xb, ya, yb, za, zb,
                         nxv, nyv, nzv, nxr, nyr, nzr,
                         "INPUT ARRAY GRID",
                         "INPUT ARRAY GRID",
                         "INPUT ARRAY GRID",
                         null, null, null,
                         null, null, null,
                         xg, yg, zg,
                         null, null, null,
                         0.0, 1.0, 1, outputTextArea, cp);
      // if the solution has been computed, get it
      if(cp) {
        double [] output = ermfe.getSolution();
        if(output != null) {
          ellipsol = new Double[nnp][1];
          for(int i=0; i<nnp; i++) ellipsol[i][0] = new Double(output[i]);
          ellipxflux = new Double[nvelx][1];
          output = ermfe.getXFlux();
          for(int i=0; i<nvelx; i++) ellipxflux[i][0] = new Double(output[i]);
          ellipyflux = new Double[nvely][1];
          output = ermfe.getYFlux();
          for(int j=0; j<nvely; j++) ellipyflux[j][0] = new Double(output[j]);
          ellipzflux = new Double[nvelz][1];
          output = ermfe.getZFlux();
          for(int k=0; k<nvelz; k++) ellipzflux[k][0] = new Double(output[k]);
          flow = new Double[nnp][1];
          for(int i=0; i<nnp; i++) flow[i][0] = new Double(0.0);
        }
      } else {
        if(ellipsol != null) {
          double [] output = new double[nnp];
          for(int i=0; i<nnp; i++) output[i] = ellipsol[i][0].doubleValue();
          ermfe.setSolution(output);
        }
        if(ellipxflux != null) {
          double [] output = new double[nvelx];
          for(int i=0; i<nvelx; i++) output[i] = ellipxflux[i][0].doubleValue();
          ermfe.setXFlux(output);
        }
        if(ellipyflux != null) {
          double [] output = new double[nvely];
          for(int i=0; i<nvely; i++) output[i] = ellipyflux[i][0].doubleValue();
          ermfe.setYFlux(output);
        }
        if(ellipzflux != null) {
          double [] output = new double[nvelz];
          for(int i=0; i<nvelz; i++) output[i] = ellipzflux[i][0].doubleValue();
          ermfe.setZFlux(output);
        }
      }
    } catch(WellPosedProblemException wpe) {
      workingWorkingWorking(false,"Well Posed Exception in Solution Process");
      return false;
    }

    // set up the pointers to the appropriate maps
    if(comparisonMapVector == null) comparisonMapVector = new Vector();
    comparisonMapVector.add(workingMapObject);
    if(graphicsComparisonList != null)
                 graphicsComparisonList.add(workingMapObject.getMapName());

    workingWorkingWorking(false, "Elliptic Solution Computation Completed");
    return true;

  }

  public boolean testForEllipticSolution() {
    if(ermfe == null) {
      printMessage("Elliptic Solver Object is null\n\n"
                       + "Click on the Elliptic Solver button\n\n");
      return false;
    }
    if(ellipsol == null) {
      printMessage("Elliptic solution array, ellipsol, is null");
      return false;
    } else {
      if(ellipsol.length <= 0) {
        printMessage("Elliptic solution array, ellipsol, has zero length.");
        return false;
      }
    }
    if(ellipxflux == null) {
      printMessage("Elliptic flux array, ellipxflux, is null");
      return false;
    } else {
      if(ellipxflux.length <= 0) {
        printMessage("Elliptic flux array, ellipxflux, has zero length.");
        return false;
      }
    }
    if(ellipyflux == null) {
      printMessage("Elliptic flux array, ellipyflux, is null");
      printMessage("Elliptic Y-Flux Not Computed");
      return false;
    } else {
      if(ellipyflux.length <= 0) {
        printMessage("Elliptic flux array, ellipyflux, has zero length.");
        return false;
      }
    }
    if(ellipzflux == null) {
      printMessage("Elliptic flux array, ellipzflux, is null");
      return false;
    } else {
      if(ellipzflux.length <= 0) {
        printMessage("Elliptic flux array, ellipzflux, has zero length.");
        return false;
      }
    }
    return true;
  }

  public void showEllipticSolutionImage() {
    ellipticSolutionFrame = new Frame("Elliptic Solution and Flux Graphics");
    ellipticSolutionFrame.setSize(3*(imageWidth+20), (imageHeight+100));
      solutionDisplayPanel = new Panel();
      solutionDisplayPanel.setLayout(new GridLayout(1,0));
        ellipticGraphicsPanel = new Panel();
        ellipticGraphicsPanel.setLayout(new BorderLayout());
        ellipticGraphicsPanel.setBackground(Color.black);
        ellipticGraphicsPanel.setForeground(Color.white);
          if(ellipsolImageCanvas == null)
            ellipsolImageCanvas = new ImageCanvas(imageWidth, imageHeight);
          ellipticSolutionLabel = new Label("Elliptic Solution", Label.CENTER);
          ellipticSolutionLabel.setBackground(Color.black);
          ellipticSolutionLabel.setForeground(Color.white);
        ellipticGraphicsPanel.add("Center", ellipsolImageCanvas);
        ellipticGraphicsPanel.add("South", ellipticSolutionLabel);
        fluxGraphicsPanel = new Panel();
        fluxGraphicsPanel.setLayout(new BorderLayout());
        fluxGraphicsPanel.setBackground(Color.black);
        fluxGraphicsPanel.setForeground(Color.white);
          if(ellipfluxImageCanvas == null)
            ellipfluxImageCanvas = new ImageCanvas(imageWidth, imageHeight);
          fluxSolutionLabel = new Label("Flux Magnitude", Label.CENTER);
          fluxSolutionLabel.setBackground(Color.black);
          fluxSolutionLabel.setForeground(Color.white);
        fluxGraphicsPanel.add("Center", ellipfluxImageCanvas);
        fluxGraphicsPanel.add("South", fluxSolutionLabel);
        flowGraphicsPanel = new Panel();
        flowGraphicsPanel.setLayout(new BorderLayout());
        flowGraphicsPanel.setBackground(Color.black);
        flowGraphicsPanel.setForeground(Color.white);
          if(flowImageCanvas == null)
               flowImageCanvas = new ImageCanvas(imageWidth, imageHeight);
          flowSolutionLabel = new Label("Flow Solution", Label.CENTER);
          flowSolutionLabel.setBackground(Color.black);
          flowSolutionLabel.setForeground(Color.white);
        flowGraphicsPanel.add("Center", flowImageCanvas);
        flowGraphicsPanel.add("South", flowSolutionLabel);
      solutionDisplayPanel.add(ellipticGraphicsPanel);
      solutionDisplayPanel.add(fluxGraphicsPanel);
      solutionDisplayPanel.add(flowGraphicsPanel);
    ellipticSolutionFrame.add(solutionDisplayPanel);
    Point pt = this.getLocation();
    ellipticSolutionFrame.setLocation(new Point(((int)pt.getX())+40,
                                               ((int)pt.getY())+40));
    workingWorkingWorking(true, "Building Elliptic Solution Image");

    ermfe.setImageDimensions(imageWidth, imageHeight);
    int nzv = workingMapObject.getNumberOfZBlocks();
    if(nzv > 1) {
      for(int i=0; i<nzv; i++) {
        ellipsolImageCanvas.redraw(0, 0, ermfe.getSolPPMImageXY(this, i));
        try {
          Thread.sleep(2500);
        } catch(InterruptedException ie) {
        }
      }
    } else {
      ellipsolImageCanvas.redraw(0, 0, ermfe.getSolPPMImageXY(this, 0));
    }
    if(nzv > 1) {
      for(int i=0; i<nzv; i++) {
        ellipfluxImageCanvas.redraw(0, 0, ermfe.getFluxPPMImageXY(this, i));
        try {
          Thread.sleep(2500);
        } catch(InterruptedException ie) {
        }
      }
    } else {
      ellipfluxImageCanvas.redraw(0, 0, ermfe.getFluxPPMImageXY(this, 0));
    }

    workingWorkingWorking(false, "Done Building Elliptic Solution Image");

    ellipticSolutionFrame.show();
    paintAll(ellipticSolutionFrame.getGraphics());

  }

  public void createEllipticSolutionImage() {
    if(ermfe == null) return;
    if(!ermfe.testSolution()) return;
    workingWorkingWorking(true, "Writing Elliptic Solution Image");
    ermfe.writeSolPPMImageXY(this, 0, new File("ellip.ppm"));
    workingWorkingWorking(false, "Done Writing Elliptic Solution Image");
  }

  public void createEllipticFluxImage() {
    if(ermfe == null) return;
    if(!ermfe.testSolution()) return;
    workingWorkingWorking(true, "Writing Elliptic Flux Image");
    ermfe.writeFluxPPMImageXY(this, 0, new File("ellip.ppm"));
    workingWorkingWorking(false, "Done Writing Elliptic Flux Image");
  }

  public void flowSolution() {
    if(upwindSaturationObjects == null) {
      int nxv = workingMapObject.getNumberOfXBlocks();
      int nyv = workingMapObject.getNumberOfYBlocks();
      int nzv = workingMapObject.getNumberOfZBlocks();
      int nvelx = ( nxv + 1 ) * nyv * nzv;
      int nvely = nxv * ( nyv + 1 ) * nzv;
      int nvelz = nxv * nyv * ( nzv + 1 );
      double [] xg = workingMapObject.getxGrid();
      double [] yg = workingMapObject.getyGrid();
      double [] zg = workingMapObject.getzGrid();
      int nnp = workingMapObject.getTotalBlocks();
      flow = new Double[nnp][1];
      for(int i=0; i<nnp; i++) flow[i][0] = new Double(0.0);
      double [] data = new double[nnp];
      for(int i=0; i<nnp; i++) data[i] = flow[i][0].doubleValue();
      double [] vx = new double[nvelx];
      for(int i=0; i<nvelx; i++) vx[i] = ellipxflux[i][0].doubleValue();
      double [] vy = new double[nvely];
      for(int j=0; j<nvely; j++) vy[j] = ellipyflux[j][0].doubleValue();
      double [] vz = new double[nvelz];
      for(int k=0; k<nvelz; k++) vz[k] = ellipzflux[k][0].doubleValue();

      // set up sources for the flow - the source is to put a source in the
      // corners
      PointSources ps = new PointSources();
      double xx = 0.5 * ( xg[1] + xg[0] );
      double yy = 0.5 * ( yg[1] + yg[0] );
      double zz = 0.5 * ( zg[1] + zg[0] );
      ps.addSource(xx, yy, zz, 1.0);
      xx = 0.5 * ( xg[nxv-1] + xg[nxv] );
      yy = 0.5 * ( yg[nyv-1] + yg[nyv] );
      zz = 0.5 * ( zg[nzv-1] + zg[nzv] );
      ps.addSource(xx, yy, zz, 0.0);
      if(bcType.equals(EllipticProblemDataInputDialog.X_FLUX)) {
        ps = new PointSources();
        for(int k=0; k<nzv; k++) {
          double zzl = 0.5 * ( zg[k] + zg[k+1] );
          for(int j=0; j<nyv; j++) {
            double yyl = 0.5 * ( yg[j] + yg[j+1] );
            xx = 0.5 * ( xg[1] + xg[0] );
            ps.addSource(xx, yyl, zzl, 1.0);
            xx = 0.5 * ( xg[nxv-1] + xg[nxv] );
            ps.addSource(xx, yyl, zzl, 0.0);
          }
        }
      } else if(bcType.equals(EllipticProblemDataInputDialog.Y_FLUX)) {
        ps = new PointSources();
        for(int k=0; k<nzv; k++) {
          double zzl = 0.5 * ( zg[k] + zg[k+1] );
          for(int i=0; i<nxv; i++) {
            double xxl = 0.5 * ( xg[i] + xg[i+1] );
            yy = 0.5 * ( yg[1] + yg[0] );
            ps.addSource(xxl, yy, zzl, 1.0);
            yy = 0.5 * ( yg[nyv-1] + yg[nyv] );
            ps.addSource(xxl, yy, zzl, 0.0);
          }
        }
      } else if(bcType.equals(EllipticProblemDataInputDialog.Z_FLUX)) {
        ps = new PointSources();
        for(int j=0; j<nyv; j++) {
          double yyl = 0.5 * ( yg[j] + yg[j+1] );
          for(int i=0; i<nxv; i++) {
            double xxl = 0.5 * ( xg[i] + xg[i+1] );
            zz = 0.5 * ( zg[1] + zg[0] );
            ps.addSource(xxl, yyl, zz, 1.0);
            zz = 0.5 * ( zg[nzv-1] + zg[nzv] );
            ps.addSource(xxl, yyl, zz, 0.0);
          }
        }
      }
      try {
        upwindSaturationObjects = new UpwindSaturationObject[1];
        upwindSaturationObjects[0]
                   = new UpwindSaturationObject(data, ps,
                                                nxv, nyv, nzv,
                                                xg, yg, zg,
                                                vx, vy, vz,
                                                1.0,
                                                outputTextArea);
        fsbPanel.removeAll();
        fsbPanel.add(flowSolverStepButton);
        fsbPanel.add(flowSolverStepTextField);
        fsbPanel.add(flowSolverQuitButton);
        paintAll(fsbPanel.getGraphics());
      } catch(WellPosedProblemException wpe) {
        workingWorkingWorking(false,"Well Posed Exception in Solution Process");
        return;
      }
    }
  }

  public void flowSolutionStep() {

    workingWorkingWorking(true, "Taking a flow solution step");
    int nstp = 1;
    String s = flowSolverStepTextField.getText();
    try{
      Integer intval = Integer.valueOf(s);
      nstp = intval.intValue();
    } catch (NumberFormatException nfe) {
    }
    for(int i=0; i<nstp; i++) upwindSaturationObjects[0].takeAStep();

    int nnp = workingMapObject.getTotalBlocks();
    if(flow == null) flow = new Double[nnp][1];
    double [] output = upwindSaturationObjects[0].getUnew();
    for(int i=0; i<nnp; i++) flow[i][0] = new Double(output[i]);
    // do the graph
    workingWorkingWorking(true, "Building the graphics for flow solution step");
    int nzv = workingMapObject.getNumberOfZBlocks();
    if(nzv > 1) {
      for(int i=0; i<nzv; i++) {
        flowImageCanvas.redraw(0, 0,
                     upwindSaturationObjects[0].getSolPPMImageXY(this, i));
        try {
          Thread.sleep(2500);
        } catch(InterruptedException ie) {
        }
      }
    } else {
      flowImageCanvas.redraw(0, 0,
                         upwindSaturationObjects[0].getSolPPMImageXY(this, 0));
    }
    workingWorkingWorking(false, "Done with flow solution step");
  }

  public void flowSolutionQuit() {
    upwindSaturationObjects[0] = null;
    fsbPanel.removeAll();
    fsbPanel.add(flowSolverButton);
    paintAll(fsbPanel.getGraphics());
  }

  public void doComparison() {
    if(graphicsComparisonList == null) return;
    int nl = graphicsComparisonList.getItemCount();
    if(nl <= 0) return;
    if(nl > 3) nl = 3;
  }

  //
  ///////////////////////////////////////////////////////////////////////////
  //                                                                       //
  //   V. Input Output Methods                                             //
  //                                                                       //
  //     - loadMap()                                                       //
  //     - importMap()                                                     //
  //     - printOutputArea()                                               //
  //     - saveMap()                                                       //
  //     - exportMap()                                                     //
  //     - mapNameConflict(String s)                                       //
  //     - outputLoadInstructions()                                        //
  //     - displayValues()                                                 //
  //     - exit()                                                          //
  //     - initializeProgressFrame()                                       //
  //     - updateProgressFrame()                                           //
  //     - dismissProgressFrame()                                          //
  //     - digitFilter()                                                   //
  //                                                                       //
  ///////////////////////////////////////////////////////////////////////////
  //

  public boolean loadMap() {
    MapObject mo;
    // print out some loading instructions
    outputLoadInstructions();
    workingWorkingWorking(true,"Loading Map");
    String mapNameString = null;
    // first get the map file to use
    FileDialog fd = new FileDialog(this);
    fd.show();
    String dirString = fd.getDirectory();
    mapNameString = fd.getFile();
    if(mapNameString == null) {
      workingWorkingWorking(false, "Map Load Failed - null mapNameString");
      printMessage("No Map File Folder Specified");
      return false;
    }
    // at this point we can set the name of the map
    StringTokenizer st = new StringTokenizer(mapNameString.trim(),
                                             ". \n\t\r" + fileSeparator);
    String s = "zztempzz.zzzz";
    int n = st.countTokens();
    for(int i=0; i<n-1; i++) s = st.nextToken();
    // test for the existence of the map file info
    //File f = new File(s, mapNameString);
    File f = new File(dirString, mapNameString);
    if(!f.exists()) {
      workingWorkingWorking(false, "Map Load Failed - null file f");
      printMessage("Map File Not Found");
      return false;
    }
    if(mapNameConflict(mapNameString)) {
      workingWorkingWorking(false, "Map Load Failed - map name conflict");
      printMessage("Map File Already Loaded or Name is null");
      return false;
    }
    if(mapNameConflict(mapNameString + ".map")) {
      workingWorkingWorking(false, "Map Load Failed - map already loaded");
      printMessage("Map File Already Loaded or Name is null");
      return false;
    }
    currentMapName = s;
    // set up a map object
    try {
      mo = new MapObject();
    } catch(BadArraySizeException bse) {
      workingWorkingWorking(false, "Map Load Failed - cannot create mapObject");
      printMessage("Map Load Failed: " + bse.toString());
      return false;
    }
    mo.setMapName(s);
    if(!mo.loadMaps()) {
      workingWorkingWorking(false, "Map Load Failed - loadMaps()");
      printMessage("Map Load Failed");
      return false;
    }
    // now add the map to the map object vector
    mapObjectVector.addElement(mo);
    currentMapObject = mo;
    currentMapIndex = mapObjectVector.size() - 1;
    currentLevel1MapIndex = -1;
    currentLevel2MapIndex = -1;
    workingMapObject = currentMapObject;
    workingMapObjectLevel = 0;
    workingMapObjectIndex = mapObjectVector.size() - 1;
    getWorkingMapData();
    buildMapLists();
    workingWorkingWorking(false, "Done Loading Map");
    return true;
  }

  public boolean importMap() {
    MapObject mo;
    // print out some loading instructions
    outputImportInstructions();
    workingWorkingWorking(true,"Importing Map");
    String mapNameString = null;
    // first get the map file to use
    FileDialog fd = new FileDialog(this);
    fd.show();
    String dirString = fd.getDirectory();
    String fileString = fd.getFile();
    if(fileString == null) {
      workingWorkingWorking(false, "Map Import Failed - null fileString");
      printMessage("No Map File Folder Specified");
      return false;
    }
    File f = new File(dirString, fileString);
    if(!f.exists()) {
      workingWorkingWorking(false, "Map Import Failed - null file f");
      printMessage("Map File Not Found");
      return false;
    }
    // determine a map name - pop up a dialog to get a name for the data set
    MapNameInputDialog mnid = new MapNameInputDialog(this);
    mnid.show();
    if(mnid.getState()) {
      mapNameString = mnid.getMapName();
      if(mapNameString == null) {
        workingWorkingWorking(false, "Map Import Failed");
        printMessage("Name Input Error - Name Is Null");
        return false;
      }
    } else {
      workingWorkingWorking(false, "Map Import Failed");
      printMessage("Name Input Error - Name Input Aborted");
      return false;
    }
    // test for a conflict
    if(mapNameConflict(mapNameString)) {
      workingWorkingWorking(false, "Map Import Failed - map name conflict");
      printMessage("Map File Already Exists or Name is null");
      return false;
    }
    if(mapNameConflict(mapNameString + ".map")) {
      workingWorkingWorking(false, "Map Import Failed - map already loaded");
      printMessage("Map File Already Import or Name is null");
      return false;
    }
    if(mapNameString == null) {
      workingWorkingWorking(false, "Map Import Failed - Null Map Name");
      printMessage("Map File Name is null");
      return false;
    }
    currentMapName = mapNameString;
    // set up a map object
    try {
      mo = new MapObject();
    } catch(BadArraySizeException bse) {
      workingWorkingWorking(false,"Map Import Failed - can't create mapObject");
      printMessage("Map Import Failed: " + bse.toString());
      return false;
    }
    mo.setMapName(mapNameString);
    if(!mo.loadMap(dirString + fileSeparator + fileString)) {
      workingWorkingWorking(false, "Map Import Failed - loadMap()");
      printMessage("Map Import Failed");
      return false;
    }
    // now add the map to the map object vector
    mapObjectVector.addElement(mo);
    currentMapObject = mo;
    currentMapIndex = mapObjectVector.size() - 1;
    currentLevel1MapIndex = -1;
    currentLevel2MapIndex = -1;
    workingMapObject = currentMapObject;
    workingMapObjectLevel = 0;
    workingMapObjectIndex = mapObjectVector.size() - 1;
    getWorkingMapData();
    buildMapLists();
    workingWorkingWorking(false, "Done Importing Map");
    return true;
  }

  public boolean printOutputArea() {
    workingWorkingWorking(true,"Printing Output Area Information");
    // first get the map file to use
    FileDialog fd = new FileDialog(this);
    fd.show();
    String dirString = fd.getDirectory();
    String fileString = fd.getFile();
    if(fileString == null) {
      workingWorkingWorking(false, "Output Area Print - null fileString");
      return false;
    }
    File f = new File(dirString, fileString);
    // check the file for a few things
    if(f == null) {
      workingWorkingWorking(false, "Output Area Print Failed - file is null!");
      return false;
    }
    // if the file exists check to see if it is ok to overwrite
    if(f.exists()) {
      if(f.isDirectory()) {
        workingWorkingWorking(false,
                              "Output Area Print Failed - file is a directory");
        return false;
      }
      if(!askQuestion("File Exists: Overwrite?")) {
        workingWorkingWorking(false,
                              "Output Area Print Failed - no overwrite!");
        return false;
      }
    }

    // get the information from the window
    String oaString = outputTextArea.getText();
    if(oaString == null) {
      workingWorkingWorking(false,
                            "Output Area Print Failed - output info null");
      return false;
    }

    // output the information to the file chosen
    try {
      FileWriter fw = new FileWriter(f);
      try {
        fw.write(oaString,0,oaString.length());
      } catch (IOException ioea) {
        workingWorkingWorking(false, "Output Area Print Failed - write failed");
        return false;
      }
      fw.close();
    } catch (IOException ioe) {
      workingWorkingWorking(false, "Output Area Print Failed - write failed");
      return false;
    }

    workingWorkingWorking(false, "Output Area Print Complete");
    return true;

  }

  public boolean saveMap() {
    workingWorkingWorking(true,"Saving Map");
    if(currentMapObject == null) {
      workingWorkingWorking(false,"Map Save Failed - null currentMapObject");
      printMessage("No Map to Save");
      return false;
    }
    if(!currentMapObject.writeMapHierarchy()) {
      workingWorkingWorking(false,"Map Save Failed - writeMapHierarchy()");
      printMessage("Map Hierarchy Save Failed");
      return false;
    }
    if(!currentMapObject.writeMaps()) {
      workingWorkingWorking(false,"Map Save Failed - writeMaps()");
      printMessage("Map Save Failed");
      return false;
    }
    workingWorkingWorking(false,"Done Saving Map");
    return true;
  }

  public boolean exportMap() {
    workingWorkingWorking(true,"Export Map");
    if(workingMapObject == null) {
      workingWorkingWorking(false,"Map Export Failed - null workingMapObject");
      printMessage("No Map to Export");
      return false;
    }
    // first get the map file to use
    FileDialog fd = new FileDialog(this);
    fd.show();
    String dirString = fd.getDirectory();
    String fileString = fd.getFile();
    if(fileString == null) {
      workingWorkingWorking(false, "Map Export Failed - null fileString");
      printMessage("No File Specified");
      return false;
    }
    File f = new File(dirString, fileString);
    if(f.exists()) {
      if(!askQuestion("File Exists: Overwrite?")) {
        workingWorkingWorking(false, "Map Export Failed - no overwrite!");
        printMessage("Map File Not Overwritten");
        return false;
      }
    }
    if(!workingMapObject.writeMap(dirString + fileSeparator + fileString)) {
      workingWorkingWorking(false,"Map Export Failed - writeMap()");
      printMessage("Map Export Failed");
      return false;
    }
    workingWorkingWorking(false,"Done Exporting Map");
    return true;
  }

  public boolean mapNameConflict(String s) {
    if(s == null) return true;
    if(s.equals("")) return true;
    StringTokenizer st = new StringTokenizer(s.trim(),
                                             ". \n\t\r" + fileSeparator);
    int n = st.countTokens();
    for(int i=0; i<n-1; i++) s = st.nextToken();
    if(mapObjectVector == null) return false;
    n = mapObjectVector.size();
    if(n <= 0) return false;
    for(int i=0; i<n; i++) {
      MapObject mo = (MapObject) mapObjectVector.get(i);
      String mn = mo.getMapName();
      if(mn != null) {
        String s1 = s.trim();
        String s2 = mn.trim();
        if(s1.equals(s2)) return true;
      }
    }
    return false;
  }

  public void outputLoadInstructions() {
    String s = "Choose a folder and file so that:\n\n"
             + "  Folder is:        mapname\n"
             + "  File is:          mapname.map\n\n"
             + "  For Example:\n\n"
             + "  Folder is:        elfdata\n"
             + "  File is:          elfdata.map\n\n"
             + "The JHomogenizer expects this naming convention!";
    outputTextArea.setText(s);
    outputTextArea.setCaretPosition(10000000);
  }

  public void outputBatchInstructions() {
    String s = "Working on Batch Homogenization.\n\n";
    outputTextArea.setText(s);
  }

  public void outputImportInstructions() {
    String s = "Choose a folder and file to import:\n\n";
    outputTextArea.setText(s);
  }

  public void displayValues(Double [][] d, int nx, int ny, int nz, int htag) {
    String s;
    int ival;
    int jval;
    outputTextArea.setText("\nSelected Map Values\n");
    if(d == null) {
      s = "\n  Input values null - no output";
      return;
    }
    int nval = nx * ny * nz;

    // check to see if the user really wants to do this
    if(nval > 25) {
      s = "\nThere are "
        + nval
        + " blocks in current map - display?";
      if(!askQuestion(s)) return;
    }

    if(nval != d.length) return;
    if(d[0].length != 9) return;
    for(int i=0; i<nval; i++) {
      jval = i / nx + 1;
      ival = i - ( jval - 1 ) * nx;
      int index = i + 1;
      s = "\nBlock: ind = " + index + "  ival= "
                        + ival + "  jval= " + jval + "\n";
      outputTextArea.append(s);
      outputTextArea.setCaretPosition(10000000);
      if(htag == 1) {
        if(methods != null && homogenizationMask != null) {
          int nh = homogenizationMask.length;
          s = "\nMethod of Averaging: ";
          if(i<nh) s = s + methods[homogenizationMask[i]-1];
          outputTextArea.append(s);
          outputTextArea.setCaretPosition(10000000);
        }
      }
      if(d[i] != null) {
        String s0 = "null";
        String s1 = "null";
        String s2 = "null";
        if(d[i][0] != null) s0 = digitFilter(d[i][0].doubleValue(), 4);
        if(d[i][1] != null) s1 = digitFilter(d[i][1].doubleValue(), 4);
        if(d[i][2] != null) s2 = digitFilter(d[i][2].doubleValue(), 4);
        s = "\n\n txx= " + s0
          + "		txy= " + s1
          + "		txz= " + s2
          + "\n";
        outputTextArea.append(s);
        outputTextArea.setCaretPosition(10000000);
        s0 = "null";
        s1 = "null";
        s2 = "null";
        if(d[i][3] != null) s0 = digitFilter(d[i][3].doubleValue(), 4);
        if(d[i][4] != null) s1 = digitFilter(d[i][4].doubleValue(), 4);
        if(d[i][5] != null) s2 = digitFilter(d[i][5].doubleValue(), 4);
        s = "\n	tyx= " + s0
          + "		tyy= " + s1
          + "		tyz= " + s2
          + "\n";
        outputTextArea.append(s);
        outputTextArea.setCaretPosition(10000000);
        s0 = "null";
        s1 = "null";
        s2 = "null";
        if(d[i][6] != null) s0 = digitFilter(d[i][6].doubleValue(), 4);
        if(d[i][7] != null) s1 = digitFilter(d[i][7].doubleValue(), 4);
        if(d[i][8] != null) s2 = digitFilter(d[i][8].doubleValue(), 4);
        s = "\n	tzx= " + s0
          + "		tzy= " + s1
          + "		tzz= " + s2
          + "\n";
        outputTextArea.append(s);
        outputTextArea.setCaretPosition(10000000);
      }
    }
  }

  public void exit() {
    boolean saveAll = false;
    if(mapObjectVector != null) {
      int n = mapObjectVector.size();
      for(int i=0; i<n; i++) {
        MapObject mo = (MapObject) mapObjectVector.get(i);
        if(!mo.isSaved()) {
          if(!saveAll) {
            String name = mo.getMapName();
            MapSaveInputDialog msid = new MapSaveInputDialog(this, name);
            msid.show();
            if(msid.getState()) {
              mo.writeMapHierarchy();
              mo.writeMaps();
              if(msid.saveAll()) saveAll = true;
            } else {
              // this short circuits the exit deal
              if(!msid.getExitState()) return;
            }
          } else {
            mo.writeMapHierarchy();
            mo.writeMaps();
          }
        }
      }
    }
    System.exit(0);
  }

  public void initializeProgressFrame() {
    int xval, yval;
    int dx = progressImageWidth / nxhom;
    int dy = progressImageHeight / nyhom;
    int i, j;
    int j2, ind;
    float rval, gval, bval;

    // check to make sure the frame exists
    if(progressPanel == null || progressFrame == null) return;

    // show the frame
    progressFrame.show();

    // remove the stuff on the panel and start over below
    progressPanel.removeAll();

    // set the image size
    progressImageWidth = 480;
    progressImageHeight = 480;
   
    progressImageCanvas = new ImageCanvas(progressImageWidth,
                                          progressImageHeight);

    // build the off screen image to draw to for the computations
    offScreenImage = null;
    offScreenImage = createImage(progressImageWidth,
                                 progressImageHeight);
    offScreenImageGraphics = offScreenImage.getGraphics();

    // clear the whole deal first
    offScreenImageGraphics.setColor(backGroundColor);
    offScreenImageGraphics.fillRect(0,
                                    0,
                                    progressImageWidth,
                                    progressImageHeight);

    // draw the boundaries
    offScreenImageGraphics.setColor(foreGroundColor);
    offScreenImageGraphics.drawLine(0,
                                    0,
                                    progressImageWidth-1,
                                    0);
    offScreenImageGraphics.drawLine(progressImageWidth-1,
                                    0,
                                    progressImageWidth-1,
                                    progressImageHeight-1);
    offScreenImageGraphics.drawLine(0,
                                    progressImageHeight-1,
                                    progressImageWidth-1,
                                    progressImageHeight-1);
    offScreenImageGraphics.drawLine(0,
                                    0,
                                    0,
                                    progressImageHeight-1);

    // draw the horizontal grid lines
    j = 1;
    yval = 0;
    while (j<nyhom) {
      yval = yval + dy;
      offScreenImageGraphics.drawLine(0,
                                      yval,
                                      progressImageWidth-1,
                                      yval);
      j++;
    }
 
    // draw the horizontal grid lines
    i = 1;
    xval = 0;
    while (i<nxhom) {
      xval = xval + dx;
      offScreenImageGraphics.drawLine(xval,
                                      0,
                                      xval,
                                      progressImageHeight-1);
      i++;
    }

    // get rid of the graphics context for now
    offScreenImageGraphics.dispose();

    // add the components
    progressPanel.add("Center", progressImageCanvas);
    progressPanel.add("South", progressDismissButton);

    // draw the blank image to the canvas
    progressImageCanvas.redraw(0,0,offScreenImage);
    progressFrame.paintAll(progressFrame.getGraphics());
    paintAll(this.getGraphics());

  }

  public void updateProgressFrame() {
    int xval, yval;
    int dx = progressImageWidth / nxhom;
    int dy = progressImageHeight / nyhom;
    int imageWidth = progressImageWidth + 2 * xoffset;
    int imageHeight = progressImageHeight + 2 * yoffset;
    int i, j;
    int j2, ind;
    float rval, gval, bval;

    // check to make sure the frame exists
    if(progressPanel == null || progressFrame == null
          || progressImageCanvas == null) return;

    // set the canvas size
    progressImageCanvas.setSize(imageWidth, imageHeight);

    // build the off screen image to draw to for the computations
    offScreenImage = null;
    offScreenImage = createImage(imageWidth, imageHeight);
    offScreenImageGraphics = offScreenImage.getGraphics();

    // clear the whole deal first
    offScreenImageGraphics.setColor(backGroundColor);
    offScreenImageGraphics.fillRect(0, 0, imageWidth, imageHeight);

    // set the colors of the blocks in the map
    for (j=1; j<=nyhom; j++) {
      j2 = ( j - 1 ) * nxhom;
      // get the cell boundary pixels in the second coordinate
      yval = ( j - 1 ) * dy + 1 + yoffset;
      for (i=1; i<=nxhom; i++) {
        ind = i + j2 - 1;
        // get the cell boundary pixels in the first coordinate
        xval = ( i - 1 ) * dx + 1 + xoffset;

        // get the value in the map block
        if(doneVector[ind]) {
          rval = (float) 1.0;
          gval = (float) 0.0;
          bval = (float) 0.0;
        } else {
          rval = (float) 1.0;
          gval = (float) 1.0;
          bval = (float) 1.0;
        }

        offScreenImageGraphics.setColor(new Color(rval,gval,bval));
        offScreenImageGraphics.fillRect(xval, yval, dx, dy);

      }
    }

    // draw the boundaries
    offScreenImageGraphics.setColor(foreGroundColor);
    offScreenImageGraphics.drawLine(xoffset,yoffset,imageWidth-xoffset,yoffset);
    offScreenImageGraphics.drawLine(imageWidth-xoffset,yoffset,
                                    imageWidth-xoffset,imageHeight-yoffset);
    offScreenImageGraphics.drawLine(xoffset,imageHeight-yoffset,
                                    imageWidth-xoffset,imageHeight-yoffset);
    offScreenImageGraphics.drawLine(xoffset,yoffset,
                                    xoffset,imageHeight-yoffset);

    // check to see if the grid is toggled on
    offScreenImageGraphics.setColor(foreGroundColor);

    // draw the horizontal grid lines
    j = 1;
    yval = yoffset;
    while (j<nyhom) {
      yval = yval + dy;
      offScreenImageGraphics.drawLine(xoffset,yval,imageWidth-xoffset,yval);
      j++;
    }
 
    // draw the horizontal grid lines
    i = 1;
    xval = xoffset;
    while (i<nxhom) {
      xval = xval + dx;
      offScreenImageGraphics.drawLine(xval,yoffset,xval,imageHeight-yoffset);
      i++;
    }

    // get rid of the graphics context for now
    offScreenImageGraphics.dispose();

    // draw the blank image to the canvas
    progressImageCanvas.redraw(0,0,offScreenImage);

    //progressFrame.paintAll(progressFrame.getGraphics());

  }

  public void dismissProgressFrame() {
    if(progressFrame == null) return;
    progressFrame.setVisible(false);
    progressFrame = null;
  }

  public static String digitFilter(double value, int n) {
    String integerPart = "0";
    String dot = ".";
    String fractionPart = "0";
    String exponent = "";
    String t = null;

    String s = Double.toString(value);
    StringTokenizer st = new StringTokenizer(s, " .eE");
    if(st.hasMoreTokens()) integerPart = st.nextToken();
    if(st.hasMoreTokens()) t = st.nextToken();
    if(t != null) {
      fractionPart = t;
      if(t.length() > n) fractionPart = t.substring(0,n);
    }
    if(st.hasMoreTokens()) exponent = st.nextToken();
    if(exponent.length() > 0) exponent = "E" + exponent;

    return integerPart + dot + fractionPart + exponent;
  }

  //
  ///////////////////////////////////////////////////////////////////////////
  //                                                                       //
  //  VI. Main Method Testing                                              //
  //                                                                       //
  //     - main()                                                          //
  //                                                                       //
  //                                                                       //
  ///////////////////////////////////////////////////////////////////////////
  //

  public static void main(String args[]) {
    JHomogenizer jh = new JHomogenizer();
    jh.pack();
    jh.show();
    //jh.setSize(600,800);
  }


  //
  ///////////////////////////////////////////////////////////////////////////
  //                                                                       //
  //  VI. Main Method Testing                                              //
  //                                                                       //
  //     - main()                                                          //
  //                                                                       //
  //                                                                       //
  ///////////////////////////////////////////////////////////////////////////
  //

  // the final static variables below determine the methods implemented into
  // the code at present
  private int nmethods = 0;
  private int homogenizationMethod = -1;
  private int [] homogenizationMask;
  private boolean homogenizationDone = false;
  private boolean moreHomogenizationCells = true;
  public final static int PERIODIC_HOMOGENIZATION= 1;
  public final static int ARITMETIC = 2;
  public final static int HARMONIC = 3;
  public final static int GEOMTRIC = 4;
  public final static int APERIODIC_HOMOGENIZATION = 5;
  // the following set up will make the Arithmetic Average the default in
  // most places in the code
  public final static String setmethods = "Set Block By Block";

//
// STEP I:
//
//    Add your method to the following list - note that the location you choose
//    in this list will determine the number to use in the switch statement in
//    STEP II of this process.
//
//    NOTE: The 0 choice in the list cooresponds to the case when the method of
//          homogenization for each block is set by the user. The following list
//          of choices is applied to all blocks.
//
//    In the following the Arithmetic average corresponds to the case when the
//    input to the switch statement is an integer value of one. The list of
//    choices is the following:
//
//      switch value      homogenization method
//      ------------      ---------------------
//
//           1            java object arithmetic average
//           2            java object harmonic average
//           3            java object geometric average
//           4            java object periodic homogenization - not working
//           5            java object periodic homogenization lbc - not working
//           6            f77 periodic homogenization code - homcode
//           7            f90 periodic homogenization code - perm
//           8            homogenization code - disp
//           9            java object periodic homogenization via wavelets
//
  public final static String [] methods = {"Arithmetic Average",
                                  "Harmonic Average",
                                  "Geometric Average",
// first version of the wavelet transform in two dimensions
                                  "Periodic Homogenization via Wavelet Transformation - 2D",
// this is the Hontans code
                                  "Linear Boundary Condition Homogenization",
                                  "HomCode: Periodic Homogenization: Fortran 77",
// this is the Vidal code
                                  "Periodic Fractured Media Homogenization",
// this is the Vidal code
                                  "Periodic Dispersion Tensor Homogenization",
// periodic homogenization with wavelets
                                  "Linear Boundary Condition via wavelets"};
  public final static int DEFAULT_METHOD = ARITMETIC;

  // the multithreading variables 
  private int maxThreads = 10;
  
  private Frame applicationFrame;
  private boolean isApplicationFrameSet = false;
  private Color dialogColor = Color.blue;

  private Panel homogenizationPanel;
    private Label titleLabel;
    private Panel displayPanel;
      private Panel inputPanel;
      private Panel inputDisplayPanel;
        private Label inputLabel;
        private Panel methodInputPanel;
          private Label methodLabel;
          private List methodList;

        private Panel mapObjectPanel;
          private Label mapObjectLabel;
          private Panel mapObjectListPanel;
            private Panel currentMapObjectPanel;
              private Label currentMapObjectLabel;
              private List level0MapObjectList;
            private Panel level1MapObjectPanel;
              private Label level1MapObjectLabel;
              private List level1MapObjectList;
            private Panel level2MapObjectPanel;
              private Label level2MapObjectLabel;
              private List level2MapObjectList;

        private Panel homogenizationInputPanel;
          private Label homogenizationInputLabel;
          private Panel hiPanel;
            private Panel hetInputPanel;
              private Label hetGridLabel;
              private Label xhetGridLabel;
              private TextField xhetGridTextField;
              private Label yhetGridLabel;
              private TextField yhetGridTextField;
              private Label zhetGridLabel;
              private TextField zhetGridTextField;
          private Panel homInputPanel;
            private Panel homGridPanel;
              private Label homGridLabel;
              private Label xhomGridLabel;
              private TextField xhomGridTextField;
              private Label yhomGridLabel;
              private TextField yhomGridTextField;
              private Label zhomGridLabel;
              private TextField zhomGridTextField;
          private Panel compRefInputPanel;
            private Panel compRefGridPanel;
              private Label compRefGridLabel;
              private Label xcompRefGridLabel;
              private TextField xcompRefGridTextField;
              private Label ycompRefGridLabel;
              private TextField ycompRefGridTextField;
              private Label zcompRefGridLabel;
              private TextField zcompRefGridTextField;
      private Panel outputPanel;
        private Label outputLabel;
        private TextArea outputTextArea;
        private boolean textOutputShowing;
        private ScrollPane graphicsOutputArea;
            private int imageWidth = 256;
            private int imageHeight = 256;
    private Frame ellipticSolutionFrame;
      private Panel solutionDisplayPanel;
        private Panel ellipticGraphicsPanel;
          private ImageCanvas ellipsolImageCanvas = null;
          private Label ellipticSolutionLabel;
        private Panel fluxGraphicsPanel;
          private ImageCanvas ellipfluxImageCanvas = null;
          private Label fluxSolutionLabel;
        private Panel flowGraphicsPanel;
          private ImageCanvas flowImageCanvas = null;
          private Label flowSolutionLabel;
    private boolean ellipticSolutionShowing = false;
    private boolean ellipticFluxShowing = false;

    private Panel bottomPanel;
      private Panel centerbottomPanel;
        private Panel statusPanel;
          private Label statusLineLabel;
          private TextField statusLine;
        private Panel datacontrols;
          private Button mapEditorButton;
          private Button getAllDataButton;
          private Button getSubregionButton;
          private Button displayValuesButton;
          private Button homogenizeButton;
          private Button batchHomogenizeButton;
        private Panel simulationPanel;
          private Panel esbPanel;
            private Button ellipticSolverButton;
            private Button ellipticBoundaryConditionsButton;
          private Panel seibPanel;
            private Button showEllipticImageButton;
          private Panel eibPanel;
            private Button ellipticImageButton;
          private Panel fibPanel;
            private Button fluxImageButton;
          private Panel fsbPanel;
            private Button flowSolverButton;
              private Button flowSolverStepButton;
              private TextField flowSolverStepTextField;
              private Button flowSolverQuitButton;
        private Panel controls;
          private Button loadMapButton;
          private Button importMapButton;
          private Button printOutputAreaButton;
          private Button saveMapButton;
          private Button exportMapButton;
          private Button exitButton;
      private Panel rightbottomPanel;
        private Label graphicsComparisonLabel;
        private List graphicsComparisonList;
        private Button graphicsComparisonButton;

  private Frame progressFrame;
    private Panel progressPanel;
      private ImageCanvas progressImageCanvas;
        private int progressImageWidth = 480;
        private int progressImageHeight = 480;
        private Image offScreenImage = null;
        private Graphics offScreenImageGraphics;
      private Button progressDismissButton;
    private int xoffset = 5;
    private int yoffset = 5;

  private WarningDialog warningDialog = null;
  private OKPromptDialog okpd = null;

  // color variables
  private Color backGroundColor = Color.white;
  private Color foreGroundColor = Color.black;
  private Color panelColor = new Color(196,196,132);
  private Color labelColor = new Color(110,132,255);

  // working variable
  private boolean working = false;
  private String workingString = "working";
  private Thread workingThread = null;

  // the mapeditor to attach to the application
  private Frame mapEditorFrame;
  private MapEditor mapEditor;
  private boolean mapEditorShowing = false;
  private boolean [] controlMask;
  private Double [][] data;
  private boolean dataSetLoaded = false;
  private Vector mapObjectVector = null;
  private MapObject currentMapObject;
  private String currentMapName;
  private int currentMapIndex = -1;
  private MapObject currentLevel1MapObject = null;
  private String currentLevel1MapName = null;
  private int currentLevel1MapIndex = -1;
  private MapObject currentLevel2MapObject = null;
  private String currentLevel2MapName = null;
  private int currentLevel2MapIndex = -1;
  private MapObject workingMapObject = null;
  private int workingMapObjectLevel = 0;
  private int workingMapObjectIndex = 0;
  private Double [][] hdata;
  private boolean [] doneVector;
  private Double [] kxx;
  private Double [] kxy;
  private Double [] kxz;
  private Double [] kyx;
  private Double [] kyy;
  private Double [] kyz;
  private Double [] kzx;
  private Double [] kzy;
  private Double [] kzz;
  private int nxdat;
  private int nydat;
  private int nzdat;
  private int nxnydat;
  private int nndat;
  private int nxhom;
  private int nyhom;
  private int nzhom;
  private int nxnyhom;
  private int nnhom;
  private int nxref;
  private int nyref;
  private int nzref;

  private EllipticRectMFEObject ermfe = null;
  private Double [][] ellipsol = null;
  private Double [][] ellipxflux = null;
  private Double [][] ellipyflux = null;
  private Double [][] ellipzflux = null;
  private Double [][] flow = null;
  private UpwindSaturationObject [] upwindSaturationObjects = null;
  private int noOfUpwindSaturationObjects;
  private String bcType = EllipticProblemDataInputDialog.POINT_SOURCE;

  private Vector comparisonMapVector = null;
  private int noOfComparisonMaps = 0;
  private Vector tensorImageVector = null;
  private Vector ellipsolImageVector = null;
  private Vector ellipfluxImageVector = null;
  private Vector flowImageVector = null;
  private Image tensorImage = null;
  private Image ellipsolImage = null;
  private Image ellipfluxImage = null;
  private Image flowImage = null;

  // the runtime object for the application
  private Runtime runtime;
  private Toolkit toolkit;
  private String fileSeparator;

  // screen size variables
  private Dimension screensize;
  private int screenresolution;

  // frame size
  private static int preferredWidth = 650;
  private static int preferredHeight = 800;
  private static int minimumWidth = 350;
  private static int minimumHeight = 500;

}
